Package: MSstatsPTM
Type: Package
Title: Statistical Characterization of Post-translational Modifications
Version: 2.13.0
Date: 2024-11-25
Description: MSstatsPTM provides general statistical methods for quantitative 
    characterization of post-translational modifications (PTMs). Supports DDA, 
    DIA, SRM, and tandem mass tag (TMT) labeling. Typically, the 
    analysis involves the quantification of PTM sites (i.e., modified residues) 
    and their corresponding proteins, as well as the integration of the 
    quantification results. MSstatsPTM provides functions for summarization, 
    estimation of PTM site abundance, and detection of changes in PTMs across 
    experimental conditions. 
Authors@R: c(
    person("Devon", "Kohler", email = "kohler.d@northeastern.edu", role = c("aut")),
    person("Tsung-Heng", "Tsai", email="tsai.tsungheng@gmail.com", role=c("aut")),
    person("Anthony", "Wu", email = "wu.anthon@northeastern.edu", role = c("aut", "cre")),
    person("Deril", "Raju", email = "raju.d@northeastern.edu", role = "aut"),
    person("Ting", "Huang", email = "thuang0703@gmail.com", role=c("aut")),
    person("Mateusz", "Staniak", email = "mtst@mstaniak.pl", role = c("aut")),
  	person("Meena", "Choi", email = "mnchoi67@gmail.com", role="aut"),
  	person("Olga", "Vitek", email = "o.vitek@northeastern.edu", role = "aut"))
License: Artistic-2.0
Depends: R (>= 4.3)
Imports: dplyr, gridExtra, stringr, stats, ggplot2, stringi, grDevices,
        MSstatsTMT, MSstatsConvert, MSstats, data.table, Rcpp,
        Biostrings, checkmate, ggrepel, plotly, htmltools
Suggests: knitr, rmarkdown, tinytest, covr, mockery, testthat (>=
        3.0.0)
LazyData: true
LinkingTo: Rcpp
VignetteBuilder: knitr
biocViews: ImmunoOncology, MassSpectrometry, Proteomics, Software,
        DifferentialExpression, OneChannel, TwoChannel, Normalization,
        QualityControl
BugReports: https://github.com/Vitek-Lab/MSstatsPTM/issues
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/MSstatsPTM
git_branch: devel
git_last_commit: c7e1180
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: yes
Packaged: 2025-10-31 23:04:14 UTC; biocbuild
Author: Devon Kohler [aut],
  Tsung-Heng Tsai [aut],
  Anthony Wu [aut, cre],
  Deril Raju [aut],
  Ting Huang [aut],
  Mateusz Staniak [aut],
  Meena Choi [aut],
  Olga Vitek [aut]
Maintainer: Anthony Wu <wu.anthon@northeastern.edu>

# Generated by roxygen2: do not edit by hand

export(DIANNtoMSstatsPTMFormat)
export(FragPipetoMSstatsPTMFormat)
export(MSstatsPTMSiteLocator)
export(MaxQtoMSstatsPTMFormat)
export(MetamorpheusToMSstatsPTMFormat)
export(PDtoMSstatsPTMFormat)
export(PStoMSstatsPTMFormat)
export(ProgenesistoMSstatsPTMFormat)
export(ProteinProspectortoMSstatsPTMFormat)
export(SkylinetoMSstatsPTMFormat)
export(SpectronauttoMSstatsPTMFormat)
export(annotSite)
export(dataProcessPTM)
export(dataProcessPlotsPTM)
export(dataSummarizationPTM)
export(dataSummarizationPTM_TMT)
export(designSampleSizePTM)
export(groupComparisonPTM)
export(groupComparisonPlotsPTM)
export(locateMod)
export(locatePTM)
export(tidyFasta)
import(ggplot2)
importFrom(Biostrings,readAAStringSet)
importFrom(MSstats,MSstatsContrastMatrix)
importFrom(MSstats,dataProcess)
importFrom(MSstats,designSampleSize)
importFrom(MSstats,groupComparison)
importFrom(MSstats,theme_msstats)
importFrom(MSstatsConvert,DIANNtoMSstatsFormat)
importFrom(MSstatsConvert,FragPipetoMSstatsFormat)
importFrom(MSstatsConvert,MSstatsBalancedDesign)
importFrom(MSstatsConvert,MSstatsLogsSettings)
importFrom(MSstatsConvert,MSstatsSaveSessionInfo)
importFrom(MSstatsConvert,MaxQtoMSstatsFormat)
importFrom(MSstatsConvert,MetamorpheusToMSstatsFormat)
importFrom(MSstatsConvert,PDtoMSstatsFormat)
importFrom(MSstatsConvert,ProgenesistoMSstatsFormat)
importFrom(MSstatsConvert,ProteinProspectortoMSstatsTMTFormat)
importFrom(MSstatsConvert,SkylinetoMSstatsFormat)
importFrom(MSstatsConvert,SpectronauttoMSstatsFormat)
importFrom(MSstatsTMT,MaxQtoMSstatsTMTFormat)
importFrom(MSstatsTMT,PDtoMSstatsTMTFormat)
importFrom(MSstatsTMT,PhilosophertoMSstatsTMTFormat)
importFrom(MSstatsTMT,groupComparisonTMT)
importFrom(MSstatsTMT,proteinSummarization)
importFrom(Rcpp,sourceCpp)
importFrom(checkmate,assertCharacter)
importFrom(checkmate,assertChoice)
importFrom(checkmate,assertLogical)
importFrom(checkmate,assertNumeric)
importFrom(data.table,`%like%`)
importFrom(data.table,`:=`)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,dcast)
importFrom(data.table,is.data.table)
importFrom(data.table,melt)
importFrom(data.table,rbindlist)
importFrom(data.table,setDT)
importFrom(data.table,setcolorder)
importFrom(data.table,setnames)
importFrom(data.table,tstrsplit)
importFrom(dplyr,bind_rows)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,ungroup)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,dev.off)
importFrom(grDevices,hcl)
importFrom(grDevices,pdf)
importFrom(gridExtra,grid.arrange)
importFrom(plotly,add_trace)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotly,style)
importFrom(plotly,subplot)
importFrom(rlang,.data)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,qnorm)
importFrom(stats,xtabs)
importFrom(stringi,stri_sub_replace_all)
importFrom(stringr,fixed)
importFrom(stringr,regex)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_match)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_split_i)
importFrom(stringr,str_trim)
useDynLib(MSstatsPTM, .registration=TRUE)

# MSstatsPTM

<!-- badges: start -->
[![Travis build status](https://travis-ci.org/Vitek-Lab/MSstatsPTM.svg?branch=master)](https://travis-ci.org/Vitek-Lab/MSstatsPTM)
[![Codecov test coverage](https://codecov.io/gh/Vitek-Lab/MSstatsPTM/branch/master/graph/badge.svg)](https://codecov.io/gh/Vitek-Lab/MSstatsPTM?branch=master)
<!-- badges: end -->


MSstatsPTM provides general statistical methods for quantitative characterization of post-translational modifications (PTMs). Supports DDA, DIA, and tandem mass tag (TMT) labeling. Typically, the analysis involves the quantification of PTM sites (i.e., modified residues) and their corresponding proteins, as well as the integration of the quantification results. MSstatsPTM provides functions for summarization, estimation of PTM site abundance, and detection of changes in PTMs across experimental conditions. 

## Installation 

The most recent version of the package is available on Bioconductor:

```
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("MSstatsPTM")
```

Optionally, the package can be installed directly from Github:

```
devtools::install_github("Vitek-Lab/MSstatsPTM", build_vignettes = TRUE)
```

## Contributing

We welcome contributions from the community. For details on how to contribute to the
development of MSstatsPTM, please refer to the [CONTRIBUTING](https://github.com/Vitek-Lab/MSstatsPTM/blob/master/.github/CONTRIBUTING.md) file.

## License

[Artistic-2.0](https://opensource.org/licenses/Artistic-2.0)

#' Check input data and parameters
#' @noRd
.check.plotting.data = function(data, type, sig, FCcutoff, logBase.pvalue, 
                                 ylimUp, ylimDown, xlimUp, x.axis.size, 
                                 y.axis.size, dot.size, text.size, text.angle, 
                                 legend.size, ProteinName,colorkey, numProtein, 
                                 width, height, which.Comparison, which.Protein,
                                 address){
  
  ## Column check
  assertChoice(toupper(type), c("HEATMAP", "VOLCANOPLOT"),
               .var.name = "Type") 
  assertNumeric(sig, .var.name = "Sig")
  if (!is.logical(FCcutoff)){
    assertNumeric(FCcutoff, .var.name = "FCcutoff")
  }
  
  assertChoice(logBase.pvalue, c(2, 10),
               .var.name = "Logbase Pvalue") 
  
  if (!is.logical(ylimUp)){
    assertNumeric(ylimUp, .var.name = "ylimUp")
  }
  if (!is.logical(ylimDown)){
    assertNumeric(ylimDown, .var.name = "ylimDown")
  }
  if (!is.logical(xlimUp)){
    assertNumeric(xlimUp, .var.name = "xlimUp")
  }
  
  ## Check plotting size vars
  assertNumeric(x.axis.size, .var.name = ("x.axis.size"))
  assertNumeric(y.axis.size, .var.name = ("y.axis.size"))
  assertNumeric(dot.size, .var.name = ("dot.size"))
  assertNumeric(text.size, .var.name = ("text.size"))
  assertNumeric(text.angle, .var.name = ("text.angle"))
  assertNumeric(legend.size, .var.name = ("legend.size"))
  
  assertLogical(colorkey, .var.name = ("colorkey"))
  
  assertNumeric(width, .var.name = ("width"))
  assertNumeric(height, .var.name = ("height"))
  
  ## Check data columns
  min.cols = c("Protein", "Label", "log2FC", "SE", "DF", "pvalue","adj.pvalue")
  
  if (!is.null(data[["PTM.Model"]])){
    missing = setdiff(min.cols, colnames(data[["PTM.Model"]]))
    if (length(missing) > 0){
      msg = paste("Missing columns in the PTM input:",
                  paste(missing, collapse = " "))
      stop(msg)
    }
  }
  if (!is.null(data[["PROTEIN.Model"]])){
    missing = setdiff(min.cols, colnames(data[["PROTEIN.Model"]]))
    if (length(missing) > 0){
      msg = paste("Missing columns in the Protein input:",
                   paste(missing, collapse = " "))
      stop(msg)
    }
  }
  if (!is.null(data[["ADJUSTED.Model"]])){
    missing = setdiff(min.cols, colnames(data[["ADJUSTED.Model"]]))
    if (length(missing) > 0){
      msg = paste("Missing columns in the Adjusted PTM input:",
                   paste(missing, collapse = " "))
      stop(msg)
    }
  }
                
}

#' Format models into plotting format
#' @noRd
.format.model.plots = function(data, which.Comparison, which.Protein){
  
  ptm.model = data[['PTM.Model']]
  ptm.model = as.data.table(ptm.model)
  protein.model = data[['PROTEIN.Model']]
  protein.model = as.data.table(protein.model)
  adjusted.model = data[['ADJUSTED.Model']]
  adjusted.model = as.data.table(adjusted.model)
  
  ptm.model$Protein = factor(ptm.model$Protein)
  ptm.model$Label = factor(ptm.model$Label)
  
  if (nrow(protein.model) > 0){
    protein.model$Protein = factor(protein.model$Protein)
    protein.model$Label = factor(protein.model$Label)
    adjusted.model$Protein = factor(adjusted.model$Protein)
    adjusted.model$Label = factor(adjusted.model$Label)
  }
  
  if (which.Comparison[[1]] != "all") {
    ## check which.comparison is name of comparison
    if (is.character(which.Comparison)) {
      
      temp.name = which.Comparison
      ## message if name of comparison is wrong.
      if (length(setdiff(temp.name, unique(ptm.model$Label))) > 0) {
        
        ## TODO: Logging        
        # processout = rbind(processout, 
        #                     paste0("Please check labels of comparions. ", 
        #                            "Result does not have this comparison. - ", 
        #                            toString(temp.name)))
        # write.table(processout, file=finalfile, row.names=FALSE)
        
        stop(paste0("Please check labels of comparisons. ", 
                    "Result does not have this comparison. - ", 
                    toString(temp.name)))
      }
    }
    ## check which.comparison is order number of comparison
    if (is.numeric(which.Comparison)) {
      
      temp.name = levels(ptm.model$Label)[which.Comparison]
      
      ## message if name of comparison is wrong.
      if (length(levels(ptm.model$Label))<max(which.Comparison)) {
        stop(paste0("Please check your selection of comparisons. There are ",
                    length(levels(ptm.model$Label)), 
                    " comparisons in this result."))
      }
    }  
    
    ## use only assigned comparisons
    ptm.model = ptm.model[which(ptm.model$Label %in% temp.name), ]
    if (nrow(protein.model) > 0){
      protein.model = protein.model[which(
        protein.model$Label %in% temp.name), ]
      adjusted.model = adjusted.model[which(
        adjusted.model$Label %in% temp.name),]
    }

  }
  
  if (which.Protein[[1]] != "all") {
    ## check which.comparison is name of comparison
    if (is.character(which.Protein)) {
      
      temp.name = which.Protein
      ## message if name of comparison is wrong.
      if (length(setdiff(temp.name, unique(ptm.model$Protein))) > 0) {
        
        ## TODO: Logging        
        # processout = rbind(processout, 
        #                     paste0("Please check labels of comparions. ", 
        #                            "Result does not have this comparison. - ", 
        #                            toString(temp.name)))
        # write.table(processout, file=finalfile, row.names=FALSE)
        
        stop(paste0("Please check labels of proteins ", 
                    "Result does not have all these proteins. - ", 
                    toString(temp.name)))
      }
    }
    ## check which.comparison is order number of comparison
    if (is.numeric(which.Protein)) {
      
      temp.name = levels(ptm.model$Protein)[which.Protein]
      
      ## message if name of comparison is wrong.
      if (length(levels(ptm.model$Protein)) < max(which.Protein)) {
        stop(paste0("Please check your selection of proteins. There are ",
                    length(levels(ptm.model$Protein)), 
                    " comparisons in this result."))
      }
    }
    
    ## use only assigned comparisons
    ptm.model = ptm.model[which(ptm.model$Protein %in% temp.name), ]
    if (nrow(protein.model) > 0){
      adjusted.model = adjusted.model[
        which(adjusted.model$Protein %in% temp.name), ]
      protein.model = protein.model[which(protein.model$Protein %in% unique(
        adjusted.model$GlobalProtein)), ]
    }
  }
  
  ptm.model$Protein = factor(ptm.model$Protein)
  ptm.model$Label = factor(ptm.model$Label)
  
  if (nrow(protein.model) > 0){
    protein.model$Protein = factor(protein.model$Protein)
    protein.model$Label = factor(protein.model$Label)
    adjusted.model$Protein = factor(adjusted.model$Protein)
    adjusted.model$Label = factor(adjusted.model$Label)
  }
  
  return(list('PTM.Model' = ptm.model, 'PROTEIN.Model' = protein.model, 
              'ADJUSTED.Model' = adjusted.model))
}

#' Wrapper for plotting heatmap for every model provided
#' @noRd
.plotHeatmap = function(data, sig, FCcutoff, logBase.pvalue, ylimUp, ylimDown, 
                     text.angle, x.axis.size, y.axis.size, dot.size, colorkey, 
                     numProtein, width, height, address, isPlotly){
  ## If there are the file with the same name, 
  ## add next numbering at the end of file name
  if (!isPlotly && address != FALSE) {
    allfiles = list.files()
    
    num = 0
    filenaming = paste0(address, "Heatmap")
    finalfile = paste0(address, "Heatmap.pdf")
    
    while (is.element(finalfile, allfiles)) {
      num = num + 1
      finalfile = paste0(paste(filenaming, num, sep="-"), ".pdf")
    }	
    
    pdf(finalfile, width=width, height=height)
  }
  final_plots = list()
  ptm_heatmap_plot <- .plot.model.heatmap(data[['PTM.Model']], sig, FCcutoff, logBase.pvalue, 
                      ylimUp, ylimDown, x.axis.size, y.axis.size, text.angle,
                      colorkey, numProtein,
                      "Unadjusted PTM", isPlotly)
  final_plots[[1]] <- ptm_heatmap_plot
  if (nrow(data[['PROTEIN.Model']]) > 0){
    protein_heatmap_plot <- .plot.model.heatmap(data[['PROTEIN.Model']], sig, FCcutoff, logBase.pvalue,
                        ylimUp, ylimDown, x.axis.size, y.axis.size, text.angle,
                        colorkey, numProtein,
                        "Protein", isPlotly)
    final_plots[[2]] <- protein_heatmap_plot
    adjusted_heatmap_plot <- .plot.model.heatmap(data[['ADJUSTED.Model']], sig, FCcutoff, logBase.pvalue,
                        ylimUp, ylimDown, x.axis.size, y.axis.size, text.angle,
                        colorkey, numProtein,
                        "Adjusted PTM", isPlotly)
    final_plots[[3]] <- adjusted_heatmap_plot
  }
  
  if (address != FALSE && !isPlotly) {
    dev.off()
  }
  if (isPlotly) {
    final_plots
  }
}



#' Plots individual model heatmap
#' @noRd
.plot.model.heatmap = function(data, sig, FCcutoff, logBase.pvalue, ylimUp,
                                 ylimDown, x.axis.size, y.axis.size, text.angle,
                                 colorkey, numProtein, model, isPlotly){

  Label = Protein = sign_adj_pval = NULL

  if (logBase.pvalue == 2) {
    y.limUp = 30
  } else if (logBase.pvalue == 10) {
    y.limUp = 10
  }

  if (is.numeric(ylimUp)) {
    y.limUp = ylimUp
  }


  ## if FCcutoff is assigned, make p-value insignificant.
  if (is.numeric(FCcutoff)) {
    if (colnames(data)[3] == "log2FC") {
      data$adj.pvalue[data[, 3] < log2(FCcutoff) & data[, 3] > (
        -log2(FCcutoff))] = 1
    }
    if (colnames(data)[3] == "log10FC") {
      data$adj.pvalue[data[, 3] < log10(FCcutoff) & data[, 3] > (
        -log10(FCcutoff))] = 1
    }
  }

  ## based on p-value
  if (logBase.pvalue == 2) {
      temp =  -log2(data$adj.pvalue) * sign(data[, 3])
  } else if (logBase.pvalue == 10) {
      temp =  -log10(data$adj.pvalue) * sign(data[, 3])
  }

  data$sign_adj_pval = temp
  
  obj = data[, c("Protein", "Label", "sign_adj_pval")]
  
  if(!isPlotly) {
    ## maximum number of proteins per heatmap
    unique_prot = unique(obj$Protein)
    totalpro = length(unique_prot)
    if(is.null(numProtein)) {
      numProtein = 50
    }
    numheatmap = totalpro %/% numProtein + 1
    ## draw heatmap
    ## loop for numProtein
    for (j in seq_len(numheatmap)) {
      
      plot_prot = unique_prot[((j - 1) * numProtein + 1):(j * numProtein)]
      temp_obj = obj[obj$Protein %in% plot_prot]
      
      limits = c(-1,1)*max(abs(temp_obj$sign_adj_pval[
        is.finite(temp_obj$sign_adj_pval)]))
      temp_heatmap = ggplot(temp_obj, aes(.data$Label, .data$Protein, fill = .data$sign_adj_pval)
      ) + geom_tile() + scale_fill_distiller(
        palette = "RdBu", name = "(sign) Adj pvalue",
        limits = limits) + labs(
          title = paste0(model, " - Page #", as.character(j)), y = model,
          x = "Comparison") +
        theme_msstats(type = "COMPARISONPLOT", x.axis.size, y.axis.size, 13,
                      element_rect(fill = "gray95"),
                      element_text(colour = c("#00B0F6"), size = 14),
                      "bottom", text_angle = text.angle)
      print(temp_heatmap)
    }
  } else {
    limits = c(-1, 1) * max(abs(obj$sign_adj_pval[is.finite(obj$sign_adj_pval)]))
    data$sign_adj_pval[is.infinite(data$sign_adj_pval)] <- NaN
    obj = data[, c("Protein", "Label", "sign_adj_pval")]
    if (!is.null(numProtein)) {
      # subset numProtein unique proteins
      obj$Protein <- as.character(obj$Protein)
      unique_proteins <- unique(obj$Protein)
      top_10_proteins <- unique_proteins[1:numProtein]
      obj <- obj[obj$Protein %in% top_10_proteins, ]
      limits = c(-1, 1) * max(abs(obj$sign_adj_pval[is.finite(obj$sign_adj_pval)]))
    }
    
    colors <- c("#2166AC", "#4393C3", "#92C5DE", "#D1E5F0", "#FFFFFF", "#FDDBC7", "#F4A582", "#D6604D", "#B2182B")
    heatmap_plot <- plot_ly(
      data = obj,
      x = as.character(obj$Label),
      y = obj$Protein,
      z = ~sign_adj_pval,
      width = 800,
      height = 600,
      zmin = limits[1],
      zmax = limits[2],
      xgap = 0,
      ygap = 0,
      colorbar = list(title = "(sign) Adj pvalue", tickfont = list(size = 13)),
      colors = colors,
      zauto = FALSE,
      type = "heatmap"
    )
    
    # Customize the layout
    heatmap_plot <- plotly::layout(
      heatmap_plot,
      title = paste0(model, " : All Pages"),
      xaxis = list(title = "Comparison", tickangle = 45, tickfont = list(size = x.axis.size)),
      yaxis = list(title = model, tickfont = list(size = y.axis.size)),
      plot_bgcolor = "grey"
    )
    return(heatmap_plot)
  }

  
}

#' Wrapper for plotting volcano plot for every model provided
#' @noRd
.plotVolcano = function(data, sig, FCcutoff, logBase.pvalue, ylimUp, ylimDown, 
                     xlimUp, x.axis.size, y.axis.size, dot.size, text.size, 
                     legend.size, ProteinName, colorkey, numProtein,
                     width, height, address, plot_name_list, isPlotly){
  ## If there are the file with the same name
  ## add next numbering at the end of file name
  if (!isPlotly && address != FALSE) {
    allfiles = list.files()
    
    num = 0
    filenaming = paste0(address, "VolcanoPlot")
    finalfile = paste0(address, "VolcanoPlot.pdf")
    
    while (is.element(finalfile, allfiles)) {
      num = num + 1
      finalfile = paste0(paste(filenaming, num, sep="-"), ".pdf")
    }	
    
    pdf(finalfile, width=width, height=height)
  }

  ptm_plots <- .plot.model.volcano(data[['PTM.Model']], sig, FCcutoff, logBase.pvalue, 
                      ylimUp, ylimDown, xlimUp, x.axis.size, y.axis.size, 
                      dot.size, text.size, legend.size, ProteinName,
                      plot_name_list[[1]], isPlotly)
  final_plots = ptm_plots
  if (nrow(data[['PROTEIN.Model']])) {
    protein_plots <- .plot.model.volcano(data[['PROTEIN.Model']], sig, FCcutoff, logBase.pvalue, 
                        ylimUp, ylimDown, xlimUp, x.axis.size, y.axis.size, 
                        dot.size, text.size, legend.size, ProteinName,
                        plot_name_list[[2]], isPlotly)
    adjusted_plots <- .plot.model.volcano(data[['ADJUSTED.Model']], sig, FCcutoff, logBase.pvalue,
                        ylimUp, ylimDown, xlimUp, x.axis.size, y.axis.size, 
                        dot.size, text.size, legend.size, ProteinName,
                        plot_name_list[[3]], isPlotly)
    final_plots <- c(final_plots, protein_plots, adjusted_plots)
  }
  if (address != FALSE) {
    dev.off()
  }
  if (isPlotly) {
    final_plots
  }
}

#' Plots individual model volcano plot
#' @noRd
.plot.model.volcano = function(data, sig, FCcutoff, logBase.pvalue, ylimUp,
                                ylimDown, xlimUp, x.axis.size, y.axis.size, 
                                dot.size, text.size, legend.size, ProteinName,
                                model, isPlotly){
  log2FC = Protein = NULL
  
  if (logBase.pvalue == 2) {
    y.limUp  = 30
  } else if (logBase.pvalue == 10) {
    y.limUp  = 10
  }
  
  if (is.numeric(ylimUp)) {
    y.limUp = ylimUp 
  }
  
  ## remove the result, NA and Inf
  data = data[!is.na(data$adj.pvalue), ]
  data = data[is.finite(data$log2FC)]
  
  ## group for coloring dots
  if (!FCcutoff) {  
    data[data$adj.pvalue >= sig, "colgroup"] = "black"
    data[which(data$adj.pvalue < sig & data[, 3] > 0), "colgroup"] = "red"
    data[which(data$adj.pvalue < sig & data[, 3] < 0), "colgroup"] = "blue" 
  }
  
  if (is.numeric(FCcutoff)) {
    data$colgroup = "black"
    
    if (colnames(data)[3] == "log2FC") {
      data[which(data$adj.pvalue < sig & data[, 3] > log2(FCcutoff)), 
           "colgroup"] = "red"
      data[which(data$adj.pvalue < sig & data[, 3] < (-log2(FCcutoff))), 
           "colgroup"] = "blue"
    }
    
    if (colnames(data)[3] == "log10FC") {
      data = data[is.finite(data$log10FC)]
      data[which(data$adj.pvalue < sig & data[, 3] > log10(FCcutoff)), 
           "colgroup"] = "red"
      data[which(data$adj.pvalue < sig & data[, 3] < (-log10(FCcutoff))), 
           "colgroup"] = "blue"
    }
  }
  
  data$colgroup = factor(data$colgroup, levels=c("black", "blue", "red"))
  plots <- vector("list", nlevels(data$Label))
  ## for multiple volcano plots, 
  for (i in seq_len(nlevels(data$Label))) {
    
    sub = data[data$Label == levels(data$Label)[i], ]
    
    if (logBase.pvalue == 2) {
      sub$adj.pvalue[sub$adj.pvalue < 2^(-y.limUp)] = 2^(-y.limUp)
    } else if (logBase.pvalue == 10) {
      sub$adj.pvalue[sub$adj.pvalue < 10^(-y.limUp)] = 10^(-y.limUp)
    }
    
    sub = as.data.table(sub)
    
    if (logBase.pvalue == 2) {
      y.limup = ceiling(max(-log2(sub[!is.na(sub$adj.pvalue), "adj.pvalue"])))
      if (y.limup < (-log2(sig))) {
        y.limup = (-log2(sig) + 1) ## for too small y.lim
      }
    } else if (logBase.pvalue == 10) {
      y.limup = ceiling(max(-log10(sub[!is.na(sub$adj.pvalue), "adj.pvalue"])))
      if (y.limup < (-log10(sig))) {
        y.limup = (-log10(sig) + 1) ## for too small y.lim
      }
    }
    
    ## ylimDown
    y.limdown = 0 ## default is zero
    if (is.numeric(ylimDown)) {
      y.limdown = ylimDown
    }
    
    ## x.lim
    x.lim = ceiling(max(abs(sub[is.finite(log2FC), log2FC])))
    
    if (x.lim < 3) {
      x.lim = 3
    }
    if (is.numeric(xlimUp)) {
      x.lim = xlimUp
    }
    ## for assigning x in ggplot2
    subtemp = sub
    colnames(subtemp)[3] = "logFC"
    
    if (logBase.pvalue == 2) {
      subtemp$logadjp = (-log2(subtemp$adj.pvalue))
    } else if (logBase.pvalue == 10) {
      subtemp$logadjp = (-log10(subtemp$adj.pvalue))
    }
    
    ## for x limit for inf or -inf
    subtemp$newlogFC = subtemp$logFC
    subtemp[!is.na(subtemp$issue) &
              subtemp$issue == "oneConditionMissing" & 
              subtemp$logFC == Inf, "newlogFC"] = (x.lim - 0.2)
    subtemp[!is.na(subtemp$issue) & 
              subtemp$issue == "oneConditionMissing" & 
              subtemp$logFC == (-Inf), "newlogFC"] = (x.lim - 0.2) * (-1)
    
    ## add (*) in Protein name for Inf or -Inf
    subtemp$Protein = as.character(subtemp$Protein)
    subtemp[!is.na(subtemp$issue) & 
              subtemp$issue == "oneConditionMissing", "Protein"] = 
      paste0("*", subtemp[!is.na(subtemp$issue) & 
                            subtemp$issue == "oneConditionMissing", "Protein"])
    
    if (logBase.pvalue == 2) {
      log_title = 'Log2'
    } else if (logBase.pvalue == 10) {
      log_title = 'Log10'
    }
    
    ptemp = ggplot(aes(x = .data$logFC, y = .data$logadjp,
                               color = .data$colgroup,
                               label = .data$Protein),
                    data=subtemp) +
      geom_point(size=dot.size) +
      scale_colour_manual(values=c("gray65", "blue", "red"), 
                          limits=c("black", "blue", "red"), 
                          breaks=c("black", "blue", "red"), 
                          labels=c("No regulation", "Down-regulated", 
                                   "Up-regulated")) +
      scale_y_continuous(paste0('-', log_title, ' (adjusted p-value)'), 
                         limits=c(y.limdown, y.limup)) + 
      scale_x_continuous(paste0(log_title, ' fold change'), 
                         limits=c(-x.lim, x.lim)) +
      labs(title= paste0(model, ": ", unique(sub$Label)))
    
    ## add protein name
    if (ProteinName) {
      if (length(unique(subtemp$colgroup)) == 1 & 
          any(unique(subtemp$colgroup) == 'black')) {
        message(paste0("The volcano plot for ", unique(subtemp$Label), 
      " does not show the protein names because none of them are significant."))
      } else {
        ptemp = ptemp +
          geom_text_repel(data=subtemp[subtemp$colgroup != "black", ],
                          aes(label = .data$Protein),
                          size=text.size,
                          col='black')
      }
    }
    
    ## For legend of linetype for cutoffs
    ## first assign line type
    ltypes = c("type1"="twodash", "type2"="dotted")
    
    ## cutoff lines, FDR only
    if (!FCcutoff) {
      if (logBase.pvalue == 2){
        sigcut = data.table("Protein"='sigline', 
                             "logFC"=seq(-x.lim, x.lim, length.out=20),
                             "logadjp"=(-log2(sig)),
                             "line"='twodash')
      } else if (logBase.pvalue == 10) {
        sigcut = data.table("Protein"='sigline', 
                             "logFC"=seq(-x.lim, x.lim, length.out=20),
                             "logadjp"=(-log10(sig)),
                             "line"='twodash')
      }
      
      pfinal = ptemp +
        geom_line(data=sigcut,
                  aes(x = .data$logFC, y = .data$logadjp, linetype = .data$line),
                  colour="darkgrey",
                  linewidth=0.6,
                  show.legend=TRUE) +
        scale_linetype_manual(values=c('twodash'=6),
                              labels=c(paste0("Adj p-value cutoff (", sig, ")"))
                              ) +
        guides(colour=guide_legend(override.aes=list(linetype=0)),
               linetype=guide_legend())
    }
    if (is.numeric(FCcutoff)) {
      if (logBase.pvalue == 2) {
        ## three different lines
        sigcut = data.table("Protein"='sigline', 
                             "logFC"=seq(-x.lim, x.lim, length.out=10), 
                             "logadjp"=(-log2(sig)),
                             "line"='twodash')
        FCcutpos = data.table("Protein"='sigline', 
                               "logFC"=log2(FCcutoff), 
                               "logadjp"=seq(y.limdown, y.limup, length.out=10), 
                               "line"='dotted')
        FCcutneg = data.table("Protein"='sigline', 
                               "logFC"=(-log2(FCcutoff)), 
                               "logadjp"=seq(y.limdown, y.limup, length.out=10), 
                               "line"='dotted')
      } else if (logBase.pvalue == 10) {
        ## three different lines
        sigcut = data.table("Protein"='sigline', 
                             "logFC"=seq(-x.lim, x.lim, length.out=10), 
                             "logadjp"=(-log10(sig)),
                             "line"='twodash')
        FCcutpos = data.table("Protein"='sigline', 
                               "logFC"=log10(FCcutoff), 
                               "logadjp"=seq(y.limdown, y.limup, length.out=10), 
                               "line"='dotted')
        FCcutneg = data.table("Protein"='sigline', 
                               "logFC"=(-log10(FCcutoff)), 
                               "logadjp"=seq(y.limdown, y.limup, length.out=10), 
                               "line"='dotted')
      }
      ## three lines, with order color first and then assign linetype manual
      pfinal = ptemp +
        geom_line(data=sigcut, 
                  aes(x = .data$logFC, y = .data$logadjp, linetype = .data$line),
                  colour="darkgrey",
                  linewidth=0.6,
                  show.legend=TRUE) +
        geom_line(data=FCcutpos,
                  aes(x = .data$logFC, y = .data$logadjp, linetype = .data$line),
                  colour="darkgrey",
                  linewidth=0.6,
                  show.legend=TRUE) +
        geom_line(data=FCcutneg,
                  aes(x = .data$logFC, y = .data$logadjp, linetype = .data$line),
                  colour="darkgrey",
                  linewidth=0.6) +
        scale_linetype_manual(values=c('dotted'=3, 'twodash'=6),
                              labels=c(paste0("Fold change cutoff (", FCcutoff, 
                                              ")"),
                                       paste0("Adj p-value cutoff (", sig, ")"))
                              ) +
        guides(colour=guide_legend(override.aes=list(linetype=0)),
               linetype=guide_legend())
    }
    
    pfinal = pfinal +
      theme_msstats(type = "VOLCANO", x.axis.size, y.axis.size, legend.size, 
                    element_rect(fill = "gray95"),
                    element_text(colour = c("#00B0F6"), size = 14),
                    "bottom", text_angle = text.angle, 
                    legend.title = element_blank())
      # theme(
      #   panel.background = element_rect(fill='white', colour="black"),
      #   panel.grid.minor = element_blank(),
      #   axis.text.x = element_text(size=x.axis.size, colour="black"),
      #   axis.text.y = element_text(size=y.axis.size, colour="black"),
      #   axis.ticks = element_line(colour="black"),
      #   axis.title.x = element_text(size=x.axis.size+5, vjust=-0.4),
      #   axis.title.y = element_text(size=y.axis.size+5, vjust=0.3),
      #   title = element_text(size=x.axis.size+8, vjust=1.5),
      #   legend.position="bottom",
      #   legend.key = element_rect(fill='white', colour='white'),
      #   legend.text = element_text(size=legend.size),
      #   legend.title = element_blank()
      # )
    
    print(pfinal)
    plots[[i]] = pfinal
  }
  if (isPlotly) {
    plots
  }
}

#' Data summarization function for TMT labelled MS experiments targeting PTMs.
#'
#' Utilizes functionality from MSstatsTMT to clean, summarize, and
#' normalize PTM and protein level data. Imputes missing values, performs 
#' normalization, and summarizes data. PTM data is summarized up to the 
#' modification and protein data is summarized up to the protein level. Takes 
#' as input the output of the included converters (see included `raw.input.tmt` 
#' data object for required input format).
#'
#' @export
#' @importFrom MSstatsTMT proteinSummarization
#' @importFrom MSstatsConvert MSstatsLogsSettings MSstatsSaveSessionInfo
#' @importFrom data.table as.data.table is.data.table
#' 
#' @param data Name of the output of MSstatsPTM converter function or 
#' peptide-level quantified data from other tools. It should be a list 
#' containing one or two data tables, named PTM and PROTEIN for modified and 
#' unmodified datasets. The list must at least contain the PTM dataset. The data
#' should have columns ProteinName, PeptideSequence, Charge, PSM, Mixture, 
#' TechRepMixture, Run, Channel, Condition, BioReplicate, Intensity
#' @param method Four different summarization methods to protein-level can be 
#' performed : "msstats"(default), "MedianPolish", "Median", "LogSum".
#' @param global_norm Global median normalization on for unmodified peptide 
#' level data (equalizing the medians across all the channels and MS runs). 
#' Default is TRUE. It will be performed before protein-level summarization.
#' @param global_norm.PTM Same as above for modified peptide level data. Default
#' is TRUE.
#' @param reference_norm Reference channel based normalization between MS runs 
#' on unmodified protein level data. TRUE(default) needs at least one reference 
#' channel in each MS run, annotated by 'Norm' in Condtion column. It will be 
#' performed after protein-level summarization. FALSE will not perform this 
#' normalization step. If data only has one run, then reference_norm=FALSE.
#' @param reference_norm.PTM Same as above for modified peptide level data. 
#' Default is TRUE.
#' @param remove_norm_channel TRUE(default) removes 'Norm' channels from 
#' protein level data.
#' @param remove_empty_channel TRUE(default) removes 'Empty' channels from 
#' protein level data.
#' @param MBimpute only for method="msstats". TRUE (default) imputes missing 
#' values by Accelated failure model. FALSE uses minimum value to impute the 
#' missing value for each peptide precursor ion.
#' @param MBimpute.PTM Same as above for modified peptide level data. Default is
#' TRUE
#' @param maxQuantileforCensored We assume missing values are censored. 
#' maxQuantileforCensored is Maximum quantile for deciding censored missing 
#' value, for instance, 0.999. Default is Null.
#' @param use_log_file logical. If TRUE, information about data processing
#' will be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing will be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. 
#' If not provided, such a file will be created automatically.
#' If `append = TRUE`, has to be a valid path to a file.
#' @return list of two data.tables
#' @examples
#' head(raw.input.tmt$PTM)
#' head(raw.input.tmt$PROTEIN)
#' 
#' quant.tmt.msstatsptm = dataSummarizationPTM_TMT(raw.input.tmt,
#'                                                  method = "msstats", 
#'                                                  verbose = FALSE)
#' head(quant.tmt.msstatsptm$PTM$ProteinLevelData)
dataSummarizationPTM_TMT = function(
  data,
  method = "msstats",
  global_norm = TRUE,
  global_norm.PTM = TRUE,
  reference_norm = TRUE,
  reference_norm.PTM = TRUE,
  remove_norm_channel = TRUE,
  remove_empty_channel = TRUE,
  MBimpute = TRUE,
  MBimpute.PTM = TRUE,
  maxQuantileforCensored = NULL,
  use_log_file = TRUE, 
  append = FALSE,
  verbose = TRUE, 
  log_file_path = NULL
  ) {
  
  ## Start log
  if (is.null(log_file_path) & use_log_file == TRUE){
    time_now = Sys.time()
    path = paste0("MSstatsPTM_log_", gsub("[ :\\-]", "_", time_now),
                  ".log")
    file.create(path)
  } else {path = log_file_path}

  MSstatsLogsSettings(use_log_file, append,
                      verbose, log_file_path = path,
                      base = "MSstatsPTM_log_", pkg_name = "MSstatsTMT")
  
  getOption("MSstatsTMTLog")("INFO", "Starting parameter and data checks..")
  
  .checkDataProcessParams.TMT(method, global_norm, global_norm.PTM,
                          reference_norm, reference_norm.PTM,
                          remove_norm_channel, remove_empty_channel,
                          MBimpute, maxQuantileforCensored)

  adj.protein = FALSE
  
  PTM.dataset = data[["PTM"]]
  
  protein.dataset = data[["PROTEIN"]]
  
  # Check PTM and PROTEIN data for correct format
  adj.protein = .summarizeCheck(data, 'TMT')
  PTM.dataset = as.data.table(PTM.dataset)
  
  getOption("MSstatsTMTLog")("INFO", "Parameter and data checks complete.")
  
  ## Determine if protein level should also be summarized
  if (adj.protein) {
    getOption("MSstatsTMTLog")("INFO", "Protein dataset was included.")
    protein.dataset = as.data.frame(protein.dataset)
  }
  
  ## Run summarization function from MSstatsTMT
  getOption("MSstatsTMTLog")("INFO", "Starting PTM summarization..")
  ptm.summarized = proteinSummarization(PTM.dataset,
                                         method, global_norm.PTM, 
                                         reference_norm.PTM,
                                         remove_norm_channel,
                                         remove_empty_channel, 
                                         MBimpute.PTM,
                                         maxQuantileforCensored, use_log_file, 
                                         append, verbose, log_file_path = path,
                                         msstats_log_path = path)
  
  if (adj.protein) {
    getOption("MSstatsTMTLog")("INFO", "Starting Protein summarization..")
    protein.summarized = proteinSummarization(protein.dataset,
                                               method, global_norm, 
                                               reference_norm,
                                               remove_norm_channel,
                                               remove_empty_channel, 
                                               MBimpute,
                                               maxQuantileforCensored, 
                                               use_log_file, append, verbose, 
                                               log_file_path = path)
  }
  
  ## Compile and return summarized results
  getOption("MSstatsTMTLog")("INFO", "Summarization complete. Returning output")
  msstatsptm.summarized = list("PTM" = ptm.summarized)
  if (adj.protein) {
    msstatsptm.summarized = c(msstatsptm.summarized,
                               "PROTEIN" = list(protein.summarized))
  }
  
  return(msstatsptm.summarized)
}

data("raw.input.tmt", package = "MSstatsPTM")

## Test missing columns are handled
expect_error(dataSummarizationPTM_TMT())
expect_error(dataSummarizationPTM_TMT(list(PTM = raw.input.tmt$PTM[, -1],
                                       PROTEIN = raw.input.tmt$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM,
                                       PROTEIN = raw.input.tmt$PROTEIN[, -1])))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM[, -2],
                                       PROTEIN = raw.input.tmt$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM,
                                       PROTEIN = raw.input.tmt$PROTEIN[, -2])))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM[, -3],
                                       PROTEIN = raw.input.tmt$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM,
                                       PROTEIN = raw.input.tmt$PROTEIN[, -3])))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM[, -4],
                                       PROTEIN = raw.input.tmt$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM,
                                       PROTEIN = raw.input.tmt$PROTEIN[, -4])))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM[, -5],
                                       PROTEIN = raw.input.tmt$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM,
                                       PROTEIN = raw.input$PROTEIN[, -5])))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM[, -6],
                                       PROTEIN = raw.input.tmt$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM,
                                       PROTEIN = raw.input.tmt$PROTEIN[, -6])))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM[, -7],
                                       PROTEIN = raw.input.tmt$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM,
                                       PROTEIN = raw.input.tmt$PROTEIN[, -7])))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM[, -8],
                                       PROTEIN = raw.input.tmt$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM,
                                       PROTEIN = raw.input.tmt$PROTEIN[, -8])))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM[, -9],
                                       PROTEIN = raw.input.tmt$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM,
                                       PROTEIN = raw.input.tmt$PROTEIN[, -9])))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM[, -10],
                                       PROTEIN = raw.input.tmt$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input.tmt$PTM,
                                       PROTEIN = raw.input.tmt$PROTEIN[, -10])))

## Test misspecified parameters
expect_error(dataSummarizationPTM(raw.input.tmt, method  = ""))
expect_error(dataSummarizationPTM(raw.input.tmt, global_norm = 12))
expect_error(dataSummarizationPTM(raw.input.tmt, global_norm.PTM = 12))
expect_error(dataSummarizationPTM(raw.input.tmt, reference_norm = 12))
expect_error(dataSummarizationPTM(raw.input.tmt, reference_norm.PTM = 12))
expect_error(dataSummarizationPTM(raw.input.tmt, remove_norm_channel = 12))
expect_error(dataSummarizationPTM(raw.input.tmt, remove_empty_channel = ""))
expect_error(dataSummarizationPTM(raw.input.tmt, MBimpute = ""))
expect_error(dataSummarizationPTM(raw.input.tmt, MBimpute.PTM = ""))
expect_error(dataSummarizationPTM(raw.input.tmt, maxQuantileforCensored  = ""))

#' Check annotation file to ensure it is formatted correctly
#' @noRd
.checkAnnotation = function(annotation, label_type){
  columns = colnames(annotation)
  
  if (!("Run" %in% columns | "Raw.file" %in% columns)){
    msg = "Run column missing in annotation file. Annotation must include either
    `Run` or `Raw.file` column to match with input data."
    stop(msg)
  }
  
  if (label_type == "LF"){
    max_columns = c("Run", "Raw.file", "Condition", 
                    "BioReplicate", "IsotopeLabelType", "Fraction")
  } else if (label_type == "TMT"){
    max_columns = c("Run", "Raw.file", "Fraction", "TechRepMixture", "Channel", 
                    "Condition", "Mixture", "BioReplicate")
  } else {
    msg = paste("Labeling type must be either LF or TMT")
    stop(msg)
  }
  
  if (sum(columns %in% max_columns) != length(columns)){
    msg = paste0(
    "Extra columns included in the annotation file that are not required. 
    This can cause problems in the converter dependencies. Please 
    only include the following columns in the annotation file: 
                 ", paste0(max_columns, collapse=", "))
    stop(msg)
  }
}



#' Check validity of parameters to MaxQ Converter function
#' @noRd
.checkMaxQconverterParams = function(mod.num = 'Total',
                                     keyword = "phos",
                                     which.proteinid.ptm = "Protein",
                                     which.proteinid.protein = "Leading.razor.protein",
                                     removeMpeptides = FALSE) {
  
  assertChoice(toupper(mod.num), c('SINGLE', 'TOTAL'), 
                          .var.name = 'NumberOfModifications')
  if (is.null(keyword)){
    msg = ('Keyword must not be null - stop')
    # getOption("MSstatsLog")("ERROR", msg)
    stop(msg)
  }
  assertChoice(which.proteinid.ptm, c("Proteins", "Leading.proteins",
                                                 "Protein"),
                          .var.name = "ProteinColumnNamePTM") 
  ## TODO: Use log to track these var choices
  # getOption("MSstatsLog")("INFO", paste("Summary method:", 
  #                                       summarization$method))
  assertChoice(which.proteinid.protein, c("Leading.proteins", 
                                                     "Leading.razor.protein", 
                                                     "Gene.names"),
                          .var.name = "ProteinColumnNameProtein")
  # getOption("MSstatsLog")("INFO", paste("cutOffCensored:", imputation$cutoff))
  assertLogical(removeMpeptides, .var.name = "removeMpeptides")
}

#' Function to check for global protein data
#' @noRd
.check.global.protein = function(evidence = NULL, 
                                  proteinGroups = NULL){
  
  if ((is.null(evidence) & !is.null(proteinGroups))|
      (!is.null(evidence) & is.null(proteinGroups))){
    msg = paste("To convert global protein data",
                "both the evidence and proteinGroups",
                "files must be provided - stop")
    #getOption("MSstatsLog")("ERROR", msg)
    stop(msg)
  
  } else if (is.null(evidence) & is.null(proteinGroups)){
    convert.global = FALSE
  } else {
    convert.global = TRUE
  }
  
  return(convert.global)
}

#' Check PTM and protein datasets
#' @noRd
.summarizeCheck = function(data, type) {
  # Check the PTM data
  if (is.null(data[["PTM"]])) {
    msg = paste('PTM peak list is missing. Input data must be of type list with',
                  'an element named \"PTM\" - stop')
    # getOption("MSstatsLog")("ERROR", msg)
    stop(msg)
  }
  ##TODO: Add a check to make sure Site was added into protein name?
  if (type == 'LF') {
    
    LF.cols = c("BioReplicate", "Condition", "FragmentIon", "Intensity", 
                 "IsotopeLabelType", "PeptideSequence", "PrecursorCharge",
                 "ProductCharge", "ProteinName", "Run")
    provided_cols = intersect(LF.cols, colnames(data[["PTM"]]))
    
    if (length(provided_cols) < 10){
      msg = paste("Missing columns in the PTM input:",
                   paste(setdiff(LF.cols, colnames(data[["PTM"]])), 
                         collapse = " "))
      #getOption("MSstatsLog")("ERROR", msg)
      stop(msg)
    }
  } else if (type == 'TMT'){
    ## Add peptide sequence if not available
    if (!'PeptideSequence' %in% colnames(data)){
      data[["PTM"]]$PeptideSequence = data[["PTM"]]$ProteinName
    }
    
    tmt.columns = c('ProteinName', 'PeptideSequence', 'Charge', 'PSM', 
                     'Mixture', 'TechRepMixture', 'Run', 'Channel', 'Condition',
                     'BioReplicate', 'Intensity')
    provided_cols = intersect(tmt.columns, colnames(data[["PTM"]]))
    
    if (length(provided_cols) < 11){
      msg = paste("Missing columns in the PTM input:",
                   paste(setdiff(tmt.columns, colnames(data[["PTM"]])),
                         collapse = " "))
      #getOption("MSstatsLog")("ERROR", msg)
      stop(msg)
    }
  }
  # Check the PROTEIN data
  if (is.null(data[["PROTEIN"]])) {
    adjust.protein = FALSE
  } else {
    adjust.protein = TRUE
    
    ## Check Protein data.table columns
    if (type == 'LF') {
      LF.cols = c("BioReplicate", "Condition", "FragmentIon", "Intensity", 
                   "IsotopeLabelType", "PeptideSequence", "PrecursorCharge",
                   "ProductCharge", "ProteinName", "Run")
      provided_cols = intersect(LF.cols, colnames(data[["PROTEIN"]]))
      
      if (length(provided_cols) < 10){
        msg = paste("Missing columns in the PROTEIN input:",
                     paste(setdiff(LF.cols, colnames(data[["PROTEIN"]])), 
                           collapse = " "))
        #getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
      }
    } else if (type == 'TMT'){
      tmt.columns = c('ProteinName', 'PeptideSequence', 'Charge', 'PSM', 
                       'Mixture', 'TechRepMixture', 'Run', 'Channel', 
                       'Condition', 'BioReplicate', 'Intensity')
      provided_cols = intersect(tmt.columns, colnames(data[["PROTEIN"]]))
      
      if (length(provided_cols) < 11){
        msg = paste("Missing columns in the PROTEIN input:",
                     paste(setdiff(tmt.columns, colnames(data[["PROTEIN"]])),
                           collapse = " "))
        #getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
      }
    }
  }
  ## Conditional for protein adjustment
  return(adjust.protein)
}

#' Check validity of parameters to dataProcess function.
#' @param log_base of logarithmic transformation
#' @param normalization_method string: "quantile", "equalizemedians", "FALSE",
#' "NONE" or "globalStandards"
#' @param normalization_method.PTM string: "quantile", "equalizemedians", 
#' "FALSE", "NONE" or "globalStandards"
#' @param address string
#' @param fill_rows logical, if TRUE, missing run observations for each feature
#' will be added with INTENSITY = NA
#' @param feature_selection list with elements: remove_uninformative
#' @param feature_selection.PTM list with elements: remove_uninformative
#' @param summarization list with elements: method.
#' @param imputation list with elements: cutoff, symbol.
#' @param n_clusters integer
#' @noRd
.checkDataProcessParams = function(log_base, normalization_method,
                                   normalization_method.PTM,
                                   standards_names, standards_names.PTM, 
                                   address,
                                   feature_selection, feature_selection.PTM,
                                   summarization, imputation) {
  
  checkmate::assertChoice(log_base, c(2, 10), .var.name = "logTrans")
  checkmate::assertChoice(summarization$method, c("linear", "TMP"),
                          .var.name = "summaryMethod")
  #getOption("MSstatsLog")("INFO", paste("Summary method:", 
  #                                      summarization$method))
  #getOption("MSstatsLog")("INFO", paste("cutOffCensored:", imputation$cutoff))
  checkmate::assertChoice(imputation$symbol, c("0", "NA"), 
                          null.ok = TRUE, .var.name = "censoredInt")
  #getOption("MSstatsLog")("INFO", paste("censoredInt:", imputation$symbol))
  if (summarization$method == "TMP" & imputation$MB & is.null(imputation$symbol)
      ) {
    msg = paste("The combination of required input",
                "MBimpute == TRUE and censoredInt = NULL",
                "has no censore missing values.",
                "Imputation will not be performed.- stop")
    getOption("MSstatsLog")("ERROR", msg)
    stop(msg)
  }
  checkmate::assertChoice(toupper(as.character(normalization_method)),
                          c("FALSE", "EQUALIZEMEDIANS", "QUANTILE", 
                            "GLOBALSTANDARDS"), .var.name = "normalization")
  checkmate::assertChoice(toupper(as.character(normalization_method.PTM)),
                          c("FALSE", "EQUALIZEMEDIANS", "QUANTILE", 
                            "GLOBALSTANDARDS"), .var.name = "PTM normalization")
  if (toupper(as.character(normalization_method)) == "GLOBALSTANDARDS" &
      is.null(standards_names)) {
    msg = paste("For normalization with global standards,",
                "the names of global standards are needed.",
                "Please add 'nameStandards' input.")
    #getOption("MSstatsLog")("ERROR", msg)
    stop(msg)
  }
  ## Remove this if global standards is not needed for PTM data
  if (toupper(as.character(normalization_method.PTM)) == "GLOBALSTANDARDS" &
      is.null(standards_names.PTM)) {
    msg = paste("For normalization with global standards,",
                "the names of global standards are needed.",
                "Please add 'nameStandards' input.")
    #getOption("MSstatsLog")("ERROR", msg)
    stop(msg)
  }
}

#' Check validity of parameters to TMT proteinSummarization function.
#' @param method string: "msstats", "MedianPolish", "Median", "LogSum"
#' @param global_norm logical: if True will normalize to protein-level
#' @param global_norm.PTM logical: if True will normalize to PTM-level
#' @param reference_norm logical: Indicates if reference channel was included
#' @param reference_norm,PTM logical: Indicates if reference channel was included
#' @param remove_norm_channel logical: remove Norm channel
#' @param remove_empty_channel logical: remove empty channels
#' @param MBimpute logical: only for method "msstats"
#' @param maxQuantileforCensored: Null or double
#' @noRd
.checkDataProcessParams.TMT = function(method, global_norm, global_norm.PTM,
                            reference_norm, reference_norm.PTM, 
                            remove_norm_channel, remove_empty_channel,
                            MBimpute, maxQuantileforCensored) {
  
  checkmate::assertChoice(method, c("msstats", "MedianPolish", "Median", 
                                    "LogSum"), .var.name = "Method")
  checkmate::assertLogical(global_norm, .var.name = "global_norm")
  checkmate::assertLogical(global_norm.PTM, .var.name = "global_norm.PTM")
  checkmate::assertLogical(reference_norm, .var.name = "reference_norm")
  checkmate::assertLogical(reference_norm.PTM, .var.name = "reference_norm.PTM")
  checkmate::assertLogical(remove_norm_channel, 
                           .var.name = "remove_norm_channel")
  checkmate::assertLogical(remove_empty_channel, 
                           .var.name = "remove_empty_channel")
  checkmate::assertLogical(MBimpute, .var.name = "MBimpute")
  
  if(!(is.double(maxQuantileforCensored)|is.null(maxQuantileforCensored))){
    msg = "maxQuantileforCensored must be either NULL or double - stop"
    stop(msg)
  }
}

# MSstatsPTM:::.checkAnnotation tests
## Test missing Run/Raw.file columns
annotation_no_run <- data.frame(
    Condition = c("A", "B"),
    BioReplicate = c(1, 2)
)

expect_error(
    MSstatsPTM:::.checkAnnotation(annotation_no_run, "LF"),
    pattern = "Run column missing in annotation file"
)

expect_error(
    MSstatsPTM:::.checkAnnotation(annotation_no_run, "TMT"),
    pattern = "Run column missing in annotation file"
)

## Test valid Run column
annotation_with_run <- data.frame(
    Run = c("run1", "run2"),
    Condition = c("A", "B")
)

expect_silent(MSstatsPTM:::.checkAnnotation(annotation_with_run, "LF"))

## Test valid Raw.file column
annotation_with_rawfile <- data.frame(
    Raw.file = c("file1.raw", "file2.raw"),
    Condition = c("A", "B")
)

expect_silent(MSstatsPTM:::.checkAnnotation(annotation_with_rawfile, "LF"))

## Test both Run and Raw.file columns
annotation_with_both <- data.frame(
    Run = c("run1", "run2"),
    Raw.file = c("file1.raw", "file2.raw"),
    Condition = c("A", "B")
)

expect_silent(MSstatsPTM:::.checkAnnotation(annotation_with_both, "LF"))

## Test LF label_type with all valid columns
annotation_lf_valid <- data.frame(
    Run = c("run1", "run2"),
    Raw.file = c("file1.raw", "file2.raw"),
    Condition = c("A", "B"),
    BioReplicate = c(1, 2),
    IsotopeLabelType = c("L", "H"),
    Fraction = c(1, 1)
)

expect_silent(MSstatsPTM:::.checkAnnotation(annotation_lf_valid, "LF"))

## Test LF label_type with subset of valid columns
annotation_lf_subset <- data.frame(
    Run = c("run1", "run2"),
    Condition = c("A", "B"),
    BioReplicate = c(1, 2)
)

expect_silent(MSstatsPTM:::.checkAnnotation(annotation_lf_subset, "LF"))

## Test TMT label_type with all valid columns
annotation_tmt_valid <- data.frame(
    Run = c("run1", "run2"),
    Raw.file = c("file1.raw", "file2.raw"),
    Fraction = c(1, 1),
    TechRepMixture = c(1, 1),
    Channel = c("126", "127"),
    Condition = c("A", "B"),
    Mixture = c(1, 1),
    BioReplicate = c(1, 2)
)

expect_silent(MSstatsPTM:::.checkAnnotation(annotation_tmt_valid, "TMT"))

## Test TMT label_type with subset of valid columns
annotation_tmt_subset <- data.frame(
    Run = c("run1", "run2"),
    Channel = c("126", "127"),
    Condition = c("A", "B")
)

expect_silent(MSstatsPTM:::.checkAnnotation(annotation_tmt_subset, "TMT"))

## Test extra columns for LF
annotation_lf_extra <- data.frame(
    Run = c("run1", "run2"),
    Condition = c("A", "B"),
    ExtraColumn = c("extra1", "extra2")  # This should not be allowed
)

expect_error(
    MSstatsPTM:::.checkAnnotation(annotation_lf_extra, "LF"),
    pattern = "Extra columns included in the annotation file"
)

## Test that error message contains allowed columns for LF
expect_error(
    MSstatsPTM:::.checkAnnotation(annotation_lf_extra, "LF"),
    pattern = "Run, Raw.file, Condition, BioReplicate, IsotopeLabelType, Fraction"
)

## Test extra columns for TMT
annotation_tmt_extra <- data.frame(
    Run = c("run1", "run2"),
    Channel = c("126", "127"),
    Condition = c("A", "B"),
    InvalidColumn = c("invalid1", "invalid2")  # This should not be allowed
)

expect_error(
    MSstatsPTM:::.checkAnnotation(annotation_tmt_extra, "TMT"),
    pattern = "Extra columns included in the annotation file"
)

## Test that error message contains allowed columns for TMT
expect_error(
    MSstatsPTM:::.checkAnnotation(annotation_tmt_extra, "TMT"),
    pattern = "Run, Raw.file, Fraction, TechRepMixture, Channel, Condition, Mixture, BioReplicate"
)

## Test empty data frame
annotation_empty <- data.frame()

expect_error(
    MSstatsPTM:::.checkAnnotation(annotation_empty, "LF"),
    pattern = "Run column missing in annotation file"
)

## Test only Run column
annotation_only_run <- data.frame(
    Run = c("run1", "run2")
)

expect_silent(MSstatsPTM:::.checkAnnotation(annotation_only_run, "LF"))
expect_silent(MSstatsPTM:::.checkAnnotation(annotation_only_run, "TMT"))

## Test only Raw.file column
annotation_only_rawfile <- data.frame(
    Raw.file = c("file1.raw", "file2.raw")
)

expect_silent(MSstatsPTM:::.checkAnnotation(annotation_only_rawfile, "LF"))
expect_silent(MSstatsPTM:::.checkAnnotation(annotation_only_rawfile, "TMT"))

## Test different label_type values
annotation_basic <- data.frame(
    Run = c("run1", "run2"),
    Condition = c("A", "B")
)

expect_silent(MSstatsPTM:::.checkAnnotation(annotation_basic, "LF"))
expect_silent(MSstatsPTM:::.checkAnnotation(annotation_basic, "TMT"))

## Test unknown label_type
expect_error(
    MSstatsPTM:::.checkAnnotation(annotation_basic, "UNKNOWN"),
    pattern = "Labeling type must be either LF or TMT"
)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width=8, 
  fig.height=8
)

## ----eval = FALSE-------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("MSstatsPTM")

## ----message=FALSE, warning=FALSE---------------------------------------------
library(MSstatsPTM)
library(MSstats)

## ----maxq, eval=TRUE----------------------------------------------------------
# TMT experiment
head(maxq_tmt_evidence)
head(maxq_tmt_annotation)
 
msstats_format_tmt = MaxQtoMSstatsPTMFormat(evidence=maxq_tmt_evidence,
                                    annotation=maxq_tmt_annotation,
                                    fasta=system.file("extdata", 
                                                      "maxq_tmt_fasta.fasta", 
                                                      package="MSstatsPTM"),
                                    fasta_protein_name="uniprot_ac",
                                    mod_id="\\(Phospho \\(STY\\)\\)",
                                    use_unmod_peptides=TRUE,
                                    labeling_type = "TMT",
                                    which_proteinid_ptm = "Proteins")

head(msstats_format_tmt$PTM)
head(msstats_format_tmt$PROTEIN)

# LF experiment
head(maxq_lf_evidence)
head(maxq_lf_annotation)

msstats_format_lf = MaxQtoMSstatsPTMFormat(evidence=maxq_lf_evidence,
                                     annotation=maxq_lf_annotation,
                                     fasta=system.file("extdata", 
                                                       "maxq_lf_fasta.fasta", 
                                                       package="MSstatsPTM"),
                                     fasta_protein_name="uniprot_ac",
                                     mod_id="\\(Phospho \\(STY\\)\\)",
                                     use_unmod_peptides=TRUE,
                                     labeling_type = "LF",
                                     which_proteinid_ptm = "Proteins")

head(msstats_format_lf$PTM)
head(msstats_format_lf$PROTEIN)


## ----fragpipe, eval=TRUE------------------------------------------------------
# TMT example
head(fragpipe_input)
head(fragpipe_annotation)
head(fragpipe_input_protein)
head(fragpipe_annotation_protein)

msstats_data = FragPipetoMSstatsPTMFormat(fragpipe_input,
                                          fragpipe_annotation,
                                          fragpipe_input_protein, 
                                          fragpipe_annotation_protein,
                                          mod_id_col = "STY",
                                          localization_cutoff=.75,
                                          remove_unlocalized_peptides=TRUE)
head(msstats_data$PTM)
head(msstats_data$PROTEIN)

# LFQ Example
input = system.file("tinytest/raw_data/Fragpipe/MSstats.csv", 
                                        package = "MSstatsPTM")
input = data.table::fread(input)
annot = system.file("tinytest/raw_data/Fragpipe/experiment_annotation.tsv", 
                                        package = "MSstatsPTM")
annot = data.table::fread(annot)    
input_protein = system.file("tinytest/raw_data/Fragpipe/msstats_proteome_lf.csv",
                                        package = "MSstatsPTM")

input_protein = data.table::fread(input_protein) # Global profiling run

msstats_data = FragPipetoMSstatsPTMFormat(input,
                                          annot,
                                          input_protein = input_protein,
                                          label_type="LF",
                                          mod_id_col = "STY",
                                          localization_cutoff=.75,
                                          protein_id_col = "ProteinName",
                                          peptide_id_col = "PeptideSequence")
head(msstats_data$PTM)
head(msstats_data$PROTEIN)

## ----pd, eval=TRUE------------------------------------------------------------
head(pd_psm_input)
head(pd_annotation)

msstats_format = PDtoMSstatsPTMFormat(pd_psm_input, 
                                 pd_annotation, 
                                 system.file("extdata", "pd_fasta.fasta", 
                                             package="MSstatsPTM"),
                                 use_unmod_peptides=TRUE, 
                                 which_proteinid = "Master.Protein.Accessions")

head(msstats_format$PTM)
head(msstats_format$PROTEIN)

## ----Spectronaut, eval=TRUE---------------------------------------------------
head(spectronaut_input)
head(spectronaut_annotation)

msstats_input = SpectronauttoMSstatsPTMFormat(spectronaut_input, 
                  annotation=spectronaut_annotation, 
                  fasta_path=system.file("extdata", "spectronaut_fasta.fasta", 
                                         package="MSstatsPTM"),
                  use_unmod_peptides=TRUE,
                  mod_id = "\\[Phospho \\(STY\\)\\]",
                  fasta_protein_name = "uniprot_iso"
                  )
 
head(msstats_input$PTM)
head(msstats_input$PROTEIN)


## ----raw_data, eval = FALSE---------------------------------------------------
# # Add site into ProteinName column
# raw_ptm_df$ProteinName = paste(raw_ptm_df$ProteinName,
#                                 raw_ptm_df$Site, sep = "_")
# 
# # Run MSstats Converters
# PTM_data = MSstats::DIANNtoMSstatsFormat(raw_ptm_df, annotation)
# PROTEIN_data = MSstats::DIANNtoMSstatsFormat(raw_protein_df, annotation)
# 
# # Combine into one list
# msstatsptm_input_data = list(PTM = PTM_data, PROTEIN = PROTEIN_data)

## ----summarize, message=FALSE, warning=FALSE----------------------------------

MSstatsPTM.summary = dataSummarizationPTM(raw.input, verbose = FALSE, 
                                          use_log_file = FALSE, append = FALSE)

head(MSstatsPTM.summary$PTM$ProteinLevelData)
head(MSstatsPTM.summary$PROTEIN$ProteinLevelData)

## ----qcplot, message=FALSE, warning=FALSE-------------------------------------
dataProcessPlotsPTM(MSstatsPTM.summary,
                    type = 'QCPLOT',
                    which.PTM = "allonly",
                    address = FALSE)

## ----profileplot, message=FALSE, warning=FALSE--------------------------------

dataProcessPlotsPTM(MSstatsPTM.summary,
                    type = 'ProfilePlot',
                    which.Protein = "Q9Y6C9",
                    address = FALSE)

## ----model, message=FALSE, warning=FALSE--------------------------------------

# Specify contrast matrix
comparison = matrix(c(-1,0,1,0),nrow=1)
row.names(comparison) = "CCCP-Ctrl"
colnames(comparison) = c("CCCP", "Combo", "Ctrl", "USP30_OE")

MSstatsPTM.model = groupComparisonPTM(MSstatsPTM.summary, 
                                      ptm_label_type = "LF",
                                      protein_label_type = "LF",
                                      contrast.matrix = comparison,
                                      use_log_file = FALSE, append = FALSE,
                                      verbose = FALSE)
head(MSstatsPTM.model$PTM.Model)
head(MSstatsPTM.model$PROTEIN.Model)
head(MSstatsPTM.model$ADJUSTED.Model)

## ----volcano, message=FALSE, warning=FALSE------------------------------------
groupComparisonPlotsPTM(data = MSstatsPTM.model,
                        type = "VolcanoPlot",
                        FCcutoff= 2,
                        logBase.pvalue = 2,
                        address=FALSE)

## ----heatmap, message=FALSE, warning=FALSE------------------------------------
groupComparisonPlotsPTM(data = MSstatsPTM.model,
                        type = "Heatmap",
                        which.PTM = 1:30,
                        address=FALSE)

## ----sample_size, message=FALSE, warning=FALSE--------------------------------

# Specify contrast matrix
sample_size = designSampleSizePTM(MSstatsPTM.model, c(2.0, 2.75), FDR = 0.05, 
                                  numSample = TRUE, power = 0.8)

head(sample_size)

## ----sample_size_plot, message=FALSE, warning=FALSE---------------------------

MSstats::designSampleSizePlots(sample_size)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width=8, 
  fig.height=8
)

## ----message=FALSE, warning=FALSE---------------------------------------------
library(MSstatsPTM)

## ----eval = FALSE-------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("MSstatsPTM")

## ----summarize, echo=FALSE, message=FALSE, warning=FALSE----------------------

MSstatsPTM.summary <- dataSummarizationPTM_TMT(raw.input.tmt, 
                                               use_log_file = FALSE, 
                                               append = FALSE, verbose = FALSE)

## ----show_summ----------------------------------------------------------------
head(MSstatsPTM.summary$PTM$ProteinLevelData)
head(MSstatsPTM.summary$PROTEIN$ProteinLevelData)

## ----qcplot, message=FALSE, warning=FALSE-------------------------------------

dataProcessPlotsPTM(MSstatsPTM.summary,
                    type = 'QCPLOT',
                    which.PTM = "allonly",
                    address = FALSE)

## ----profileplot, message=FALSE, warning=FALSE--------------------------------
dataProcessPlotsPTM(MSstatsPTM.summary,
                    type = 'PROFILEPLOT',
                    which.Protein = c("Protein_12"),
                    address = FALSE)

## ----model, message=FALSE, warning=FALSE--------------------------------------

# Specify contrast matrix
comparison <- matrix(c(1,0,0,-1,0,0,
                       0,1,0,0,-1,0,
                       0,0,1,0,0,-1,
                       1,0,-1,0,0,0,
                       0,1,-1,0,0,0,
                       0,0,0,1,0,-1,
                       0,0,0,0,1,-1),nrow=7, ncol=6, byrow=TRUE)

# Set the names of each row
row.names(comparison)<-c('1-4', '2-5', '3-6', '1-3', 
                         '2-3', '4-6', '5-6')
colnames(comparison) <- c('Condition_1','Condition_2','Condition_3',
                          'Condition_4','Condition_5','Condition_6')
MSstatsPTM.model <- groupComparisonPTM(MSstatsPTM.summary,
                                       ptm_label_type = "TMT",
                                       protein_label_type = "TMT",
                                       contrast.matrix = comparison,
                                       use_log_file = FALSE, append = FALSE)
head(MSstatsPTM.model$PTM.Model)
head(MSstatsPTM.model$PROTEIN.Model)
head(MSstatsPTM.model$ADJUSTED.Model)

## ----volcano, message=FALSE, warning=FALSE------------------------------------
groupComparisonPlotsPTM(data = MSstatsPTM.model,
                        type = "VolcanoPlot",
                        which.Comparison = c('1-4'),
                        which.PTM = 1:50,
                        address=FALSE)

## ----meatmap, message=FALSE, warning=FALSE------------------------------------
groupComparisonPlotsPTM(data = MSstatsPTM.model,
                        type = "Heatmap",
                        which.PTM = 1:49,
                        address=FALSE)

#' Determine if data is label free or TMT
#' @noRd
.check.dataProcess.plotting.data = function(data, type, ylimUp, ylimDown, 
                                             x.axis.size, y.axis.size, 
                                             text.size, text.angle, legend.size,
                                             dot.size.profile, ncol.guide, 
                                             width, height, ptm.title, 
                                             protein.title, which.Protein, 
                                             originalPlot, summaryPlot, address
                                             ) {
  
  ## Check input columns
  assertChoice(type, c("PROFILEPLOT", "QCPLOT"), .var.name = "Type") 
  if (!is.logical(ylimUp)){assertNumeric(ylimUp, .var.name = "ylimUp")}
  if (!is.logical(ylimDown)){assertNumeric(ylimDown, .var.name = "ylimDown")}
  assertNumeric(x.axis.size, .var.name = ("x.axis.size"))
  assertNumeric(y.axis.size, .var.name = ("y.axis.size"))
  assertNumeric(dot.size.profile, .var.name = ("dot.size.profile"))
  assertNumeric(text.size, .var.name = ("text.size"))
  assertNumeric(text.angle, .var.name = ("text.angle"))
  assertNumeric(legend.size, .var.name = ("legend.size"))
  assertNumeric(ncol.guide, .var.name = ("ncol.guide"))
  assertNumeric(width, .var.name = ("width"))
  assertNumeric(height, .var.name = ("height"))
  assertCharacter(ptm.title, .var.name = ("ptm.title"))
  assertCharacter(protein.title, .var.name = ("protein.title"))
  assertLogical(originalPlot, .var.name = c("originalPlot"))
  assertLogical(summaryPlot, .var.name = c("summaryPlot"))
  
  
  ## Test if input is labelfree or tmt
  data.ptm = data[['PTM']]$FeatureLevelData
  if ('Mixture' %in% colnames(data.ptm)){
    label = "TMT"
  }
  else {
    label = "LabelFree"
  }
  return(label)
}

#' Extract global protein from combined protein and site column
#' @noRd
.extractProteinPlot = function(ptm_model, protein_model){
  
  ## All proteins
  available_proteins = unique(as.character(protein_model$PROTEINNAME))
  available_proteins = available_proteins[order(nchar(available_proteins),
                                                 available_proteins,
                                                 decreasing = TRUE)]
  available_ptms = unique(as.character(ptm_model$PROTEINNAME))
  
  ## Call Rcpp function
  ptm_proteins = extract_protein_name(available_ptms,
                                       available_proteins)
  global_protein_lookup = data.table(PROTEINNAME = available_ptms, 
                                      GLOBALPROTEIN = ptm_proteins)

  return(global_protein_lookup)
}

#' Prepare data into format needed for plotting
#' @noRd
.format.data.process.plots = function(data, label) {
  
  data.ptm = data[['PTM']]
  data.protein = data[['PROTEIN']]
  
  datafeature.ptm = data.ptm$FeatureLevelData
  datafeature.ptm = as.data.table(datafeature.ptm)
  
  datarun.ptm = data.ptm$ProteinLevelData
  datarun.ptm = as.data.table(datarun.ptm)
  
  data.list = list(datafeature.ptm, datarun.ptm)
  
  if (!is.null(data.protein)) {
    datafeature.protein = data.protein$FeatureLevelData
    datafeature.protein = as.data.table(datafeature.protein)
    
    datarun.protein = data.protein$ProteinLevelData
    datarun.protein = as.data.table(datarun.protein)
    
    data.list = list(datafeature.protein, datafeature.ptm, 
                      datarun.protein, datarun.ptm)
  }
  
  ## Adjust colnames to avoid repeating code
  for (i in seq_along(data.list)) {
    colnames(data.list[[i]]) = toupper(colnames(data.list[[i]]))
    if ('INTENSITY' %in% colnames(data.list[[i]])){
      data.list[[i]]$ABUNDANCE = log2(data.list[[i]]$INTENSITY)
    }
    setnames(data.list[[i]], c('LOGINTENSITIES', 'GROUP', 'PROTEIN', 
                               'Protein', 'LOG2INTENSITY'), 
             c('ABUNDANCE', 'CONDITION', 'PROTEINNAME', 'PROTEINNAME',
               'ABUNDANCE'), 
             skip_absent = TRUE)
    
    data.list[[i]][!is.na(data.list[[i]]$ABUNDANCE) &
                     data.list[[i]]$ABUNDANCE < 1, 'ABUNDANCE'] = 0
    data.list[[i]] = data.list[[i]][ABUNDANCE > 0, ]
  }
  
  ## Run Rcpp code to match proteins
  if (!is.null(data.protein)) {
    protein_lookup = .extractProteinPlot(data.list[[2]], data.list[[1]])
    
    ## Add extracted protein name into model
    data.list[[2]] = merge(data.list[[2]], protein_lookup,
                       all.x = TRUE, by = 'PROTEINNAME')
    data.list[[4]] = merge(data.list[[4]], protein_lookup,
                            all.x = TRUE, by = 'PROTEINNAME')
  
    # conditions in feature data
    fea.conds.protein = as.character(unique(data.list[[1]]$CONDITION))
    fea.conds.ptm = as.character(unique(data.list[[2]]$CONDITION))
    # conditions in protein data
    run.conds.protein = as.character(unique(data.list[[3]]$CONDITION))
    run.conds.ptm = as.character(unique(data.list[[4]]$CONDITION))
    
    # only keep the overlapped conditions between feature and run data
    shared.conds = Reduce(intersect, list(fea.conds.protein, fea.conds.ptm,
                                           run.conds.protein, run.conds.ptm))
    for (i in seq_along(data.list)) {
      data.list[[i]]$PROTEINNAME = factor(data.list[[i]]$PROTEINNAME)
      data.list[[i]]$RUN = factor(data.list[[i]]$RUN)
      data.list[[i]]$CONDITION = factor(data.list[[i]]$CONDITION)	
      data.list[[i]] = data.list[[i]][data.list[[i]]$CONDITION %in% shared.conds
                                       ,]
    }
  } else {
    fea.conds.ptm = as.character(unique(data.list[[1]]$CONDITION))
    run.conds.ptm = as.character(unique(data.list[[2]]$CONDITION))
    
    # only keep the overlapped conditions between feature and run data
    shared.conds = Reduce(intersect, list(fea.conds.ptm, run.conds.ptm))
    for (i in seq_along(data.list)) {
      data.list[[i]]$PROTEINNAME = factor(data.list[[i]]$PROTEINNAME)
      data.list[[i]]$RUN = factor(data.list[[i]]$RUN)
      data.list[[i]]$CONDITION = factor(data.list[[i]]$CONDITION)	
      data.list[[i]] = data.list[[i]][data.list[[i]]$CONDITION %in% shared.conds
                                       ,]
    }
  }
  
  return(data.list)
}

#' Format TMT data for profile plot and qcplot
#' @noRd
.preplot.format.tmt = function(datafeature, datarun, y.limup, type){
  
  RUN = CONDITION = CHANNEL = groupAxis = .N = cumGroupAxis = NULL
  
  datafeature = datafeature[with(datafeature, order(RUN, CONDITION, CHANNEL)),]

  datafeature$CONDITION = factor(datafeature$CONDITION)
  datarun$CONDITION = factor(datarun$CONDITION)
  
  datafeature$RUN = factor(datafeature$RUN)
  datarun$RUN = factor(datarun$RUN)
  
  ## !! important: order of x-axis
  ## can be reorder by group and then channel, WITHIN Run
  ## first make new column for x-axis
  datafeature$group.channel = paste(datafeature$CONDITION, datafeature$CHANNEL, 
                                     sep = "_")
  
  ## not sure better way for coding
  ## potentially change it.
  datafeature$xorder = factor()
  
  for (k in seq_along(unique(datafeature$RUN))) {
    
    runid = as.character(unique(datafeature$RUN)[k])
    datafeature[datafeature$RUN == runid, ]$xorder = factor(
      datafeature[datafeature$RUN == runid, ]$group.channel,
      levels = unique(datafeature[datafeature$RUN == runid, ]$group.channel),
      labels = seq(1, length(unique(datafeature[datafeature$RUN == runid, 
                                                 ]$group.channel))))
  }
  
  ## need to make data.frame with same variables for condition name
  datafeature$xorder = as.numeric(datafeature$xorder)

  ## keep unique information for x-axis labeling. will be used in plotting
  tempGroupName = unique(datafeature[, c("CONDITION", "xorder", "RUN", 
                                          "CHANNEL")])
  groupline = copy(tempGroupName)
  groupline[, groupAxis := .N, by=list(CONDITION, RUN)]
  groupline[, c("xorder", "CHANNEL") := NULL][]
  groupline = groupline[!duplicated(groupline), ]
  
  groupline[, cumGroupAxis := cumsum(groupAxis), by = list(RUN)]
  groupline$cumGroupAxis = groupline$cumGroupAxis + 0.5

  ## add coordinate for group id
  groupline$xorder = groupline$cumGroupAxis - groupline$groupAxis / 2
  groupline$abundance = y.limup - 0.5
  
  groupline.all = groupline

  ## remove last condition for vertical line between groups
  groupline = groupline[-which(groupline$CONDITION %in% levels(
      groupline$CONDITION)[nlevels(groupline$CONDITION)]), ]
  
  ## need to fill in incomplete rows for Runlevel data
  if (type == 'PROFILEPLOT'){
    haverun = FALSE
    
    if (sum(is.element(colnames(datarun), "RUN")) != 0) {
      datamat = dcast(PROTEINNAME + CHANNEL ~ RUN, data = datarun,
                       value.var = 'ABUNDANCE', keep = TRUE)
  
      datarun = melt(datamat, id.vars=c('PROTEINNAME', 'CHANNEL'))
      colnames(datarun)[colnames(datarun) %in% c("variable", "value")
                        ] = c('RUN', 'ABUNDANCE')
  
      ## match x axis order
      datarun = merge(datarun, tempGroupName, by = c('RUN', 'CHANNEL'))
      haverun = TRUE
    }
  }
  
  return(list(datafeature, datarun, groupline, groupline.all))
}

#' Plot standard profile plot for TMT
#' @noRd
.plot.profile.tmt = function(data.list, protein, y.limup, y.limdown, 
                              x.axis.size, y.axis.size, text.size, text.angle, 
                              legend.size, dot.size.profile, ncol.guide){
  
  cumGroupAxis = xorder = abundance = CONDITION = NULL
  
  datafeature = data.list[[1]]
  datarun = data.list[[2]]
  groupline = data.list[[3]]
  groupline.all = data.list[[4]]
  
  sub = datafeature[datafeature$PROTEINNAME == protein, ]
  
  sub$PEPTIDESEQUENCE = factor(as.character(sub$PEPTIDESEQUENCE))
  sub$CHARGE = factor(as.character(sub$CHARGE))
  sub$PSM = factor(as.character(sub$PSM))
  
  # if all measurements are NA,
  if (nrow(sub) == sum(is.na(sub$ABUNDANCE))|
      nrow(sub) == sum(!is.na(sub$ABUNDANCE) & sub$ABUNDANCE == 0)) {
    message(paste0("Can't the Profile plot for ", protein,
      " because all measurements are NAs or zero."))
  }
  
  ## seq for peptide and charge
  ## for seting up color and linetype
  b = unique(sub[, c("PEPTIDESEQUENCE", "PSM")])
  ## add because if there are missing value, orders are different.
  b = b[with(b, order(PEPTIDESEQUENCE, PSM)), ]
  
  temp1 = xtabs(~PEPTIDESEQUENCE, b)
  
  ## unique charge id within peptide sequence, for line type
  ss = NULL
  ## unique peptide sequence id, for color
  s = NULL
  
  for (j in seq_along(temp1)) {
    temp3 = rep(j, temp1[j])
    s = c(s, temp3)
    temp2 = seq(1, temp1[j])
    ss = c(ss, temp2)
  }
  
  ## for annotation of condition
  groupline.tmp = data.table(groupline, "PSM" = unique(sub$PSM)[1], 
                              "PEPTIDESEQUENCE" = unique(sub$PEPTIDESEQUENCE)[1]
                              )
  
  groupline.all.tmp = data.table(groupline.all, "PSM" = unique(sub$PSM)[1],
                                  "PEPTIDESEQUENCE" = unique(sub$PEPTIDESEQUENCE
                                                             )[1])
  
  ## 2019. 12. 17, MC : for profile plot, define color for dot
  cbp = c("#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2",
           "#D55E00", "#CC79A7")
  check.length = length(unique(s)) %/% length(cbp)
  if ( check.length > 0 ){
    cbp = rep(cbp, times=check.length + 1)
  } else {
    cbp = cbp
  }
  
  yaxis.name = 'Log2-intensities'
  ## 1st plot for Protein plot
  protein_temp = ggplot(aes(x = .data$xorder, y = .data$ABUNDANCE,
                                    color = .data$PSM, linetype = .data$PSM),
                         data = sub) +
    facet_grid(~RUN) +
    geom_point(data = sub, aes(shape=.data$CENSORED), size=dot.size.profile, na.rm=TRUE) +
    geom_line(linewidth = 0.5, na.rm=TRUE) +
    scale_colour_manual(values=cbp[s]) +
    scale_linetype_manual(values = ss) +
    scale_shape_manual(values = c(16, 1),labels = c("Detected data", "Censored missing data")) +
    labs(title = unique(sub$PROTEINNAME),
         x = 'MS runs') +
    scale_y_continuous(yaxis.name, limits = c(y.limdown, y.limup)) +
    scale_x_continuous('MS runs') +
    geom_vline(data = groupline.tmp,
               aes(xintercept = .data$cumGroupAxis),
               colour = "grey", linetype = "longdash") +
    geom_text(data = groupline.all.tmp,
              aes(x = .data$xorder, y = .data$abundance, label = .data$CONDITION),
              size = text.size,
              angle = text.angle, hjust = .9,
              color = "black") +
    theme_msstats(type = "PROFILEPLOT",x.axis.size, y.axis.size, legend.size,
                  text_angle = text.angle) +
    guides(color = guide_legend(title = paste("# peptide:", nlevels(
      sub$PEPTIDESEQUENCE)),
      title.theme = element_text(size = 13, angle = 0),
      keywidth = 0.4,
      keyheight = 0.1,
      default.unit = 'inch',
      ncol = ncol.guide),
      linetype = guide_legend(
        title = paste("# peptide:",
                      nlevels(sub$PEPTIDESEQUENCE)),
        title.theme = element_text(size = 13, angle = 0),
        keywidth = 0.4,
        keyheight = 0.1,
        default.unit = 'inch',
        ncol = ncol.guide))
  
  return(protein_temp)
  
}

#' Plot summarized profile plot for TMT
#' @noRd
.plot.profile.summary.tmt = function(data.list, protein, y.limup, y.limdown, 
                                      x.axis.size, y.axis.size, text.size, 
                                      text.angle, legend.size, dot.size.profile,
                                      ncol.guide){
  
  cumGroupAxis = xorder = abundance = CONDITION = ABUNDANCE = analysis = NULL
  
  datafeature = data.list[[1]]
  datarun = data.list[[2]]
  groupline = data.list[[3]]
  groupline.all = data.list[[4]]
  
  sub = datafeature[datafeature$PROTEINNAME == protein, ]
  
  sub$PEPTIDESEQUENCE = factor(as.character(sub$PEPTIDESEQUENCE))
  sub$CHARGE = factor(as.character(sub$CHARGE))
  sub$PSM = factor(as.character(sub$PSM))
  
  # if all measurements are NA,
  if (nrow(sub) == sum(is.na(sub$ABUNDANCE))|
      nrow(sub) == sum(!is.na(sub$ABUNDANCE) & sub$ABUNDANCE == 0)) {
    message(paste0("Can't the Profile plot for ", protein,
                   " because all measurements are NAs or zero."))
  }
  
  ## for annotation of condition
  groupline.tmp = data.table(groupline, "PSM" = unique(sub$PSM)[1],
                              "PEPTIDESEQUENCE" = unique(
                                sub$PEPTIDESEQUENCE)[1])
  
  groupline.all.tmp = data.table(groupline.all, "PSM" = unique(sub$PSM)[1],
                                          "PEPTIDESEQUENCE" = unique(
                                            sub$PEPTIDESEQUENCE)[1])
  
  subrun = datarun[datarun$PROTEINNAME == protein, ]
  
  if (nrow(subrun) != 0) {
    
    quantrun = sub[1, ]
    quantrun = quantrun[rep(seq_len(nrow(subrun))), ]
    
    quantrun$PROTEINNAME = subrun$PROTEINNAME
    quantrun$GLOBALPROTEIN = subrun$GLOBALPROTEIN
    quantrun$group.channel = NA
    quantrun$INTENSITY = NA
    quantrun$CONDITION = NA
    quantrun$MIXTURE = NA
    quantrun$TECHREPMIXTURE = NA
    quantrun$BIOREPLICATE = NA
    quantrun$PEPTIDESEQUENCE = "Run summary"
    quantrun$CHARGE = "Run summary"
    quantrun$PSM = "Run summary"
    quantrun$CHANNEL = subrun$CHANNEL
    quantrun$RUN = subrun$RUN
    quantrun$ABUNDANCE = subrun$ABUNDANCE
    quantrun$xorder = subrun$xorder
    
  } else {
    ## if there is only one Run measured across all runs
    ## no Run information for linear with censored
    quantrun = datafeature[1, ]
    quantrun[, 2:ncol(quantrun)] = NA
    
    quantrun$PROTEINNAME = levels(datafeature$PROTEINNAME)[1]
    quantrun$PEPTIDESEQUENCE = "Run summary"
    quantrun$CHARGE = "Run summary"
    quantrun$PSM = "Run summary"
    quantrun$ABUNDANCE = NA
    quantrun$INTENSITY = NA
  }
  
  quantrun$analysis = "Run summary"
  sub$analysis = "Processed feature-level data"
  
  final = rbindlist(list(sub, quantrun), fill=TRUE)
  final$analysis = factor(final$analysis)
  final$PSM = factor(final$PSM)
  yaxis.name = 'Log2-intensities'

  ## Draw summarized ptm plot
  ptempall = ggplot(aes(x = .data$xorder, y = .data$ABUNDANCE, 
                                color = .data$analysis, linetype = .data$PSM, 
                                size = .data$analysis), data = final) +
    facet_grid(~RUN) +
    # geom_point(size = dot.size.profile, na.rm=TRUE) +
    geom_point(data = final, aes(shape=.data$CENSORED), size=dot.size.profile, na.rm=TRUE) +
    geom_line(linewidth = 0.5, na.rm=TRUE) +
    scale_colour_manual(values = c("lightgray", "darkred")) +
    scale_shape_manual(values = c(16, 1),labels = c("Detected data", "Censored missing data")) +
    scale_size_manual(values = c(1.7, 2), guide = "none") +
    scale_linetype_manual(values = c(rep(1, times = length(
      unique(final$PSM))-1), 2), guide = "none") +
    labs(title = unique(sub$PROTEINNAME),
         x = 'MS runs') +
    scale_y_continuous(yaxis.name, limits = c(y.limdown, y.limup)) +
    geom_vline(data = groupline.tmp,
               aes(xintercept = .data$cumGroupAxis),
               colour = "grey", linetype = "longdash") +
    geom_text(data = groupline.all.tmp,
              aes(x = .data$xorder, y = .data$abundance, label = .data$CONDITION),
              size = text.size,
              angle = text.angle, hjust = .9,
              color = "black") +
    theme_msstats(type = "PROFILEPLOT",x.axis.size, y.axis.size, legend.size,
                  text_angle = text.angle) +
    guides(color = guide_legend(order = 1,
                                title = NULL,
                                label.theme = element_text(
                                  size = 10, angle = 0)))
  
  ## draw point again because some red summary dots could be hiden
  ptempall = ptempall + geom_point(data = final, aes(
    x = .data$xorder, y = .data$ABUNDANCE, size = .data$analysis, color = .data$analysis)
    )

  return(ptempall)
}

#' Wrapper function for TMT profile plot
#' @noRd
.profile.tmt = function(data.table.list, type, ylimUp, ylimDown, 
                          x.axis.size, y.axis.size,text.size,text.angle, 
                          legend.size, dot.size.profile, ncol.guide, width, 
                          height,which.Protein,originalPlot,summaryPlot,
                          address, isPlotly){
  
  PROTEINNAME = GLOBALPROTEIN = NULL
  
  if (length(data.table.list) == 4){
    datafeature.protein = data.table.list[[1]]
    datafeature.ptm = data.table.list[[2]]
    datarun.protein = data.table.list[[3]]
    datarun.ptm = data.table.list[[4]]
    y.limup = ceiling(max(datafeature.protein$ABUNDANCE,
                           datafeature.ptm$ABUNDANCE, na.rm = TRUE) + 5)
  } else {
    datafeature.ptm = data.table.list[[1]]
    datarun.ptm = data.table.list[[2]]
    
    y.limup = ceiling(max(datafeature.ptm$ABUNDANCE, na.rm = TRUE) + 5)
  }
  ## TODO: Add log
  # processout = rbind(processout,
  #                     c("ProfilePlot plotting started."))
  
  
  if (length(data.table.list) == 4){
    plot_global = TRUE
  } else {
    plot_global = FALSE
  }
  
  ## choose Proteins or not
  if (which.Protein[[1]] != "all") {
    ## check which.Protein is name of Protein
    if (is.character(which.Protein)) {
      temp.name = which.Protein
      
      ## message if name of Protein is wrong.
      if (length(setdiff(temp.name,unique(datafeature.ptm$PROTEINNAME))) > 0) {
        stop("Please check protein name. Dataset does not
             have this protein. - ",
             toString(temp.name))
      }
    }
    
    ## check which.Protein is order number of Protein
    else if (is.numeric(which.Protein)) {
      temp.name = levels(datafeature.ptm$PROTEINNAME)[which.Protein]
      
      ## message if name of Protein is wrong.
      if (length(levels(datafeature.ptm$PROTEINNAME)) < max(which.Protein)) {
        stop("Please check your number of proteins. There are ",
             length(levels(datafeature.ptm$PROTEINNAME))," proteins in this
             dataset.")
      }
    }
    
    ## use only assigned proteins
    datafeature.ptm = datafeature.ptm[which(datafeature.ptm$PROTEINNAME %in%
                                               temp.name), ]
    datafeature.ptm$PROTEINNAME = factor(datafeature.ptm$PROTEINNAME)
    datarun.ptm = datarun.ptm[which(datarun.ptm$PROTEINNAME %in% temp.name), ]
    datarun.ptm$PROTEINNAME = factor(datarun.ptm$PROTEINNAME)
    
    if (length(data.table.list) == 4){
      temp_proteins = as.character(unique(datafeature.ptm$GLOBALPROTEIN))
      
      plot_global = TRUE
      ## Check if there is a corresponding protein for the PTM
      if (!temp_proteins %in% datafeature.protein$PROTEINNAME){
        message(paste0("Global Protein data not available for ", 
                       as.character(temp_proteins), 
                       ", only PTM will be plotted.")
        )
        plot_global = FALSE
      }
      
      datafeature.protein = datafeature.protein[which(
        datafeature.protein$PROTEINNAME %in% temp_proteins), ]
      datafeature.protein$PROTEINNAME = factor(datafeature.protein$PROTEINNAME)
      
      datarun.protein = datarun.protein[which(datarun.protein$PROTEINNAME %in%
                                                 temp_proteins), ]
      datarun.protein$PROTEINNAME = factor(datarun.protein$PROTEINNAME)
    }
  }
  
  ## assign upper or lower limit
  if (is.numeric(ylimUp)) {
    y.limup = ylimUp
  }
  if (is.numeric(ylimDown)) {
    y.limdown = ylimDown
  } else {
    y.limdown = 0
  }
  
  ## Apply pre-plot formatting
  ptm.list = .preplot.format.tmt(datafeature.ptm, datarun.ptm, 
                                       y.limup, type)
  datafeature.ptm = ptm.list[[1]]
  datarun.ptm = ptm.list[[2]]
  groupline.ptm = ptm.list[[3]]
  groupline.all.ptm = ptm.list[[4]]
  
  if (plot_global){
    protein.list = .preplot.format.tmt(datafeature.protein, datarun.protein, 
                                        y.limup, type)
    datafeature.protein = protein.list[[1]]
    datarun.protein = protein.list[[2]]
    groupline.protein = protein.list[[3]]
    groupline.all.protein = protein.list[[4]]
    
    ## Only plot proteins that occur in both datasets
    global_proteins = unique(datafeature.protein[, PROTEINNAME])
    ptm_proteins = unique(datafeature.ptm[, GLOBALPROTEIN])
    plot_proteins = intersect(ptm_proteins, global_proteins)
    datafeature.ptm = datafeature.ptm[GLOBALPROTEIN %in% plot_proteins,]
    plot_proteins = unique(datafeature.ptm[, c('PROTEINNAME', 'GLOBALPROTEIN')]
                            )
  } else {
    plot_proteins = unique(datafeature.ptm[, c('PROTEINNAME')])
  }
  
  # init list to return plots
  output_plots <- list()
  output_plots[["original_plot"]] = list()
  output_plots[["summary_plot"]] = list()

  if (originalPlot) {
    if (!isPlotly && address != FALSE) {
      allfiles = list.files()
      
      num = 0
      filenaming = paste0(address, "ProfilePlot")
      finalfile = paste0(address, "ProfilePlot.pdf")
      
      while (is.element(finalfile, allfiles)) {
        num = num + 1
        finalfile = paste0(paste(filenaming, num, sep = "-"), ".pdf")
      }
      
      pdf(finalfile, width = width, height = height)
    }
    
    ## factoring for run, channel, condition should be done before loop
    
    for (i in seq_len(nrow(plot_proteins))) {
      
      ptm_temp = .plot.profile.tmt(ptm.list, 
                           as.character(plot_proteins[, PROTEINNAME][i]), 
                           y.limup, y.limdown, x.axis.size,
                           y.axis.size, text.size, text.angle, legend.size,
                           dot.size.profile, ncol.guide)
      if (plot_global){
        protein_temp = .plot.profile.tmt(protein.list, 
                                 as.character(
                                   plot_proteins[, GLOBALPROTEIN][i]), 
                                 y.limup, y.limdown, x.axis.size,
                                 y.axis.size, text.size, text.angle, 
                                 legend.size, dot.size.profile, ncol.guide)
        original_profile_plots <- list(PTEMP.PTM = ptm_temp, PTEMP.PROTEIN = protein_temp)
        grid.arrange(ptm_temp, protein_temp, ncol=1)
      } else {
        original_profile_plots <- list(PTEMP.PTM = ptm_temp, PTEMP.PROTEIN = NULL)
        print(ptm_temp)
      }
      output_plots[["original_plot"]][[paste("plot",i)]] <- original_profile_plots
      message(paste0("Drew the Profile plot for ", 
                    as.character(plot_proteins[, PROTEINNAME][i]),
                    " (", i, " of ", nrow(plot_proteins), ")"))
    }
    # end-loop for each protein
    
    if (address != FALSE & !isPlotly) {
      dev.off()
    }
    
  } # end original plot
  
  ############################################
  ## 2nd plot for original plot : summary
  ############################################
  
  if (summaryPlot) {
    if (!isPlotly && address != FALSE) {
      allfiles = list.files()
      
      num = 0
      filenaming = paste0(address, "ProfilePlot_wSummarization")
      finalfile = paste0(address, "ProfilePlot_wSummarization.pdf")
      
      while (is.element(finalfile, allfiles)) {
        num = num + 1
        finalfile = paste0(paste(filenaming, num, sep = "-"), ".pdf")
      }
      
      pdf(finalfile, width = width, height = height)
    }
    
    for (i in seq_len(nrow(plot_proteins))) {
      
      ptm_temp = .plot.profile.summary.tmt(ptm.list, as.character(
        plot_proteins[, PROTEINNAME][i]), y.limup, y.limdown, x.axis.size,
        y.axis.size, text.size, text.angle, legend.size,dot.size.profile, 
        ncol.guide)
      if (plot_global){
        protein_temp = .plot.profile.summary.tmt(protein.list, as.character(
          plot_proteins[, GLOBALPROTEIN][i]), y.limup, y.limdown, x.axis.size,
          y.axis.size, text.size, text.angle, legend.size,
          dot.size.profile, ncol.guide)
        summary_profile_plots <- list(PTEMP.PTM = ptm_temp, PTEMP.PROTEIN = protein_temp)
        grid.arrange(ptm_temp, protein_temp, ncol=1)
      } else {
        summary_profile_plots <- list(PTEMP.PTM = ptm_temp, PTEMP.PROTEIN = NULL)
        print(ptm_temp)
      }
      output_plots[["summary_plot"]][[paste("plot",i)]] <- summary_profile_plots
      message(paste("Drew the Profile plot for ", 
                    as.character(plot_proteins[, PROTEINNAME][i]),
                    "(", i, " of ", nrow(plot_proteins), ")"))

      }
      
    if (address!=FALSE & !isPlotly) {
      dev.off()
    }
  } # end summarization plot
  if(isPlotly) {
    output_plots
  }
}

#' Plot boxplot of all proteins
#' @noRd
.qc.all.plot = function(datafeature, groupline, groupline.all, title, ylimup, 
                         ylimdown, x.axis.size, y.axis.size, text.size, 
                         text.angle) {
  
  cumGroupAxis = xorder = abundance = CONDITION = NULL
  
  ## for annotation of condition
  groupline.tmp = data.table(groupline, "PSM" = unique(datafeature$PSM)[1], 
                              "PEPTIDESEQUENCE" = unique(
                                datafeature$PEPTIDESEQUENCE)[1])
  
  groupline.all.tmp = data.table(groupline.all, 
                                  "PSM" = unique(datafeature$PSM)[1], 
                                  "PEPTIDESEQUENCE" = unique(
                                    datafeature$PEPTIDESEQUENCE)[1])
  
  ## 1st plot for original plot
  ## for boxplot, x-axis, xorder should be factor
  datafeature$xorder = factor(datafeature$xorder)

  ## y-axis labeling
  yaxis.name = 'Log2-intensities'
  
  ptemp = ggplot(aes(x = .data$xorder, y = .data$ABUNDANCE),
                      data = datafeature) +
    facet_grid(~RUN) +
    geom_boxplot(aes(fill = .data$CONDITION), outlier.shape = 1,
                 outlier.size = 1.5) +
    labs(title = title, x = 'MS runs') +
    scale_y_continuous(yaxis.name, limits = c(ylimdown, ylimup)) +
    geom_vline(data = groupline.tmp,
               aes(xintercept = .data$cumGroupAxis),
               colour = "grey", linetype = "longdash") +
    geom_text(data = groupline.all.tmp,
              aes(x = .data$xorder, y = .data$abundance, label = .data$CONDITION),
              size = text.size,
              angle = text.angle, hjust = .9,
              color = "black") +
    theme_msstats(type = "PROFILEPLOT", x.axis.size, y.axis.size, 13, 
                  element_rect(fill = "gray95"),
                  element_text(colour = c("#00B0F6"), size = 14),
                  "none", text_angle = text.angle) +
    theme(strip.text.x = element_text(
            size = 5,angle = 15))
    # theme(
    #   panel.background = element_rect(fill = 'white', colour = "black"),
    #   legend.key = element_rect(fill = 'white', colour = 'white'),
    #   panel.grid.minor = element_blank(),
    #   strip.background = element_rect(fill = 'gray95'),
    #   axis.ticks.x = element_blank(),
    #   axis.text.x = element_blank(),
    #   axis.text.y = element_text(size = y.axis.size, colour = "black"),
    #   axis.ticks = element_line(colour = "black"),
    #   axis.title.x = element_text(size = x.axis.size + 5, vjust = -0.4),
    #   axis.title.y = element_text(size = y.axis.size + 5, vjust = 0.3),
    #   title = element_text(size = x.axis.size + 8, vjust = 1.5),
    #   legend.position = "none")

  return(ptemp)
}

#' Plot boxplot of single protein
#' @noRd
.qc.single.plot = function(datafeature, groupline, groupline.all, protein, 
                            ylimup, ylimdown, x.axis.size, y.axis.size, 
                            text.size, text.angle){
  
  ABUNDANCE = cumGroupAxis = xorder = abundance = CONDITION = NULL
  
  sub = datafeature[datafeature$PROTEINNAME == protein]
  sub = sub[!is.na(sub$ABUNDANCE)]
  
  ## if all protein measurements are NA,
  if (nrow(sub) == sum(sub[!is.na(sub$ABUNDANCE), ABUNDANCE])) {
    message(paste("Can't the Quality Control plot for ", unique(
      sub$PROTEINNAME), " because all measurements are NAs."))
  }
  
  ## for annotation of condition
  groupline.tmp = data.table(groupline, "PSM" = unique(sub$PSM)[1], 
                              "PEPTIDESEQUENCE" = unique(sub$PEPTIDESEQUENCE)[1]
                              )
  
  groupline.all.tmp = data.table(groupline.all, "PSM" = unique(sub$PSM)[1],
                                  "PEPTIDESEQUENCE" = unique(
                                    sub$PEPTIDESEQUENCE)[1])
  
  ## 1st plot for original plot
  ## for boxplot, x-axis, xorder should be factor
  sub$xorder = factor(sub$xorder)
  yaxis.name = 'Log2-intensities'
  
  ptemp = ggplot(aes(x = .data$xorder, y = .data$ABUNDANCE),
                      data = sub) +
    facet_grid(~RUN) +
    geom_boxplot(aes(fill = .data$CONDITION), outlier.shape = 1,
                 outlier.size = 1.5) +
    labs(title = protein, x = 'MS runs') +
    scale_y_continuous(yaxis.name, limits = c(ylimdown, ylimup)) +
    geom_vline(data = groupline.tmp,
               aes(xintercept = .data$cumGroupAxis),
               colour = "grey", linetype = "longdash") +
    geom_text(data = groupline.all.tmp,
              aes(x = .data$xorder, y = .data$abundance, label = .data$CONDITION),
              size = text.size,
              angle = text.angle, hjust = .9,
              color = "black") +
    theme_msstats(type = "PROFILEPLOT", x.axis.size, y.axis.size, 13, 
                  element_rect(fill = "gray95"),
                  element_text(colour = c("#00B0F6"), size = 14),
                  "none", text_angle = text.angle) +
    theme(strip.text.x = element_text(
      size = 5,angle = 15))
    # theme(
    #   panel.background = element_rect(fill = 'white', colour = "black"),
    #   legend.key = element_rect(fill = 'white', colour = 'white'),
    #   panel.grid.minor = element_blank(),
    #   strip.background = element_rect(fill = 'gray95'),
    #   axis.ticks.x = element_blank(),
    #   axis.text.x = element_blank(),
    #   axis.text.y = element_text(size = y.axis.size, colour = "black"),
    #   axis.ticks = element_line(colour = "black"),
    #   axis.title.x = element_text(size = x.axis.size + 5, vjust = -0.4),
    #   axis.title.y = element_text(size = y.axis.size + 5, vjust = 0.3),
    #   title = element_text(size = x.axis.size + 8, vjust = 1.5),
    #   legend.position = "none")
  
  return(ptemp)
  
}

#' Wrapper function for plotting QC Plots
#' @noRd
.qc.tmt = function(data.table.list, type, ylimUp, ylimDown, width, height,
                    x.axis.size, y.axis.size,text.size, text.angle, 
                    which.Protein, address, ptm_title, protein_title, isPlotly) {
  
  PROTEINNAME = GLOBALPROTEIN = NULL
  
  if (length(data.table.list) == 4){
    datafeature.protein = data.table.list[[1]]
    datafeature.ptm = data.table.list[[2]]
    datarun.protein = data.table.list[[3]]
    datarun.ptm = data.table.list[[4]]
    
    y.limup = ceiling(max(datafeature.protein$ABUNDANCE,
                           datafeature.ptm$ABUNDANCE, na.rm = TRUE) + 3)
  } else {
    datafeature.ptm = data.table.list[[1]]
    datarun.ptm = data.table.list[[2]]
    
    y.limup = ceiling(max(datafeature.ptm$ABUNDANCE, na.rm = TRUE) + 3)
  }
  
  ## save the plots as pdf or not
  ## If there are the file with the same name
  ## add next numbering at the end of file name
  if (!isPlotly && address != FALSE) {
    allfiles = list.files()
    
    num = 0
    filenaming = paste0(address,"QCPlot")
    finalfile = paste0(address,"QCPlot.pdf")
    
    while (is.element(finalfile, allfiles)) {
      num = num + 1
      finalfile = paste0(paste(filenaming, num, sep = "-"), ".pdf")
    }
    
    pdf(finalfile, width = width, height = height)
  }
  
  ## assign upper or lower limit
  if (is.numeric(ylimUp)) {
    y.limup = ylimUp
  }
  
  y.limdown = 0
  if (is.numeric(ylimDown)) {
    y.limdown = ylimDown
  }
  
  ## Apply pre-plot formatting
  ptm.list = .preplot.format.tmt(datafeature.ptm, datarun.ptm, y.limup, 
                                       type)
  datafeature.ptm = ptm.list[[1]]
  datarun.ptm = ptm.list[[2]]
  groupline.ptm = ptm.list[[3]]
  groupline.all.ptm = ptm.list[[4]]
  
  if (length(data.table.list) == 4){
    protein.list = .preplot.format.tmt(datafeature.protein, datarun.protein,
                                        y.limup, type)
    datafeature.protein = protein.list[[1]]
    datarun.protein = protein.list[[2]]
    groupline.protein = protein.list[[3]]
    groupline.all.protein = protein.list[[4]]
  }
  
  # Init list to return plots
  output_plots = list()
    
  ## all protein
  if (which.Protein[[1]] == 'all' | which.Protein[[1]] == 'allonly') {
    
    ## Plot all QC
    ptemp.ptm = .qc.all.plot(datafeature.ptm, groupline.all.ptm, 
                              groupline.all.ptm, ptm_title, y.limup, y.limdown, 
                              x.axis.size, y.axis.size, text.size, text.angle)
    if (length(data.table.list) == 4){
      ptemp.protein = .qc.all.plot(datafeature.protein, groupline.protein, 
                                    groupline.all.protein, protein_title, y.limup,
                                    y.limdown, x.axis.size, y.axis.size,text.size, 
                                    text.angle)
      grid.arrange(ptemp.ptm, ptemp.protein, ncol=1)
      all_protein_plots <- list(PTEMP.PTM = ptemp.ptm, PTEMP.PROTEIN = ptemp.protein)
    } else {
      all_protein_plots <- list(PTEMP.PTM = ptemp.ptm, PTEMP.PROTEIN = NULL)
      print(ptemp.ptm)
    }
    output_plots[[1]] = all_protein_plots
    message("Drew the Quality Contol plot(boxplot) for all ptms/proteins.")
  }
  
  if (length(data.table.list) == 4){
    plot_global = TRUE
  } else {
    plot_global = FALSE
  }
  
  ## each protein
  ## choose Proteins or not
  if (which.Protein[[1]] != 'allonly') {
    if (which.Protein[[1]] != "all") {
      ## check which.Protein is name of Protein
      if (is.character(which.Protein)) {
        temp.name = which.Protein
        
        ## message if name of Protein is wrong.
        if (length(setdiff(temp.name,unique(datafeature.ptm$PROTEINNAME))) > 0){
          stop("Please check protein name.
               Data set does not have this protein. - ",
               toString(temp.name))
        }
      }
      
      ## check which.Protein is order number of Protein
      if (is.numeric(which.Protein)) {
        temp.name = levels(datafeature.ptm$PROTEINNAME)[which.Protein]
        
        ## message if name of Protein is wrong.
        if (length(levels(datafeature.ptm$PROTEINNAME)) < max(which.Protein)) {
          stop("Please check your number of proteins. There are ",
               length(levels(datafeature.ptm$PROTEINNAME)),
               " proteins in this dataset.")
        }
      }
      
      ## use only assigned proteins
      datafeature.ptm = datafeature.ptm[which(
        datafeature.ptm$PROTEINNAME %in% temp.name), ]
      datafeature.ptm$PROTEINNAME = factor(datafeature.ptm$PROTEINNAME)
      if (length(data.table.list) == 4){
        temp_proteins = unique(datafeature.ptm$GLOBALPROTEIN)
      
        plot_global = TRUE
        ## Check if there is a corresponding protein for the PTM
        if (!temp_proteins %in% datafeature.protein$PROTEINNAME){
          message(paste0("Global Protein data not available for ", 
                         as.character(temp_proteins), 
                         ", only PTM will be plotted.")
          )
          plot_global = FALSE
        }
        
        datafeature.protein = datafeature.protein[
          which(datafeature.protein$PROTEINNAME %in% temp_proteins), ]
        datafeature.protein$PROTEINNAME = factor(
          datafeature.protein$PROTEINNAME)
      }
    }

    ## Only plot proteins that occur in both datasets
    if (plot_global){
      global_proteins = unique(datafeature.protein[, PROTEINNAME])
      ptm_proteins = unique(datafeature.ptm[, GLOBALPROTEIN])
      plot_proteins = intersect(ptm_proteins, global_proteins)
      datafeature.ptm = datafeature.ptm[GLOBALPROTEIN %in% plot_proteins,]
      plot_proteins = unique(datafeature.ptm[, c('PROTEINNAME', 'GLOBALPROTEIN')]
      )
    } else {
      plot_proteins = unique(datafeature.ptm[, c('PROTEINNAME')])
    }
    
    for (i in seq_len(nrow(plot_proteins))) {

      ptemp.ptm = .qc.single.plot(datafeature.ptm, groupline.ptm, 
                                   groupline.all.ptm, as.character(
                                     plot_proteins[, PROTEINNAME][[i]]), 
                                   y.limup, y.limdown, x.axis.size, y.axis.size, 
                                   text.size, text.angle)
      if (plot_global){
        ptemp.protein = .qc.single.plot(datafeature.protein, groupline.protein, 
                                         groupline.all.protein, as.character(
                                           plot_proteins[, GLOBALPROTEIN][i]), 
                                         y.limup, y.limdown, x.axis.size, 
                                         y.axis.size, text.size, text.angle)
        grid.arrange(ptemp.ptm, ptemp.protein, ncol=1)
        single_protein_plots <- list(PTEMP.PTM = ptemp.ptm, PTEMP.PROTEIN = ptemp.protein)
      } else {
        print(ptemp.ptm)
        single_protein_plots <- list(PTEMP.PTM = ptemp.ptm, PTEMP.PROTEIN = NULL)
      }
      output_plots[[i+1]] = single_protein_plots # to accomodate all proteins
      message(paste("Drew the Quality Contol plot(boxplot) for",
                    as.character(plot_proteins[, PROTEINNAME][i]), "(", i, 
                    " of ", nrow(plot_proteins), ")"))
      
    } # end-loop
    }
  
  if (address != FALSE) {
    dev.off()
  }
  
  if(isPlotly) {
    output_plots <- Filter(function(x) !is.null(x), output_plots)
    output_plots
  }
}

#' Format labelfree data for plotting
#' @noRd
.preplot.format.lf = function(datafeature, datarun, y.limup, type){
  
  datafeature = datafeature[with(datafeature, 
                                  order(CONDITION, SUBJECT, LABEL)),]
  
  ## Set factors
  datafeature$CONDITION = factor(datafeature$CONDITION)
  datarun$CONDITION = factor(datarun$CONDITION)
  datafeature$RUN = factor(datafeature$RUN)
  datarun$RUN = factor(datarun$RUN)
  
  tempGroupName = unique(datafeature[, c("CONDITION", "RUN")])
  
  groupAxis = as.numeric(xtabs(~CONDITION, tempGroupName))
  cumGroupAxis = cumsum(groupAxis)
  lineNameAxis = cumGroupAxis[-nlevels(datafeature$CONDITION)]
  
  groupName = data.table(RUN=c(0, lineNameAxis) + groupAxis / 2 + 0.5,
                          lineNameAxis = c(0, lineNameAxis),
                          ABUNDANCE=rep(y.limup-1, length(groupAxis)), 
                          Name=levels(datafeature$CONDITION))
  
  
  if (length(unique(datafeature$LABEL)) == 2) {
    datafeature$LABEL = factor(datafeature$LABEL, labels=c("Reference", 
                                                            "Endogenous"))	
  } else {
    if (unique(datafeature$LABEL) == "L") {
      datafeature$LABEL = factor(datafeature$LABEL, labels=c("Endogenous"))	
    }
    if (unique(datafeature$LABEL) == "H") {
      datafeature$LABEL = factor(datafeature$LABEL, labels=c("Reference"))
    }
  }
  
  ## need to fill in incomplete rows for Runlevel data
  if (type == 'PROFILEPLOT'){
    haverun = FALSE
    
    if (sum(is.element(colnames(datarun), "RUN")) != 0) {
      datamat = dcast(PROTEINNAME ~ RUN, data=datarun, value.var='ABUNDANCE', 
                       keep=TRUE) 
      
      datarun = melt(datamat, id.vars=c('PROTEINNAME'))
      colnames(datarun)[colnames(datarun) %in% c("variable", "value")
                        ] = c('RUN', 'ABUNDANCE')
      
      haverun = TRUE
    }
  }
  
  return(list(datafeature, datarun, groupName))
}

#' Plot individual profile for label free
#' @noRd
.plot.profile.lf = function(data.list, protein, y.limup, y.limdown, 
                              x.axis.size, y.axis.size, text.size, text.angle, 
                              legend.size, dot.size.profile, ncol.guide){
  
  lineNameAxis = RUN = ABUNDANCE = Name = NULL
  
  datafeature = data.list[[1]]
  datarun = data.list[[2]]
  groupname = data.list[[3]]
  
  sub = datafeature[datafeature$PROTEINNAME == protein, ]
  sub$RUN = as.numeric(sub$RUN)
  sub$PEPTIDE = factor(as.character(sub$PEPTIDE))
  
  # if all measurements are NA,
  if (nrow(sub) == sum(is.na(sub$ABUNDANCE))|
      nrow(sub) == sum(!is.na(sub$ABUNDANCE) & sub$ABUNDANCE == 0)) {
    message(paste0("Can't the Profile plot for ", protein,
                   " because all measurements are NAs or zero."))
  }
  
  ## seq for peptide and charge
  ## for seting up color and linetype
  b = unique(sub[, c("PEPTIDE", "FEATURE")])
  ## add because if there are missing value, orders are different.
  b = b[with(b, order(PEPTIDE, FEATURE)), ]
  
  temp1 = xtabs(~data.frame(b)[, 1])
  
  ## unique charge id within peptide sequence, for line type
  ss = NULL
  ## unique peptide sequence id, for color
  s = NULL
  
  for (j in seq_len(length(temp1))) {
    temp3 = rep(j, temp1[j])
    s = c(s, temp3)
    temp2 = seq(1, temp1[j])
    ss = c(ss, temp2)	
  }
  
  ## for annotation of condition
  groupNametemp = data.table(groupname, 
                              "FEATURE"=unique(sub$FEATURE)[1], 
                              "PEPTIDE"=unique(sub$PEPTIDE)[1])
  
  ## 2019. 12. 17, MC : for profile plot, define color for dot
  cbp = c("#E69F00", "#56B4E9", "#009E73", "#F0E442", 
           "#0072B2", "#D55E00", "#CC79A7")
  check.length = length(unique(s)) %/% length(cbp)
  if ( check.length > 0 ){
    cbp = rep(cbp, times=check.length + 1)
  }
  
  yaxis.name = 'Log-intensities'
  sub$group_aes = paste(sub$CONDITION, sub$FEATURE, sep = "_")
  ## 1st plot for Protein plot
  protein_temp = ggplot(data=sub) + facet_grid(~LABEL) +
    geom_point(aes(x=.data$RUN, y=.data$ABUNDANCE, 
                          color=.data$FEATURE,  shape=.data$CENSORED), #
               size=dot.size.profile, na.rm=TRUE) +
    geom_line(aes(x=.data$RUN, y=.data$ABUNDANCE, 
                         color=.data$FEATURE, linetype=.data$FEATURE), #
              linewidth = 0.5, na.rm=TRUE) +
    scale_colour_manual(values=cbp[s]) +
    scale_linetype_manual(values = ss) +
    scale_shape_manual(values = c(16, 1),labels = c("Detected data", "Censored missing data")) +
    labs(title = unique(sub$PROTEINNAME),
         x = 'MS runs') +
    scale_y_continuous(yaxis.name, limits = c(y.limdown, y.limup)) +
    scale_x_continuous('MS runs', breaks=groupNametemp$lineNameAxis) +
    geom_vline(data = groupNametemp[lineNameAxis != 0],
               aes(xintercept = .data$lineNameAxis + 0.5),
               colour = "grey", linetype = "longdash") +
    geom_text(data = groupNametemp,
              aes(x = .data$RUN, y = .data$ABUNDANCE, label = .data$Name),
              size = text.size,
              angle = text.angle, hjust = .9,
              color = "black") +
    theme_msstats(type = "PROFILEPLOT",x.axis.size, y.axis.size, legend.size,
                  text_angle = text.angle) +
    guides(color = guide_legend(title = paste("# peptide:", nlevels(
      sub$PEPTIDE)),
      title.theme = element_text(size = 13, angle = 0),
      keywidth = 0.4,
      keyheight = 0.1,
      default.unit = 'inch',
      ncol = ncol.guide),
      linetype = guide_legend(
        title = paste("# peptide:",
                      nlevels(sub$PEPTIDE)),
        title.theme = element_text(size = 13, angle = 0),
        keywidth = 0.4,
        keyheight = 0.1,
        default.unit = 'inch',
        ncol = ncol.guide))
  
  return(protein_temp)
}

#' Plot summary profile for label free
#' @noRd
.plot.profile.summary.lf = function(data.list, protein, y.limup, y.limdown, 
                             x.axis.size, y.axis.size, text.size, text.angle, 
                             legend.size, dot.size.profile, ncol.guide){
  
  lineNameAxis = RUN = ABUNDANCE = Name = NULL
  
  datafeature = data.list[[1]]
  datarun = data.list[[2]]
  groupname = data.list[[3]]
  
  sub = datafeature[datafeature$PROTEINNAME == protein, ]
  sub$RUN = as.numeric(sub$RUN)
  sub$PEPTIDE = factor(as.character(sub$PEPTIDE))
  
  # if all measurements are NA,
  if (nrow(sub) == sum(is.na(sub$ABUNDANCE))|
      nrow(sub) == sum(!is.na(sub$ABUNDANCE) & sub$ABUNDANCE == 0)) {
    message(paste0("Can't the Profile plot for ", protein,
                   " because all measurements are NAs or zero."))
  }
  
  ## seq for peptide and charge
  ## for seting up color and linetype
  b = unique(sub[, c("PEPTIDE", "FEATURE")])
  ## add because if there are missing value, orders are different.
  b = b[with(b, order(PEPTIDE, FEATURE)), ]
  
  temp1 = xtabs(~data.frame(b)[, 1])
  
  ## unique charge id within peptide sequence, for line type
  ss = NULL
  ## unique peptide sequence id, for color
  s = NULL
  
  for (j in seq_len(length(temp1))) {
    temp3 = rep(j, temp1[j])
    s = c(s, temp3)
    temp2 = seq(1, temp1[j])
    ss = c(ss, temp2)	
  }
  
  ## for annotation of condition
  groupNametemp = data.table(groupname, 
                              "FEATURE"=unique(sub$FEATURE)[1], 
                              "PEPTIDE"=unique(sub$PEPTIDE)[1])

  subrun = datarun[datarun$PROTEINNAME == protein, ]
  
  subrun$analysis = "Run summary"
  subrun$FEATURE = "Run summary"
  sub$analysis = "Processed feature-level data"
  final = rbindlist(list(sub,subrun), fill = TRUE)
  final$RUN = as.numeric(levels(final$RUN))[final$RUN]
  final = final[order(RUN)]
  #final$RUN = as.factor(final$RUN)
  
  yaxis.name = 'Log2-intensities'
  final$group_aes = paste(final$CONDITION, final$FEATURE, 
                           final$analysis, sep = "_")
  
  ## Draw summarized ptm plot
  ptempall = ggplot(data=final) +
    geom_point(aes(x=.data$RUN, y=.data$ABUNDANCE, 
                          color=.data$analysis, shape=.data$CENSORED), size = dot.size.profile, na.rm=TRUE) +
    geom_line(aes(x=.data$RUN, y=.data$ABUNDANCE, 
                         color=.data$analysis, linetype=.data$FEATURE), linewidth = 0.5, na.rm=TRUE) + 
    scale_colour_manual(values = c("lightgray", "darkred")) +
    scale_shape_manual(values = c(16, 1),labels = c("Detected data", "Censored missing data")) +
    scale_size_manual(values = c(1.7, 2), guide = "none") +
    scale_linetype_manual(values = c(rep(1, times = length(
      unique(final$FEATURE))-1), 2), guide = "none") +
    labs(title = unique(sub$PROTEINNAME),
         x = 'MS runs') +
    scale_y_continuous(yaxis.name, limits = c(y.limdown, y.limup)) +
    geom_vline(data = groupNametemp[lineNameAxis != 0],
               aes(xintercept = .data$lineNameAxis + 0.5),
               colour = "grey", linetype = "longdash") +
    geom_text(data = groupNametemp,
              aes(x = .data$RUN, y = .data$ABUNDANCE, label = .data$Name),
              size = text.size,
              angle = text.angle, hjust = .9,
              color = "black") +
    theme_msstats(type = "PROFILEPLOT", x.axis.size, y.axis.size, legend.size,
                  text_angle = text.angle) +
    guides(color = guide_legend(order = 1,
                                title = NULL,
                                label.theme = element_text(
                                  size = 10, angle = 0)))
  
  return(ptempall)
}

#' Wrapper function for label free profile plot 
#' @noRd
.profile.lf = function(data.table.list, type, ylimUp, ylimDown, 
                             x.axis.size, y.axis.size,text.size,text.angle, 
                             legend.size, dot.size.profile, ncol.guide, width, 
                             height,which.Protein,originalPlot,summaryPlot,
                             address, isPlotly) {
  
  PROTEINNAME = GLOBALPROTEIN = NULL
  
  if (length(data.table.list) == 4){
    datafeature.protein = data.table.list[[1]]
    datafeature.ptm = data.table.list[[2]]
    datarun.protein = data.table.list[[3]]
    datarun.ptm = data.table.list[[4]]
    
    y.limup = ceiling(max(datafeature.protein$ABUNDANCE,
                           datafeature.ptm$ABUNDANCE, na.rm = TRUE) + 5)
  } else {
    datafeature.ptm = data.table.list[[1]]
    datarun.ptm = data.table.list[[2]]
    
    y.limup = ceiling(max(datafeature.ptm$ABUNDANCE, na.rm = TRUE) + 5)
  }
  
  ## TODO: Add log
  # processout = rbind(processout,
  #                     c("ProfilePlot plotting started."))
  
  if (length(data.table.list) == 4){
    plot_global = TRUE
  } else {
    plot_global = FALSE
  }
  
  ## choose Proteins or not
  if (which.Protein[[1]] != "all") {
    ## check which.Protein is name of Protein
    if (is.character(which.Protein)) {
      temp.name = which.Protein
      
      ## message if name of Protein is wrong.
      if (length(setdiff(temp.name,unique(datafeature.ptm$PROTEINNAME))) > 0) {
        stop("Please check protein name. Dataset does not
             have this protein. - ",
             toString(temp.name))
      }
    }
    
    ## check which.Protein is order number of Protein
    else if (is.numeric(which.Protein)) {
      temp.name = levels(datafeature.ptm$PROTEINNAME)[which.Protein]
      
      ## message if name of Protein is wrong.
      if (length(levels(datafeature.ptm$PROTEINNAME)) < max(which.Protein)) {
        stop("Please check your number of proteins. There are ",
             length(levels(datafeature.ptm$PROTEINNAME))," proteins in this
             dataset.")
      }
    }
    
    ## use only assigned proteins
    datafeature.ptm = datafeature.ptm[which(datafeature.ptm$PROTEINNAME %in%
                                               temp.name), ]
    datarun.ptm = datarun.ptm[which(datarun.ptm$PROTEINNAME %in% temp.name), ]
    datarun.ptm$PROTEINNAME = factor(datarun.ptm$PROTEINNAME)
    datafeature.ptm$PROTEINNAME = factor(datafeature.ptm$PROTEINNAME)
    
    if (length(data.table.list) == 4){
      temp_proteins = as.character(unique(datafeature.ptm$GLOBALPROTEIN))
      
      plot_global = TRUE
      ## Check if there is a corresponding protein for the PTM
      if (!temp_proteins %in% datafeature.protein$PROTEINNAME){
        message(paste0("Global Protein data not available for some PTMs,", 
                        " only PTM will be plotted.")
                )
        plot_global = FALSE
      }
      
      datafeature.protein = datafeature.protein[which(
        datafeature.protein$PROTEINNAME %in% temp_proteins), ]
      datafeature.protein$PROTEINNAME = factor(datafeature.protein$PROTEINNAME)
      
      datarun.protein = datarun.protein[which(datarun.protein$PROTEINNAME %in%
                                                 temp_proteins), ]
      datarun.protein$PROTEINNAME = factor(datarun.protein$PROTEINNAME)
    }
  }
  
  ## assign upper or lower limit
  if (is.numeric(ylimUp)) {
    y.limup = ylimUp
  }
  if (is.numeric(ylimDown)) {
    y.limdown = ylimDown
  } else {
    y.limdown = 0
  }
  
  ptm.list = .preplot.format.lf(datafeature.ptm, datarun.ptm, 
                                  y.limup, type)
  datafeature.ptm = ptm.list[[1]]
  datarun.ptm = ptm.list[[2]]
  groupName.ptm = ptm.list[[3]]
  
  if (plot_global) {
    ## Apply pre-plot formatting
    protein.list = .preplot.format.lf(datafeature.protein, datarun.protein, 
                                       y.limup, type)
    datafeature.protein = protein.list[[1]]
    datarun.protein = protein.list[[2]]
    groupName.protein = protein.list[[3]]
    
    ## Only plot proteins that occur in both datasets
    global_proteins = unique(datafeature.protein[, PROTEINNAME])
    ptm_proteins = unique(datafeature.ptm[, GLOBALPROTEIN])
    plot_proteins = intersect(ptm_proteins, global_proteins)
    datafeature.ptm = datafeature.ptm[GLOBALPROTEIN %in% plot_proteins,]
    plot_proteins = unique(datafeature.ptm[, c('PROTEINNAME', 'GLOBALPROTEIN')]
                            )
  } else {
    plot_proteins = unique(datafeature.ptm[, c('PROTEINNAME')])
  }
  
  # init list to return plots
  output_plots <- list()
  output_plots[["original_plot"]] = list()
  output_plots[["summary_plot"]] = list()
  
  if (originalPlot) {
    if (!isPlotly && address != FALSE) {
      allfiles = list.files()
      
      num = 0
      filenaming = paste0(address, "ProfilePlot")
      finalfile = paste0(address, "ProfilePlot.pdf")
      
      while (is.element(finalfile, allfiles)) {
        num = num + 1
        finalfile = paste0(paste(filenaming, num, sep = "-"), ".pdf")
      }
      
      pdf(finalfile, width = width, height = height)
    }
    
    ## factoring for run, channel, condition should be done before loop
    
    for (i in seq_len(nrow(plot_proteins))) {
      
      ptm_temp = .plot.profile.lf(ptm.list,
                                   as.character(
                                     plot_proteins[, PROTEINNAME][i]), 
                                   y.limup, y.limdown, x.axis.size,
                                   y.axis.size, text.size, text.angle, 
                                   legend.size, dot.size.profile, ncol.guide)
      if (plot_global) {
        protein_temp = .plot.profile.lf(protein.list, 
                                         as.character(
                                           plot_proteins[, GLOBALPROTEIN][i]), 
                                         y.limup, y.limdown, x.axis.size,
                                         y.axis.size, text.size, text.angle, 
                                         legend.size, dot.size.profile, 
                                         ncol.guide)
        original_profile_plots <- list(PTEMP.PTM = ptm_temp, PTEMP.PROTEIN = protein_temp)
        grid.arrange(ptm_temp, protein_temp, ncol=1)
      } else{
        original_profile_plots <- list(PTEMP.PTM = ptm_temp, PTEMP.PROTEIN = NULL)
        print(ptm_temp)
      }
      output_plots[["original_plot"]][[paste("plot",i)]] <- original_profile_plots
      message(paste0("Drew the Profile plot for ", 
                    as.character(plot_proteins[, PROTEINNAME][i]),
                    " (", i, " of ", nrow(plot_proteins), ")"))
    }
    # end-loop for each protein
    
    if (address != FALSE & !isPlotly) {
      dev.off()
    }
    
  } # end original plot
  
  ############################################
  ## 2nd plot for original plot : summary
  ############################################
  
  if (summaryPlot) {
    if (!isPlotly && address != FALSE) {
      allfiles = list.files()
      
      num = 0
      filenaming = paste0(address, "ProfilePlot_wSummarization")
      finalfile = paste0(address, "ProfilePlot_wSummarization.pdf")
      
      while (is.element(finalfile, allfiles)) {
        num = num + 1
        finalfile = paste0(paste(filenaming, num, sep = "-"), ".pdf")
      }
      
      pdf(finalfile, width = width, height = height)
    }
    
    for (i in seq_len(nrow(plot_proteins))) {
      
      ptm_temp = .plot.profile.summary.lf(ptm.list, as.character(
        plot_proteins[, PROTEINNAME][i]), y.limup, y.limdown, x.axis.size,
        y.axis.size, text.size, text.angle, legend.size,dot.size.profile, 
        ncol.guide)
      
      if (plot_global){
        protein_temp = .plot.profile.summary.lf(protein.list, as.character(
          plot_proteins[, GLOBALPROTEIN][i]), y.limup, y.limdown, x.axis.size,
          y.axis.size, text.size, text.angle, legend.size, dot.size.profile, 
          ncol.guide)
        summary_profile_plots <- list(PTEMP.PTM = ptm_temp, PTEMP.PROTEIN = protein_temp)
        grid.arrange(ptm_temp, protein_temp, ncol=1)
      } else {
        summary_profile_plots <- list(PTEMP.PTM = ptm_temp, PTEMP.PROTEIN = NULL)
        print(ptm_temp)
      }
      output_plots[["summary_plot"]][[paste("plot",i)]] <- summary_profile_plots
      message(paste("Drew the Profile plot for ", 
                    as.character(plot_proteins[, PROTEINNAME][i]),
                    "(", i, " of ", nrow(plot_proteins), ")"))
    }
    if (address!=FALSE & !isPlotly) {
      dev.off()
    }
  } # end summarization plot
  if(isPlotly) {
    output_plots
  }
  
}

#' Plot boxplot of all proteins LF
#' @noRd
.qc.all.plot.lf = function(datafeature, groupName, title, 
                            y.limdown, y.limup, x.axis.size, y.axis.size, 
                            text.size) {
  lineNameAxis = RUN = ABUNDANCE = Name = NULL
  
  ## for annotation of condition
  groupName.tmp = data.table(groupName, "PSM" = unique(datafeature$FEATURE)[1], 
                              "PEPTIDESEQUENCE" = unique(datafeature$PEPTIDE)[1]
                              )
  
  ## y-axis labeling
  yaxis.name = 'Log-intensities'
  
  ptemp = ggplot(aes(x = .data$RUN, y = .data$ABUNDANCE),
                  data = datafeature) +
    geom_boxplot(aes(fill = .data$CONDITION), outlier.shape = 1,
                 outlier.size = 1.5) +
    labs(title = title, x = 'MS runs') +
    scale_y_continuous(yaxis.name, y.limdown, y.limup) +
    geom_vline(data = groupName.tmp[lineNameAxis != 0],
               aes(xintercept = .data$lineNameAxis + 0.5),
               colour = "grey", linetype = "longdash") +
    geom_text(data = groupName.tmp,
              aes(x = .data$RUN, y = .data$ABUNDANCE - 1, label = .data$Name),
              size = text.size,
              angle = 0,color = "black") +
    theme_msstats(type = "PROFILEPLOT", x.axis.size, y.axis.size,
                  13, 
                  element_rect(fill = "gray95"),
                  element_text(colour = c("#00B0F6"), size = 14),
                  "none", text_angle = text.angle)
    # theme(
    #   panel.background = element_rect(fill = 'white', colour = "black"),
    #   legend.key = element_rect(fill = 'white', colour = 'white'),
    #   panel.grid.minor = element_blank(),
    #   strip.background = element_rect(fill = 'gray95'),
    #   axis.ticks.x = element_blank(),
    #   axis.text.x = element_blank(),
    #   axis.text.y = element_text(size = y.axis.size, colour = "black"),
    #   axis.ticks = element_line(colour = "black"),
    #   axis.title.x = element_text(size = x.axis.size + 5, vjust = -0.4),
    #   axis.title.y = element_text(size = y.axis.size + 5, vjust = 0.3),
    #   title = element_text(size = x.axis.size + 8, vjust = 1.5),
    #   legend.position = "none")
  
  return(ptemp)
}

#' Plot boxplot of single proteins LF
#' @noRd
.qc.single.plot.lf = function(datafeature, groupname, protein, y.limdown, y.limup,
                            x.axis.size, y.axis.size,text.size){
  PROTEINNAME = ABUNDANCE = lineNameAxis = RUN = Name = NULL
  
  sub = datafeature[PROTEINNAME == protein, ]
  sub = sub[!is.na(sub$ABUNDANCE), ]
  
  ## if all protein measurements are NA,
  if (nrow(sub) == sum(sub[!is.na(sub$ABUNDANCE), ABUNDANCE])) {
    message(paste("Can't the Quality Control plot for ", unique(
      sub$PROTEINNAME), " because all measurements are NAs."))
  }
  
  ## for annotation of condition
  groupname.tmp = data.table(groupname, "FEATURE" = unique(sub$FEATURE)[1], 
                              "PEPTIDESEQUENCE" = unique(sub$PEPTIDE)[1]
  )

  ## 1st plot for original plot
  ## for boxplot, x-axis, xorder should be factor
  yaxis.name = 'Log-intensities'
  
  ptemp = ggplot(aes(x = .data$RUN, y = .data$ABUNDANCE),
                  data = sub) +
    geom_boxplot(aes(fill = .data$CONDITION), outlier.shape = 1,
                 outlier.size = 1.5) +
    labs(title = protein, x = 'MS runs') +
    scale_y_continuous(yaxis.name, limits = c(y.limdown, y.limup)) +
    geom_vline(data = groupname.tmp[lineNameAxis != 0],
               aes(xintercept = .data$lineNameAxis + .5),
               colour = "grey", linetype = "longdash") +
    geom_text(data = groupname.tmp,
              aes(x = .data$RUN, y = .data$ABUNDANCE, label = .data$Name),
              size = text.size,
              color = "black") +
    theme_msstats(type = "PROFILEPLOT", x.axis.size, y.axis.size,
                  13, 
                  element_rect(fill = "gray95"),
                  element_text(colour = c("#00B0F6"), size = 14),
                  "none", text_angle = text.angle)
    # theme(
    #   panel.background = element_rect(fill = 'white', colour = "black"),
    #   legend.key = element_rect(fill = 'white', colour = 'white'),
    #   panel.grid.minor = element_blank(),
    #   strip.background = element_rect(fill = 'gray95'),
    #   axis.ticks.x = element_blank(),
    #   axis.text.x = element_blank(),
    #   axis.text.y = element_text(size = y.axis.size, colour = "black"),
    #   axis.ticks = element_line(colour = "black"),
    #   axis.title.x = element_text(size = x.axis.size + 5, vjust = -0.4),
    #   axis.title.y = element_text(size = y.axis.size + 5, vjust = 0.3),
    #   title = element_text(size = x.axis.size + 8, vjust = 1.5),
    #   legend.position = "none")
  return(ptemp)
}

#' Wrapper function for LF QC Plot
#' @noRd
.qc.lf = function(data.table.list, type, ylimUp, ylimDown, width, height,
                        x.axis.size, y.axis.size,text.size, which.Protein,
                         address, ptm_title, protein_title, isPlotly) {
  
  PROTEINNAME = GLOBALPROTEIN = NULL
  
  if (length(data.table.list) == 4){
    datafeature.protein = data.table.list[[1]]
    datafeature.ptm = data.table.list[[2]]
    datarun.protein = data.table.list[[3]]
    datarun.ptm = data.table.list[[4]]
    
    y.limup = ceiling(max(datafeature.protein$ABUNDANCE,
                           datafeature.ptm$ABUNDANCE, na.rm = TRUE) + 3)
  } else {
    datafeature.ptm = data.table.list[[1]]
    datarun.ptm = data.table.list[[2]]
    
    y.limup = ceiling(max(datafeature.ptm$ABUNDANCE, na.rm = TRUE) + 3)
  }
  
  ## save the plots as pdf or not
  ## If there are the file with the same name
  ## add next numbering at the end of file name
  if (!isPlotly && address != FALSE) {
    allfiles = list.files()
    
    num = 0
    filenaming = paste0(address,"QCPlot")
    finalfile = paste0(address,"QCPlot.pdf")
    
    while (is.element(finalfile, allfiles)) {
      num = num + 1
      finalfile = paste0(paste(filenaming, num, sep = "-"), ".pdf")
    }
    
    pdf(finalfile, width = width, height = height)
  }
  
  ## assign upper or lower limit
  if (is.numeric(ylimUp)) {
    y.limup = ylimUp
  }
  
  y.limdown = 0
  if (is.numeric(ylimDown)) {
    y.limdown = ylimDown
  }
  
  ## Apply pre-plot formatting
  ptm.list = .preplot.format.lf(datafeature.ptm, datarun.ptm, y.limup, type)
  datafeature.ptm = ptm.list[[1]]
  datarun.ptm = ptm.list[[2]]
  groupName.ptm = ptm.list[[3]]
  
  if (length(data.table.list) == 4){
    protein.list = .preplot.format.lf(datafeature.protein, datarun.protein,
                                        y.limup, type)
    datafeature.protein = protein.list[[1]]
    datarun.protein = protein.list[[2]]
    groupName.protein = protein.list[[3]]
  }
  
  # Init list to return plots
  output_plots = list()

  ## all protein
  if (which.Protein[[1]] == 'all' | which.Protein[[1]] == 'allonly') {
    ## Plot all QC
    ptemp.ptm = .qc.all.plot.lf(datafeature.ptm, groupName.ptm, ptm_title,
                                 y.limdown, y.limup, x.axis.size, y.axis.size, 
                                 text.size)
    if (length(data.table.list) == 4){
      ptemp.protein = .qc.all.plot.lf(datafeature.protein, groupName.protein,
                                       protein_title, y.limdown, y.limup, 
                                    x.axis.size, y.axis.size, text.size)
      grid.arrange(ptemp.ptm, ptemp.protein, ncol=1)
      all_protein_plots <- list(PTEMP.PTM = ptemp.ptm, PTEMP.PROTEIN = ptemp.protein)
    } else{
      all_protein_plots <- list(PTEMP.PTM = ptemp.ptm, PTEMP.PROTEIN = NULL)
      print(ptemp.ptm)
    }
    output_plots[[1]] = all_protein_plots
    message("Drew the Quality Contol plot(boxplot) for all ptms/proteins.")
  }
  
  ## each protein
  ## choose Proteins or not
  if (length(data.table.list) == 4){
    plot_global = TRUE
  } else {
    plot_global = FALSE
  }
    
  if (which.Protein[[1]] != 'allonly') {
    if (which.Protein[[1]] != "all") {
      ## check which.Protein is name of Protein
      if (is.character(which.Protein)) {
        temp.name = which.Protein
        
        ## message if name of Protein is wrong.
        if (length(setdiff(temp.name,unique(datafeature.ptm$PROTEINNAME))) > 0){
          stop("Please check protein name.
               Data set does not have this protein. - ",
               toString(temp.name))
        }
      }
      
      ## check which.Protein is order number of Protein
      if (is.numeric(which.Protein)) {
        temp.name = levels(datafeature.ptm$PROTEINNAME)[which.Protein]
        
        ## message if name of Protein is wrong.
        if (length(levels(datafeature.ptm$PROTEINNAME)) < max(which.Protein)) {
          stop("Please check your number of proteins. There are ",
               length(levels(datafeature.ptm$PROTEINNAME)),
               " proteins in this dataset.")
        }
      }
      
      ## use only assigned proteins
      datafeature.ptm = datafeature.ptm[which(
        datafeature.ptm$PROTEINNAME %in% temp.name), ]
      datafeature.ptm$PROTEINNAME = factor(datafeature.ptm$PROTEINNAME)
      
      if (length(data.table.list) == 4){
        temp_proteins = as.character(unique(datafeature.ptm$GLOBALPROTEIN))
        
        plot_global = TRUE
        ## Check if there is a corresponding protein for the PTM
        if (!temp_proteins %in% datafeature.protein$PROTEINNAME){
          message(paste0("Global Protein data not available for ", 
                         as.character(temp_proteins), 
                         ", only PTM will be plotted.")
          )
          plot_global = FALSE
        }
        
        datafeature.protein = datafeature.protein[
          which(datafeature.protein$PROTEINNAME %in% temp_proteins), ]
        datafeature.protein$PROTEINNAME = factor(datafeature.protein$PROTEINNAME)
      }
    }
    
    ## Only plot proteins that occur in both datasets
    if (plot_global){
      global_proteins = unique(datafeature.protein[, PROTEINNAME])
      ptm_proteins = unique(datafeature.ptm[, GLOBALPROTEIN])
      plot_proteins = intersect(ptm_proteins, global_proteins)
      datafeature.ptm = datafeature.ptm[GLOBALPROTEIN %in% plot_proteins,]
      plot_proteins = unique(datafeature.ptm[, c('PROTEINNAME', 
                                                  'GLOBALPROTEIN')])
    } else {
      plot_proteins = unique(datafeature.ptm[, c('PROTEINNAME')])
    }
    
    for (i in seq_len(nrow(plot_proteins))) {
      ptemp.ptm = .qc.single.plot.lf(datafeature.ptm, groupName.ptm, 
                                   as.character(plot_proteins[, PROTEINNAME][i]
                                                ),
                                   y.limdown, y.limup, x.axis.size, y.axis.size, 
                                   text.size)
      if (plot_global){
        ptemp.protein = .qc.single.plot.lf(datafeature.protein, groupName.protein, 
                                         as.character(
                                           plot_proteins[, GLOBALPROTEIN][i]),
                                         y.limdown, y.limup, x.axis.size, 
                                         y.axis.size, text.size)
        grid.arrange(ptemp.ptm, ptemp.protein, ncol=1)
        single_protein_plots <- list(PTEMP.PTM = ptemp.ptm, PTEMP.PROTEIN = ptemp.protein)
      } else {
        print(ptemp.ptm)
        single_protein_plots <- list(PTEMP.PTM = ptemp.ptm, PTEMP.PROTEIN = NULL)
      }
      output_plots[[i+1]] = single_protein_plots # to accomodate all proteins
      message(paste0("Drew the Quality Contol plot(boxplot) for ",
                    as.character(plot_proteins[, PROTEINNAME][i]), " (", i, 
                    " of ", nrow(plot_proteins), ")"))
    } # end-loop
    }
  
  if (address != FALSE & !isPlotly) {
    dev.off()
  }
  
  if(isPlotly) {
    output_plots <- Filter(function(x) !is.null(x), output_plots)
    output_plots
  }
}

#' @noRd
.convert.ptm.data = function(pho.data,
                              annot,
                              mod.num,
                              keyword,
                              which.proteinid,
                              removeMpeptides){
  
  ## Remove Contaminant --------------------------------------------------------
  if (is.element("Contaminant", colnames(pho.data))) {
    pho.data = pho.data["Contaminant" == "", "Contaminant":=NA]
    pho.data = pho.data["Contaminant" != "+"]
  }
  
  if (is.element("Potential.contaminant", colnames(pho.data))) {
    pho.data = pho.data["Potential.contaminant" == "", 
                         "Potential.contaminant" :=NA]
    pho.data = pho.data["Potential.contaminant" != "+"]
  }
  
  ## Remove Reverse ------------------------------------------------------------
  if (is.element("Reverse", colnames(pho.data))) {
    pho.data = pho.data["Reverse" == "", "Reverse" := NA]
    pho.data = pho.data["Reverse" != "+"]
  }
  
  message('** + Contaminant, + Reverse, + Only.identified.by.site, PTMs are removed.')
  
  ## column for protein  -------------------------------------------------------
  ## default : Proteins
  which.pro = NULL
  
  if (which.proteinid == 'Proteins') {
    which.pro = 'Proteins'
  } else if (which.proteinid == 'Leading.proteins') {
    which.pro = 'Leading.proteins'
  } else if (which.proteinid == 'Protein') {
    which.pro = 'Protein'
  }
  
  if (which.proteinid == 'Protein' & !is.element('Protein', colnames(pho.data))) {
    which.pro = 'Leading.proteins'
    message('** Use Leading.proteins instead of Protein.')
  }
  
  if (which.proteinid == 'Leading.proteins' &
      !is.element('Leading.proteins', colnames(pho.data))) {
    which.pro = 'Proteins'
    message('** Use Proteins instead of Leading.proteins.')
  }
  
  ## at least 'Proteins' should be in the evidence.txt
  if (!is.element(which.pro, colnames(pho.data))) {
    stop('** Please select which columns should be used for protein ids,
         among three options (Proteins, Leading.proteins, Protein).')
  }
  
  if (mod.num == 'Single'){
    pep = "___1"
    pho.abun = .convert.peps(pho.data,
                              annot,
                              pep,
                              keyword,
                              which.pro,
                              removeMpeptides)
  }
  else if (mod.num == 'Total'){
    ## Extract and convert peps
    pho.pep.list = unique(unlist(lapply(
      colnames(pho.data), function(x) {str_extract(x, '___\\w+')})))
    pho.pep.list = pho.pep.list[!is.na(pho.pep.list)]
    
    temp.data.table.list = list()
    for (pep in pho.pep.list) {
      temp_pep = .convert.peps(pho.data,
                                annot,
                                pep,
                                keyword,
                                which.pro,
                                removeMpeptides)
      temp.data.table.list = append(temp.data.table.list, list(temp_pep))
    }
    names(temp.data.table.list) = pho.pep.list
    
    ## Combine into one datatable
    pho.abun = .pho.combine.pep(temp.data.table.list)
  }
  else{
    stop("mod.num must be one of Single or Total.")
  }
  
  #pho.abun = pho.abun[Abundance != 0 & !is.na(Abundance)]
  pho.abun$Run = paste(pho.abun$Mixture, pho.abun$TechRepMixture, sep = "_")
  
  colnames(pho.abun) = c('Replicate', 'PSM', 'Intensity', 'Channel', 
                          'Condition', 'Mixture', 'TechRepMixture', 
                          'BioReplicate', 'PeptideSequence', 'Charge', 
                          'ProteinName', 'Run')
  return(pho.abun)
  }


#' @noRd
.convert.peps = function(data,
                          annot,
                          pho_pep,
                          keyword,
                          which.pro,
                          removeMpeptides) {
  
  
  ## extract required columns --------------------------------------------------
  if (keyword == ""){
    required_data_columns = colnames(data)[
      grepl("Reporter.intensity.corrected.", colnames(data)) &
        lengths(regmatches(colnames(data), gregexpr("\\.", colnames(data)))
                ) >= 4
      & grepl(pho_pep, colnames(data))]
  } else {
    required_data_columns = colnames(data)[
    grepl("Reporter.intensity.corrected.", colnames(data)) &
      grepl(keyword, colnames(data)) & grepl(pho_pep, colnames(data))]
  }
  
  data = data[, c(which.pro, "Positions.within.proteins", "Amino.acid",
                   "Phospho..STY..Probabilities","Position.in.peptide",
                   "Charge", required_data_columns), with=FALSE]
  
  if (is.element('Proteins', colnames(data))){
    colnames(data)[colnames(data) == 'Proteins'] = 'ProteinName'
  }
  if (is.element('Leading.proteins', colnames(data))){
    colnames(data)[colnames(data) == 'Leading.proteins'] = 'ProteinName'
    ## Remove sites that are not in lead protein
    data$Positions.within.proteins = vapply(
      paste0(data$Positions.within.proteins, ';'), function(x){
        str_match(x, regex("^.*?(?=;)"))[1]})
  }
  if (is.element('Protein', colnames(data))){
    colnames(data)[colnames(data) == 'Protein'] = 'ProteinName'
    ## Remove sites that are not in lead protein
    data$Positions.within.proteins = sapply(
      paste0(data$Positions.within.proteins, ';'), function(x){
        str_match(x, regex("^.*?(?=;)"))[1]})
  }
  
  ## remove M site -------------------------------------------------------------
  ## only when removeMpeptides=TRUE
  if(removeMpeptides){
    remove_m_sequence = unique(data[
      grep("M", data$Phospho..STY..Probabilities),
      "Phospho..STY..Probabilities"])
    
    if(length(remove_m_sequence) > 0){
      data = data[-grep("M", data$Phospho..STY..Probabilities),]
    }
  }
  
  ## remove rows with all zero values ------------------------------------------
  ### some rows have all zero values across all MS runs. They should be removed.
  tmp = data[, c(required_data_columns), with=FALSE]
  count = apply(tmp, 1, function(x) sum(x==0))
  ## remove
  data = data[count != ncol(tmp), ]
  
  ## check whether there are multiple rows or not.------------------------------
  data = data[data$ProteinName!="", ]
  
  ## unique identifier for each row : phospho size + position + charge
  data$fea = paste(data$Phospho..STY..Probabilities, data$Charge,
                    data$ProteinName,
                    paste0(data$Amino.acid, data$Positions.within.proteins),
                    sep="_")
  
  count = xtabs(~fea, data)
  if(length(count[count > 1]) > 0){
    ## keep the features that have no issue.
    pho.single = data[which(data$fea %in% names(count[count == 1])), ]

    for(i in seq_len(length(names(count[count > 1])))){

      ## keep the feature that have issue.
      pho.mul = data[which(data$fea %in% names(count[count > 1])[i]), ]

      ## keep the feature with maximal summation of intensities
      pho.new = rbind(pho.single[, c(required_data_columns, 'fea'), with=FALSE],
                       pho.mul[
                         which.max(rowSums(pho.mul[, c(required_data_columns),
                                                   with=FALSE])),
                         c(required_data_columns, 'fea'), with=FALSE])
    }

  } else{
    pho.new = data[, c(required_data_columns, 'fea'), with=FALSE]
  }
  
  ## Assign the column names ---------------------------------------------------
  colnames(pho.new)[colnames(pho.new) == 'fea'] = 'PSM'
  
  ## make long format
  pho.l = melt(pho.new, id=c("PSM"))
  colnames(pho.l)[which(
    colnames(pho.l) %in% c('variable','value'))] = c("Replicate","Abundance")
  pho.l$Replicate = gsub(pho_pep, "", pho.l$Replicate)
  
  ## merge with annotation
  pho.l = merge(pho.l, annot, by = 'Replicate', all.x = TRUE)
  
  ## replace zero with NA for MQ output
  pho.l[pho.l$Abundance == 0, 'Abundance'] = NA
  pho.l$Abundance = as.numeric(as.character(pho.l$Abundance))
  #pho.l$Abundance = log2(pho.l$Abundance)
  pho.l$PeptideSequence = str_match(pho.l$PSM, regex("^(?:[^_]*){1}"))
  pho.l$Charge = str_match(pho.l$PSM, regex("(?<=_)[0-9](?=_)")[1])
  pho.l$Protein = str_replace(pho.l$PSM, fixed(paste0(pho.l$PeptideSequence, 
                                                       '_', pho.l$Charge, '_')),
                               "")
  pep.pho.abun = pho.l
  
  return(pep.pho.abun)
}

#' @noRd
.pho.combine.pep = function(pep.list){
  
  ## Columns to join on
  join.cols = colnames(pep.list[[1]])[
    colnames(pep.list[[1]]) != 'Abundance']
  
  ## Combine pep data.tables
  pho.abun = Reduce(function(x,y) {
    merge(x, y, by = join.cols, all = TRUE)}, pep.list)
  
  ## sum pep abundance
  pho.abun$Total = rowSums(
    pho.abun[, c(colnames(pho.abun)[
      colnames(pho.abun) %like% 'Abundance']),
      with=FALSE], na.rm=TRUE)
  
  ## Clean combined data.table
  pho.abun = pho.abun[, c(colnames(pho.abun)[
    !colnames(pho.abun) %like% 'Abundance']), with = FALSE]
  colnames(pho.abun)[colnames(pho.abun) == 'Total'] = 'Abundance'
  setcolorder(pho.abun, c("PSM", "Replicate", "Abundance", "Channel", 
                          "Condition", "Mixture", "TechRepMixture", 
                          "BioReplicate", "Protein", "Charge"))
  
  combined.pho.abun = pho.abun
  return(combined.pho.abun)
}

#' Add modified data into protein for Progenesis input
#' @noRd
.progensis.add.sites = function(ptm_input, fasta_path, col_order){
  
  X.8 = X.9 = X.10 = NULL
  
  ## Load fasta file
  format_fasta = tidyFasta(fasta_path)
  
  mod_df = ptm_input[X.9 != "", ]
  
  ## Extract peptide and protein info
  peptide = mod_df[3:nrow(mod_df), X.8]
  protein = mod_df[3:nrow(mod_df), X.10]
  mods = mod_df[3:nrow(mod_df), X.9]
  
  ## Extract modifications
  loc_df = data.table("uniprot_iso" = protein, "peptide" = peptide, 
                       "mods" = mods)
  loc_df = merge(loc_df, format_fasta, by = c("uniprot_iso"), all.x = TRUE)
  loc_df = loc_df[!is.na(sequence),]
  
  mod_locations = regmatches(loc_df[, mods], gregexpr("[[:digit:]]+", 
                                                       loc_df[, mods]))
  
  start = sapply(seq_len(nrow(loc_df)),
                  function(i) gregexpr(loc_df$peptide[i], 
                                       loc_df$sequence[i])[[1]])
  mod_index = sapply(seq_len(length(mod_locations)), function(i){
    mod_locations[i] = list(
      as.integer(unlist(mod_locations[i])) + start[i])})
  
  peptide_mod = mapply(get_sites, mod_locations, mod_index,
                        loc_df$peptide)
  
  loc_df$site = peptide_mod
  loc_df$site = paste(loc_df$uniprot_iso, loc_df$site, sep = "_")
  
  ptm_input = merge(ptm_input, 
                     unique(loc_df[, c("uniprot_iso", "peptide", 
                                       "mods", "site")]), 
                     by.x = c("X.10" , "X.8", "X.9"), 
                     by.y = c("uniprot_iso", "peptide", "mods"), all.x = TRUE)
  
  ## Put data back into Progenesis format
  ptm_input = ptm_input[order(ptm_input$id), ]
  ptm_input$X.10 = ptm_input$site
  ptm_input[2, "X.10"] = "Accession"
  ptm_input[1, "X.10"] = ""
  ptm_input = ptm_input[, ..col_order]
  
  ## Remove proteins missing from fasta file
  ptm_input = ptm_input[!is.na(X.10), ]
  
  return(ptm_input)
}

#' Helper function for progenesis site identification
#' @noRd
get_sites = function(str_idx, pep_idx, pep) {
  
  if (!is.na(pep_idx[1])){
    mod_site = ""
    for (i in seq(str_idx)){
      site = substr(pep, as.numeric(str_idx[[i]]), as.numeric(str_idx[[i]]))
      temp = paste0(site, as.character(pep_idx[i]), "_")
      mod_site = paste0(mod_site, temp)
    }
    mod_site = substr(mod_site,1,nchar(mod_site)-1)
  } else {mod_site = NA}
  return(mod_site)
}

#' Helper function for spectronaut site identification
#' combine with prot one at some point
#' @noRd
spectro_get_sites = function(str_idx, start, pep) {
  
  mod_site = ""
  for (i in seq_along(str_idx)){
    site = substr(pep, as.numeric(str_idx[[i]])-i, as.numeric(str_idx[[i]])-i)
    pep_idx = start[[1]] + as.numeric(str_idx[[i]])-i-1
    temp = paste0(site, as.character(pep_idx), "_")
    mod_site = paste0(mod_site, temp)
  }
  mod_site = substr(mod_site,1,nchar(mod_site)-1)
  
  return(mod_site)
}


#' Locate modification site number and amino acid
#' 
#' @param data `data.table` of enriched experimental run. Must include 
#' `ProteinName`, `PeptideSequence`, `PeptideModifiedSequence`, and (optionally)
#' `Start` columns.
#' @param protein_name_col Name of column indicating protein. Default is 
#' `ProteinName`.
#' @param unmod_pep_col Name of column indicating unmodified peptide sequence. Default
#' is `PeptideSequence`.
#' @param mod_pep_col Name of column indicating modified peptide sequence. Default
#' is `PeptideModifiedSequence`.
#' @param clean_mod Remove special characters and numbers around modification 
#' name. Default is `FALSE`
#' @param fasta_file File path to FASTA file that matches with proteins in 
#' `data`. Can be either string or `data.table` processed with `tidyFasta()` 
#' function. Default to NULL if peptide number included in `data`.
#' @param fasta_protein_name Name of fasta file column that matches with 
#' `protein_name_col`. Default is `header`.
#' @param mod_id String that indicates what amino acid was modified in 
#' `PeptideSequence`.
#' @param localization_scores Boolean indicating if mod id is a localization 
#' score. If TRUE, `mod_id` will be ignored and localization cutoff will be 
#' used to determine sites. Default is FALSE.
#' @param localization_cutoff Default is .75. Localization probabilities below 
#' cutoffs will be removed. `localization_scores` must be TRUE.
#' @param remove_unlocalized_peptides Default is TRUE. If `localization_scores`
#' is TRUE and probabilities are below `localization_cutoff`, the modification 
#' site will not be able to be determined. These unlocalized peptides can be 
#' kept or removed. If FALSE the unlocalized peptides will still be used in 
#' modeling the sites that could be localized.
#' @param terminus_included Boolean indicating if the `PeptideSequence` includes
#' the terminus amino acid.
#' @param terminus_id String that indicates what the terminus amino acid is. 
#' Default is '.'.
#' @param mod_id_is_numeric Boolean indicating if modification identifier is 
#' a number instead of a character (i.e. +80 vs *).
#' @param remove_underscores Boolean indicating if underscores around peptide 
#' exist. These should be removed to properly count where in sequence the 
#' modification occurred. 
#' @param remove_other_mods keeping mods that are not of interest can mess up 
#' the amino acid count. Remove them if they are causing issues.
#' @param bracket bracket type that encompasses PTM (usually `[` or `(`). Always
#' pass opening bracket (there is a function to grab the close bracket). Default 
#' is FALSE (i.e. no bracket).
#' @param replace_text If PTM is noted by text (i.e. `Phospho`) and needs to be 
#' replaced by an indicator (`*`)
#' 
#' @importFrom data.table as.data.table
#' @importFrom stringr str_extract_all str_replace_all
#' 
#' @return `data.table` with site location added into `Protein` column.
#' @export 
#' 
#' @examples
#' ##TODO
#' 
MSstatsPTMSiteLocator = function(data, 
                                 protein_name_col= "ProteinName",
                                 unmod_pep_col = "PeptideSequence",
                                 mod_pep_col = "PeptideModifiedSequence",
                                 clean_mod=FALSE,
                                 fasta_file=NULL, 
                                 fasta_protein_name="header",
                                 mod_id="\\*", 
                                 localization_scores=FALSE,
                                 localization_cutoff=.75,
                                 remove_unlocalized_peptides=TRUE,
                                 terminus_included=FALSE, 
                                 terminus_id="\\.",
                                 mod_id_is_numeric=FALSE,
                                 remove_underscores=FALSE,
                                 remove_other_mods=FALSE,
                                 bracket=FALSE,
                                 replace_text=FALSE){
  
  ## Check if peptide number included in data
  if ("Start" %in% colnames(data)){
    id_data = data[,c(protein_name_col, unmod_pep_col, 
                      mod_pep_col, "Start"), with=FALSE]
  } else {
    if (is.null(fasta_file)){
      stop("FASTA file not provided and `Start` column missing from data. \
           MSstatsPTMSiteLocator requires one of these to be provided to \
           identify modification site number.")
    }
    
    id_data = data[,c(protein_name_col, unmod_pep_col, mod_pep_col), with=FALSE]
    id_data = .joinFasta(id_data, fasta_file, fasta_protein_name,
                         protein_name_col, unmod_pep_col, mod_pep_col)
  }
  
  id_data[,"original_mod"] = id_data[, ..mod_pep_col]
  
  if (localization_scores){
    id_data = .removeCutoffSites(id_data, mod_pep_col, localization_cutoff,
                                 remove_unlocalized_peptides)
    mod_id="\\*"
  }
  
  if (terminus_included){
    id_data = .fixTerminus(id_data, terminus_id, unmod_pep_col)
  }
  
  if (clean_mod){
    id_data[,mod_pep_col] = lapply(id_data[,mod_pep_col, with=FALSE], 
                                   function(x){gsub("[0-9]+|[[:punct:]]", "", x)})
  }
  
  if (remove_underscores){
    id_data[,mod_pep_col] = lapply(id_data[,mod_pep_col, with=FALSE], 
                                   function(x){gsub("[_]", "", x)})
  }
  
  
  if (typeof(bracket) == "character"){
    close_bracket = get_closing_bracket(bracket)
    bracket_regex = paste0("\\", bracket, ".*?\\", close_bracket)
    
    if (remove_other_mods){
      keep_id = gsub("[^[:alpha:]]", "", str_split(mod_id, " ")[[1]][[1]])
      remove = unlist(lapply(id_data[,mod_pep_col, with=FALSE][[1]], function(x){
          all(grepl(keep_id, stringr::str_extract_all(x, bracket_regex)[[1]]))
          }))
      id_data = id_data[remove]
      
    } else {
      keep_id = paste0(gsub("[^[:alpha:]]", "", str_split(mod_id, " ")[[1]]),collapse = "|")
      pattern = paste0("\\", bracket, "(?!", keep_id, ")(.*?)\\)\\", close_bracket)
      
      id_data[, (mod_pep_col) := lapply(id_data[,mod_pep_col, with=FALSE], 
                                        function(x){
                                          gsub(pattern, "", x, perl = TRUE)})
      ]
    }
  }

  id_data = .locateSites(id_data, mod_id, protein_name_col, 
                         unmod_pep_col, mod_pep_col, mod_id_is_numeric,
                         replace_text)
  id_data = id_data[!duplicated(id_data),]
  
  id_data[["new_peptide_col"]] = id_data[[mod_pep_col]]
  
  data = merge(data, id_data, by.x=c(protein_name_col, unmod_pep_col, 
                                     mod_pep_col),
               by.y=c(protein_name_col, unmod_pep_col, 
                      "original_mod"))
  
  data[, "ProteinNameUnmod"] = data[, protein_name_col, with=FALSE]
  data[, protein_name_col] = data$ProteinName_mod
  data=as.data.table(data)
  
  if ("Start.y" %in% colnames(data)){
    data$Start = data$Start.y
  }
  remove = intersect(c("PeptideModifiedSequence_adj", "ProteinName_mod", 
                       "start_fix", "Start.x", "Start.y", "original_mod",
                       "number_sites"),
                     colnames(data))
  data[, (remove):=NULL]
  return(data)
}


#' Remove sites below cutoff probability
#' @param data data.table
#' @param mod_pep_col column in data with modified sites
#' @param cutoff numeric cutoff. Default is .75.
#' @param remove_unlocalized_peptides Boolean if to remove peptides that 
#' could not be fully localized.
#' @importFrom stringr str_replace_all
#' @return data.table with modifications below cutoff removed
#' @keywords internal
.removeCutoffSites = function(data, mod_pep_col, cutoff,
                              remove_unlocalized_peptides){
  
  # data[,"number_sites"] = unlist(lapply(data[, ..mod_pep_col][[1]], function(x){
  #   round(sum(as.numeric(regmatches(str_replace_all(x,"\\.",""),
  #                                   gregexpr("[[:digit:]]+",
  #                                            str_replace_all(x,"\\.","")))[[1]])
  #             )/10000)}))
  extract_sites = function(peptide){
    total_sum = sum(as.numeric(regmatches(peptide,
                                    gregexpr("[[:digit:]]+\\.*[[:digit:]]*",
                                             peptide))[[1]]))
    if (total_sum > 100){
      total_sum = round(total_sum/100)
    }
    return(total_sum)
  }
  
  
  data[,"number_sites"] = unlist(lapply(data[, ..mod_pep_col][[1]], extract_sites))
  
  remove_sites = function(peptide){
    if (peptide == "" | is.na(peptide)){
      mod_pep=""
    } else{
      # localized_site = as.numeric(
      #   regmatches(peptide, gregexpr("0.[[:digit:]]+|1.000", peptide))[[1]]) >= cutoff
      localized_site = as.numeric(
        regmatches(peptide, gregexpr("[[:digit:]]+.[[:digit:]]+|1.00", peptide))[[1]]) >= cutoff
      split_peptide = strsplit(peptide, "[0-9.()]")[[1]][
        strsplit(peptide, "[0-9.()]")[[1]] !=""]
      if (sum(localized_site) > 0){
        i=0
        for (mod_site in seq_along(localized_site)){
          if (localized_site[mod_site]){
            split_peptide = append(split_peptide, "*", 
                                   after=mod_site+i)
            i = i+1
          }
        }
        
      }
      mod_pep = paste(split_peptide, collapse='')
    }
    return(mod_pep)
  }
  
  data[,mod_pep_col] = unlist(lapply(data[,..mod_pep_col][[1]], remove_sites))
  
  if (remove_unlocalized_peptides){
    count = unlist(lapply(data[,..mod_pep_col][[1]], function(x){
      lengths(regmatches(x, gregexpr("\\*", x)))}))
    
    data = data[count == data[,"number_sites"][[1]],]
  }
  return(data)
  
}

#' Add FASTA data into dataframe
#' @param data data.table
#' @param fasta_file string or data.table
#' @return data.table
#' @keywords internal
.joinFasta = function(data, fasta_file, fasta_protein_name,
                      protein_name_col, unmod_pep_col, mod_pep_col){
  
  ## Load FASTA if not already loaded
  if (is.character(fasta_file)){
    fasta_file = tidyFasta(fasta_file)
  }
  
  ## Join data and FASTA
  data = merge(data, fasta_file, by.x=protein_name_col, 
                  by.y=fasta_protein_name, all.x=TRUE)

  missing_prot = unique(data[is.na(data$sequence), 
                             protein_name_col, with=FALSE])
  if (nrow(missing_prot) > 0){
    print(paste0("FASTA file missing ", as.character(nrow(missing_prot)),
                 " Proteins. These will be removed. This may be due to non-unique identifications."))
  }
  
  data = data[!is.na(data$sequence),]
              
  ## Locate peptide sequence start
  data$Start = mapply(function(x,y){unlist(gregexpr(pattern=x, y))[[1]]}, 
                      data[, unmod_pep_col, with=FALSE][[1]], 
                      data[, "sequence"][[1]])
  data$Start = unlist(data$Start)
  
  data = data[, c(protein_name_col, unmod_pep_col, mod_pep_col, "Start"), 
              with=FALSE]
  
  return(data)
}

#' Fix terminus location adjustments
#' @param data data.table containing peptide data
#' @param terminus_id character string identifying the terminus 
#' (e.g. N-terminus)
#' @param unmod_pep_col character string specifying the column name containing 
#' unmodified peptide sequences
#' @return data.table with corrected Start positions
.fixTerminus = function(data, terminus_id, unmod_pep_col){
  
  ## Terminus makes start location off
  data$start_fix = unlist(lapply(data[, unmod_pep_col, with=FALSE], function(x){
    as.integer(unlist(gregexpr(terminus_id, 
                               substr(x, start=1, stop=2))) == 2)}))
  data$Start = data$Start - unlist(data$start_fix)
  
  return(data)
}

#' Add site location and aa
#' @param data data.table
#' @param mod_id string
#' @return data.table
#' @keywords internal
.locateSites = function(data, mod_id, protein_name_col, 
                        unmod_pep_col, mod_pep_col, mod_id_is_numeric,
                        replace_text=FALSE){
  
  data[, "PeptideModifiedSequence_adj"] = data[,mod_pep_col, with=FALSE]
  
  if (mod_id_is_numeric){
    mod_id="\\*"

    ## In case brackets are round
    data$PeptideModifiedSequence_adj = lapply(data$PeptideModifiedSequence_adj,
                                              function(x){gsub("\\(", "\\[", x)})
    data$PeptideModifiedSequence_adj = lapply(data$PeptideModifiedSequence_adj,
                                              function(x){gsub("\\)", "\\]", x)})

    ## Replace mod locations with *
    data$PeptideModifiedSequence_adj = lapply(data$PeptideModifiedSequence_adj,
                                           function(x){gsub("[0-9.()]", "", x)})
    data$PeptideModifiedSequence_adj = lapply(data$PeptideModifiedSequence_adj,
                                           function(x){gsub("\\[", "", x)})
    data$PeptideModifiedSequence_adj = lapply(data$PeptideModifiedSequence_adj,
                                              function(x){gsub("\\]", mod_id, x)})
    data$PeptideModifiedSequence_adj = lapply(data$PeptideModifiedSequence_adj,
                                              function(x){gsub("\\+",
                                                               "", x)})
    ## Fix instances of double mod aa (caused by Acetylation)
    data$PeptideModifiedSequence_adj = lapply(data$PeptideModifiedSequence_adj,
                                           function(x){
                                             gsub("\\*\\*", mod_id, x)})
    data$PeptideModifiedSequence_adj = unlist(data$PeptideModifiedSequence_adj)
  }
  if(replace_text){
    temp_mod_id = mod_id
    mod_id = "\\*"
    data$PeptideModifiedSequence_adj = unlist(lapply(data$PeptideModifiedSequence_adj,
                                              function(x){
                                                gsub(temp_mod_id, mod_id, x)}))
  }
  
  unmod_data = data[!grepl(mod_id, data[,"PeptideModifiedSequence_adj"][[1]]), ]
  mod_data = data[grepl(mod_id, data[,"PeptideModifiedSequence_adj"][[1]]),]
  
  ## Locate number and aa  
  string_num = lapply(mod_data$PeptideModifiedSequence_adj, 
                      function(x){unlist(gregexpr(mod_id, x))})
  
  string_num = lapply(string_num, function(x){x - 1:length(x)})
  site_num = mapply(function(x,y){x + y - 1}, string_num, mod_data$Start, 
                    SIMPLIFY=FALSE)
  
  site_aa = lapply(mod_data$PeptideModifiedSequence_adj, function(x){
    unlist(lapply(unlist(gregexpr(mod_id, x)) - 1, 
                  function(y){substr(x, y, y)}))})
  
  ## Combine number and aa
  full_site = lapply(mapply(function(x,y){paste(x, y, sep="")}, 
                            site_aa, site_num, SIMPLIFY=FALSE),
                     function(z){paste(z, collapse='_')})
  mod_data$ProteinName_mod = paste(mod_data[, protein_name_col, with=FALSE][[1]], 
                                   full_site, sep="_")
  
  ## Ensure columns are not lists
  mod_data[,unmod_pep_col] = mod_data[, unmod_pep_col, with=FALSE][[1]]
  mod_data$PeptideModifiedSequence_adj = unlist(
    mod_data$PeptideModifiedSequence_adj)
  mod_data$ProteinName_mod = unlist(mod_data$ProteinName_mod)
  
  unmod_data$ProteinName_mod = unmod_data[, protein_name_col, with=FALSE][[1]]
  data = rbindlist(list(mod_data, unmod_data))
  
  return(data)
}

#' Pivot Peak Studio data into long format
#' @noRd
#' @keywords internal
.pivotPS = function(input){
  
  columns = colnames(input)
  len_cols = length(columns)
  end_runs = which(grepl("os", columns))[[1]] - 1
  keep_cols = c(3, 4, len_cols, len_cols-1, len_cols-2, 11:end_runs)
  
  
  filter_input = input[, ..keep_cols]
  
  
  pivot_input = melt(filter_input, 
                     measure.vars = 6:length(colnames(filter_input)), 
                     variable.name = "Raw.File", value.name = "Intensity")
  colnames(pivot_input) = c("ProteinName", "PeptideSequence", 
                            "Mod", "End", "Start", "Raw.File", "Intensity")
  
  pivot_input$FragmentIon <- NA
  pivot_input$ProductCharge <- NA
  pivot_input$PrecursorCharge <- NA
  pivot_input$IsotopeLabelType <- 'L'
  
  return(pivot_input)
}

#' Remove modifications from pep sequence that are not required
#' @noRd
#' @keywords internal
.MSstatsPTMRemoveMods = function(sequence, remove_special_char=TRUE){
  if (remove_special_char){
    sequence = lapply(sequence, function(x){gsub("\\[(.*?)\\]|[0-9]+|[[:punct:]]", "", x)})
  } else {
    sequence = lapply(sequence, function(x){gsub("\\[(.*?)\\]|[0-9]+", "", x)})
  }
  
  return(unlist(sequence))
}


#' Remake MaxQ MSstatsConvert::.cleanRawMaxQuant function to not require proteinGroups file
#' 
#' @noRd
#' @keywords internal
.cleanRawMaxQuantPTM = function(msstats_object, protein_id_col, data.type,
                                remove_by_site = FALSE,
                                channel_columns = "Reporterintensitycorrected"){
  
  ProteinIDs = id = PSM = PeptideSequence = NULL
  ProteingroupIDs = PrecursorCharge = Intensity = NULL
  channels = character(0)
  
  mq_input = MSstatsConvert::getInputFile(msstats_object, "evidence")
  
  filter_cols = c("Contaminant", "Potentialcontaminant", "Reverse")
  msg = paste("** + Contaminant, + Reverse, + Potential.contaminant",
              "proteins are removed.")
  if (remove_by_site) {
    filter_cols = c(filter_cols, "Onlyidentifiedbysite")
    msg = paste("** + Contaminant, + Reverse, + Potential.contaminant,",
                "+ Only.identified.by.site proteins are removed.")
  }
  
  mq_input = MSstatsConvert:::.filterManyColumns(mq_input, filter_cols, "+")
  
  
  if (data.type == "TMT") {
    channels = MSstatsConvert:::.getChannelColumns(colnames(mq_input), 
                                                   channel_columns)
  }
  
  data.table::setnames(
    mq_input, 
    c(protein_id_col, "Modifiedsequence", "Charge", "Rawfile"), 
    c("ProteinName", "PeptideSequence", "PrecursorCharge", "Run"),
    skip_absent = TRUE)
  mq_input[["PeptideSequence"]] = gsub("_", "", mq_input[["PeptideSequence"]])
  mq_cols = c("ProteinName", "PeptideSequence", "Modifications", 
              "PrecursorCharge", "Run", "Intensity", 
              "Fraction", "TechReplicate", "Run", "BioReplicate",
              "PSM", "Score")
  mq_cols = intersect(c(mq_cols, channels),
                      colnames(mq_input))
  mq_input = mq_input[, mq_cols, with = FALSE]
  mq_input = unique(mq_input)
  
  if (data.type == "TMT") {
    mq_input[, PSM := paste(PeptideSequence, PrecursorCharge, 
                            1:nrow(mq_input), sep = "_")]
    mq_input = melt(mq_input, measure.vars = channels,
                    id.vars = c("ProteinName", "PeptideSequence", 
                                "PrecursorCharge", "PSM", "Run", "Score"),
                    variable.name = "Channel", value.name = "Intensity")
    mq_input$Channel = gsub(channel_columns, "channel", mq_input$Channel)
    mq_input$Channel = MSstatsConvert:::.standardizeColnames(mq_input$Channel)
    suppressWarnings({
      mq_input$Intensity = ifelse(mq_input$Intensity == 0, NA,
                                  mq_input$Intensity)
    })
    mq_input = MSstatsConvert:::.filterFewMeasurements(mq_input, 0, FALSE, 
                                      c("PeptideSequence", 
                                        "PrecursorCharge", "Run"))
  }
  
  mq_input = mq_input[!is.na(Intensity), ]
  
  return(mq_input)
}

#' Remake MSstatstoMaxQuant converter to work without proteinGroups file
#' 
#' @noRd
#' @keywords internal
MaxQtoMSstatsFormatHelper = function(
    evidence, annotation, data.type = "LF",   
    proteinID = "Proteins",
    useUniquePeptide = TRUE,
    summaryforMultipleRows = max,
    removeFewMeasurements = TRUE,
    removeMpeptides = FALSE,
    removeOxidationMpeptides = FALSE,
    removeProtein_with1Peptide = FALSE,
    use_log_file = TRUE,
    append = FALSE,
    verbose = TRUE,
    log_file_path = NULL) {
  MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                      log_file_path)
  
  input = MSstatsConvert::MSstatsImport(list(evidence = evidence), 
                                        type = "MSstats",
                                        tool = "MaxQuant")
  input = .cleanRawMaxQuantPTM(input, proteinID, data.type)
  annotation = MSstatsConvert::MSstatsMakeAnnotation(input, 
                                                     annotation, 
                                                     "Run" = "Rawfile")
  
  m_filter = list(col_name = "PeptideSequence", 
                  pattern = "M", 
                  filter = removeMpeptides, 
                  drop_column = FALSE)
  
  oxidation_filter = list(col_name = "Modifications", 
                          pattern = "Oxidation", 
                          filter = removeOxidationMpeptides, 
                          drop_column = TRUE)
  
  feature_columns = c("PeptideSequence", "PrecursorCharge")
  input = MSstatsConvert::MSstatsPreprocess(
    input, 
    annotation,
    feature_columns,
    remove_shared_peptides = useUniquePeptide, 
    remove_single_feature_proteins = removeProtein_with1Peptide,
    pattern_filtering = list(oxidation = oxidation_filter,
                             m = m_filter),
    feature_cleaning = list(
      remove_features_with_few_measurements = removeFewMeasurements,
      summarize_multiple_psms = summaryforMultipleRows),
    columns_to_fill = list("FragmentIon" = NA,
                           "ProductCharge" = NA,
                           "IsotopeLabelType" = "L"))
  input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns)
  
  msg_final = paste("** Finished preprocessing. The dataset is ready",
                    "to be processed by the dataProcess function.")
  getOption("MSstatsLog")("INFO", msg_final)
  getOption("MSstatsMsg")("INFO", msg_final)
  getOption("MSstatsLog")("INFO", "\n")
  
  return(input)
}

#' Remake MSstatstoMaxQuantTMT converter to work without proteinGroups file
#' 
#' @noRd
#' @keywords internal
MaxQtoMSstatsTMTFormatHelper = function(
    evidence, annotation, data.type="TMT", which.proteinid = 'Proteins',
    rmProt_Only.identified.by.site = FALSE, useUniquePeptide = TRUE,
    rmPSM_withfewMea_withinRun = TRUE, rmProtein_with1Feature = FALSE, 
    summaryforMultipleRows = sum, 
    use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL
) {
  MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                      log_file_path, 
                                      base = "MSstatsTMT_converter_log_")
  
  input = MSstatsConvert::MSstatsImport(list(evidence = evidence), 
                                        "MSstatsTMT", "MaxQuant")
  input = .cleanRawMaxQuantPTM(input, which.proteinid, data.type,
                               remove_by_site = rmProt_Only.identified.by.site,
                               channel_columns = "Reporterintensitycorrected")
  annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)
  
  feature_columns = c("PeptideSequence", "PrecursorCharge")
  input = MSstatsConvert::MSstatsPreprocess(
    input, 
    annotation, 
    feature_columns,
    remove_shared_peptides = useUniquePeptide,
    remove_single_feature_proteins = rmProtein_with1Feature,
    feature_cleaning = list(remove_features_with_few_measurements = rmPSM_withfewMea_withinRun,
                            summarize_multiple_psms = summaryforMultipleRows))
  input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                fix_missing = "zero_to_na")
  data.table::setnames(input, "PrecursorCharge", "Charge", skip_absent = TRUE)
  
  msg_final = paste("** Finished preprocessing. The dataset is ready",
                    "to be processed by the proteinSummarization function.")
  getOption("MSstatsLog")("INFO", msg_final)
  getOption("MSstatsMsg")("INFO", msg_final)
  getOption("MSstatsLog")("INFO", "\n")
  input
}

#' Extracts full mod id column from Philosopher data. Is a combo of AA and number
#' 
#' @noRd
#' @keywords internal
.getFullModID = function(input, mod_id_col){
  columns = colnames(input)
  
  full_id = columns[grepl(mod_id_col, columns) & 
                    !grepl("Best.Localization", columns)]
  
  if (length(full_id) > 1){
    stop("Multiple columns found containing mod_id_col string. Please be more specific and add full column name (including mass).")
  }
  return(full_id)
}

#' Extract PD mod sites using probability
#' @noRd
#' @keywords internal
.getPDmods = function(input){
  
  message("INFO: Extracting modifications")
  probability_column = colnames(input)[grepl("Best.Site.Probabilities", 
                                             colnames(input))]
  
  probs = input[, probability_column, with=FALSE][[1]]
  probs = str_split(probs, ";")
  insert_prob = lapply(probs, function(x){
    paste0("(",str_trim(gsub(".*:","",x)), ")")}
  )
  insert_prob = ifelse(insert_prob=="()", NA, insert_prob)
  
  insert_position = lapply(probs, function(x){gsub(".*?([0-9]+).*", "\\1", 
                                                   str_trim(gsub(":.*","",x)))})
  insert_position = suppressWarnings(lapply(insert_position, 
                                            function(x){as.numeric(x)}))
  
  inject <- function(string, index, replacement){
    stri_sub_replace_all(string, from = index+1,
                         to = index,
                         replacement = replacement)
  }
  
  inserted_string = mapply(inject, input[, "Sequence"][[1]], insert_position, insert_prob)
  inserted_string = ifelse(is.na(inserted_string), input[, "Sequence"][[1]], inserted_string)
  input[, "ModSequence"] = inserted_string
  
  return(input)
}

#' Pull out modifications from PD PTM data for input into protein name
#' @noRd
#' @keywords internal
.extract_pd_mods = function(input, mod_id, keep_all_mods, sequence_col){
  
  message("INFO: Extracting modifications")
  modifications = input$Modifications
  split_mods = str_split(modifications, ";")
  
  quick_filter = function(mod_list){
    mod_list = str_trim(mod_list)
    mask = ifelse(grepl(mod_id, mod_list), TRUE, FALSE)
    return(mod_list[mask])
  }
  
  if(!keep_all_mods){
    target_mods = lapply(split_mods, quick_filter)
    target_mods = lapply(target_mods, function(x){str_trim(
      str_replace(x, mod_id, ""))})
  } else{
    target_mods = lapply(split_mods, function(x){str_trim(
      str_replace(x, "\\s*\\([^\\)]+\\)", ""))})
  }
  insert_position = lapply(target_mods, function(x){gsub(".*?([0-9]+).*", 
                                                        "\\1", x)})
  
  insert_position = suppressWarnings(lapply(insert_position, 
                                            function(x){as.numeric(x)}))
  
  inject <- function(string, index, replacement){
    index = index[!is.na(index)]
    stri_sub_replace_all(string, from = index+1,
                         to = index,
                         replacement = replacement)
  }
  
  inserted_string = mapply(inject, input[, ..sequence_col][[1]], insert_position, "*")
  inserted_string = ifelse(is.na(inserted_string), input[, ..sequence_col][[1]], inserted_string)
  input[, "ModSequence"] = inserted_string
  
  return(input)
}

#' simple function to grab closing bracket
#' @noRd
#' @keywords internal
get_closing_bracket <- function(open_bracket) {
  closing_brackets <- c(")", "]", "}")
  return(closing_brackets[which(open_bracket == c("(", "[", "{"))])
}

#' Visualization for model-based analysis and summarization
#'
#' To analyze the results of modeling changes in abundance of modified peptides 
#' and overall protein, groupComparisonPlotsPTM takes as input the results of 
#' the groupComparisonPTM function. It asses the results of three models: 
#' unadjusted PTM, adjusted PTM, and overall protein. To asses the results of
#' the model, the following visualizations can be created:
#' (1) VolcanoPlot (specify "VolcanoPlot" in option type), to plot peptides or
#' proteins and their significance for each model.
#' (2) Heatmap (specify "Heatmap" in option type), to evaluate the fold change
#' between conditions and peptides/proteins
#' 
#' @export
#' @import ggplot2
#' @importFrom grDevices dev.off hcl pdf
#' @importFrom data.table data.table as.data.table 
#' @importFrom ggrepel geom_text_repel
#' @importFrom checkmate assertNumeric assertChoice assertLogical
#' @importFrom MSstats theme_msstats
#' @importFrom plotly ggplotly style add_trace plot_ly subplot layout
#' 
#' @param data name of the list with models, which can be the output of the 
#' MSstatsPTM \code{\link[MSstatsPTM]{groupComparisonPTM}} function
#' @param type choice of visualization, one of VolcanoPlot or Heatmap
#' @param sig FDR cutoff for the adjusted p-values in heatmap and volcano plot. 
#' level of significance for comparison plot. 100(1-sig)% confidence interval 
#' will be drawn. sig=0.05 is default.
#' @param FCcutoff For volcano plot or heatmap, whether involve fold change 
#' cutoff or not. FALSE (default) means no fold change cutoff is applied for 
#' significance analysis. FCcutoff = specific value means specific fold change 
#' cutoff is applied.
#' @param logBase.pvalue for volcano plot or heatmap, (-) logarithm 
#' transformation of adjusted p-value with base 2 or 10(default).
#' @param ylimUp for all three plots, upper limit for y-axis. FALSE (default) 
#' for volcano plot/heatmap use maximum of -log2 (adjusted p-value) or -log10 
#' (adjusted p-value). FALSE (default) for comparison plot uses maximum of 
#' log-fold change + CI.
#' @param ylimDown for all three plots, lower limit for y-axis. FALSE (default) 
#' for volcano plot/heatmap use minimum of -log2 (adjusted p-value) or -log10 
#' (adjusted p-value). FALSE (default) for comparison plot uses minimum of 
#' log-fold change - CI.
#' @param xlimUp for Volcano plot, the limit for x-axis. FALSE (default) for 
#' use maximum for absolute value of log-fold change or 3 as default if maximum 
#' for absolute value of log-fold change is less than 3.
#' @param x.axis.size size of axes labels, e.g. name of the comparisons in 
#' heatmap, and in comparison plot. Default is 10.
#' @param y.axis.size size of axes labels, e.g. name of targeted proteins in 
#' heatmap. Default is 10.
#' @param dot.size size of dots in volcano plot and comparison plot. Default is 
#' 3.
#' @param text.size size of ProteinName label in the graph for Volcano Plot. 
#' Default is 4.
#' @param text.angle angle of x-axis labels represented each comparison at the 
#' bottom of graph in comparison plot. Default is 0.
#' @param legend.size size of legend for color at the bottom of volcano plot. 
#' Default is 7.
#' @param ProteinName for volcano plot only, whether display protein names or 
#' not. TRUE (default) means protein names, which are significant, are 
#' displayed next to the points. FALSE means no protein names are displayed.
#' @param colorkey TRUE(default) shows colorkey.
#' @param numProtein The number of proteins which will be presented in each 
#' heatmap. Default is 50.
#' @param width width of the saved file. Default is 10.
#' @param height height of the saved file. Default is 10.
#' @param which.Comparison list of comparisons to draw plots. List can be 
#' labels of comparisons or order numbers of comparisons from levels(data$Label)
#' , such as levels(testResultMultiComparisons$ComparisonResult$Label). 
#' Default is "all", which generates all plots for each protein.
#' @param which.PTM Protein list to draw comparison plots. List can be 
#' names of Proteins or order numbers of Proteins from 
#' levels(testResultMultiComparisons$ComparisonResult$Protein). Default is 
#' "all", which generates all comparison plots for each protein.
#' @param address the name of folder that will store the results. Default 
#' folder is the current working directory. The other assigned folder has to 
#' be existed under the current working directory. An output pdf file is 
#' automatically created with the default name of "VolcanoPlot.pdf" or 
#' "Heatmap.pdf". The command address can help to specify where to store the 
#' file as well as how to modify the beginning of the file name. If 
#' address=FALSE, plot will be not saved as pdf file but showed in window
#' @param isPlotly Parameter to use Plotly or ggplot2. If set to TRUE, MSstats 
#' will save Plotly plots as HTML files. If set to FALSE MSstats will save ggplot2 plots
#' as PDF files
#' @return plot or pdf
#' @examples 
#' 
#' model.lf.msstatsptm = groupComparisonPTM(summary.data, 
#'                                      ptm_label_type="LF",
#'                                      protein_label_type="LF")
#' groupComparisonPlotsPTM(data = model.lf.msstatsptm,
#'                         type = "VolcanoPlot",
#'                         FCcutoff= 2,
#'                         logBase.pvalue = 2,
#'                         address=FALSE)
groupComparisonPlotsPTM = function(data = data,
                                    type,
                                    sig=0.05,
                                    FCcutoff=FALSE,
                                    logBase.pvalue=10,
                                    ylimUp=FALSE,
                                    ylimDown=FALSE,
                                    xlimUp=FALSE,
                                    x.axis.size=10,
                                    y.axis.size=10,
                                    dot.size=3,
                                    text.size=4,
                                    text.angle=0,
                                    legend.size=13,
                                    ProteinName=TRUE,
                                    colorkey=TRUE,
                                    numProtein=NULL, 
                                    width=10, 
                                    height=10, 
                                    which.Comparison = "all", 
                                    which.PTM = "all",
                                    address="",
                                    isPlotly = FALSE) {
  
  .check.plotting.data(data, type, sig, FCcutoff, logBase.pvalue, ylimUp,
                       ylimDown, xlimUp, x.axis.size, y.axis.size, dot.size,
                       text.size, text.angle, legend.size, ProteinName,
                       colorkey, numProtein, width, height, which.Comparison, 
                       which.PTM, address)
  
  data = .format.model.plots(data, which.Comparison, which.PTM)
  type = toupper(type)
  
  plot_name_list = c("Unadjusted Peptides", "Global Protein", 
                     "Adjusted Peptide")
  
  if(isPlotly & address != FALSE) {
    print("Plots will be saved as .HTML file as plotly is selected, set isPlotly = FALSE, if 
            you want to generate PDF using ggplot2")
  }
  
  if (type == 'HEATMAP') {
    plotly_plots <- .plotHeatmap(data, sig, FCcutoff, logBase.pvalue, ylimUp, ylimDown, text.angle,
                  x.axis.size, y.axis.size, dot.size, colorkey, numProtein, 
                  width, height, address, isPlotly)
    
    if(isPlotly) {
      if(address != FALSE) {
        .savePlotlyPlotHTML(plotly_plots,address,"Heatmap" ,width, height)
      }
      plotly_plots
    }
    
  } else if (type == 'VOLCANOPLOT') {
    plots <- .plotVolcano(data, sig, FCcutoff, logBase.pvalue, ylimUp, ylimDown, xlimUp, 
                  x.axis.size, y.axis.size, dot.size, text.size, legend.size, 
                  ProteinName, colorkey, numProtein, width, height, 
                  address, plot_name_list, isPlotly)
    
    plotly_plots <- vector("list", length(plots))
    if(isPlotly) {
      for(i in seq_along(plots)) {
        plot <- plots[[i]]
        plotly_plot <- .convertGgplot2Plotly(plot, width = 1000)
        plotly_plot <- .fixLegendPlotlyPlotsVolcano(plotly_plot)
        plotly_plots[[i]] = list(plotly_plot)
      }
      if(address != FALSE) {
        .savePlotlyPlotHTML(plotly_plots,address,"VolcanoPlot" ,width, height)
      }
      plotly_plots <- unlist(plotly_plots, recursive = FALSE)
      plotly_plots
    }
  }
}

.fixLegendPlotlyPlotsVolcano = function(plot) {
  df <- data.frame(id = seq_along(plot$x$data), legend_entries = unlist(lapply(plot$x$data, `[[`, "name")))
  # Create a mapping
  color_mapping <- c("black" = "No regulation", "red" = "Up-regulated", "blue" = "Down-regulated")
  # Update the legend_entries column
  df$legend_group <- sapply(df$legend_entries, function(entry) {
    for (color in names(color_mapping)) {
      if (grepl(color, entry)) {
        entry <- gsub(color, color_mapping[color], entry)
        break
      }
    }
    entry <- gsub(",.+", "", entry)
    entry <- gsub("\\(|\\)", "", entry)  # Remove any remaining parentheses
    entry
  })
  for (i in df$id) {
    if(length(grep(df$legend_group[[i]], color_mapping)) == 0) { # keep only 3 legends
      plot$x$data[[i]]$showlegend <- FALSE
    }
    plot$x$data[[i]]$name <- df$legend_group[[i]]
    plot$x$data[[i]]$legendgroup <- plot$x$data[[i]]$name
  }
  plot <- plotly::layout(plot,legend=list(title=list(text="")))
  plot
}
  
  

#' Visualization for explanatory data analysis
#'
#' To illustrate the quantitative data and quality control of MS runs,
#' dataProcessPlotsPTM takes the quantitative data from dataSummarizationPTM or
#' dataSummarizationPTM_TMT to plot the following :
#' (1) profile plot (specify "ProfilePlot" in option type), to identify the
#' potential sources of variation for each protein;
#' (2) quality control plot (specify "QCPlot" in option type), to evaluate the
#' systematic bias between MS runs.
#'
#' @export
#' @import ggplot2
#' @importFrom grDevices dev.off hcl pdf
#' @importFrom gridExtra grid.arrange
#' @importFrom stringr str_match
#' @importFrom data.table data.table as.data.table melt dcast `:=` setnames copy rbindlist
#' @importFrom checkmate assertCharacter assertNumeric assertChoice
#' @importFrom MSstats theme_msstats
#' @importFrom plotly ggplotly style add_trace plot_ly subplot layout
#' 
#' @param data name of the list with PTM and (optionally) Protein data, which
#' can be the output of the MSstatsPTM 
#' \code{\link[MSstatsPTM]{dataSummarizationPTM}} or 
#' \code{\link[MSstatsPTM]{dataSummarizationPTM_TMT}} functions.
#' @param type choice of visualization. "ProfilePlot" represents profile plot of
#'  log intensities across MS runs.
#' "QCPlot" represents box plots of log intensities across channels and MS runs.
#' @param ylimUp upper limit for y-axis in the log scale.
#' FALSE(Default) for Profile Plot and QC Plot uses the upper limit as rounded
#' off maximum of log2(intensities) after normalization + 3..
#' @param ylimDown lower limit for y-axis in the log scale. FALSE(Default) for
#' Profile Plot and QC Plot uses 0..
#' @param x.axis.size size of x-axis labeling for "Run" and "channel in Profile
#' Plot and QC Plot.
#' @param y.axis.size size of y-axis labels. Default is 10.
#' @param text.size size of labels represented each condition at the top of
#' Profile plot and QC plot. Default is 4.
#' @param text.angle angle of labels represented each condition at the top of
#' Profile plot and QC plot. Default is 0.
#' @param legend.size size of legend above Profile plot. Default is 7.
#' @param dot.size.profile size of dots in Profile plot. Default is 2.
#' @param ncol.guide number of columns for legends at the top of plot. Default
#' is 5.
#' @param width width of the saved pdf file. Default is 10.
#' @param height height of the saved pdf file. Default is 10.
#' @param ptm.title title of overall PTM QC plot
#' @param protein.title title of overall Protein QC plot
#' @param which.PTM PTM list to draw plots. List can be names of
#' PTMs or order numbers of PTMs.
#' Default is "all", which generates all plots for each protein. For QC plot,
#' "allonly" will generate one QC plot with all proteins.
#' @param which.Protein List of proteins to plot. Will plot all PTMs associated 
#' with listed Proteins. Default is NULL which will default to which.PTM.
#' @param originalPlot TRUE(default) draws original profile plots, without
#' normalization.
#' @param summaryPlot TRUE(default) draws profile plots with protein
#' summarization for each channel and MS run.
#' @param address the name of folder that will store the results. Default folder
#'  is the current working directory.
#' @param isPlotly Parameter to use Plotly or ggplot2. If set to TRUE, MSstats 
#' will save Plotly plots as HTML files. If set to FALSE MSstats will save ggplot2 plots
#' as PDF files
#' The other assigned folder has to be existed under the current working
#' directory.
#' An output pdf file is automatically created with the default name of
#' "ProfilePlot.pdf" or "QCplot.pdf".
#' The command address can help to specify where to store the file as well as
#' how to modify the beginning of the file name.
#' If address=FALSE, plot will be not saved as pdf file but showed in window.
#' @return plot or pdf
#' @examples
#' 
#' # QCPlot
#' dataProcessPlotsPTM(summary.data,
#'                     type = 'QCPLOT',
#'                     which.PTM = "allonly",
#'                     address = FALSE)
#'                     
#' #ProfilePlot
#' dataProcessPlotsPTM(summary.data,
#'                     type = 'PROFILEPLOT',
#'                     which.PTM = "Q9UQ80_K376",
#'                     address = FALSE)
dataProcessPlotsPTM = function(data,
                                type = 'PROFILEPLOT',
                                ylimUp = FALSE,
                                ylimDown = FALSE,
                                x.axis.size = 10,
                                y.axis.size = 10,
                                text.size = 4,
                                text.angle = 90,
                                legend.size = 7,
                                dot.size.profile = 2,
                                ncol.guide = 5,
                                width = 10,
                                height = 12,
                                ptm.title = "All PTMs",
                                protein.title = "All Proteins",
                                which.PTM = "all", 
                                which.Protein = NULL,
                                originalPlot = TRUE,
                                summaryPlot = TRUE,
                                address = "",
                                isPlotly = FALSE) {
  
  type = toupper(type)
  label = .check.dataProcess.plotting.data(data, type, ylimUp, ylimDown, 
                                            x.axis.size,
                                y.axis.size, text.size, text.angle, legend.size,
                                dot.size.profile, ncol.guide, width, height,
                                ptm.title, protein.title, which.PTM, 
                                originalPlot, summaryPlot, address)
  
  data.table.list = .format.data.process.plots(data, label)
  
  if(isPlotly & address != FALSE) {
    print("Plots will be saved as .HTML file as plotly is selected, set isPlotly = FALSE, if 
            you want to generate PDF using ggplot2")
  }
  
  ## Filter for all PTMs in one proteins
  if (!is.null(which.Protein)){
    
    data.table.list[[1]] = data.table.list[[1]][PROTEINNAME %in% which.Protein]
    data.table.list[[2]] = data.table.list[[2]][
      GLOBALPROTEIN %in% which.Protein]
    data.table.list[[3]] = data.table.list[[3]][PROTEINNAME %in% which.Protein]
    data.table.list[[4]] = data.table.list[[4]][
      GLOBALPROTEIN %in% which.Protein]
    
    if (sum(nrow(data.table.list[[1]]), nrow(data.table.list[[2]]), 
           nrow(data.table.list[[3]]), nrow(data.table.list[[4]])) == 0){
      msg = paste0("The protein ", which.Protein, " specified in Which.Protein",
                   " is not in the global protein run. Please specify ",
                   "individual peptides only for this Protein.")
      stop(msg)
    }
  }
  
  ## Profile plot ##
  ## ---------------
  if (type == "PROFILEPLOT") {
    if (label == 'TMT'){
      plots <- .profile.tmt(data.table.list, type, ylimUp, ylimDown, 
                   x.axis.size, y.axis.size,text.size,text.angle, 
                   legend.size, dot.size.profile, ncol.guide, width, 
                   height, which.PTM, originalPlot, summaryPlot, 
                   address, isPlotly)
    } else if (label == 'LabelFree'){
      plots <- .profile.lf(data.table.list, type, ylimUp, ylimDown, 
                  x.axis.size, y.axis.size,text.size,text.angle, 
                  legend.size, dot.size.profile, ncol.guide, width, 
                  height, which.PTM, originalPlot, 
                  summaryPlot, address, isPlotly)
    }
    
    plotly_plots <- list()
    if(isPlotly) {
      og_plotly_plot = NULL
      summ_plotly_plot = NULL
      if("original_plot" %in% names(plots)) {
        for(i in seq_along(plots[["original_plot"]])) {
          plot_i <- plots[["original_plot"]][[paste("plot",i)]]
          plotly_plot_ptm <- .convertGgplot2Plotly(plot_i[["PTEMP.PTM"]])
          plotly_plot_ptm = .fixLegendPlotlyPlotsDataprocess(plotly_plot_ptm, "OriginalPlot")
          if (!is.null(plot_i[["PTEMP.PROTEIN"]])) {
            plotly_plot_protein <- .convertGgplot2Plotly(plot_i[["PTEMP.PROTEIN"]])
            plotly_plot_protein = .fixLegendPlotlyPlotsDataprocess(plotly_plot_protein, "OriginalPlot")
            plotly_plot_combined_original <- .combineSubPlotsPlotly(plotly_plot_ptm, plotly_plot_protein)
            plotly_plots = c(plotly_plots, list(plotly_plot_combined_original))
          } else {
            plotly_plots = c(plotly_plots, list(plotly_plot_ptm))
          }
        }
      }
      if("summary_plot" %in% names(plots)) {
        for(i in seq_along(plots[["summary_plot"]])) {
          plot_i <- plots[["summary_plot"]][[paste("plot",i)]]
          plotly_plot_ptm <- .convertGgplot2Plotly(plot_i[["PTEMP.PTM"]])
          plotly_plot_ptm = .fixLegendPlotlyPlotsDataprocess(plotly_plot_ptm, "SummaryPlot")
          if (!is.null(plot_i[["PTEMP.PROTEIN"]])) {
            plotly_plot_protein <- .convertGgplot2Plotly(plot_i[["PTEMP.PROTEIN"]])
            plotly_plot_combined_summary <- .combineSubPlotsPlotly(plotly_plot_ptm, plotly_plot_protein)
            plotly_plot_combined_summary = .fixLegendPlotlyPlotsDataprocess(plotly_plot_combined_summary, "SummaryPlot")
            plotly_plots = c(plotly_plots, list(plotly_plot_combined_summary))
          } else {
            plotly_plots = c(plotly_plots, list(plotly_plot_ptm))
          }
        }
      }
      if(address != FALSE) {
        .savePlotlyPlotHTML(plotly_plots,address,"ProfilePlot" ,width, height)
      }
      plotly_plots
    }
  }

  ## QC plot (Quality control plot) ##
  ## ---------------------------------
  else if (type == "QCPLOT") {
    if (label == 'TMT'){
      plots <- .qc.tmt(data.table.list, type, ylimUp, ylimDown, width, height, 
              x.axis.size, y.axis.size, text.size, text.angle,
              which.PTM, address, ptm.title, protein.title, isPlotly)
    } else if (label == 'LabelFree'){
      plots <- .qc.lf(data.table.list, type, ylimUp, ylimDown, width, height, 
             x.axis.size, y.axis.size, text.size,
             which.PTM, address, ptm.title, protein.title, isPlotly)
    } 
    
    plotly_plots <- list()
    if(isPlotly) {
      for(i in seq_along(plots)) {
        plot <- plots[[i]]
        plotly_plot_ptm <- .convertGgplot2Plotly(plot[["PTEMP.PTM"]])
        if(label == "TMT") {
          plotly_plot_ptm <- facet_strip_bigger(plotly_plot_ptm)
        }
        if (!is.null(plot[["PTEMP.PROTEIN"]])) {
          plotly_plot_protein <- .convertGgplot2Plotly(plot[["PTEMP.PROTEIN"]])
          if(label == "TMT") {
            plotly_plot_protein <- facet_strip_bigger(plotly_plot_protein)
          }
          plotly_plot_combined <- .combineSubPlotsPlotly(plotly_plot_ptm, plotly_plot_protein)
          plotly_plots[[i]] = list(plotly_plot_combined)
        } else {
          plotly_plots[[i]] = list(plotly_plot_ptm)
        }
        
      }
      if(address != FALSE) {
        .savePlotlyPlotHTML(plotly_plots,address,"QCPlot" ,width, height)
      }
      plotly_plots <- unlist(plotly_plots, recursive = FALSE)
      plotly_plots
    }
  }
}

.combineSubPlotsPlotly = function(plotly_plot_ptm, plotly_plot_protein) {
  title_ptm <- plotly_plot_ptm$x$layout$title$text
  title_protein <- plotly_plot_protein$x$layout$title$text
  
  plotly_plot_ptm <- plotly::layout(plotly_plot_ptm, title = "")
  plotly_plot_protein <- plotly::layout(plotly_plot_protein, title = "")
  
  plotly_plot_combined <- subplot(plotly_plot_ptm, plotly_plot_protein, nrows = 2, margin=0.1,titleX = TRUE, titleY = TRUE)
  plotly_plot_combined <- plotly::layout(plotly_plot_combined,
                                         annotations = list(
                                           list(
                                             x = 0.04,  # Centered horizontally
                                             y = 1.05,  # Above the first plot
                                             text = title_ptm,  # Use extracted title
                                             showarrow = FALSE,
                                             xref = 'paper',
                                             yref = 'paper',
                                             xanchor = 'center',
                                             yanchor = 'bottom',
                                             font = list(size = 16)
                                           ),
                                           list(
                                             x = 0.04,  # Centered horizontally
                                             y = 0.45,  # Above the second plot (adjust as necessary)
                                             text = title_protein,  # Use extracted title
                                             showarrow = FALSE,
                                             xref = 'paper',
                                             yref = 'paper',
                                             xanchor = 'center',
                                             yanchor = 'bottom',
                                             font = list(size = 16)
                                           )
                                         )
  )
  plotly_plot_combined
}

.fixLegendPlotlyPlotsDataprocess = function(plot, type) {
  df <- data.frame(id = seq_along(plot$x$data), legend_entries = unlist(lapply(plot$x$data, `[[`, "name")))
  df$legend_group <- gsub("^\\((.*?),.*", "\\1", df$legend_entries)
  df$is_first <- !duplicated(df$legend_group)
  df$is_bool <- ifelse(grepl("TRUE|FALSE", df$legend_group), TRUE, FALSE)
  df$is_valid_column <- ifelse(grepl("Processed feature-level data|Run summary", df$legend_entries), TRUE, FALSE)
  plot$x$data[[nrow(df)]]$showlegend <- FALSE # remove text legend
  
  for (i in df$id) {
    is_first <- df$is_first[[i]]
    is_bool <- df$is_bool[[i]]
    plot$x$data[[i]]$name <- df$legend_group[[i]]
    plot$x$data[[i]]$legendgroup <- plot$x$data[[i]]$name
    if (!is_first) {
      plot$x$data[[i]]$showlegend <- FALSE
    } 
    if(type == "SummaryPlot") {
      is_valid_column <- df$is_valid_column[[i]]
      if (!is_valid_column) plot$x$data[[i]]$showlegend <- FALSE
    }
    if(is_bool) plot$x$data[[i]]$showlegend <- FALSE
  }
  plot
}

facet_strip_bigger <- function(gp){
  if (!is.null(gp$x$layout$annotations)) {
    for (i in seq_along(gp$x$layout$annotations)) {
      if(gp$x$layout$annotations[[i]]$text != "Log2-intensities" && gp$x$layout$annotations[[i]]$text != "MS runs") {
        gp$x$layout$annotations[[i]]$font$size <- 7
        # gp$x$layout$annotations[[i]]$xanchor <- "center"
        gp$x$layout$annotations[[i]]$xshift <- 50
      }
    }
  }
  return(gp)
}

#' converter for plots from ggplot to plotly
#' @noRd
.convertGgplot2Plotly = function(plot, tips = "all", width = 1800, height = 600) {
  converted_plot <- ggplotly(plot,tooltip = tips)
  converted_plot <- plotly::layout(
    converted_plot,
    width = width,   # Set the width of the chart in pixels
    height = height,  # Set the height of the chart in pixels
    title = list(
      font = list(
        size = 18
      )
    ),
    legend = list(
      x = 0,     # Set the x position of the legend
      y = -0.25,    # Set the y position of the legend (negative value to move below the plot)
      orientation = "h",  # Horizontal orientation
      font = list(
        size = 12  # Set the font size for legend item labels
      ),
      title = list(
        font = list(
          size = 12  # Set the font size for the legend title
        )
      )
    )
  ) 
  converted_plot
}

.savePlotlyPlotHTML = function(plots, address, file_name, width, height) {
  print("Saving plots as HTML")
  pb <- txtProgressBar(min = 0, max = 4, style = 3)
  
  setTxtProgressBar(pb, 1)
  file_name = getFileName(address, file_name, width, height)
  file_name = paste0(file_name,".html")
  
  setTxtProgressBar(pb, 2)
  doc <- .getPlotlyPlotHTML(plots, width, height)
  
  setTxtProgressBar(pb, 3)
  htmltools::save_html(html = doc, file = file_name) # works but lib same folder
  
  setTxtProgressBar(pb, 4)
  zip(paste0(gsub("\\.html$", "", file_name),".zip"), c(file_name, "lib"))
  unlink(file_name)
  unlink("lib",recursive = T)
  
  close(pb)
}

getFileName = function(name_base, file_name, width, height) {
  all_files = list.files(".")
  if(file_name == 'ProfilePlot'){
    num_same_name = sum(grepl(paste0("^", name_base, file_name, "_[0-9]?"), all_files))
  } else {
    num_same_name = sum(grepl(paste0("^", name_base, file_name, "[0-9]?"), all_files))
  }
  if (num_same_name > 0) {
    file_name = paste(file_name, num_same_name + 1, sep = "_")
  }
  file_path = paste0(name_base, file_name)
  return(file_path)
}

.getPlotlyPlotHTML = function(plots, width, height) {
  doc <- htmltools::tagList(lapply(plots,function(x) htmltools::div(x, style = "float:left;width:100%;")))
  # Set a specific width for each plot
  plot_width <- 800
  plot_height <- 600
  
  # Create a div for each plot with style settings
  divs <- lapply(plots, function(x) {
    htmltools::div(x, style = paste0("width:", plot_width, "px; height:", plot_height, "px; margin: 10px;"))
  })
  
  # Combine the divs into a tagList
  doc <- htmltools::tagList(divs)
  doc
}

#' Data processing and summarization of peptide-level quantification to PTM and protein level quantification
#' 
#' Function to perform data processing and summarization on an experiment 
#' targeting post-translational modifications. Performs normalization, missing 
#' value imputation, feature selection, and summarization. Can optionally take 
#' an additional global protein quantification experiment for protein-level 
#' correction of PTM changes. Can take either label free or tandem mass tag 
#' (TMT) labeled data.
#' 
#' @export
#' @importFrom MSstats dataProcess
#' @importFrom MSstatsTMT proteinSummarization
#' 
#' @param data Name of the output of MSstatsPTM converter function or 
#' peptide-level quantified data from other tools. It should be a list 
#' containing one or two data tables, named PTM and PROTEIN for modified and 
#' unmodified datasets. The list must at least contain the PTM dataset, however
#' the PROTEIN dataset is optional.
#' @param ptm_label_type Indicator of labeling type for PTM dataset. Must be one of
#' `LF` or `TMT`
#' @param protein_label_type Indicator of labeling type for PROTEIN dataset. Must be 
#' one of `LF` or `TMT`
#' @param MBimpute_ptm Missing value imputation on the feature-level for the PTM
#' dataset. TRUE (default) imputes missing values by Accelated 
#' failure model. FALSE uses minimum value to impute the missing value for each 
#' peptide precursor ion.
#' @param MBimpute_protein Missing value imputation on the feature-level for the
#' PROTEIN dataset. TRUE (default) imputes missing values by Accelated 
#' failure model. FALSE uses minimum value to impute the missing value for each 
#' peptide precursor ion.
#' @param ... Additional parameters passed to either the 
#' \link[MSstats]{dataProcess} function from `MSstats` or the 
#' \link[MSstatsTMT]{proteinSummarization} function from `MSstatsTMT`.
dataProcessPTM = function(data, 
                          ptm_label_type,
                          protein_label_type,
                          MBimpute_ptm=FALSE,
                          MBimpute_protein=TRUE, 
                          use_log_file=TRUE, 
                          append=FALSE,
                          verbose=TRUE, 
                          log_file_path=NULL,
                          ...){
  
  ## Start log
  if (is.null(log_file_path) & use_log_file == TRUE){
    time_now = Sys.time()
    path = paste0("MSstatsPTM_log_", gsub("[ :\\-]", "_", time_now),
                  ".log")
    file.create(path)
  } else {
    path = log_file_path
  }
  
  ars = list(...)
  
  potential_lf_ars = c("logTrans", "normalization", "nameStandards", 
                       "featureSubset", "remove_uninformative_feature_outlier", 
                       "min_feature_count", "n_top_feature", "summaryMethod", 
                       "equalFeatureVar", "censoredInt", "remove50missing", 
                       "fix_missing", "maxQuantileforCensored", "numberOfCores")
  potential_tmt_ars = c("method", "global_norm", "reference_norm", 
                        "remove_norm_channel", "remove_empty_channel", 
                        "maxQuantileforCensored")
  
  lf_ars = intersect(names(ars), potential_lf_ars)
  tmt_ars = intersect(names(ars), potential_tmt_ars)
  
  ## Determine if PTM should be adjusted for protein level.
  if (!is.null(data$PROTEIN)){
    adj_protein = TRUE
  } else{
    adj_protein = FALSE
  }
  
  if (ptm_label_type == "LF"){
    
    MSstatsLogsSettings(use_log_file, append,
                        verbose, log_file_path = path)
    
    summarized_ptm = do.call(
      dataProcess, c(list(raw=data$PTM), 
                     ars[lf_ars],
                     list(MBimpute=MBimpute_ptm))
    )
  } else if (ptm_label_type == "TMT"){
    
    MSstatsLogsSettings(use_log_file, append,
                        verbose, log_file_path = path,
                        pkg_name = "MSstatsTMT")
    
    summarized_ptm = do.call(
      proteinSummarization, c(list(data=data$PTM), 
                              ars[tmt_ars],
                              list(MBimpute=MBimpute_ptm))
    )

  }
  if (adj_protein){
    if (protein_label_type == "LF"){
      
      MSstatsLogsSettings(use_log_file, append,
                          verbose, log_file_path = path)
      
      summarized_protein = do.call(
        dataProcess, c(list(raw=data$PROTEIN), 
                       ars[lf_ars],
                       list(MBimpute=MBimpute_protein))
      )
    } else if (protein_label_type == "TMT"){
      
      MSstatsLogsSettings(use_log_file, append,
                          verbose, log_file_path = path,
                          pkg_name = "MSstatsTMT")
      
      summarized_protein = do.call(
        proteinSummarization, c(list(data=data$PROTEIN), 
                                ars[tmt_ars],
                                list(MBimpute=MBimpute_protein))
      )
    }
  }
  
  return(
    list(
      PTM=summarized_ptm,
      PROTEIN=summarized_protein
      )
    )
  
}

#' Data summarization function for label-free MS experiments targeting PTMs.
#'
#' Utilizes functionality from MSstats to clean, summarize, and
#' normalize PTM and protein level data. Imputes missing values, performs 
#' normalization, and summarizes data. PTM data is summarized up to the 
#' modification and protein data is summarized up to the protein level. Takes 
#' as input the output of the included converters (see included `raw.input` 
#' data object for required input format).
#'
#' @export
#' @importFrom MSstats dataProcess
#' @importFrom MSstatsConvert MSstatsLogsSettings MSstatsSaveSessionInfo
#' @importFrom data.table as.data.table is.data.table
#' 
#' @param data name of the list with PTM and (optionally) unmodified protein 
#' data.tables, which can be the output of the MSstatsPTM converter functions
#' @param logTrans logarithm transformation with base 2(default) or 10
#' @param normalization normalization for the protein level dataset, to remove 
#' systematic bias between MS runs. There are three different normalizations 
#' supported. 'equalizeMedians'(default) represents constant normalization 
#' (equalizing the medians) based on reference signals is performed. 'quantile' 
#' represents quantile normalization based on reference signals is performed. 
#' 'globalStandards' represents normalization with global standards proteins. 
#' FALSE represents no normalization is performed
#' @param normalization.PTM normalization for PTM level dataset. Default is 
#' "equalizeMedians" Can be adjusted to any of the options described above.
#' @param nameStandards vector of global standard peptide names for protein 
#' dataset. only for normalization with global standard peptides.
#' @param nameStandards.PTM Same as above for PTM dataset.
#' @param featureSubset "all" (default) uses all features that the data set has. 
#' "top3" uses top 3 features which have highest average of log-intensity across runs. 
#' "topN" uses top N features which has highest average of log-intensity across runs. 
#' It needs the input for n_top_feature option. 
#' "highQuality" flags uninformative feature and outliers.
#' @param featureSubset.PTM For PTM dataset only. Options same as above.
#' @param remove_uninformative_feature_outlier For protein dataset only. It only
#'  works after users used featureSubset="highQuality" in dataProcess. TRUE 
#'  allows to remove 1) the features are flagged in the column, 
#'  feature_quality="Uninformative" which are features with bad quality, 2) 
#'  outliers that are flagged in the column, is_outlier=TRUE, for run-level 
#'  summarization. FALSE (default) uses all features and intensities for 
#'  run-level summarization.
#' @param remove_uninformative_feature_outlier.PTM For PTM dataset only. Options
#' same as above.
#' @param min_feature_count optional. Only required if featureSubset = "highQuality".
#' Defines a minimum number of informative features a protein needs to be considered
#' in the feature selection algorithm.
#' @param min_feature_count.PTM For PTM dataset only. Options the same as above.
#' Default is 1 due to low average feature count for PTMs. 
#' @param n_top_feature For protein dataset only. The number of top features for
#'  featureSubset='topN'. Default is 3, which means to use top 3 features.
#' @param n_top_feature.PTM For PTM dataset only. Options same as above.
#' @param summaryMethod "TMP"(default) means Tukey's median polish, which is 
#' robust estimation method. "linear" uses linear mixed model. 
#' @param equalFeatureVar only for summaryMethod="linear". default is TRUE. 
#' Logical variable for whether the model should account for heterogeneous 
#' variation among intensities from different features. Default is TRUE, which 
#' assume equal variance among intensities from features. FALSE means that we 
#' cannot assume equal variance among intensities from features, then we will 
#' account for heterogeneous variation from different features.
#' @param censoredInt Missing values are censored or at random. 'NA' (default) 
#' assumes that all 'NA's in 'Intensity' column are censored. '0' uses zero 
#' intensities as censored intensity. In this case, NA intensities are missing 
#' at random. The output from Skyline should use '0'. Null assumes that all NA 
#' intensites are randomly missing.
#' @param MBimpute For protein dataset only. only for summaryMethod="TMP" and 
#' censoredInt='NA' or '0'. TRUE (default) imputes 'NA' or '0' (depending on 
#' censoredInt option) by Accelated failure model. FALSE uses the values 
#' assigned by cutoffCensored.
#' @param MBimpute.PTM For PTM dataset only. Options same as above.
#' @param remove50missing only for summaryMethod="TMP". TRUE removes the runs 
#' which have more than 50% missing values. FALSE is default.
#' @param maxQuantileforCensored Maximum quantile for deciding censored missing 
#' values. default is 0.999
#' @param fix_missing Default is Null. Optional, same as the 'fix_missing' 
#' parameter in MSstatsConvert::MSstatsBalancedDesign function
#' @param use_log_file logical. If TRUE, information about data processing
#' will be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing will be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. 
#' If not provided, such a file will be created automatically.
#' If `append = TRUE`, has to be a valid path to a file.
#' @param base start of the file name.
#' @return list of summarized PTM and Protein results. These results contain 
#' the reformatted input to the summarization function, as well as run-level
#' summarization results.
#' @examples
#' head(raw.input$PTM)
#' head(raw.input$PROTEIN)
#' 
#' quant.lf.msstatsptm = dataSummarizationPTM(raw.input, verbose = FALSE)
#' head(quant.lf.msstatsptm$PTM$ProteinLevelData)
dataSummarizationPTM = function(
  data,
  logTrans = 2, 
  normalization = "equalizeMedians", 
  normalization.PTM = "equalizeMedians", 
  nameStandards = NULL,
  nameStandards.PTM = NULL,
  featureSubset = "all",
  featureSubset.PTM = "all",
  remove_uninformative_feature_outlier = FALSE, 
  remove_uninformative_feature_outlier.PTM = FALSE,
  min_feature_count = 2,
  min_feature_count.PTM = 1,
  n_top_feature = 3, 
  n_top_feature.PTM = 3,
  summaryMethod = "TMP", 
  equalFeatureVar = TRUE, 
  censoredInt = "NA",
  MBimpute = TRUE, 
  MBimpute.PTM = TRUE, 
  remove50missing = FALSE,
  fix_missing = NULL,
  maxQuantileforCensored = 0.999,
  use_log_file = TRUE, 
  append = TRUE,
  verbose = TRUE, 
  log_file_path = NULL,
  base = "MSstatsPTM_log_") {

  ## Start log  
  if (is.null(log_file_path) & use_log_file == TRUE){
    time_now = Sys.time()
    path = paste0(base, gsub("[ :\\-]", "_", time_now), 
                  ".log")
    file.create(path)
  } else {path = log_file_path}
  
  MSstatsLogsSettings(use_log_file, append,
                      verbose, log_file_path = path)
  
  getOption("MSstatsLog")("INFO", "Starting parameter and data checks..")
  
  .checkDataProcessParams(logTrans, normalization, normalization.PTM, 
                          nameStandards, nameStandards.PTM, 
                          address,
                          list(method = featureSubset, n_top = n_top_feature,
                               remove_uninformative = 
                                 remove_uninformative_feature_outlier),
                          list(method = featureSubset.PTM, 
                               n_top = n_top_feature.PTM,
                               remove_uninformative = 
                                 remove_uninformative_feature_outlier.PTM),
                          list(method = summaryMethod, 
                               equal_var = equalFeatureVar),
                          list(symbol = censoredInt,
                               MB = MBimpute))
  
  adj.protein = FALSE

  PTM.dataset = data[["PTM"]]
  protein.dataset = data[["PROTEIN"]]
  
  # Check PTM and PROTEIN data for correct format
  adj.protein = .summarizeCheck(data, 'LF')
  
  getOption("MSstatsLog")("INFO", "Parameter and data checks complete.")
  
  message("Starting PTM summarization...")
  ## Run summarization function from MSstats
  getOption("MSstatsLog")("INFO", "Starting PTM summarization..")
  ptm.summarized = dataProcess(as.data.frame(PTM.dataset),
                                logTrans, normalization.PTM, nameStandards.PTM,
                                featureSubset.PTM, 
                                remove_uninformative_feature_outlier.PTM,
                                min_feature_count.PTM, n_top_feature.PTM,
                                summaryMethod, equalFeatureVar, censoredInt,
                                MBimpute.PTM, remove50missing, fix_missing,
                                maxQuantileforCensored, use_log_file, append,
                                verbose, log_file_path = path)
  if (adj.protein) {
    message("Starting Protein summarization...")
    getOption("MSstatsLog")("INFO", "Starting Protein summarization..")
    protein.summarized = dataProcess(as.data.frame(protein.dataset),
                                  logTrans, normalization, nameStandards,
                                  featureSubset, 
                                  remove_uninformative_feature_outlier,
                                  min_feature_count, n_top_feature,
                                  summaryMethod, equalFeatureVar, censoredInt,
                                  MBimpute, remove50missing, fix_missing,
                                  maxQuantileforCensored, use_log_file, append,
                                  verbose, log_file_path = path)
  }

  ## Compile and return summarized results
  getOption("MSstatsLog")("INFO", "Summarization complete. Returning output")
  msstatsptm.summarized = list("PTM" = ptm.summarized)
  if (adj.protein) {
    msstatsptm.summarized = c(msstatsptm.summarized,
                               "PROTEIN" = list(protein.summarized))
  }

  return(msstatsptm.summarized)
}

data("raw.input", package = "MSstatsPTM")

## Test missing columns are handled
expect_error(dataSummarizationPTM())
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM[, -1],
                                       PROTEIN = raw.input$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM,
                                       PROTEIN = raw.input$PROTEIN[, -1])))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM[, -2],
                                       PROTEIN = raw.input$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM,
                                       PROTEIN = raw.input$PROTEIN[, -2])))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM[, -3],
                                       PROTEIN = raw.input$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM,
                                       PROTEIN = raw.input$PROTEIN[, -3])))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM[, -4],
                                       PROTEIN = raw.input$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM,
                                       PROTEIN = raw.input$PROTEIN[, -4])))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM[, -5],
                                       PROTEIN = raw.input$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM,
                                       PROTEIN = raw.input$PROTEIN[, -5])))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM[, -6],
                                       PROTEIN = raw.input$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM,
                                       PROTEIN = raw.input$PROTEIN[, -6])))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM[, -7],
                                       PROTEIN = raw.input$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM,
                                       PROTEIN = raw.input$PROTEIN[, -7])))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM[, -8],
                                       PROTEIN = raw.input$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM,
                                       PROTEIN = raw.input$PROTEIN[, -8])))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM[, -9],
                                       PROTEIN = raw.input$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM,
                                       PROTEIN = raw.input$PROTEIN[, -9])))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM[, -10],
                                       PROTEIN = raw.input$PROTEIN)))
expect_error(dataSummarizationPTM(list(PTM = raw.input$PTM,
                                       PROTEIN = raw.input$PROTEIN[, -10])))

## Test misspecified parameters
expect_error(dataSummarizationPTM(raw.input, logTrans = 20))
expect_error(dataSummarizationPTM(raw.input, logTrans = "test"))
expect_error(dataSummarizationPTM(raw.input, normalization = TRUE))
expect_error(dataSummarizationPTM(raw.input, normalization.PTM = TRUE))
expect_error(dataSummarizationPTM(raw.input, fillIncompleteRows = 20))
expect_error(dataSummarizationPTM(raw.input, summaryMethod = TRUE))
expect_error(dataSummarizationPTM(raw.input, censoredInt = ""))
expect_error(dataSummarizationPTM(raw.input, cutoffCensored = ""))
expect_error(dataSummarizationPTM(raw.input, MBimpute = ""))
expect_error(dataSummarizationPTM(raw.input, MBimpute.PTM = ""))

#' Extract global protein from combined protein and site column
#' @noRd
.extractProtein = function(ptm_model, protein_model){
  
  ## All proteins
  available_proteins = unique(as.character(protein_model$Protein))
  available_proteins = available_proteins[order(nchar(available_proteins),
                                                 available_proteins,
                                                 decreasing = TRUE)]
  available_ptms = unique(as.character(ptm_model$Protein))
  available_proteins = available_proteins[!grepl(";", available_proteins)]
  ## Set site
  ptm_model$Site = ptm_model$Protein
  
  ## Ensure vectors passed to Rcpp are characters
  available_ptms = as.character(available_ptms)
  available_proteins = as.character(available_proteins)
  ## Call Rcpp function
  ptm_proteins = extract_protein_name(available_ptms,
                                       available_proteins)
  global_protein_lookup = data.table(Site = available_ptms, 
                                      Protein = ptm_proteins)
  
  ## Add extracted protein name into model
  ptm_model[, "Protein" := NULL]
  ptm_model = merge(ptm_model, global_protein_lookup,
                     all.x = TRUE, by = 'Site')

  return(ptm_model)
}

#' Apply global protein adjustment to each condition
#' @noRd
.applyPtmAdjustment = function(label, ptm_model, protein_model){
  
  Label = NULL
  
  temp_ptm_model = ptm_model[Label == label]
  temp_protein_model = protein_model[Label == label]
  
  ## Function from MSstatsPTM Compare
  temp_adjusted_model = .adjustProteinLevel(temp_ptm_model, temp_protein_model)
  temp_adjusted_model$adj.pvalue = p.adjust(temp_adjusted_model$pvalue,
                                             method = 'BH')
  temp_adjusted_model
}

#' Calculate parameter changes
#' @noRd
.adjustProteinLevel = function(diff_site, diff_protein) {
  diff_ref = diff_protein[, c("Protein", "Label", "log2FC", "SE", "DF")]
  names(diff_ref)[names(diff_ref) == "log2FC"] = "log2FC_ref"
  names(diff_ref)[names(diff_ref) == "SE"] = "SE_ref"
  names(diff_ref)[names(diff_ref) == "DF"] = "DF_ref"
  joined = merge(diff_site, diff_ref, by = c("Protein", "Label"), all.x=TRUE)
  
  missing_ctrl = joined[joined$log2FC == Inf, ]  # PTM missing in control
  missing_case = joined[joined$log2FC == -Inf, ] # PTM missing in case
  res_mctrl = data.table(Protein = missing_ctrl$Protein,
                          Site = missing_ctrl$Site, Label = missing_ctrl$Label,
                          log2FC = Inf, SE = NA, Tvalue = NA, DF = NA, 
                          pvalue = NA)
  res_mcase = data.table(Protein = missing_case$Protein,
                          Site = missing_case$Site, Label = missing_case$Label,
                          log2FC = -Inf, SE = NA, Tvalue = NA, DF = NA, 
                          pvalue = NA)
  
  idx_full = abs(joined$log2FC) != Inf & abs(joined$log2FC_ref) != Inf
  full = joined[idx_full, ]
  log2fc = full$log2FC - full$log2FC_ref
  s2 = full$SE ^ 2
  s2_ref = full$SE_ref ^ 2
  stderr = sqrt(s2 + s2_ref)
  numer = (s2 + s2_ref) ^ 2
  denom = (s2 ^ 2 / full$DF + s2_ref ^ 2 / full$DF_ref)
  df = numer / denom
  tval = log2fc / stderr
  pval = 2 * stats::pt(abs(tval), df, lower.tail = FALSE)

  res_full = data.table(Protein = full$Protein,
                         Site = full$Site,
                         Label = full$Label,
                         log2FC = log2fc,
                         SE = stderr,
                         Tvalue = tval,
                         DF = df,
                         pvalue = pval)
  
  rbindlist(list(res_full, res_mctrl, res_mcase))
}

#' make contrast matrix for pairwise comparisons
#' @noRd
.makeContrast = function(groups) {
  
  ncomp = length(groups) * (length(groups) - 1) / 2 # Number of comparison
  contrast_matrix = matrix(rep(0, length(groups) * ncomp), 
                            ncol = length(groups))
  colnames(contrast_matrix) = groups
  
  count = 0
  contrast_matrix_rownames = NULL
  for(j in seq_len(length(groups)-1)){
    for(k in (j+1):length(groups)){
      
      count = count + 1
      # save row name
      contrast_matrix_rownames = c(contrast_matrix_rownames, 
                                    paste(groups[j], groups[k], sep = "-"))
      # set constrast value
      contrast_matrix[count, groups[j]] = 1
      contrast_matrix[count, groups[k]] = -1
    }
  }
  rownames(contrast_matrix) = contrast_matrix_rownames
  
  return(contrast_matrix)
}

#' Perform differential analysis on MS-based proteomics experiments targeting PTMs
#'
#' Takes summarized PTM and protein data from `dataSummarizationPTM` or 
#' `dataSummarizationPTM_TMT` functions and performs differential analysis. 
#' Leverages unmodified protein data to perform adjustment and deconvolute the 
#' effect of the PTM and unmodified protein. If protein data is unavailable, 
#' PTM data can still be passed into the function, however adjustment can not be
#' performed. All model results are returned for completeness.
#' 
#' @export
#' @importFrom data.table data.table as.data.table `:=`
#' @importFrom MSstats groupComparison MSstatsContrastMatrix
#' @importFrom MSstatsTMT groupComparisonTMT
#' @importFrom MSstatsConvert MSstatsLogsSettings MSstatsSaveSessionInfo
#' @importFrom stats p.adjust xtabs
#' @importFrom stringr str_match
#' 
#' @param data list of summarized datasets. Output of MSstatsPTM summarization 
#' function \code{\link[MSstatsPTM]{dataSummarizationPTM}}  or 
#' \code{\link[MSstatsPTM]{dataSummarizationPTM_TMT}} depending on acquisition
#' type.
#' @param data.type Type of data. Must be one of `LF` or `TMT`. Will be deprecated 
#' in favor of ptm_label_type and protein_label_type.
#' @param contrast.matrix comparison between conditions of interests. Default 
#' models full pairwise comparison between all conditions
#' @param moderated For TMT experiments only. TRUE will moderate t statistic; 
#' FALSE (default) uses ordinary t statistic. Default is FALSE.
#' @param adj.method For TMT experiments only. Adjusted method for multiple 
#' comparison. "BH" is default. "BH" is used for all other experiment types
#' @param log_base For non-TMT experiments only. The base of the logarithm used 
#' in summarization.
#' @param save_fitted_models logical, if TRUE, fitted models will be added to 
#' the output.
#' @param use_log_file logical. If TRUE, information about data processing
#' will be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing will be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. 
#' If not provided, such a file will be created automatically.
#' If `append = TRUE`, has to be a valid path to a file.
#' @param base start of the file name.
#' @param ptm_label_type Indicator of labeling type for PTM dataset. Must be one
#' of `LF` or `TMT`
#' @param protein_label_type Indicator of labeling type for PROTEIN dataset. 
#' Must be one of `LF` or `TMT`
#' @return list of modeling results. Includes PTM, PROTEIN, and ADJUSTED
#'         data.tables with their corresponding model results.
#'         
#' @examples 
#' 
#' model.lf.msstatsptm = groupComparisonPTM(summary.data, 
#'                                      ptm_label_type="LF",
#'                                      protein_label_type="LF",
#'                                      verbose = FALSE)
groupComparisonPTM = function(data, 
                              data.type = NULL,
                              contrast.matrix = "pairwise",
                              moderated = FALSE, 
                              adj.method = "BH",
                              log_base = 2,
                              save_fitted_models=TRUE,
                              use_log_file = TRUE, 
                              append = FALSE,
                              verbose = TRUE, 
                              log_file_path = NULL,
                              base = "MSstatsPTM_log_",
                              ptm_label_type = "LF",
                              protein_label_type = "LF") {
  
  ## Start log  
  # if (is.null(log_file_path) & use_log_file == TRUE){
  #   time_now = Sys.time()
  #   path = paste0(base, gsub("[ :\\-]", "_", time_now), 
  #                 ".log")
  #   file.create(path)
  # } else {path = log_file_path}
  # 
  # if (data.type == 'TMT'){
  #   pkg = "MSstatsTMT"
  #   option_log = "MSstatsTMTLog"
  # } else {
  #   pkg = "MSstats"
  #   option_log = "MSstatsLog"
  # }
  # 
  # MSstatsLogsSettings(use_log_file, append,
  #                     verbose, log_file_path = path, 
  #                     pkg_name = pkg)
  
  # getOption(option_log)("INFO", "Starting parameter and data checks..")
  if (!is.null(data.type) && (data.type == "TMT" || data.type == "LF")) {
      ptm_label_type = data.type
      protein_label_type = data.type
  }
  
  Label = Site = NULL
  
  data.ptm = data[["PTM"]]
  data.protein = data[["PROTEIN"]]
  
  ## Check for missing variables in PTM
  ## Determine if PTM should be adjusted for protein level.
  if (!is.null(data.protein)){
    adj.protein = TRUE
  } else{
    adj.protein = FALSE
  }
  
  ## Create pairwise matrix for label free
  if (contrast.matrix[1] == "pairwise"){
    # getOption(option_log)("INFO", "Building pairwise matrix.")
    if ("GROUP" %in% colnames(data.ptm$ProteinLevelData)) {
      labels <- unique(data.ptm$ProteinLevelData$GROUP)
    } else if ("Condition" %in% colnames(data.ptm$ProteinLevelData)) {
      labels <- unique(data.ptm$ProteinLevelData$Condition)
    } else {
      stop("Error: Neither 'GROUP' nor 'Condition' column exists in data.ptm$ProteinLevelData.")
    }
    contrast.matrix = MSstatsContrastMatrix('pairwise', labels)
  }
  
  ## PTM Modeling
  message("Starting PTM modeling...")
  if (ptm_label_type == "TMT"){
    # getOption(option_log)("INFO", "Starting TMT PTM Model")
    ptm_model_full = groupComparisonTMT(data.ptm, 
                                        contrast.matrix = contrast.matrix,
                                        moderated = moderated, 
                                        adj.method = adj.method,
                                        save_fitted_models=save_fitted_models)#,
                                        # save_fitted_models = TRUE,
                                        # use_log_file = use_log_file, 
                                        # append = append, verbose = verbose, 
                                        # log_file_path = path)
    ptm_model = ptm_model_full$ComparisonResult
    ptm_model_site_sep = ptm_model_full$ComparisonResult
    ptm_model_details = ptm_model_full$FittedModel
  } else if (ptm_label_type == "LF") {
    # getOption(option_log)("INFO", "Starting non-TMT PTM Model")
    ptm_model_full = groupComparison(contrast.matrix,
                                      data.ptm, save_fitted_models, log_base)#, 
                                      # use_log_file, append, verbose, 
                                      # log_file_path = path)
    ptm_model = ptm_model_full$ComparisonResult
    ptm_model_site_sep = ptm_model_full$ComparisonResult
    ptm_model_details = ptm_model_full$FittedModel
  }
  
  models = list('PTM.Model'=ptm_model, 'Model.Details'=ptm_model_details)
  
  if (adj.protein) {
    
    ## Protein Modeling
    message("Starting Protein modeling...")
    if (protein_label_type == "TMT"){
      # getOption(option_log)("INFO", "Starting TMT Protein Model")
      protein_model_full = groupComparisonTMT(data.protein, 
                                          contrast.matrix = contrast.matrix,
                                          moderated = moderated, 
                                          adj.method = adj.method,
                                          save_fitted_models = save_fitted_models)
                                          # use_log_file = use_log_file,
                                          # append = append,
                                          # verbose = verbose, 
                                          # log_file_path = path)
      protein_model = protein_model_full$ComparisonResult
      protein_model_details = protein_model_full$FittedModel
    } else if (protein_label_type == "LF") {
      # getOption(option_log)("INFO", "Starting non-TMT Protein Model")
      protein_model_full = groupComparison(contrast.matrix, 
                                           data.protein, save_fitted_models, 
                                           log_base, use_log_file)#, 
                                            # append, verbose, 
                                            # log_file_path = path)
      protein_model = protein_model_full$ComparisonResult
      protein_model_details = protein_model_full$FittedModel
    }
    
    ptm_model = as.data.table(ptm_model)
    protein_model = as.data.table(protein_model)
    
    message("Starting adjustment...")
    # getOption(option_log)("INFO", "Starting Protein Adjustment")
    ptm_model_site_sep = copy(ptm_model)
    
    ## extract global protein name
    ptm_model_site_sep = .extractProtein(ptm_model_site_sep, protein_model)
    # getOption(option_log)("INFO", "Rcpp function extracted protein info")
    
    ## adjustProteinLevel function can only compare one label at a time
    comparisons = unique(ptm_model_site_sep[, Label])
    
    adjusted_model_list = list()
    for (i in seq_len(length(comparisons))) {
      # getOption(option_log)("INFO", paste0("Adjusting for Comparison - ", 
                                             # as.character(i)))
      temp_adjusted_model = .applyPtmAdjustment(comparisons[[i]],
                                                   ptm_model_site_sep,
                                                   protein_model)
      adjusted_model_list[[i]] = temp_adjusted_model
    }
    
    adjusted_models = rbindlist(adjusted_model_list)
    
    adjusted_models$GlobalProtein = adjusted_models$Protein
    adjusted_models$Protein = adjusted_models$Site
    adjusted_models[, Site := NULL]
    
    ## Add unadjustable ptms into final dataframe
    adjusted_models$temp_check = paste(adjusted_models$Protein, 
                                       adjusted_models$Label, sep = "_")
    ptm_model$temp_check = paste(ptm_model$Protein, 
                                 ptm_model$Label, sep = "_")
    missing_ptms = setdiff(ptm_model$temp_check, adjusted_models$temp_check)
    
    missing_rows = ptm_model[ptm_model$temp_check %in% missing_ptms]
    missing_rows$GlobalProtein = "missing"
    missing_rows$Adjusted = FALSE
    missing_rows[, c('issue', 'MissingPercentage', 
                     'ImputationPercentage', 'temp_check'):=NULL]
    
    adjusted_models$Adjusted = TRUE
    
    if (ptm_label_type == 'TMT'){
      missing_rows$Tvalue = missing_rows$log2FC / missing_rows$SE
    }
    
    ptm_model$temp_check = NULL
    adjusted_models$temp_check = NULL
    
    adjusted_models = rbindlist(list(adjusted_models, missing_rows), 
                                use.names=TRUE)
    adjusted_models = adjusted_models[!is.na(adjusted_models$Protein)]
    
    # getOption(option_log)("INFO", "Adjustment complete, returning models.")
    models = list('PTM.Model'=ptm_model, 
                  'PROTEIN.Model'=protein_model,
                  'ADJUSTED.Model'=adjusted_models, 
                  'Model.Details'=list('PTM'=ptm_model_details,
                                       'PROTEIN'=protein_model_details))
  }
  
  if (!is.null(data.type) && (data.type == "TMT" || data.type == "LF")) {
      warning("DEPRECATION NOTICE: The `data.type` argument is being deprecated. 
              Please use `ptm_label_type` and `protein_label_type` instead ahead
              of Release 3.22")
  }

  return(models)

}

data("summary.data", package = "MSstatsPTM")
data("summary.data.tmt", package = "MSstatsPTM")

## Test missing data is handled
expect_error(groupComparisonPTM())
expect_error(groupComparisonPTM(list(PTM = NULL, 
                                     PROTEIN = summary.data$PROTEIN), 
                                ptm_label_type="LF",
                                protein_label_type="LF"))

expect_silent(groupComparisonPTM(list(PTM = summary.data$PTM, 
                                     PROTEIN = NULL), 
                                 ptm_label_type="LF",
                                 protein_label_type="LF"))

## Test expected output - LF
group.comp <- MSstatsPTM::groupComparisonPTM(summary.data, 
                                             ptm_label_type="LF",
                                             protein_label_type="LF")
expect_inherits(group.comp, "list")
expect_inherits(group.comp$PTM.Model, "data.table")
expect_inherits(group.comp$PROTEIN.Model, "data.table")
expect_inherits(group.comp$ADJUSTED.Model, "data.table")

expect_equal(colnames(group.comp$PTM.Model), 
             c("Protein", "Label", "log2FC", "SE", "Tvalue", "DF", "pvalue",
               "adj.pvalue", "issue", "MissingPercentage", 
               "ImputationPercentage"))
expect_equal(colnames(group.comp$PROTEIN.Model), 
             c("Protein", "Label", "log2FC", "SE", "Tvalue", "DF", "pvalue",
               "adj.pvalue", "issue", "MissingPercentage", 
               "ImputationPercentage"))
expect_equal(colnames(group.comp$ADJUSTED.Model), 
             c("Protein", "Label", "log2FC", "SE", "Tvalue", "DF", "pvalue", 
               "adj.pvalue", "GlobalProtein", "Adjusted"))

## Test expected output - TMT
group.comp.tmt <- MSstatsPTM::groupComparisonPTM(summary.data.tmt, 
                                                 ptm_label_type="TMT",
                                                 protein_label_type="TMT")
expect_inherits(group.comp.tmt, "list")
expect_inherits(group.comp.tmt$PTM.Model, "data.table")
expect_inherits(group.comp.tmt$PROTEIN.Model, "data.table")
expect_inherits(group.comp.tmt$ADJUSTED.Model, "data.table")

expect_equal(colnames(group.comp.tmt$PTM.Model), 
             c("Protein", "Label", "log2FC", "SE", "DF", "pvalue", 
               "adj.pvalue", "issue"))
expect_equal(colnames(group.comp.tmt$PROTEIN.Model), 
             c("Protein", "Label", "log2FC", "SE", "DF", "pvalue", 
               "adj.pvalue", "issue"))
expect_equal(colnames(group.comp$ADJUSTED.Model), 
             c("Protein", "Label", "log2FC", "SE", "Tvalue", "DF", "pvalue", 
               "adj.pvalue", "GlobalProtein", "Adjusted"))

#' Example annotation file for a label-free MaxQuant experiment.
#' 
#' Must be manually created by the user and input into the 
#' MaxQtoMSstatsPTMFormat converter. Requires the correct columns and maps the 
#' experimental desing into the MSstats format. Specify unique bioreplicates for
#' group comparison designs, and the same bioreplicate for repeated measure 
#' designs. The columns and descriptions are below.
#' 
#' \itemize{
#'   \item Run : Run name that matches exactly with MaxQuant run. Used to join 
#'   evidence and metadata in annotation file.
#'   \item Condition : Name of condition that was used for each run.
#'   \item BioReplicate : Name of biological replicate. Repeating the same name 
#'   here will tell MSstatsPTM that the experiment is a repeated measure design.
#'   \item Raw.file : Run name that matches exactly with MaxQuant run. Used to join 
#'   evidence and metadata in annotation file.
#'   \item IsotopeLabelType: Name of isotope label. May be all `L` or unique 
#'   depending on experimental design.
#' }
#' 
#' @format A data.table with 5 columns.
#' @examples
#' head(maxq_lf_annotation)
"maxq_lf_annotation"

#' Example MaxQuant evidence file from the output of a label free experiment
#' 
#' Experiment was performed by the Olsen lab and published on Nat. Commun.
#' (citation below).
#' 
#' Bekker-Jensen, D.B., Bernhardt, O.M., Hogrebe, A. et al. Rapid and 
#' site-specific deep phosphoproteome profiling by data-independent acquisition 
#' without the need for spectral libraries. Nat Commun 11, 787 (2020). 
#' https://doi.org/10.1038/s41467-020-14609-1
#' 
#' The experiment was processed using MaxQuant by the computational proteomics 
#' team at Pfizer (Liang Xue and Pierre Jean). 
#' 
#' The experiment did not contain a global profiling run, but we show an example 
#' of extracting the unmodified peptides and using them in place of the 
#' profiling run.
#' 
#' @format a data.table with 63 columns and 511 rows, the output of MaxQuant
#' @examples
#' head(maxq_lf_evidence)
"maxq_lf_evidence"

#' Example annotation file for a TMT MaxQuant experiment.
#' 
#' Must be manually created by the user and input into the 
#' MaxQtoMSstatsPTMFormat converter. Requires the correct columns and maps the 
#' experimental desing into the MSstats format. Specify unique bioreplicates for
#' group comparison designs, and the same bioreplicate for repeated measure 
#' designs. The columns and descriptions are below.
#' 
#' \itemize{
#'   \item Run : Run name that matches exactly with MaxQuant run. Used to join 
#'   evidence and metadata in annotation file.
#'   \item Fraction : If multiple fractions were used (i.e. the same mixture 
#'   split into multiple fractions) enter that here.
#'   TechRepMixture : Multiple runs using the same bioreplicate
#'   \item Channel : Mixture channel used
#'   \item Condition : Name of condition that was used for each run.
#'   \item Mixture : The unique mixture (plex) name
#'   \item BioReplicate : Name of biological replicate. Repeating the same name 
#'   here will tell MSstatsPTM that the experiment is a repeated measure design.
#' }
#' 
#' @format A data.table with 7 columns.
#' @examples
#' head(maxq_tmt_annotation)
"maxq_tmt_annotation"

#' Example MaxQuant evidence file from the output of a TMT experiment
#' 
#' Experiment was performed by the Olsen lab and published on Nat. Commun.
#' (citation below).
#' 
#' Hogrebe, A., von Stechow, L., Bekker-Jensen, D.B. et al. Benchmarking common 
#' quantification strategies for large-scale phosphoproteomics. Nat Commun 9, 
#' 1045 (2018). https://doi.org/10.1038/s41467-018-03309-6
#' 
#' The experiment was processed using MaxQuant by the computational proteomics 
#' team at Pfizer (Liang Xue and Pierre Jean). 
#' 
#' The experiment did not contain a global profiling run, but we show an example 
#' of extracting the unmodified peptides and using them in place of the 
#' profiling run.
#' 
#' @format a data.table with 96 columns and 199 rows, the output of MaxQuant
#' @examples
#' head(maxq_tmt_evidence)
"maxq_tmt_evidence"

#' Example annotation file for a label-free Proteome Discoverer experiment.
#' 
#' Must be manually created by the user and input into the 
#' PDtoMSstatsPTMFormat converter. Requires the correct columns and maps the 
#' experimental desing into the MSstats format. Specify unique bioreplicates for
#' group comparison designs, and the same bioreplicate for repeated measure 
#' designs. The columns and descriptions are below.
#' 
#' \itemize{
#'   \item Run : Run name that matches exactly with PD run. Used to join 
#'   evidence and metadata in annotation file.
#'   \item Condition : Name of condition that was used for each run.
#'   \item BioReplicate : Name of biological replicate. Repeating the same name 
#'   here will tell MSstatsPTM that the experiment is a repeated measure design.
#' }
#' 
#' @format A data.table with 3 columns.
#' @examples
#' head(pd_annotation)
"pd_annotation"

#' Example Proteome Discoverer evidence file from the output of a label free experiment
#' 
#' Experiment was performed by the Olsen lab and published on Nat. Commun.
#' (citation below).
#' 
#' Bekker-Jensen, D.B., Bernhardt, O.M., Hogrebe, A. et al. Rapid and 
#' site-specific deep phosphoproteome profiling by data-independent acquisition 
#' without the need for spectral libraries. Nat Commun 11, 787 (2020). 
#' https://doi.org/10.1038/s41467-020-14609-1
#' 
#' The experiment was processed using Proteome Discoverer by the computational 
#' proteomics team at Pfizer (Liang Xue and Pierre Jean). 
#' 
#' The experiment did not contain a global profiling run, but we show an example 
#' of extracting the unmodified peptides and using them in place of the 
#' profiling run.
#' 
#' @format a data.table with 60 columns and 1657 rows, the output of PD
#' @examples
#' head(pd_psm_input)
"pd_psm_input"

#' Example output of Proteome Discoverer converter
#' 
#' output using example data provided in package
#' 
#' The experiment did not contain a global profiling run, but we show an example 
#' of extracting the unmodified peptides and using them in place of the 
#' profiling run.
#' 
#' @format a list with 2 data.frames
#' @examples
#' head(pd_testing_output)
"pd_testing_output"

#' Example annotation file for a label-free Spectronaut experiment.
#' 
#' Must be manually created by the user and input into the 
#' SpectronauttoMSstatsPTMFormat converter. Requires the correct columns and 
#' maps the experimental desing into the MSstats format. Specify unique 
#' bioreplicates for group comparison designs, and the same bioreplicate for 
#' repeated measure designs. The columns and descriptions are below.
#' 
#' \itemize{
#'   \item Run : Run name that matches exactly with Spectronaut run. Used to join 
#'   evidence and metadata in annotation file.
#'   \item Condition : Name of condition that was used for each run.
#'   \item BioReplicate : Name of biological replicate. Repeating the same name 
#'   here will tell MSstatsPTM that the experiment is a repeated measure design.
#'   \item Raw.file : Run name that matches exactly with Spectronaut run. Used to join 
#'   evidence and metadata in annotation file.
#' }
#' 
#' @format A data.table with 5 columns.
#' @examples
#' head(spectronaut_annotation)
"spectronaut_annotation"

#' Example Spectronaut evidence file from the output of a label free experiment
#' 
#' Experiment was performed by the Olsen lab and published on Nat. Commun.
#' (citation below).
#' 
#' Bekker-Jensen, D.B., Bernhardt, O.M., Hogrebe, A. et al. Rapid and 
#' site-specific deep phosphoproteome profiling by data-independent acquisition 
#' without the need for spectral libraries. Nat Commun 11, 787 (2020). 
#' https://doi.org/10.1038/s41467-020-14609-1
#' 
#' The experiment was processed using Spectronaut by the computational proteomics 
#' team at Pfizer (Liang Xue and Pierre Jean). 
#' 
#' The experiment did not contain a global profiling run, but we show an example 
#' of extracting the unmodified peptides and using them in place of the 
#' profiling run.
#' 
#' @format a data.table with 23 columns and 2683 rows, the output of Spectronaut
#' @examples
#' head(spectronaut_input)
"spectronaut_input"

#' Example annotation file for a TMT FragPipe experiment.
#' 
#' Automatically created by FragPipe, manually checked by the user and input 
#' into the FragPipetoMSstatsPTMFormat converter. Requires the correct columns 
#' and maps the experimental desing into the MSstats format. Specify unique 
#' bioreplicates for group comparison designs, and the same bioreplicate for 
#' repeated measure designs. The columns and descriptions are below.
#' 
#' \itemize{
#'   \item Run : Run name that matches exactly with FragPipe run. Used to join 
#'   evidence and metadata in annotation file.
#'   \item Fraction : If multiple fractions were used (i.e. the same mixture 
#'   split into multiple fractions) enter that here.
#'   TechRepMixture : Multiple runs using the same bioreplicate
#'   \item Channel : Mixture channel used
#'   \item Condition : Name of condition that was used for each run.
#'   \item Mixture : The unique mixture (plex) name
#'   \item BioReplicate : Name of biological replicate. Repeating the same name 
#'   here will tell MSstatsPTM that the experiment is a repeated measure design.
#' }
#' 
#' @format A data.table with 7 columns.
#' @examples
#' head(fragpipe_annotation)
"fragpipe_annotation"

#' Example annotation file for a global profiling run TMT FragPipe experiment.
#' 
#' Automatically created by FragPipe, manually checked by the user and input 
#' into the FragPipetoMSstatsPTMFormat converter. Requires the correct columns 
#' and maps the experimental desing into the MSstats format. Specify unique 
#' bioreplicates for group comparison designs, and the same bioreplicate for 
#' repeated measure designs. The columns and descriptions are below.
#' 
#' \itemize{
#'   \item Run : Run name that matches exactly with FragPipe run. Used to join 
#'   evidence and metadata in annotation file.
#'   \item Fraction : If multiple fractions were used (i.e. the same mixture 
#'   split into multiple fractions) enter that here.
#'   TechRepMixture : Multiple runs using the same bioreplicate
#'   \item Channel : Mixture channel used
#'   \item Condition : Name of condition that was used for each run.
#'   \item Mixture : The unique mixture (plex) name
#'   \item BioReplicate : Name of biological replicate. Repeating the same name 
#'   here will tell MSstatsPTM that the experiment is a repeated measure design.
#' }
#' 
#' @format A data.table with 7 columns.
#' @examples
#' head(fragpipe_annotation_protein)
"fragpipe_annotation_protein"

#' Output of FragPipe TMT PTM experiment
#' 
#' This dataset was provided by the FragPipe team at the Nesvilab. It was 
#' processed using Philosopher and targeted Phosphorylation.
#' 
#' @format A data.table with 29 columns and 246 rows.
#' @examples
#' head(fragpipe_input)
"fragpipe_input"

#' Output of FragPipe TMT global profiling experiment
#' 
#' This dataset was provided by the FragPipe team at the Nesvilab. It was 
#' processed using Philosopher and targeted Phosphorylation.
#' 
#' @format A data.table with 27 columns and 47 rows.
#' @examples
#' head(fragpipe_input_protein)
"fragpipe_input_protein"

#' Example of input PTM dataset for TMT experiments.
#' 
#' It can be the output of MSstatsPTM converter MaxQtoMSstatsPTMFormat or other 
#' MSstatsTMT converter functions (Please see MSstatsPTM_TMT_Workflow vignette).
#' The dataset is formatted as a list with two data.tables named PTM and 
#' PROTEIN. In each data.table the variables are as follows:
#' 
#' \itemize{
#'   \item ProteinName : Name of protein with modification site mapped in with
#'    an underscore. ie "Protein_4_Y474"
#'   \item PeptideSequence
#'   \item Charge
#'   \item PSM
#'   \item Mixture : Mixture of samples labeled with different TMT reagents,
#'    which can be analyzed in
#'   a single mass spectrometry experiment. If the channal doesn't have sample,
#'    please add `Empty' under Condition.
#'   \item TechRepMixture : Technical replicate of one mixture. One mixture may
#'    have multiple technical replicates.
#'   For example, if `TechRepMixture' = 1, 2 are the two technical replicates of
#'    one mixture, then they should match
#'   with same `Mixture' value.
#'   \item Run : MS run ID.
#'   \item Channel : Labeling information (126, ... 131).
#'   \item Condition : Condition (ex. Healthy, Cancer, Time0)
#'   \item BioReplicate : Unique ID for biological subject. If the channal
#'   doesn't have sample, please add `Empty' under BioReplicate.
#'   \item Intensity
#' }
#'
#' @format A list of two data.tables named PTM and PROTEIN with 1716 and 29221 
#' rows respectively.
#' @examples
#' head(raw.input.tmt$PTM)
#' head(raw.input.tmt$PROTEIN)
#'
"raw.input.tmt"

#' Example of input PTM dataset for LabelFree/DDA/DIA experiments.
#' 
#' It can be the output of MSstatsPTM converter ProgenesistoMSstatsPTMFormat or 
#' other MSstats converter functions (Please see MSstatsPTM_LabelFree_Workflow 
#' vignette). The dataset is formatted as a list with two data.tables named PTM 
#' and PROTEIN. In each data.table the variables are as follows:
#' 
#' \itemize{
#'   #'   \item ProteinName : Name of protein with modification site mapped in with
#'    an underscore. ie "Protein_4_Y474"
#'   \item PeptideSequence
#'   \item Condition : Condition (ex. Healthy, Cancer, Time0)
#'   \item BioReplicate : Unique ID for biological subject.
#'   \item Run : MS run ID.
#'   \item Intensity
#'   \item PrecursorCharge 
#'   \item FragmentIon
#'   \item ProductCharge
#'   \item IsotopeLabelType
#' }
#'
#' @format A list of two data.tables named PTM and PROTEIN with 1745 and 478 
#' rows respectively.
#' @examples
#' head(raw.input$PTM)
#' head(raw.input$PROTEIN)
#'
"raw.input"

#' Example of output from dataSummarizationPTM function for non-TMT data
#'
#' It is made from \code{\link{raw.input}}.
#' It is the output of dataSummarizationPTM function from MSstatsPTM.
#' It should include a list with two names \code{PTM} and \code{PROTEIN}. Each 
#' of these list values is also a list with two names \code{ProteinLevelData} 
#' and \code{FeatureLevelData}, which correspond to two data.tables.The columns 
#' in these two data.tables are listed below. The variables are as follows:
#' \itemize{
#'   \item FeatureLevelData : \itemize{
#'     \item PROTEIN : Protein ID with modification site mapped in. Ex.
#'     Protein_1002_S836
#'     \item PEPTIDE : Full peptide with charge
#'     \item TRANSITION: Charge
#'     \item FEATURE : Combination of Protien, Peptide, and Transition Columns
#'     \item LABEL : 
#'     \item GROUP : Condition (ex. Healthy, Cancer, Time0)
#'     \item RUN : Unique ID for technical replicate of one TMT
#'     mixture.
#'     \item SUBJECT : Unique ID for biological subject.
#'     \item FRACTION : Unique Fraction ID
#'     \item originalRUN : Run name
#'     \item censored : 
#'     \item INTENSITY : Unique ID for TMT mixture.
#'     \item ABUNDANCE : Unique ID for TMT mixture.
#'     \item newABUNDANCE : Unique ID for TMT mixture.
#'     \item predicted : Unique ID for TMT mixture.
#'   }
#'   \item ProteinLevelData : \itemize{
#'     \item RUN : MS run ID
#'     \item Protein : Protein ID with modification site mapped in. Ex.
#'     Protein_1002_S836
#'     \item LogIntensities: Protein-level summarized abundance
#'     \item originalRUN : Labeling information (126, ... 131)
#'     \item GROUP : Condition (ex. Healthy, Cancer, Time0)
#'     \item SUBJECT : Unique ID for biological subject.
#'     \item TotalGroupMeasurements : Unique ID for technical replicate of one TMT
#'     mixture.
#'     \item NumMeasuredFeature : Unique ID for TMT mixture.
#'     \item MissingPercentage : Unique ID for TMT mixture.
#'     \item more50missing : Unique ID for TMT mixture.
#'     \item NumImputedFeature : Unique ID for TMT mixture.
#'   }
#'    
#'   }
#'
#' @format A list of two lists with four data.tables.
#' @examples
#' head(summary.data)
#'
"summary.data"

#' Example of output from dataSummarizationPTM_TMT function for TMT data
#'
#' It is made from \code{\link{raw.input.tmt}}.
#' It is the output of dataSummarizationPTM_TMT function from MSstatsPTM.
#' It should include a list with two names \code{PTM} and \code{PROTEIN}. Each 
#' of these list values is also a list with two names \code{ProteinLevelData} 
#' and \code{FeatureLevelData}, which correspond to two data.tables.The columns 
#' in these two data.tables are listed below. The variables are as follows:
#' \itemize{
#'   \item FeatureLevelData : \itemize{
#'     \item ProteinName : MS run ID
#'     \item PSM : Protein ID with modification site mapped in. Ex.
#'     Protein_1002_S836
#'     \item censored: Protein-level summarized abundance
#'     \item predicted : Labeling information (126, ... 131)
#'     \item log2Intensity : Condition (ex. Healthy, Cancer, Time0)
#'     \item Run : Unique ID for biological subject.
#'     \item Channel : Unique ID for technical replicate of one TMT
#'     mixture.
#'     \item BioReplicate : Unique ID for TMT mixture.
#'     \item Condition : Unique ID for TMT mixture.
#'     \item Mixture : Unique ID for TMT mixture.
#'     \item TechRepMixture : Unique ID for TMT mixture.
#'     \item PeptideSequence : Unique ID for TMT mixture.
#'     \item Charge : Unique ID for TMT mixture.
#'   }
#'   \item ProteinLevelData : \itemize{
#'     \item Mixture : MS run ID
#'     \item TechRepMixture : Protein ID with modification site mapped in. Ex.
#'     Protein_1002_S836
#'     \item Run: Protein-level summarized abundance
#'     \item Channel : Labeling information (126, ... 131)
#'     \item Protein : Condition (ex. Healthy, Cancer, Time0)
#'     \item Abundance : Unique ID for biological subject.
#'     \item BioReplicate : Unique ID for technical replicate of one TMT
#'     mixture.
#'     \item Condition : Unique ID for TMT mixture.
#'   }
#'    
#'   }
#'
#' @format A list of two lists with four data.tables.
#' @examples
#' head(summary.data.tmt)
#'
"summary.data.tmt"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

extract_protein_name <- function(ptm_list, protein_list) {
    .Call(`_MSstatsPTM_extract_protein_name`, ptm_list, protein_list)
}

#' Convert the output of DIA-NN PSM file into MSstatsPTM format
#' 
#' Takes as input the `report.tsv` file from DIA-NN and converts it into 
#' MSstatsPTM format. Requires PSM and an annotation file. Optionally an 
#' additional `report.tsv` file for a corresponding global profiling run can 
#' be included.
#' 
#' @importFrom data.table as.data.table
#' @importFrom MSstatsConvert DIANNtoMSstatsFormat
#' 
#' @param input data.frame of `report.tsv` file produced by Philosopher
#' @param annotation annotation with Run, Fraction, TechRepMixture, Mixture, Channel, 
#' BioReplicate, Condition columns or a path to file. Refer to the example 'annotation' for the meaning of each column.
#' @param input_protein same as `input` for global profiling run. Default is NULL.
#' @param annotation_protein same as `annotation` for global profiling run. Default is NULL.
#' @param fasta_path A string of path to a FASTA file, used to match PTM peptides.
#' @param use_unmod_peptides Boolean if the unmodified peptides in the input 
#' file should be used to construct the unmodified protein output. Only used if
#' `input_protein` is not provided. Default is `FALSE`.
#' @param protein_id_col Use 'Protein.Groups'(default) column for protein name. 
#' @param fasta_protein_name Name of column that matches with the protein names 
#' in `protein_id_col`. The protein names in these two columns must match in 
#' order to join the FASTA file with the DIA-NN output. Default is "uniprot_ac"
#' for uniprot ID. For uniprot mnemonic ID, use "entry_name"
#' @param global_qvalue_cutoff The global qvalue cutoff. Default is 0.01.
#' @param qvalue_cutoff local qvalue cutoff for library. Default is 0.01.
#' @param pg_qvalue_cutoff local qvalue cutoff for protein groups Run should be 
#' the same as filename. Default is 0.01.
#' @param useUniquePeptide logical, if TRUE (default) removes peptides that are assigned for more than one proteins. 
#' We assume to use unique peptide for each protein.
#' @param removeFewMeasurements TRUE (default) will remove the features that have 1 or 2 measurements within each Run.
#' @param removeOxidationMpeptides TRUE (default) will remove the peptides including oxidation (M) sequence.
#' @param removeProtein_with1Feature TRUE will remove the proteins which have only 1 peptide and charge. Defaut is FALSE.
#' @param MBR If analaysis was done with match between runs or not. Default is TRUE.
#' @param use_log_file logical. If TRUE, information about data processing will 
#' be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing wil be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. If not provided, such a file will be created 
#' automatically. If 'append = TRUE', has to be a valid path to a file.
#' 
#' @return `list` of one or two `data.frame` of class `MSstatsTMT`, named `PTM` and `PROTEIN`
#' 
#' @export
#' 
#' @examples
#' # Example from PRIDE ID PXD053502
#' input = system.file("tinytest/raw_data/DIANN/report.tsv", 
#'                                         package = "MSstatsPTM")
#' input = data.table::fread(input)
#' annot = system.file("tinytest/raw_data/DIANN/annot.csv", 
#'                                         package = "MSstatsPTM")
#' annot = data.table::fread(annot)
#' fasta_path = system.file("extdata", "diann.fasta", 
#'                        package="MSstatsPTM")
#' 
#' msstatsptm_format = DIANNtoMSstatsPTMFormat(
#'     input, 
#'     annot, 
#'     protein_id_col = "Protein.Names", 
#'     fasta_path = fasta_path, 
#'     fasta_protein_name = "entry_name", 
#'     use_log_file = FALSE
#' )
#' 
#' head(msstatsptm_format$PTM)
#' 
DIANNtoMSstatsPTMFormat = function(input,
                                   annotation,
                                   input_protein=NULL,
                                   annotation_protein=NULL,
                                   fasta_path=NULL,
                                   use_unmod_peptides=FALSE,
                                   protein_id_col = "Protein.Group",
                                   fasta_protein_name="uniprot_ac",
                                   global_qvalue_cutoff = 0.01,
                                   qvalue_cutoff = 0.01,
                                   pg_qvalue_cutoff = 0.01,
                                   useUniquePeptide = TRUE,
                                   removeFewMeasurements = TRUE,
                                   removeOxidationMpeptides = TRUE,
                                   removeProtein_with1Feature = FALSE,
                                   MBR=TRUE,
                                   use_log_file = TRUE,
                                   append = FALSE,
                                   verbose = TRUE,
                                   log_file_path = NULL){
  
  MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                      log_file_path, 
                                      base = "MSstatsPTM_converter_log_")
  
  ## Check input parameters
  checkmate::assertTRUE(!is.null(input) & !is.null(annotation))
  .checkAnnotation(annotation, "LF")
  if (!is.null(annotation_protein)){
    .checkAnnotation(annotation_protein, "LF")
  }
  
  input = as.data.table(input)
  fasta = MSstatsPTM::tidyFasta(fasta_path)
  
  input = MSstatsPTMSiteLocator(input, 
                                protein_name_col=protein_id_col, 
                                unmod_pep_col="Stripped.Sequence",
                                mod_pep_col="Modified.Sequence",
                                clean_mod=TRUE,
                                fasta_file=fasta,
                                fasta_protein_name=fasta_protein_name,
                                mod_id="UniMod",
                                replace_text=TRUE)
  
  if (use_unmod_peptides){
    input_protein = input[input[,..protein_id_col][[1]]  == input$ProteinNameUnmod]  
    annotation_protein = annotation
  } else {
    input = input[input[,..protein_id_col][[1]] != input$ProteinNameUnmod]
  }
  
  ptm_input = DIANNtoMSstatsFormat(input, 
                                   annotation,
                                   global_qvalue_cutoff,
                                   qvalue_cutoff,
                                   pg_qvalue_cutoff,
                                   useUniquePeptide,
                                   removeFewMeasurements,
                                   removeOxidationMpeptides,
                                   removeProtein_with1Feature,
                                   use_log_file,
                                   append,
                                   verbose,
                                   log_file_path,
                                   MBR)
  
  msstats_format = list(PTM=ptm_input, PROTEIN=NULL)
  
  if (!is.null(input_protein)){
    checkmate::assertTRUE(!is.null(input_protein) & 
                            !is.null(annotation_protein))
    
    protein_input = DIANNtoMSstatsFormat(input_protein, 
                                         annotation_protein,
                                         global_qvalue_cutoff,
                                         qvalue_cutoff,
                                         pg_qvalue_cutoff,
                                         useUniquePeptide,
                                         removeFewMeasurements,
                                         removeOxidationMpeptides,
                                         removeProtein_with1Feature,
                                         use_log_file,
                                         append,
                                         verbose,
                                         log_file_path,
                                         MBR)
    
    msstats_format = list(PTM=ptm_input, PROTEIN=protein_input)
    
  }
  return(msstats_format)
}



#' Convert output of TMT labeled Fragpipe data into MSstatsPTM format.
#' 
#' Takes as input TMT experiments which are the output of Fragpipe and converts
#' into MSstatsPTM format. Requires `msstats.csv` file and an annotation file. 
#' Optionally an additional `msstats.csv` file can be uploaded if a 
#' corresponding global profiling run was performed. Site localization is 
#' performed and only high probability localizations are kept.
#' 
#' @export
#' @importFrom data.table as.data.table 
#' @importFrom MSstatsConvert FragPipetoMSstatsFormat
#' @importFrom MSstatsTMT PhilosophertoMSstatsTMTFormat
#' 
#' @param input data.frame of `msstats.csv` file produced by Philosopher
#' @param annotation annotation with Run, Fraction, TechRepMixture, Mixture, Channel, 
#' BioReplicate, Condition columns or a path to file. Refer to the example 'annotation' for the meaning of each column. Channel column should be 
#' consistent with the channel columns (Ignore the prefix "Channel ") in msstats.csv file. Run column should be consistent with the Spectrum.File columns in msstats.csv file.
#' @param input_protein same as `input` for global profiling run. Default is NULL.
#' @param annotation_protein same as `annotation` for global profiling run. Default is NULL.
#' @param use_unmod_peptides Boolean if the unmodified peptides in the input 
#' file should be used to construct the unmodified protein output. Only used if
#' `input_protein` is not provided. Default is `FALSE`.
#' @param label_type Type of labeling used for experiment. Must be one of "LF" 
#' or "TMT". Default is "TMT".
#' @param protein_id_col Use 'Protein'(default) column for TMT. This needs to be
#' changed to "ProteinName" for label free. For TMT, 'Master.Protein.Accessions'
#' can be used instead to get the protein ID with single protein.
#' @param peptide_id_col Use 'Peptide.Sequence'(default) column for TMT. Must be
#' changed to "PeptideSequence" for label free. "Modified.Peptide.Sequence" can 
#' be used instead to get the modified peptide sequence.
#' @param mod_id_col Column containing the modified Amino Acids. For example, a Phosphorylation experiment may pass `STY`. The corresponding column with `STY` combined with the mass (e.x. `STY.79.9663`) will be selected. Default is `STY`.
#' @param localization_cutoff Minimum localization score required to keep modification. Default is .75.
#' @param remove_unlocalized_peptides Boolean indicating if peptides without all sites localized should be kept. Default is TRUE (non-localized sites will be removed).
#' @param Purity_cutoff Cutoff for purity. Default is 0.6. Purity refers to how much of the detected ion signal 
#' within a specific inclusion window belongs to the target molecule or its closely related forms, 
#' compared to any other unwanted signals or noise. Higher values indicate greater purity.
#' @param PeptideProphet_prob_cutoff Cutoff for the peptide identification probability. Default is 0.7. 
#' The probability is confidence score determined by PeptideProphet and higher values indicate greater confidence.
#' @param useUniquePeptide logical, if TRUE (default) removes peptides that are assigned for more than one proteins. 
#' We assume to use unique peptide for each protein.
#' @param rmPSM_withfewMea_withinRun TRUE will remove the features that have 1 or 2 measurements within each Run. Default is FALSE.
#' @param rmPeptide_OxidationM TRUE (default) will remove the peptides including oxidation (M) sequence.
#' @param rmProtein_with1Feature TRUE will remove the proteins which have only 1 peptide and charge. Default is FALSE.
#' @param summaryforMultipleRows sum (default) or max - when there are multiple measurements for certain feature in certain run, 
#' select the feature with the largest summation or maximal value.
#' @param use_log_file logical. If TRUE, information about data processing will 
#' be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing wil be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. If not provided, such a file will be created 
#' automatically. If 'append = TRUE', has to be a valid path to a file.
#' 
#' @return `list` of one or two `data.frame` of class `MSstatsTMT`, named `PTM` and `PROTEIN`
#' 
#' @export
#' 
#' @examples 
#' # TMT Example (with global profiling run)
#' head(fragpipe_input)
#' head(fragpipe_annotation)
#' head(fragpipe_input_protein)
#' head(fragpipe_annotation_protein)
#' 
#' msstats_data = FragPipetoMSstatsPTMFormat(fragpipe_input,
#'                                           fragpipe_annotation,
#'                                           fragpipe_input_protein, 
#'                                           fragpipe_annotation_protein,
#'                                           label_type="TMT",
#'                                           mod_id_col = "STY",
#'                                           localization_cutoff=.75,
#'                                           remove_unlocalized_peptides=TRUE)
#' head(msstats_data$PTM)
#' head(msstats_data$PROTEIN)
#' 
#' # LFQ Example
#' input = system.file("tinytest/raw_data/Fragpipe/MSstats.csv", 
#'                                         package = "MSstatsPTM")
#' input = data.table::fread(input)
#' annot = system.file("tinytest/raw_data/Fragpipe/experiment_annotation.tsv", 
#'                                         package = "MSstatsPTM")
#' annot = data.table::fread(annot)       
#' input_protein = system.file("tinytest/raw_data/Fragpipe/msstats_proteome_lf.csv",
#'                                         package = "MSstatsPTM")                                  
#' input_protein = data.table::fread(input_protein)
#' 
#' msstats_data = FragPipetoMSstatsPTMFormat(input,
#'                                           annot,
#'                                           input_protein = input_protein,
#'                                           label_type="LF",
#'                                           mod_id_col = "STY",
#'                                           localization_cutoff=.75,
#'                                           protein_id_col = "ProteinName",
#'                                           peptide_id_col = "PeptideSequence")
#'                                           
#' # If no global profiling run is available, omit input_protein and set:
#' # msstats_data = FragPipetoMSstatsPTMFormat(input, annot,
#' #   label_type = "LF", mod_id_col = "STY",
#' #   localization_cutoff = .75, protein_id_col = "ProteinName",
#' #   peptide_id_col = "PeptideSequence", use_unmod_peptides = FALSE)
#' 
#' head(msstats_data$PTM)
#' head(msstats_data$PROTEIN)
#' 
FragPipetoMSstatsPTMFormat = function(input,
                                       annotation=NULL,
                                       input_protein=NULL,
                                       annotation_protein=NULL,
                                       use_unmod_peptides=FALSE,
                                       label_type="TMT",
                                       protein_id_col = "Protein",
                                       peptide_id_col = "Peptide.Sequence",
                                       mod_id_col = "STY",
                                       localization_cutoff=.75,
                                       remove_unlocalized_peptides=TRUE,
                                       Purity_cutoff = 0.6,
                                       PeptideProphet_prob_cutoff = 0.7,
                                       useUniquePeptide = TRUE,
                                       rmPSM_withfewMea_withinRun = FALSE,
                                       rmPeptide_OxidationM = TRUE,
                                       rmProtein_with1Feature = FALSE,
                                       summaryforMultipleRows = sum,
                                       use_log_file = TRUE,
                                       append = FALSE,
                                       verbose = TRUE,
                                       log_file_path = NULL){
  MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                      log_file_path, 
                                      base = "MSstatsPTM_converter_log_")
  
  ## Check input parameters
  checkmate::assertTRUE(!is.null(input))
  if (label_type == "TMT"){
    checkmate::assertTRUE(!is.null(annotation))
    .checkAnnotation(annotation, "TMT")
    if (!is.null(annotation_protein)){
      .checkAnnotation(annotation_protein, "TMT")
    }
  }
  
  input = as.data.table(input)
  
  mod_id_col = .getFullModID(input, mod_id_col)
  input$Start = input$Protein.Start
  if ("Is.Unique" %in% colnames(input)){
    input$Is.Unique = as.logical(input$Is.Unique)
  }
  
  input = MSstatsPTMSiteLocator(input, 
                       protein_name_col= protein_id_col,
                       unmod_pep_col = peptide_id_col,
                       mod_pep_col = mod_id_col,
                       clean_mod=FALSE,
                       fasta_file=NULL, 
                       fasta_protein_name="header",
                       mod_id="\\*", 
                       localization_scores=TRUE,
                       localization_cutoff=localization_cutoff,
                       remove_unlocalized_peptides=remove_unlocalized_peptides,
                       terminus_included=FALSE, 
                       terminus_id="\\.")
  
  if (use_unmod_peptides){
    input_protein = input[input[[protein_id_col]] == input$ProteinNameUnmod]
    input = input[input[[protein_id_col]] != input$ProteinNameUnmod]  
    annotation_protein = annotation
  } else {
    input = input[input[[protein_id_col]] != input$ProteinNameUnmod]  
  }
  
  input[[peptide_id_col]] = input[[colnames(input)[grepl("new_peptide_col", colnames(input))][[1]]]]
  if (label_type == "TMT"){
    
    ptm_input = PhilosophertoMSstatsTMTFormat(
      input, annotation, protein_id_col, peptide_id_col, Purity_cutoff, 
      PeptideProphet_prob_cutoff, useUniquePeptide, rmPSM_withfewMea_withinRun, 
      rmPeptide_OxidationM, rmProtein_with1Feature, summaryforMultipleRows, 
      use_log_file, append, verbose, log_file_path)
  } else if (label_type == "LF"){
    msstats_cols = c("ProteinName", "PeptideSequence", "PrecursorCharge", 
                     "FragmentIon", "ProductCharge", "IsotopeLabelType", 
                     "Condition", "BioReplicate", "Run", "Intensity")
    ptm_input = FragPipetoMSstatsFormat(
      input[,..msstats_cols], useUniquePeptide, rmPSM_withfewMea_withinRun, 
      rmProtein_with1Feature, summaryforMultipleRows,
      use_log_file, append, verbose, log_file_path
    )
  }
  
  msstats_format = list(PTM=ptm_input, PROTEIN=NULL)
  
  if (!is.null(input_protein)){
    
    if (label_type == "TMT"){
      protein_input = PhilosophertoMSstatsTMTFormat(
        input_protein, annotation_protein, protein_id_col, peptide_id_col, 
        Purity_cutoff, PeptideProphet_prob_cutoff, useUniquePeptide, 
        rmPSM_withfewMea_withinRun, rmPeptide_OxidationM, rmProtein_with1Feature, 
        summaryforMultipleRows, use_log_file, append, verbose, log_file_path)
    } else if (label_type == "LF"){
      protein_input = FragPipetoMSstatsFormat(
        input_protein[,..msstats_cols], useUniquePeptide, rmPSM_withfewMea_withinRun, 
        rmProtein_with1Feature, summaryforMultipleRows,
        use_log_file, append, verbose, log_file_path)
    }
    msstats_format = list(PTM=ptm_input, PROTEIN=protein_input)
    
  }
  
  return(msstats_format)
}


#' Convert output of label-free or TMT MaxQuant experiments into MSstatsPTM format
#' 
#' Takes as input LF/TMT experiments from MaxQ and converts the data into the 
#' format needed for MSstatsPTM. Requires modified evidence.txt file from MaxQ 
#' and an annotation file for PTM data. To adjust modified peptides for changes 
#' in global protein level, unmodified TMT experimental data must also be 
#' returned. Optionally can use `Phospho(STY)Sites.txt` (or other PTM specific 
#' files) from MaxQuant, but this is not recommended. If PTM specific file 
#' provided, the raw intensities must be provided, not a ratio.
#'
#' @export
#' @importFrom stringr str_extract regex str_replace fixed str_split
#' @importFrom data.table melt as.data.table `:=` `%like%` setcolorder setDT tstrsplit setcolorder
#' @importFrom MSstatsConvert MaxQtoMSstatsFormat
#' @importFrom MSstatsTMT MaxQtoMSstatsTMTFormat
#' @importFrom checkmate assertChoice assertLogical
#' 
#' @param evidence name of 'evidence.txt' data, which includes feature-level 
#' data for enriched (PTM) data.
#' @param annotation data frame annotation file for the ptm level data.
#' Contains column Run, Fraction, TechRepMixture, Mixture, Channel, 
#' BioReplicate, Condition.
#' @param fasta_path A string of path to a FASTA file, used to match PTM peptides.
#' @param fasta_protein_name Name of fasta column that matches with protein name
#' in evidence file. Default is `uniprot_ac`.
#' @param mod_id Character that indicates the modification of interest. Default 
#' is `\\(Phospho\\)`. Note `\\` must be included before special characters.
#' @param sites_data (Not recommended. Only used if evidence file not provided. 
#' Only works for TMT labeled data) Modified peptide output from MaxQuant. For 
#' example, a phosphorylation experiment would require the Phospho(STY)Sites.txt
#' file
#' @param evidence_prot name of 'evidence.txt' data, which includes 
#' feature-level data for global profiling (unmodified) data.
#' @param proteinGroups name of 'proteinGroups.txt' data. It needs to matching 
#' protein group ID in `evidence_prot`.
#' @param annotation_protein data frame annotation file for the protein level data.
#' Contains column Run, Fraction, TechRepMixture, Mixture, Channel, 
#' BioReplicate, Condition.
#' @param use_unmod_peptides Boolean if the unmodified peptides in the input 
#' file should be used to construct the unmodified protein output. Only used if
#' `input_protein` is not provided. Default is `FALSE`.
#' @param labeling_type Either `TMT` or `LF` (Label-Free) depending on 
#' experimental design. Default is `LF`.
#' @param mod_num (Only if `sites.data` is used) For modified peptide dataset. 
#' The number modifications per peptide to be used. If "Single", only peptides 
#' with one modification will be used. Otherwise "Total" can be selected which 
#' does not cap the number of modifications per peptide. "Single" is the 
#' default. Selecting "Total" may confound the effect of different 
#' modifications.
#' @param TMT_keyword (Only if `sites.data` is used) the sub-name of columns 
#' in sites.data file. Default is `TMT`. This corresponds to the columns in the 
#' format `Reporter.intensity.corrected.1.TMT1phos___1`. Specifically, this 
#' parameter indicates the first section of the string `TMT1phos` (Before the 
#' mixture number). If `TMT` is present in the string, set this value to `TMT`. 
#' Else if `TMT` is not there (ie string is in the format `1phos`) leave this 
#' parameter as an empty string ('').
#' @param ptm_keyword (Only if `sites.data` is used) the sub-name of columns in 
#' the sites.data file. Default is 
#' `phos`. This corresponds to the columns in the format 
#' `Reporter.intensity.corrected.1.TMT1phos___1`. Specifically, this parameter 
#' indicates the second section of the string `TMT1phos` (After the mixture 
#' number). If the string is present, set this parameter. Else if this part of 
#' the string is empty (ie string is in the format `TMT1`) leave this parameter 
#' as an empty string ('').
#' @param which_proteinid_ptm For PTM dataset, which column to use for protein 
#' name. Use 'Proteins'(default) column for protein name. 'Leading.proteins' or 
#' 'Leading.razor.protein' or 'Gene.names' can be used instead to get the 
#' protein ID with single protein. However, those can potentially have the 
#' shared peptides.
#' @param remove_other_mods Remove peptides which include modfications other 
#' than the one listed in `mod_id`. Default is `TRUE`. For example, in an 
#' experiment targeting Phosphorylation, setting this parameter to `TRUE` would 
#' remove peptides like 
#' (Acetyl (Protein N-term))AAAAPDSRVS(Phospho (STY))EEENLK. Set this parameter 
#' to `FALSE` to keep peptides with extraneous modifications.
#' @param which_proteinid_protein For Protein dataset, which column to use for 
#' protein name. Same options as above.
#' @param removeMpeptides If Oxidation (M) modifications should be removed. 
#' Default is TRUE.
#' @param removeOxidationMpeptides TRUE will remove the peptides including 
#' 'oxidation (M)' in modification. FALSE is default.
#' @param removeProtein_with1Peptide TRUE will remove the proteins which have 
#' only 1 peptide and charge. FALSE is default.
#' @param use_log_file logical. If TRUE, information about data processing will 
#' be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing wil be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. If not provided, such a file will be created 
#' automatically. If 'append = TRUE', has to be a valid path to a file.
#' 
#' @return a list of two data.tables named 'PTM' and 'PROTEIN' in the format 
#' required by MSstatsPTM.
#' 
#' @examples
#' # TMT experiment
#' head(maxq_tmt_evidence)
#' head(maxq_tmt_annotation)
#' 
#' msstats_format_tmt = MaxQtoMSstatsPTMFormat(evidence=maxq_tmt_evidence,
#'                         annotation=maxq_tmt_annotation,
#'                         fasta=system.file("extdata", "maxq_tmt_fasta.fasta", package="MSstatsPTM"),
#'                         fasta_protein_name="uniprot_ac",
#'                         mod_id="\\(Phospho \\(STY\\)\\)",
#'                         use_unmod_peptides=TRUE,
#'                         labeling_type = "TMT",
#'                         which_proteinid_ptm = "Proteins")
#' 
#' head(msstats_format_tmt$PTM)
#' head(msstats_format_tmt$PROTEIN)
#' 
#' # LF experiment
#' head(maxq_lf_evidence)
#' head(maxq_lf_annotation)
#' 
#' msstats_format_lf = MaxQtoMSstatsPTMFormat(evidence=maxq_lf_evidence,
#'                         annotation=maxq_lf_annotation,
#'                         fasta=system.file("extdata", "maxq_lf_fasta.fasta", package="MSstatsPTM"),
#'                         fasta_protein_name="uniprot_ac",
#'                         mod_id="\\(Phospho \\(STY\\)\\)",
#'                         use_unmod_peptides=TRUE,
#'                         labeling_type = "LF",
#'                         which_proteinid_ptm = "Proteins")
#' head(msstats_format_lf$PTM)
#' head(msstats_format_lf$PROTEIN)
MaxQtoMSstatsPTMFormat = function(evidence=NULL,
                                  annotation=NULL,
                                  fasta_path,
                                  fasta_protein_name="uniprot_ac",
                                  mod_id="\\(Phospho \\(STY\\)\\)",
                                  sites_data=NULL,
                                  evidence_prot = NULL,
                                  proteinGroups = NULL,
                                  annotation_protein = NULL,
                                  use_unmod_peptides=FALSE,
                                  labeling_type = "LF",
                                  mod_num = 'Single',
                                  TMT_keyword = "TMT",
                                  ptm_keyword = "phos",
                                  which_proteinid_ptm = "Proteins",
                                  which_proteinid_protein = "Proteins",
                                  remove_other_mods=TRUE,
                                  removeMpeptides = FALSE,
                                  removeOxidationMpeptides = FALSE,
                                  removeProtein_with1Peptide = FALSE,
                                  use_log_file = TRUE,
                                  append = FALSE,
                                  verbose = TRUE,
                                  log_file_path = NULL) {
  
  ## TODO: add this code to check function
  if (is.null(evidence) & is.null(sites_data)){
    stop("Either evidence and proteinGroups or sites_data files must be included.")
  }
  
  .checkAnnotation(annotation, labeling_type)
  if (!is.null(annotation_protein)){
    .checkAnnotation(annotation_protein, labeling_type)
  }
  
  if (!is.null(evidence_prot)){
    if (is.null(proteinGroups)) {
      stop("proteinGroups is NULL.  Please input the proteinGroups file from your global proteome dataset using the proteinGroups parameter")
    }
  }
  
  # .checkMaxQconverterParams(mod.num,
  #                           ptm.keyword,
  #                           which.proteinid.ptm,
  #                           which.proteinid.protein,
  #                           removeMpeptides)
  
  if (is.null(sites_data)){
    evidence = as.data.table(evidence)
  } else {
    pho.data = as.data.table(sites_data)
  }
  
  annot.ptm = as.data.table(annotation)
  
  # clean.prot = .check.global.protein(evidence, proteinGroups)
  
  ## Format annotation for PTM
  if (is.null(sites_data)){
    if(labeling_type == "TMT"){
      
      evidence_sites = MSstatsPTMSiteLocator(evidence, 
                                             protein_name_col = which_proteinid_ptm,
                                             unmod_pep_col = "Sequence",
                                             mod_pep_col = "Modified.sequence",
                                             fasta_file = fasta_path,
                                             fasta_protein_name = "uniprot_ac", 
                                             mod_id=mod_id, 
                                             terminus_included = FALSE,
                                             remove_underscores=TRUE,
                                             remove_other_mods=remove_other_mods,
                                             bracket="(",
                                             replace_text=TRUE)
      
      msstatsptm_input = MaxQtoMSstatsTMTFormatHelper(evidence_sites,
                                                      annotation,
                                                      which.proteinid = which_proteinid_ptm,
                                                      rmPSM_withfewMea_withinRun = removeProtein_with1Peptide,
                                                      use_log_file=use_log_file,
                                                      append=append,
                                                      verbose=verbose,
                                                      log_file_path=log_file_path)
      
    } else if (labeling_type == "LF"){
      
      evidence_sites = MSstatsPTMSiteLocator(evidence, 
                                             protein_name_col= which_proteinid_ptm,
                                             unmod_pep_col = "Sequence",
                                             mod_pep_col = "Modified.sequence",
                                             fasta_file=fasta_path,
                                             fasta_protein_name="uniprot_ac", 
                                             mod_id=mod_id, 
                                             terminus_included=FALSE,
                                             remove_underscores=TRUE,
                                             remove_other_mods=remove_other_mods,
                                             bracket="(",
                                             replace_text=TRUE)
      
      msstatsptm_input = MaxQtoMSstatsFormatHelper(evidence_sites,
                                                   annotation,
                                                   proteinID=which_proteinid_ptm,
                                                   removeMpeptides = removeMpeptides,
                                                   removeOxidationMpeptides = removeOxidationMpeptides,
                                                   removeProtein_with1Peptide = removeProtein_with1Peptide,
                                                   use_log_file=use_log_file,
                                                   append=append,
                                                   verbose=verbose,
                                                   log_file_path=log_file_path)
      
      
    }
  } else {
    setcolorder(annot.ptm, c("Run", "Channel", "Condition", "Mixture", 
                             "TechRepMixture", "Fraction", "BioReplicate"))
    
    annot.ptm = annot.ptm[, "Fraction" := NULL] ## TODO: should this be removed?
    annot.ptm = annot.ptm[, "Run" :=NULL] ## We recreate this column to match with PTM data
    annot.ptm = unique(annot.ptm)
    annot.ptm$Replicate = paste0("Reporter.intensity.corrected.",
                                 gsub('channel.', '', annot.ptm$Channel), 
                                 ".", TMT.keyword, 
                                 gsub("mixture", "", annot.ptm$Mixture), 
                                 ptm.keyword)
    
    msstatsptm_input = .convert.ptm.data(pho.data,
                                         annot.ptm,
                                         mod.num,
                                         ptm.keyword,
                                         which.proteinid.ptm,
                                         removeMpeptides)
    
    ## MaxQ phospho file has duplicate peptides per site 
    ## (if site has equal probability)
    ## Add protein into site to create unique identifier
    msstatsptm_input$PeptideSequence = paste(
      msstatsptm_input$PeptideSequence, msstatsptm_input$ProteinName, 
      sep = ':')
    
    setDT(msstatsptm_input)[, PeptideSequence := tstrsplit(PeptideSequence, ":",
                                                           keep = 1)]
  }
  
  if (!is.null(evidence_prot)){
    annotation_protein = as.data.table(annotation_protein)
    
    if(labeling_type == "TMT"){
      msstats.abun = MaxQtoMSstatsTMTFormat(evidence = evidence_prot,
                                            proteinGroups = proteinGroups,
                                            annotation = annotation_protein,
                                            which.proteinid = which_proteinid_protein)
    } else if (labeling_type == "LF"){
      msstats.abun = MaxQtoMSstatsFormat(evidence = evidence_prot,
                                         proteinGroups = proteinGroups,
                                         annotation = annotation_protein,
                                         proteinID = which_proteinid_protein)
    }
    
    msstatsptm_input = msstatsptm_input[grepl(mod_id, msstatsptm_input$PeptideSequence),]
    MSstatsPTMformat = list('PTM' = msstatsptm_input, 
                            "PROTEIN" = msstats.abun)
    return(MSstatsPTMformat)
  }
  
  if (use_unmod_peptides){
    msstats.abun = msstatsptm_input[!grepl(mod_id, msstatsptm_input$PeptideSequence),]
    msstatsptm_input = msstatsptm_input[grepl(mod_id, msstatsptm_input$PeptideSequence),]
    
    MSstatsPTMformat = list(PTM = msstatsptm_input, 
                            PROTEIN = msstats.abun)
  } else {
    msstatsptm_input = msstatsptm_input[grepl(mod_id, msstatsptm_input$PeptideSequence),]
    MSstatsPTMformat = list('PTM' = msstatsptm_input)
  }
  
  return(MSstatsPTMformat)
}

#' Convert Proteome Discoverer output into MSstatsPTM format
#' 
#' Import Proteome Discoverer files, identify modification site location.
#' 
#' @param input PD report corresponding with enriched experimental data.
#' @param annotation name of 'annotation.txt' or 'annotation.csv' data which 
#' includes Condition, BioReplicate, Run information. 'Run' will be matched 
#' with 'Spectrum.File'
#' @param fasta_path string containing path to the corresponding fasta file for 
#' the modified peptide dataset.
#' @param protein_input PD report corresponding with unmodified experimental 
#' data.
#' @param annotation_protein Same format as `annotation` corresponding to 
#' unmodified data.
#' @param labeling_type type of experimental design, must be one of `LF` for 
#' label free or `TMT` for tandem mass tag.
#' @param mod_id Character that indicates the modification of interest. Default 
#' is `\\(Phospho\\)`. Note `\\` must be included before special characters.
#' @param keep_all_mods Boolean indicating whether to keep or remove peptides 
#' not in `mod_id`. Default is FALSE.
#' @param use_localization_cutoff Boolean indicating whether to use a custom 
#' localization cutoff or rely on PD's modifications column. `TRUE` is default 
#' and apply custom cutoff `localization_cutoff`.
#' @param use_unmod_peptides If `protein_input` is not provided, 
#' unmodified peptides can be extracted from `input` to be used in place of a 
#' global profiling run. Default is `FALSE`.
#' @param fasta_protein_name Name of fasta column that matches with protein name
#' in evidence file. Default is `uniprot_iso`.
#' @param localization_cutoff Minimum localization score required to keep modification. Default is .75.
#' @param remove_unlocalized_peptides Boolean indicating if peptides without all sites localized should be kept. Default is TRUE (non-localized sites will be removed).
#' @param useNumProteinsColumn TRUE removes peptides which have more than 1 in 
#' Proteins column of PD output.
#' @param useUniquePeptide TRUE (default) removes peptides that are assigned 
#' for more than one proteins. We assume to use unique peptide for each protein.
#' @param summaryforMultipleRows max(default) or sum - when there are multiple 
#' measurements for certain feature and certain run, use highest or sum of 
#' multiple intensities.
#' @param removeFewMeasurements TRUE (default) will remove the features that 
#' have 1 or 2 measurements across runs.
#' @param removeOxidationMpeptides TRUE will remove the peptides including 
#' 'oxidation (M)' in modification. FALSE is default.
#' @param removeProtein_with1Peptide TRUE will remove the proteins which have 
#' only 1 peptide and charge. FALSE is default.
#' @param which_quantification Use 'Precursor.Area'(default) column for 
#' quantified intensities. 'Intensity' or 'Area' can be used instead.
#' @param which_proteinid Use 'Protein.Accessions'(default) column for protein 
#' name. 'Master.Protein.Accessions' can be used instead.
#' @param use_log_file logical. If TRUE, information about data processing will 
#' be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing will be 
#' printed to the console
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. If not provided, such a file will be created 
#' automatically. If 'append = TRUE', has to be a valid path to a file.
#' 
#' @importFrom MSstatsConvert PDtoMSstatsFormat
#' @importFrom MSstatsTMT PDtoMSstatsTMTFormat
#' @importFrom stringr str_split str_trim str_split_i
#' @importFrom data.table setnames
#' @importFrom stringi stri_sub_replace_all
#' @return `list` of `data.table`
#' @export 
#' 
#' @examples
#' # Global profiling example
#' input = system.file("tinytest/raw_data/PD/pd-ptm-input.csv", 
#' package = "MSstatsPTM")
#' input = data.table::fread(input)
#' annot = system.file("tinytest/raw_data/PD/pd-ptm-annot.csv",
#'                     package = "MSstatsPTM")
#' annot = data.table::fread(annot)
#' input_protein = system.file("tinytest/raw_data/PD/protein-input.csv",
#'                             package = "MSstatsPTM")
#' input_protein = data.table::fread(input_protein)
#' annot_protein = system.file("tinytest/raw_data/PD/protein-annot.csv",
#'                             package = "MSstatsPTM")
#' annot_protein = data.table::fread(annot_protein)
#' fasta_path=system.file("extdata", "pd_with_proteome.fasta", 
#'                        package="MSstatsPTM")
#' pd_imported = PDtoMSstatsPTMFormat(
#'     input,
#'     annotation = annot,
#'     protein_input = input_protein,
#'     annotation_protein = annot_protein,
#'     fasta_path = fasta_path,
#'     mod_id = "\\(GG\\)",
#'     labeling_type = "TMT",
#'     use_localization_cutoff = FALSE,
#'     which_proteinid = "Master.Protein.Accessions")
#'    
#' head(pd_imported$PTM)
#' head(pd_imported$PROTEIN)
#' 
#' # No global profiling example
#' head(pd_psm_input)
#' head(pd_annotation)
#' 
#' msstats_format = PDtoMSstatsPTMFormat(pd_psm_input, 
#'                                       pd_annotation, 
#'                                       system.file("extdata", "pd_fasta.fasta", package="MSstatsPTM"),
#'                                       use_unmod_peptides=TRUE, 
#'                                       which_proteinid = "Master.Protein.Accessions")
#' 
#' head(msstats_format$PTM)
#' head(msstats_format$PROTEIN)
PDtoMSstatsPTMFormat = function(input,
                                annotation,
                                fasta_path,
                                protein_input=NULL,
                                annotation_protein=NULL,
                                labeling_type = "LF",
                                mod_id="\\(Phospho\\)",
                                use_localization_cutoff=FALSE,
                                keep_all_mods=FALSE,
                                use_unmod_peptides=FALSE,
                                fasta_protein_name="uniprot_iso",
                                localization_cutoff=75,
                                remove_unlocalized_peptides=TRUE,
                                useNumProteinsColumn = FALSE,
                                useUniquePeptide = TRUE,
                                summaryforMultipleRows = max,
                                removeFewMeasurements = TRUE,
                                removeOxidationMpeptides = FALSE,
                                removeProtein_with1Peptide = FALSE,
                                which_quantification = "Precursor.Area",
                                which_proteinid = "Protein.Group.Accessions",
                                use_log_file = TRUE,
                                append = FALSE,
                                verbose = TRUE,
                                log_file_path = NULL){
  
  input = as.data.table(input)
  
  if (is.null(fasta_path) & use_localization_cutoff == TRUE){
    stop("A FASTA file must be included if using a custom localization cutoff. Please pass a FASTA file to `fasta_path`.")
  }
  
  if (!is.null(protein_input) & use_unmod_peptides == TRUE){
    stop("Either pass protein_input data or set use_unmod_peptides = TRUE, not both")
  }
  
  .checkAnnotation(annotation, labeling_type)
  if (!is.null(annotation_protein)){
    .checkAnnotation(annotation_protein, labeling_type)
  }
  
  if (labeling_type == "LF"){
    sequence_col = "Sequence"
  } else if (labeling_type == "TMT"){
    sequence_col = "Annotated.Sequence"
    input[[sequence_col]] = toupper(str_split_i(input[[sequence_col]], 
                                                "\\.", i=2))
  }
  
  if (use_localization_cutoff){
    input = .getPDmods(input)
    
    input = MSstatsPTMSiteLocator(input, 
                                  protein_name_col= which_proteinid,
                                  unmod_pep_col = sequence_col,
                                  mod_pep_col = "ModSequence",
                                  clean_mod=FALSE,
                                  fasta_file=fasta_path, 
                                  fasta_protein_name=fasta_protein_name,
                                  mod_id="\\*", 
                                  localization_scores=TRUE,
                                  localization_cutoff=localization_cutoff,
                                  remove_unlocalized_peptides=remove_unlocalized_peptides,
                                  terminus_included=FALSE, 
                                  terminus_id="\\.")
  } else {
    input = .extract_pd_mods(input, mod_id, keep_all_mods, sequence_col)
    # input[,which_proteinid] = paste(input[,..which_proteinid][[1]], mods,sep="_")
    
    input = MSstatsPTMSiteLocator(input, 
                                  protein_name_col= which_proteinid,
                                  unmod_pep_col = sequence_col,
                                  mod_pep_col = "ModSequence",
                                  clean_mod=FALSE,
                                  fasta_file=fasta_path, 
                                  fasta_protein_name=fasta_protein_name,
                                  mod_id="\\*", 
                                  localization_scores=FALSE,
                                  localization_cutoff=localization_cutoff,
                                  remove_unlocalized_peptides=remove_unlocalized_peptides,
                                  terminus_included=FALSE, 
                                  terminus_id="\\.")
    
  }
  
  input[[sequence_col]] = input[, "ModSequence"]
  
  if (labeling_type == "LF"){
    ptm_input = PDtoMSstatsFormat(input, annotation, useNumProteinsColumn,
                                  useUniquePeptide, summaryforMultipleRows,
                                  removeFewMeasurements, removeOxidationMpeptides,
                                  removeProtein_with1Peptide, 
                                  which_quantification, which_proteinid, 
                                  sequence_col, use_log_file, append, verbose,
                                  log_file_path)
  } else if (labeling_type == "TMT"){
    ptm_input = PDtoMSstatsTMTFormat(input, annotation, which_proteinid, 
                                     useNumProteinsColumn,
                                     useUniquePeptide, TRUE,
                                     removeProtein_with1Peptide, 
                                     summaryforMultipleRows, 
                                     use_log_file, append, verbose,
                                     log_file_path)
  }
  
  if ("PeptideModifiedSequence" %in% colnames(ptm_input)){
    setnames(ptm_input, c("PeptideModifiedSequence"), c("PeptideSequence"))
  }
  
  if (!is.null(protein_input)) {
    if (labeling_type == "LF"){
      protein_input = PDtoMSstatsFormat(protein_input, annotation_protein, 
                                        useNumProteinsColumn,
                                        useUniquePeptide, summaryforMultipleRows,
                                        removeFewMeasurements, 
                                        removeOxidationMpeptides,
                                        removeProtein_with1Peptide, 
                                        which_quantification, which_proteinid, 
                                        "Sequence", use_log_file, append, 
                                        verbose, log_file_path)
    } else if (labeling_type == "TMT"){
      protein_input = PDtoMSstatsTMTFormat(protein_input, annotation_protein, 
                                       which_proteinid, useNumProteinsColumn,
                                       useUniquePeptide, TRUE,
                                       removeProtein_with1Peptide, 
                                       summaryforMultipleRows, 
                                       use_log_file, append, verbose,
                                       log_file_path)
    }
    
    if ("PeptideModifiedSequence" %in% colnames(protein_input)){
      setnames(protein_input, c("PeptideModifiedSequence"), 
               c("PeptideSequence"))
    }
      
    ptm_input = ptm_input[grepl("\\*", ptm_input[, "PeptideSequence"]), ]
    
    msstats_input = list(PTM = ptm_input, PROTEIN = protein_input)
  } else {
    if (use_unmod_peptides){
      ptm_input=as.data.frame(ptm_input)
      protein_input = ptm_input[!grepl("\\*", ptm_input$PeptideSequence),]
      ptm_input = ptm_input[grepl("\\*", ptm_input$PeptideSequence),]
      
      msstats_input = list(PTM = ptm_input, PROTEIN = protein_input)
    } else {
      ptm_input = ptm_input[grepl("\\*", ptm_input[, "PeptideSequence"]), ]
      
      msstats_input = list(PTM = ptm_input)
    }
  }
  
  return(msstats_input)
  
}

#' Converts non-TMT Progenesis output into the format needed for MSstatsPTM
#'
#' @export
#' @importFrom MSstatsConvert ProgenesistoMSstatsFormat
#' @importFrom data.table as.data.table
#'
#' @param ptm_input name of Progenesis output with modified peptides, which is
#' wide-format. 'Accession', Sequence', 'Modification', 'Charge' and one column
#' for each run are required
#' @param annotation name of 'annotation.txt' or 'annotation.csv' data which
#' includes Condition, BioReplicate, Run, and Type (PTM or Protein) 
#' information. It will be matched with the column name of input for MS runs. 
#' Please note PTM and global Protein run names are often different, which is 
#' why an additional Type column indicating Protein or PTM is required.
#' @param global_protein_input name of Progenesis output with unmodified
#' peptides, which is wide-format. 'Accession', Sequence', 'Modification',
#' 'Charge' and one column for each run are required
#' @param fasta_path string containing path to the corresponding fasta file for 
#' the modified peptide dataset.
#' @param useUniquePeptide TRUE(default) removes peptides that are assigned for
#' more than one proteins. We assume to use unique peptide for each protein.
#' @param summaryforMultipleRows max(default) or sum - when there are multiple
#' measurements for certain feature and certain run, use highest or sum of
#' multiple intensities.
#' @param removeFewMeasurements TRUE (default) will remove the features that 
#' have 1 or 2 measurements across runs.
#' @param removeOxidationMpeptides TRUE will remove the modified peptides
#' including 'Oxidation (M)' sequence. FALSE is default.
#' @param removeProtein_with1Peptide TRUE will remove the proteins which have
#' only 1 peptide and charge. FALSE is default.
#' @param mod.num For modified peptide dataset, must be one of `Single` or 
#' `Total`. The default is `Single`. The number modifications per peptide to be 
#' used. If "Single", only peptides with one modification will be 
#' used. Otherwise "Total" includes peptides with more than one modification.
#' Selecting "Total" may confound the effect of different modifications.
#' @return a list of two data.tables named 'PTM' and 'PROTEIN' in the format 
#' required by MSstatsPTM.
#' @examples
#' 
#' input = system.file("tinytest/raw_data/Progenesis/progenesis_peptide.csv", 
#'                                 package = "MSstatsPTM")
#' input = data.table::fread(input)
#' colnames(input) = unlist(input[1,])
#' input = input[-1,]
#' annot = system.file("tinytest/raw_data/Progenesis/phospho_annotation.csv", 
#'                                 package = "MSstatsPTM")
#' annot = data.table::fread(annot)
#' prog_imported = ProgenesistoMSstatsPTMFormat(
#'     input, 
#'     annot
#' )
#' head(prog_imported$PTM)
ProgenesistoMSstatsPTMFormat = function(ptm_input,
                                        annotation,
                                        global_protein_input = FALSE,
                                        fasta_path = FALSE,
                                        useUniquePeptide=TRUE,
                                        summaryforMultipleRows=max, 
                                        removeFewMeasurements=TRUE,
                                        removeOxidationMpeptides=FALSE,
                                        removeProtein_with1Peptide=FALSE,
                                        mod.num = 'Single'){
  
  ptm_input = as.data.table(ptm_input)
  annotation = as.data.table(annotation)
  col_order = colnames(ptm_input)
  ptm_input$id = 1:nrow(ptm_input)
  
  annotation = as.data.table(annotation)
  ptm_annot = annotation[Type == "PTM"]
  protein_annot = annotation[Type == "Protein"]
  
  X.10 = X.9 = X.8 = NULL
  
  ## Format PTM data
  if (fasta_path == FALSE) {
    ptm_input[,X.10 := paste(X.10, X.8, X.9, sep = "_")]
    ptm_input[2, "X.10"] = "Accession"
  } else {
    .progensis.add.sites(ptm_input, fasta_path, col_order)
  }
  
  convert.ptm = ProgenesistoMSstatsFormat(ptm_input, ptm_annot,
                                          useUniquePeptide,
                                          summaryforMultipleRows,
                                          removeFewMeasurements,
                                          removeOxidationMpeptides,
                                          removeProtein_with1Peptide)
  
  if (global_protein_input[[1]][1] != FALSE){
    
    global_protein_input = as.data.table(global_protein_input)
    
    convert.prot = ProgenesistoMSstatsFormat(global_protein_input, 
                                             protein_annot,
                                             useUniquePeptide,
                                             summaryforMultipleRows,
                                             removeFewMeasurements,
                                             removeOxidationMpeptides,
                                             removeProtein_with1Peptide)
    MSstatsPTM.data = list("PTM" = convert.ptm,
                           "PROTEIN" = convert.prot)
  } else {
    MSstatsPTM.data = list("PTM" = convert.ptm,
                           "PROTEIN" = NULL)
  }
  
  return(MSstatsPTM.data)
  
}

#' Convert Peaks Studio output into MSstatsPTM format
#' 
#' Currently only supports label-free quantification.
#' 
#' @param input name of Peaks Studio PTM output
#' @param annotation name of annotation file which includes Raw.file, Condition,
#' BioReplicate, Run. For example annotation see example below.
#' @param input_protein name of Peaks Studio unmodified protein output 
#' (optional)
#' @param annotation_protein name of annotation file which includes Raw.file, 
#' Condition,
#' BioReplicate, Run for unmodified protein output.
#' @param use_unmod_peptides Boolean if the unmodified peptides in the input 
#' file should be used to construct the unmodified protein output. Only used if
#' `input_protein` is not provided. Default is `FALSE`
#' @param target_modification Character name of modification of interest. To 
#' use all mod types, leave as `NULL`. Default is `NULL`. Note that if the name 
#' includes special characters, you must include "\\" before the characters. Ex.
#' "Phosphorylation \\(STY\\)"
#' @param remove_oxidation_peptides Boolean if Oxidation (M) modifications 
#' should be removed. Default is `FALSE`
#' @param remove_multi_mod_types Used if `target_modification` is not `NULL`. 
#' `TRUE` will remove peptides with multiple types of modifications 
#' (ie acetylation and phosphorylation). `FALSE` will keep these peptides and 
#' summarize them seperately.
#' @param summaryforMultipleRows max(default) or sum - when there are multiple 
#' measurements for certain feature and certain run, use highest or sum of 
#' multiple intensities.
#' @param use_log_file logical. If TRUE, information about data processing will 
#' be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing wil be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. If not provided, such a file will be created 
#' automatically. If 'append = TRUE', has to be a valid path to a file.
#' 
#' @importFrom data.table as.data.table melt
#' @importFrom MSstatsConvert MSstatsBalancedDesign
#' 
#' @return `list` of `data.table`
#' @export 
#' 
#' @examples
#' # The output should be in the following format.
#' head(raw.input$PTM)
#' head(raw.input$PROTEIN)
PStoMSstatsPTMFormat = function(
    input, 
    annotation, 
    input_protein = NULL, 
    annotation_protein = NULL,
    use_unmod_peptides = FALSE, 
    target_modification = NULL, 
    remove_oxidation_peptides = FALSE, 
    remove_multi_mod_types = FALSE,
    summaryforMultipleRows = max, 
    use_log_file = TRUE, 
    append = FALSE, 
    verbose = TRUE, 
    log_file_path = NULL
){
  
  input = as.data.table(input)
  if (!is.null(input_protein)){
    input_protein = as.data.table(input_protein)
  }
  
  .checkAnnotation(annotation, "LF")
  if (!is.null(annotation_protein)){
    .checkAnnotation(annotation_protein, "LF")
  }
  
  if (!"Source.File" %in% colnames(input)){
    message("Pivoting input data..")
    input = .pivotPS(input)
  }
  
  message("Merging with annotation..")
  input = merge(input, annotation, all.x = TRUE, by = "Raw.File")
  
  ## Filter for required modifications
  message("Filtering modifications based on function arguements..")
  if (remove_oxidation_peptides){
    input = input[!grepl("Oxidation", Mod)]
  }
  
  if (is.null(input_protein) & use_unmod_peptides){
    input_prot = input[is.na(Mod)]
  }
  
  if (!is.null(target_modification)){
    input = input[grepl(target_modification, Mod)]
    
    if (nrow(input) == 0){
      stop("No modifications match target. Please ensure target modification is correctly spelled.")
    }
  } else {
    input = input[!is.na(Mod)]
  }
  
  ## Can be peptides where two mods such as Phosphorylation (STY); Deamidation (NQ)
  ## Setting this will remove these peptides
  if (remove_multi_mod_types){
    input = input[!grepl(";", Mod)]
  }
  
  ## Add mod to protein name
  input$PeptideModifiedSequence = input$PeptideSequence
  input = MSstatsPTMSiteLocator(input, fasta_file=NULL, mod_id="\\*", 
                                mod_id_is_numeric=TRUE, 
                                terminus_included=TRUE, terminus_id="\\.")
  
  ## Finish converter
  input[, c("Raw.File", "Mod", "End", "Start"):=NULL]
  feature_cols = c("ProteinName", "PeptideSequence", "FragmentIon", 
                   "ProductCharge", "PrecursorCharge", "IsotopeLabelType",
                   "BioReplicate", "Condition", "Run" )
  input = MSstatsBalancedDesign(input, c('PeptideSequence', 'ProductCharge'))
  input = as.data.table(input)[, list(
    Intensity = summaryforMultipleRows(Intensity, na.rm = TRUE)), 
    by = feature_cols]

  if (!is.null(input_protein)){
    input_protein[, c("Raw.File", "Mod", "End", "Start"):=NULL]
    input_protein = MSstatsBalancedDesign(input_protein, 
                                       c('PeptideSequence', 'ProductCharge'))
    input_protein = as.data.table(input_protein)[, list(
      Intensity = summaryforMultipleRows(Intensity, na.rm = TRUE)), 
      by = feature_cols]
    
    input_protein = as.data.frame(input_protein)
  }
  
  return(list("PTM" = as.data.frame(input),
              "PROTEIN" = input_protein))
}

#' Convert Skyline output into MSstatsPTM format
#' 
#' Currently only supports label-free quantification.
#' 
#' @param input name of Skyline PTM output
#' @param fasta_path A string of path to a FASTA file, used to match PTM peptides.
#' @param fasta_protein_name Name of fasta column that matches with protein name
#' in evidence file. Default is `uniprot_iso`.
#' @param annotation name of 'annotation.txt' data which includes Condition, 
#' BioReplicate, Run. If annotation is already complete in Skyline, use 
#' annotation=NULL (default). It will use the annotation information from input.
#' @param input_protein name of Skyline unmodified protein output (optional)
#' @param annotation_protein name of 'annotation.txt' data which includes Condition, 
#' BioReplicate, Run for unmodified protein output. This can be the same as 
#' `annotation`.
#' @param use_unmod_peptides Boolean if the unmodified peptides in the input 
#' file should be used to construct the unmodified protein output. Only used if
#' `input_protein` is not provided. Default is `FALSE`.
#' @param removeiRT TRUE (default) will remove the proteins or peptides which 
#' are labeld 'iRT' in 'StandardType' column. FALSE will keep them.
#' @param filter_with_Qvalue TRUE(default) will filter out the intensities that 
#' have greater than qvalue_cutoff in DetectionQValue column. Those intensities 
#' will be replaced with zero and will be considered as censored missing values 
#' for imputation purpose.
#' @param qvalue_cutoff Cutoff for DetectionQValue. default is 0.01.
#' @param use_unique_peptide TRUE (default) removes peptides that are assigned 
#' for more than one proteins. We assume to use unique peptide for each protein.
#' @param remove_few_measurements TRUE will remove the features that 
#' have 1 or 2 measurements across runs. FALSE is default.
#' @param remove_oxidation_peptides TRUE will remove the peptides including 
#' 'oxidation (M)' in modification. FALSE is default.
#' @param removeProtein_with1Feature TRUE will remove the proteins which have 
#' only 1 feature, which is the combination of peptide, precursor charge, 
#' fragment and charge. FALSE is default.
#' @param use_log_file logical. If TRUE, information about data processing will 
#' be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing wil be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. If not provided, such a file will be created 
#' automatically. If 'append = TRUE', has to be a valid path to a file.
#' 
#' @importFrom data.table as.data.table setnames
#' @importFrom MSstatsConvert SkylinetoMSstatsFormat
#' 
#' @return `list` of `data.table`
#' @export
#' 
#' @examples
#' # The output should be in the following format.
#' head(raw.input$PTM)
#' head(raw.input$PROTEIN)
SkylinetoMSstatsPTMFormat = function(input,
                                     fasta_path,
                                     fasta_protein_name="uniprot_iso",
                                     annotation=NULL,
                                     input_protein=NULL,
                                     annotation_protein=NULL,
                                     use_unmod_peptides = FALSE, 
                                     removeiRT = TRUE,
                                     filter_with_Qvalue = TRUE,
                                     qvalue_cutoff = 0.01,
                                     use_unique_peptide = TRUE,
                                     remove_few_measurements = FALSE,
                                     remove_oxidation_peptides = FALSE,
                                     removeProtein_with1Feature = FALSE, 
                                     use_log_file = TRUE, append = FALSE, 
                                     verbose = TRUE, log_file_path = NULL
){
  
  message("Converting modified data..")
  input = as.data.table(input)
  input_prot = NULL
  if (!is.null(annotation)){
    .checkAnnotation(annotation, "LF")
    if (!is.null(annotation_protein)){
      .checkAnnotation(annotation_prot, "LF")
    }
    
    input = merge(input, annotation, all.x = TRUE, by = "File Name")
  }
  
  ## Filter for required modifications
  message("Filtering modifications based on function arguements..")
  if (is.null(input_protein) & use_unmod_peptides){
    input_protein = input[!grepl("\\[+", input$`Peptide Modified Sequence`), ]
    
    if (nrow(input_protein)){
      stop("No unmodified peptides in PTM dataset. It cannot be used for \
           adjustment.")
    }
  }
  
  setnames(input, c("Protein.Name", "Peptide", "Peptide.Modified.Sequence"), 
           c("ProteinName","PeptideSequence", "PeptideModifiedSequence"))
  
  ## Ensure only modified peptides are used
  input = input[grepl("\\[+", input$`PeptideModifiedSequence`), ]
  
  ## Locate sites
  input = MSstatsPTMSiteLocator(input, 
                                fasta_file=fasta_path,
                                fasta_protein_name=fasta_protein_name,
                                terminus_included=FALSE,
                                mod_id_is_numeric=TRUE)

  
  input = SkylinetoMSstatsFormat(input, annotation = NULL, removeiRT, 
                         filter_with_Qvalue, qvalue_cutoff,
                         use_unique_peptide, remove_few_measurements,
                         remove_oxidation_peptides, removeProtein_with1Feature,
                         use_log_file, append, verbose, log_file_path)
  
  if (!is.null(input_protein)){
    message("Converting unmodified protein data..")
    
    input_prot = SkylinetoMSstatsFormat(input_protein, annotation_protein, 
                                        removeiRT, filter_with_Qvalue, 
                                        qvalue_cutoff, use_unique_peptide, 
                                        remove_few_measurements, 
                                        remove_oxidation_peptides, 
                                        removeProtein_with1Feature, 
                                        use_log_file, append, verbose, 
                                        log_file_path)
  }
  
  return(list("PTM" = as.data.frame(input),
              "PROTEIN" = as.data.frame(input_prot)))
}


#' Convert Spectronaut output into MSstatsPTM format
#' 
#' Converters label-free Spectronaut data into MSstatsPTM format. Requires PSM 
#' output from Spectronaut and a custom made annotation file, mapping the run 
#' name to the condition and bioreplicate. Can optionally take a seperate PSM 
#' file for a global profiling run. If no global profiling run provided, the 
#' function can extract the unmodified peptides from the PTM PSM file and use 
#' them as a global profiling run (not recommended).
#' 
#' @param input name of Spectronaut PTM output, which is long-format. 
#' ProteinName, PeptideSequence, PrecursorCharge, FragmentIon, ProductCharge, 
#' IsotopeLabelType, Condition, BioReplicate, Run, Intensity, 
#' F.ExcludedFromQuantification are required. Rows with 
#' F.ExcludedFromQuantification=True will be removed.
#' @param annotation name of 'annotation.txt' data which includes Condition, 
#' BioReplicate, Run. If annotation is already complete in Spectronaut, 
#' use annotation=NULL (default). It will use the annotation information from 
#' input.
#' @param fasta_path string containing path to the corresponding fasta file for 
#' the modified peptide dataset.
#' @param protein_input name of Spectronaut global protein output, which is 
#' as in the same format as `input` parameter. 
#' @param annotation_protein name of annotation file for global protein data, in
#' the same format as above.
#' @param use_unmod_peptides If `protein_input` is not provided, 
#' unmodified peptides can be extracted from `input` to be used in place of a 
#' global profiling run. Default is `FALSE`.
#' @param intensity 'PeakArea'(default) uses not normalized peak area. 
#' 'NormalizedPeakArea' uses peak area normalized by Spectronaut. Default is 
#' NULL
#' @param mod_id Character that indicates the modification of interest. Default 
#' is `\\(Phospho\\)`. Note `\\` must be included before special characters.
#' @param fasta_protein_name Name of fasta column that matches with protein name
#' in evidence file. Default is `uniprot_iso`.
#' @param remove_other_mods Remove peptides which include modfications other 
#' than the one listed in `mod_id`. Default is `TRUE`. For example, in an 
#' experiment targeting Phosphorylation, setting this parameter to `TRUE` would 
#' remove peptides like 
#' (Acetyl (Protein N-term))AAAAPDSRVS(Phospho (STY))EEENLK. Set this parameter 
#' to `FALSE` to keep peptides with extraneous modifications.
#' @param filter_with_Qvalue TRUE(default) will filter out the intensities that 
#' have greater than qvalue_cutoff in EG.Qvalue column. Those intensities will 
#' be replaced with zero and will be considered as censored missing values for 
#' imputation purpose.
#' @param qvalue_cutoff Cutoff for EG.Qvalue. Default is 0.01.
#' @param useUniquePeptide TRUE (default) removes peptides that are assigned for
#'  more than one proteins. We assume to use unique peptide for each protein.
#' @param removeFewMeasurements TRUE (default) will remove the features that 
#' have 1 or 2 measurements across runs.
#' @param removeProtein_with1Feature TRUE will remove the proteins which have 
#' only 1 feature, which is the combination of peptide, precursor charge, 
#' fragment and charge. FALSE is default.
#' @param summaryforMultipleRows max(default) or sum - when there are multiple 
#' measurements for certain feature and certain run, use highest or sum of 
#' multiple intensities.
#' @param use_log_file logical. If TRUE, information about data processing will 
#' be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing wil be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. If not provided, such a file will be created 
#' automatically. If 'append = TRUE', has to be a valid path to a file.
#' @return a list of two data.tables named 'PTM' and 'PROTEIN' in the format 
#' required by MSstatsPTM.
#' @export
#' @importFrom MSstatsConvert SpectronauttoMSstatsFormat
#' @examples
#' 
#' head(spectronaut_input)
#' head(spectronaut_annotation)
#' 
#' msstats_input = SpectronauttoMSstatsPTMFormat(spectronaut_input, 
#'                   annotation=spectronaut_annotation, 
#'                   fasta_path=system.file("extdata", "spectronaut_fasta.fasta", package="MSstatsPTM"),
#'                   use_unmod_peptides=TRUE,
#'                   mod_id = "\\[Phospho \\(STY\\)\\]",
#'                   fasta_protein_name = "uniprot_iso"
#'                   )
#' 
#' head(msstats_input$PTM)
#' head(msstats_input$PROTEIN)
SpectronauttoMSstatsPTMFormat = function(
    input,
    annotation = NULL,
    fasta_path = NULL,
    protein_input = NULL,
    annotation_protein = NULL,
    use_unmod_peptides=FALSE,
    intensity = "PeakArea",
    mod_id="\\[Phospho \\(STY\\)\\]",
    fasta_protein_name="uniprot_iso",
    remove_other_mods=TRUE,
    filter_with_Qvalue = TRUE,
    qvalue_cutoff = 0.01,
    useUniquePeptide = TRUE,
    removeFewMeasurements = TRUE,
    removeProtein_with1Feature = FALSE,
    summaryforMultipleRows = max,
    use_log_file = TRUE,
    append = FALSE,
    verbose = TRUE,
    log_file_path = NULL){
  
  ## TODO: Add checks on input and params
  input = as.data.table(input)
  ## Needed if input only call PSM
  if (!"EG.ModifiedSequence" %in% colnames(input)){
    input$PeptideSequence = .MSstatsPTMRemoveMods(input$EG.PrecursorId)
    mod_col = "EG.PrecursorId"
  } else {
    input$PeptideSequence = .MSstatsPTMRemoveMods(input$EG.ModifiedSequence)
    mod_col = "EG.ModifiedSequence"
  }
  
  
  input = MSstatsPTMSiteLocator(input, protein_name_col= "PG.ProteinGroups",
                                unmod_pep_col = "PeptideSequence",
                                mod_pep_col = mod_col,
                                fasta_file=fasta_path, 
                                fasta_protein_name=fasta_protein_name,
                                mod_id=mod_id, 
                                terminus_included=FALSE, terminus_id="\\.",
                                remove_underscores=TRUE,
                                remove_other_mods=remove_other_mods,
                                bracket="[",
                                replace_text=TRUE)
  
  ptm_input = SpectronauttoMSstatsFormat(input, annotation, 
                                         intensity = intensity, 
                                         filter_with_Qvalue = filter_with_Qvalue,
                                         qvalue_cutoff = qvalue_cutoff, 
                                         useUniquePeptide = useUniquePeptide, 
                                         removeFewMeasurements = removeFewMeasurements, 
                                         removeProtein_with1Feature = removeProtein_with1Feature, 
                                         summaryforMultipleRows = summaryforMultipleRows)
  
  msstats_input = list(PTM = ptm_input)
  if (!is.null(protein_input)) {
    protein_input = SpectronauttoMSstatsFormat(protein_input, 
                                               annotation_protein, 
                                               intensity = intensity, 
                                               filter_with_Qvalue = filter_with_Qvalue,
                                               qvalue_cutoff = qvalue_cutoff, 
                                               useUniquePeptide = useUniquePeptide, 
                                               removeFewMeasurements = removeFewMeasurements, 
                                               removeProtein_with1Feature = removeProtein_with1Feature, 
                                               summaryforMultipleRows = summaryforMultipleRows)
    
    msstats_input = list(PTM = ptm_input, PROTEIN = protein_input)
  }
  
  if (use_unmod_peptides){
    protein_input = ptm_input[!grepl(mod_id, ptm_input$PeptideSequence),]
    ptm_input = ptm_input[grepl(mod_id, ptm_input$PeptideSequence),]
    
    msstats_input = list(PTM = ptm_input, PROTEIN = protein_input)
  }
  
  return(msstats_input)
}

#' Import Metamorpheus files into PTM format
#' 
#' @author Anthony Wu
#' 
#' @param input name of Metamorpheus output file, which is tabular format. Use the AllQuantifiedPeaks.tsv file from the Metamorpheus output.
#' @param annotation name of 'annotation.txt' data which includes Condition, BioReplicate.
#' @param fasta_path string containing path to the corresponding fasta file for 
#' the modified peptide dataset.
#' @param input_protein same as `input` for global profiling run. Default is NULL.
#' @param annotation_protein same as `annotation` for global profiling run. Default is NULL.
#' @param use_unmod_peptides If `protein_input` is not provided, 
#' unmodified peptides can be extracted from `input` to be used in place of a 
#' global profiling run. Default is `FALSE`.
#' @param mod_ids List of modifications of interest. Default 
#' is a list with only `Common Biological:Phosphorylation on S`. 
#' Please note that the 'mod_ids' parameter currently supports lists of size 1 only. 
#' Future updates aim to extend its functionality to accommodate lists of greater sizes.
#' Note `\\` must be included before special characters. 
#' @param useUniquePeptide TRUE (default) removes peptides that are assigned for
#'  more than one proteins. We assume to use unique peptide for each protein.
#' @param removeFewMeasurements TRUE (default) will remove the features that 
#' have 1 or 2 measurements across runs.
#' @param removeProtein_with1Feature TRUE will remove the proteins which have 
#' only 1 feature, which is the combination of peptide, precursor charge, 
#' fragment and charge. FALSE is default.
#' @param summaryforMultipleRows max(default) or sum - when there are multiple 
#' measurements for certain feature and certain run, use highest or sum of 
#' multiple intensities.
#' @param use_log_file logical. If TRUE, information about data processing will 
#' be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing wil be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. If not provided, such a file will be created 
#' automatically. If 'append = TRUE', has to be a valid path to a file. 
#' @return a list of two data.tables named 'PTM' and 'PROTEIN' in the format 
#' required by MSstatsPTM.
#' @importFrom MSstatsConvert MetamorpheusToMSstatsFormat
#' @export
#' 
#' @examples 
#' input = system.file("tinytest/raw_data/Metamorpheus/AllQuantifiedPeaks.tsv", 
#'                                 package = "MSstatsPTM")
#' input = data.table::fread(input)
#' annot = system.file("tinytest/raw_data/Metamorpheus/ExperimentalDesign.tsv", 
#'                                 package = "MSstatsPTM")
#' annot = data.table::fread(annot)
#' input_protein = system.file("tinytest/raw_data/Metamorpheus/AllQuantifiedPeaksGlobalProteome.tsv",
#'                                 package = "MSstatsPTM")
#' input_protein = data.table::fread(input_protein)
#' annot_protein = system.file("tinytest/raw_data/Metamorpheus/ExperimentalDesignGlobalProteome.tsv", 
#'                                 package = "MSstatsPTM")
#' annot_protein = data.table::fread(annot_protein)
#' fasta_path=system.file("extdata", "metamorpheus_fasta.fasta", 
#'                                 package="MSstatsPTM")
#' metamorpheus_imported = MetamorpheusToMSstatsPTMFormat(
#'     input, 
#'     annot, 
#'     fasta_path=fasta_path,
#'     input_protein=input_protein,
#'     annotation_protein=annot_protein,
#'     use_unmod_peptides=FALSE,
#'     mod_ids = c("\\[Common Fixed:Carbamidomethyl on C\\]")
#' )
#' head(metamorpheus_imported$PTM)
#' head(metamorpheus_imported$PROTEIN)
MetamorpheusToMSstatsPTMFormat = function(input,
                                          annotation,
                                          fasta_path,
                                          input_protein = NULL,
                                          annotation_protein = NULL,
                                          use_unmod_peptides = FALSE,
                                          mod_ids = c("\\[Common Biological:Phosphorylation on S\\]"),
                                          useUniquePeptide = TRUE, 
                                          removeFewMeasurements = TRUE,
                                          removeProtein_with1Feature = FALSE, 
                                          summaryforMultipleRows = max,
                                          use_log_file = TRUE, 
                                          append = FALSE, 
                                          verbose = TRUE, 
                                          log_file_path = NULL) {
    
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path, 
                                        base = "MSstatsPTM_converter_log_")
    
    mod_id = mod_ids[[1]]
    input = as.data.table(input)
    
    ## Check input parameters
    checkmate::assertTRUE(!is.null(input))
    .checkAnnotation(annotation, "LF")
    if (!is.null(input_protein)) {
        checkmate::assertTRUE(!is.null(annotation_protein))
    }
    
    fasta = MSstatsPTM::tidyFasta(fasta_path)

    protein_id_col = "Protein Group"
    input = MSstatsPTMSiteLocator(input,
                                  protein_name_col=protein_id_col,
                                  unmod_pep_col="Base Sequence",
                                  mod_pep_col="Full Sequence",
                                  fasta_file=fasta,
                                  mod_id=mod_id,
                                  fasta_protein_name="uniprot_iso")

    if (use_unmod_peptides){
        input_protein = input[input[,..protein_id_col][[1]]  == input$ProteinNameUnmod]
        annotation_protein = annotation
    } else {
        input = input[input[,..protein_id_col][[1]] != input$ProteinNameUnmod]
    }
    
    ptm_input = MetamorpheusToMSstatsFormat(input,
                                            annotation,
                                            useUniquePeptide = useUniquePeptide, 
                                            removeFewMeasurements = removeFewMeasurements,
                                            removeProtein_with1Feature = removeProtein_with1Feature, 
                                            summaryforMultipleRows = summaryforMultipleRows,
                                            use_log_file = use_log_file, 
                                            append = append, 
                                            verbose = verbose, 
                                            log_file_path = log_file_path)
    
    msstats_format = list(PTM = ptm_input, PROTEIN = NULL)
    
    if (!is.null(input_protein)) {
        protein_input = MetamorpheusToMSstatsFormat(input_protein,
                                                    annotation_protein,
                                                    useUniquePeptide = useUniquePeptide, 
                                                    removeFewMeasurements = removeFewMeasurements,
                                                    removeProtein_with1Feature = removeProtein_with1Feature, 
                                                    summaryforMultipleRows = summaryforMultipleRows,
                                                    use_log_file = use_log_file, 
                                                    append = append, 
                                                    verbose = verbose, 
                                                    log_file_path = log_file_path)
        
        ptm_input = ptm_input[grepl(mod_id, ptm_input$PeptideSequence),]
        
        msstats_format = list(PTM = ptm_input, PROTEIN = protein_input)
    }
    
    return(msstats_format)
    
}


#' Generate MSstatsPTM required input format from Protein Prospector output
#' 
#' @author Anthony Wu
#' 
#' @param input Input txt peptide report file from Protein Prospector with 
#' "Keep Replicates", "Mods in Peptide", and "Protein Mods" options selected.
#' @param annotation data frame which contains column Run, Fraction, 
#' TechRepMixture, Mixture, Channel, BioReplicate, Condition. 
#' @param input_protein same as `input` for global profiling run. Default is NULL.
#' @param annotation_protein same as `annotation` for global profiling run. Default is NULL.
#' @param use_unmod_peptides If `protein_input` is not provided, 
#' unmodified peptides can be extracted from `input` to be used in place of a 
#' global profiling run. Default is `FALSE`.
#' @param mod_ids List of modifications of interest. Default 
#' is a list with only `Phospho`.
#' Please note that the 'mod_ids' parameter currently supports lists of size 1 only. 
#' Future updates aim to extend its functionality to accommodate lists of greater sizes.
#' @param useUniquePeptide TRUE (default) removes peptides that are assigned for
#'  more than one proteins. We assume to use unique peptide for each protein.
#' @param removeFewMeasurements TRUE (default) will remove the features that 
#' have 1 or 2 measurements across runs.
#' @param removeProtein_with1Feature TRUE will remove the proteins which have 
#' only 1 feature, which is the combination of peptide, precursor charge, 
#' fragment and charge. FALSE is default.
#' @param summaryforMultipleRows sum(default) or max - when there are multiple 
#' measurements for certain feature and certain run, use highest or sum of 
#' multiple intensities.
#' @param use_log_file logical. If TRUE, information about data processing will 
#' be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing wil be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. If not provided, such a file will be created 
#' automatically. If 'append = TRUE', has to be a valid path to a file. 
#' @return a list of two data.tables named 'PTM' and 'PROTEIN' in the format 
#' required by MSstatsPTM.
#' @importFrom MSstatsConvert ProteinProspectortoMSstatsTMTFormat
#' 
#' @export
#' 
#' @examples
#' input = system.file("tinytest/raw_data/ProteinProspector/Prospector_PhosphoTMT.txt",
#'     package = "MSstatsPTM")
#' input = data.table::fread(input)
#' annot = system.file("tinytest/raw_data/ProteinProspector/Annotation.csv",
#'                                 package = "MSstatsPTM")
#' annot = data.table::fread(annot)
#' input_protein = system.file("tinytest/raw_data/ProteinProspector/Prospector_TotalTMT.txt",
#'     package = "MSstatsConvert")
#' input_protein = data.table::fread(input_protein)
#' annot_protein = system.file("tinytest/raw_data/ProteinProspector/Annotation.csv",
#'                                 package = "MSstatsConvert")
#' annot_protein = data.table::fread(annot_protein)
#' output <- ProteinProspectortoMSstatsPTMFormat(
#'     input, 
#'     annot, 
#'     input_protein,
#'     annot_protein
#' )
#' head(output)
#' 
ProteinProspectortoMSstatsPTMFormat = function(
        input,
        annotation,
        input_protein = NULL,
        annotation_protein = NULL,
        use_unmod_peptides = FALSE,
        mod_ids = c("Phospho"),
        useUniquePeptide = TRUE, 
        removeFewMeasurements = TRUE,
        removeProtein_with1Feature = FALSE, 
        summaryforMultipleRows = sum,
        use_log_file = TRUE, 
        append = FALSE, 
        verbose = TRUE, 
        log_file_path = NULL
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path, 
                                        base = "MSstatsPTM_converter_log_")
    
    mod_id = mod_ids[[1]]
    input = as.data.table(input)
    
    ## Check input parameters 
    checkmate::assertTRUE(!is.null(input))
    .checkAnnotation(annotation, "TMT")
    if (!is.null(input_protein)) {
        checkmate::assertTRUE(!is.null(annotation_protein))
    }
    
    input$Peptide = gsub("TMT[0-9]*plex-", "", input$Peptide) # remove TMT tags
    input = input[!grepl("\\|", input$`Protein Mods`),] # filter out rows with uncertainty in protein mods
    input$Peptide = gsub(paste0("\\((?!", mod_id, "\\)).*?\\)"), "", input$Peptide, perl=TRUE) # filter out other modifications
    
    protein_id_col = "Acc #"
    input = MSstatsPTMSiteLocator(input, 
                                  protein_name_col=protein_id_col,
                                  unmod_pep_col = "DB Peptide",
                                  mod_pep_col = "Peptide",
                                  clean_mod=FALSE,
                                  mod_id=paste0("\\(", mod_id, "\\)"),
                                  bracket = "(",
                                  replace_text = TRUE
    )
    
    if (use_unmod_peptides){
        input_protein = input[input[,..protein_id_col][[1]]  == input$ProteinNameUnmod]
        annotation_protein = annotation
    } else {
        input = input[input[,..protein_id_col][[1]] != input$ProteinNameUnmod]
    }
    
    input[["DB Peptide"]] = input[[colnames(input)[grepl("new_peptide_col", colnames(input))][[1]]]]
    ptm_input = ProteinProspectortoMSstatsTMTFormat(input,
                                            annotation,
                                            useUniquePeptide, 
                                            removeFewMeasurements,
                                            removeProtein_with1Feature, 
                                            summaryforMultipleRows,
                                            use_log_file, 
                                            append, 
                                            verbose, 
                                            log_file_path)
    
    msstats_format = list(PTM = ptm_input, PROTEIN = NULL)
    
    if (!is.null(input_protein)) {
        protein_input = ProteinProspectortoMSstatsTMTFormat(input_protein,
                                                    annotation_protein,
                                                    useUniquePeptide, 
                                                    removeFewMeasurements,
                                                    removeProtein_with1Feature, 
                                                    summaryforMultipleRows,
                                                    use_log_file, 
                                                    append, 
                                                    verbose, 
                                                    log_file_path)
        
        ptm_input = ptm_input[grepl(mod_id, ptm_input$PeptideSequence),]
        
        msstats_format = list(PTM = ptm_input, PROTEIN = protein_input)
    }
    
    return(msstats_format)
}

## Utility functions

#' Validate positive number of rows from converter output
#' 
#' @author Anthony Wu
#' 
#' @param msstats_ptm_input A list containing PTM and PROTEIN data tables
#' @param global_profiling Default TRUE indicates msstats_ptm_input should have 
#' the PROTEIN element
.validatePositiveNumberOfRows = function(msstats_ptm_input, global_profiling = TRUE) {
    expect_true(nrow(msstats_ptm_input$PTM) > 0)
    if (global_profiling) {
      expect_true(nrow(msstats_ptm_input$PROTEIN) > 0) 
    } else {
      expect_true(is.null(msstats_ptm_input$PROTEIN))
    }
}

#' Validate protein ID count in converter output
#' 
#' @author Anthony Wu
#' 
#' @param msstats_input Either PTM or PROTEIN data tables
#' @param protein_id Protein ID to validate
#' @param count Expected count of protein ID in ProteinName column
.validateProteinId = function(msstats_input, protein_id, count) {
    expect_equal(length(which(msstats_input$ProteinName==protein_id)), count)
}

#' Validate ptm ID count in converter output
#' 
#' @author Anthony Wu
#' 
#' @param msstats_input Either PTM or PROTEIN data tables
#' @param ptm_substring PTM to validate
#' @param count Expected count of PTM in PeptideSequence column
.validatePtmSubstring = function(msstats_input, ptm_substring, count) {
    expect_equal(
        length(which(grepl(ptm_substring, msstats_input$PeptideSequence))),
        count
    )
}

## MaxQ TMT useUnmodPeptides = TRUE
data("maxq_tmt_evidence", package = "MSstatsPTM")
data("maxq_tmt_annotation", package = "MSstatsPTM")

expect_silent(MaxQtoMSstatsPTMFormat(evidence=maxq_tmt_evidence,
                                     annotation=maxq_tmt_annotation,
                                     fasta=system.file("extdata", "maxq_tmt_fasta.fasta", package="MSstatsPTM"),
                                     fasta_protein_name="uniprot_ac",
                                     use_unmod_peptides=TRUE,
                                     labeling_type = "TMT"))

expect_silent(MaxQtoMSstatsPTMFormat(evidence=maxq_tmt_evidence,
                                     annotation=maxq_tmt_annotation,
                                     fasta=system.file("extdata", "maxq_tmt_fasta.fasta", package="MSstatsPTM"),
                                     fasta_protein_name="uniprot_ac",
                                     use_unmod_peptides=FALSE,
                                     labeling_type = "TMT"))

expect_error(MaxQtoMSstatsPTMFormat(evidence=maxq_tmt_evidence,
                                    annotation=maxq_tmt_annotation,
                                     fasta_protein_name="uniprot_ac",
                                     use_unmod_peptides=FALSE,
                                     labeling_type = "TMT"))

expect_error(MaxQtoMSstatsPTMFormat(evidence=maxq_tmt_evidence,
                                    fasta_protein_name="uniprot_ac",
                                    fasta=system.file("extdata", "maxq_tmt_fasta.fasta", package="MSstatsPTM"),
                                    use_unmod_peptides=FALSE,
                                    labeling_type = "TMT"))

expect_silent(MaxQtoMSstatsPTMFormat(evidence=maxq_tmt_evidence,
                                     annotation=maxq_tmt_annotation,
                                     fasta=system.file("extdata", "maxq_tmt_fasta.fasta", package="MSstatsPTM"),
                                     fasta_protein_name="uniprot_ac",
                                     use_unmod_peptides=FALSE,
                                     labeling_type = "TMT",
                                     removeOxidationMpeptides=TRUE))

mq_imported = MaxQtoMSstatsPTMFormat(evidence=maxq_tmt_evidence,
                       annotation=maxq_tmt_annotation,
                       fasta=system.file("extdata", "maxq_tmt_fasta.fasta", package="MSstatsPTM"),
                       fasta_protein_name="uniprot_ac",
                       use_unmod_peptides=TRUE,
                       labeling_type = "TMT")
.validatePositiveNumberOfRows(mq_imported)
.validateProteinId(mq_imported$PROTEIN, "P29966", 10)
.validateProteinId(mq_imported$PTM, "P29966_T150", 70)
.validateProteinId(mq_imported$PTM, "P29966_T143_S145", 10)
.validatePtmSubstring(
  mq_imported$PTM, "Phospho \\(STY\\)", 
  length(mq_imported$PTM$PeptideSequence))
.validatePtmSubstring(
  mq_imported$PROTEIN, "Phospho \\(STY\\)", 0)

## MaxQ TMT useUnmodPeptides = FALSE
mq_imported = MaxQtoMSstatsPTMFormat(evidence=maxq_tmt_evidence,
                                     annotation=maxq_tmt_annotation,
                                     fasta=system.file("extdata", "maxq_tmt_fasta.fasta", package="MSstatsPTM"),
                                     fasta_protein_name="uniprot_ac",
                                     use_unmod_peptides=FALSE,
                                     labeling_type = "TMT")
.validatePositiveNumberOfRows(mq_imported, global_profiling = FALSE)
.validateProteinId(mq_imported$PTM, "P29966_T150", 70)
.validateProteinId(mq_imported$PTM, "P29966_T143_S145", 10)
.validatePtmSubstring(
  mq_imported$PTM, "Phospho \\(STY\\)", 
  length(mq_imported$PTM$PeptideSequence))

## MaxQ LF useUnmodPeptides = TRUE
data("maxq_lf_evidence", package = "MSstatsPTM")
data("maxq_lf_annotation", package = "MSstatsPTM")

expect_silent(MaxQtoMSstatsPTMFormat(evidence=maxq_lf_evidence,
                                     annotation=maxq_lf_annotation,
                                    fasta=system.file("extdata", "maxq_lf_fasta.fasta", package="MSstatsPTM"),
                                    fasta_protein_name="uniprot_ac",
                                    mod_id="\\(Phospho \\(STY\\)\\)",
                                    use_unmod_peptides=TRUE,
                                    labeling_type = "LF",
                                    which_proteinid_ptm = "Proteins"))

expect_error(MaxQtoMSstatsPTMFormat(evidence=maxq_lf_evidence,
                                    annotation=maxq_lf_annotation,
                                     fasta=system.file("extdata", "maxq_lf_fasta.fasta", package="MSstatsPTM"),
                                     fasta_protein_name="uniprot_ac",
                                     mod_id="\\(Phospho \\(STY\\)\\)",
                                     use_unmod_peptides=TRUE,
                                     labeling_type = "TMT",
                                     which_proteinid_ptm = "Proteins"))

expect_error(MaxQtoMSstatsPTMFormat(evidence=maxq_lf_evidence,
                                    annotation=maxq_lf_annotation,
                                    fasta_protein_name="uniprot_ac",
                                    mod_id="\\(Phospho \\(STY\\)\\)",
                                    use_unmod_peptides=TRUE,
                                    labeling_type = "TMT",
                                    which_proteinid_ptm = "Proteins"))

expect_error(MaxQtoMSstatsPTMFormat(evidence=maxq_lf_evidence,
                                    annotation=maxq_lf_annotation,
                                    fasta=system.file("extdata", "maxq_lf_fasta.fasta", package="MSstatsPTM"),
                                    fasta_protein_name="uniprot_ac",
                                    mod_id="\\(Phospho \\(STY\\)\\)",
                                    use_unmod_peptides=TRUE,
                                    labeling_type = "TMT",
                                    which_proteinid_ptm = "Leading.proteins"))

expect_silent(MaxQtoMSstatsPTMFormat(evidence=maxq_lf_evidence,
                                     annotation=maxq_lf_annotation,
                                     fasta=system.file("extdata", "maxq_lf_fasta.fasta", package="MSstatsPTM"),
                                     fasta_protein_name="uniprot_ac",
                                     mod_id="\\(Phospho \\(STY\\)\\)",
                                     use_unmod_peptides=FALSE,
                                     labeling_type = "LF",
                                     which_proteinid_ptm = "Proteins"))

mq_imported = MaxQtoMSstatsPTMFormat(evidence=maxq_lf_evidence,
                                     annotation=maxq_lf_annotation,
                                     fasta=system.file("extdata", "maxq_lf_fasta.fasta", package="MSstatsPTM"),
                                     fasta_protein_name="uniprot_ac",
                                     mod_id="\\(Phospho \\(STY\\)\\)",
                                     use_unmod_peptides=TRUE,
                                     labeling_type = "LF",
                                     which_proteinid_ptm = "Proteins")
.validatePositiveNumberOfRows(mq_imported)
.validateProteinId(mq_imported$PROTEIN, "P36578", 132)
.validateProteinId(mq_imported$PTM, "P36578_S295", 66)
.validateProteinId(mq_imported$PTM, "Q13523_S431_S437", 33)
.validatePtmSubstring(
    mq_imported$PTM, "Phospho \\(STY\\)", 
    length(mq_imported$PTM$PeptideSequence))
.validatePtmSubstring(
    mq_imported$PROTEIN, "Phospho \\(STY\\)", 0)

## MaxQ LF useUnmodPeptides = FALSE
mq_imported = MaxQtoMSstatsPTMFormat(evidence=maxq_lf_evidence,
                                     annotation=maxq_lf_annotation,
                                     fasta=system.file("extdata", "maxq_lf_fasta.fasta", package="MSstatsPTM"),
                                     fasta_protein_name="uniprot_ac",
                                     mod_id="\\(Phospho \\(STY\\)\\)",
                                     use_unmod_peptides=FALSE,
                                     labeling_type = "LF",
                                     which_proteinid_ptm = "Proteins")
.validatePositiveNumberOfRows(mq_imported, global_profiling = FALSE)
.validateProteinId(mq_imported$PTM, "P36578_S295", 66)
.validateProteinId(mq_imported$PTM, "Q13523_S431_S437", 33)
.validatePtmSubstring(
  mq_imported$PTM, "Phospho \\(STY\\)", 
  length(mq_imported$PTM$PeptideSequence))

## Spectronaut
data("spectronaut_input", package = "MSstatsPTM")
data("spectronaut_annotation", package = "MSstatsPTM")
expect_silent(SpectronauttoMSstatsPTMFormat(spectronaut_input,
                  annotation=spectronaut_annotation,
                  fasta_path=system.file("extdata", "spectronaut_fasta.fasta", package="MSstatsPTM"),
                  use_unmod_peptides=TRUE,
                  mod_id = "\\[Phospho \\(STY\\)\\]",
                  fasta_protein_name = "uniprot_iso"
                  ))

expect_error(SpectronauttoMSstatsPTMFormat(spectronaut_input,
                                            annotation=spectronaut_annotation,
                                            use_unmod_peptides=TRUE,
                                            mod_id = "[[[Phospho \\(STY\\)\\]",
                                            fasta_protein_name = "uniprot_iso"
))

expect_silent(SpectronauttoMSstatsPTMFormat(spectronaut_input,
                                           fasta_path=system.file("extdata", "spectronaut_fasta.fasta", package="MSstatsPTM"),
                                           use_unmod_peptides=TRUE,
                                           mod_id = "\\[Phospho \\(STY\\)\\]",
                                           fasta_protein_name = "uniprot_iso"
))

expect_silent(SpectronauttoMSstatsPTMFormat(spectronaut_input,
                                            annotation=spectronaut_annotation,
                                            fasta_path=system.file("extdata", "spectronaut_fasta.fasta", package="MSstatsPTM"),
                                            mod_id = "\\[Phospho \\(STY\\)\\]",
                                            fasta_protein_name = "uniprot_iso"
))

spectronaut_imported = SpectronauttoMSstatsPTMFormat(spectronaut_input,
                                            annotation=spectronaut_annotation,
                                            fasta_path=system.file("extdata", "spectronaut_fasta.fasta", package="MSstatsPTM"),
                                            use_unmod_peptides=TRUE,
                                            mod_id = "\\[Phospho \\(STY\\)\\]",
                                            fasta_protein_name = "uniprot_iso"
)
.validatePositiveNumberOfRows(spectronaut_imported)
.validateProteinId(spectronaut_imported$PROTEIN, "P36578", 36)
.validateProteinId(spectronaut_imported$PTM, "P36578_S295", 36)
.validateProteinId(spectronaut_imported$PTM, "P09938_S15_S22", 36)
.validatePtmSubstring(
    spectronaut_imported$PTM, "Phospho \\(STY\\)", 
    length(spectronaut_imported$PTM$PeptideSequence))
.validatePtmSubstring(
    spectronaut_imported$PROTEIN, "Phospho \\(STY\\)", 0)

## PD
data("pd_psm_input", package = "MSstatsPTM")
data("pd_annotation", package = "MSstatsPTM")
data("pd_testing_output", package = "MSstatsPTM")

expect_message(PDtoMSstatsPTMFormat(pd_psm_input, 
    pd_annotation,
    system.file("extdata", "pd_fasta.fasta", package="MSstatsPTM"),
    use_unmod_peptides=TRUE,
    which_proteinid = "Master.Protein.Accessions"))

expect_equal(pd_testing_output, PDtoMSstatsPTMFormat(pd_psm_input, 
                                   pd_annotation,
                                   system.file("extdata", "pd_fasta.fasta", 
                                               package="MSstatsPTM"),
                                   use_unmod_peptides=TRUE,
                                   which_proteinid = "Master.Protein.Accessions"))
 
expect_equal(pd_testing_output, PDtoMSstatsPTMFormat(pd_psm_input, 
                                    pd_annotation,
                                    system.file("extdata", "pd_fasta.fasta", 
                                                package="MSstatsPTM"),
                                    use_unmod_peptides=TRUE,
                                    which_proteinid = "Master.Protein.Accessions"))

expect_message(PDtoMSstatsPTMFormat(pd_psm_input, 
                                   pd_annotation,
                                   system.file("extdata", "pd_fasta.fasta", 
                                               package="MSstatsPTM"),
                                   use_unmod_peptides=TRUE,
                                   which_proteinid = "Master.Protein.Accessions",
                                   use_localization_cutoff = FALSE))

expect_error(PDtoMSstatsPTMFormat(pd_psm_input, 
                                   pd_annotation,
                                   use_unmod_peptides=TRUE,
                                   which_proteinid = "Master.Protein.Accessions"))

expect_message(PDtoMSstatsPTMFormat(pd_psm_input, 
                                  pd_annotation,
                                  system.file("extdata", "pd_fasta.fasta", 
                                              package="MSstatsPTM"),
                                  which_proteinid = "Master.Protein.Accessions"))

expect_error(PDtoMSstatsPTMFormat(pd_psm_input, 
                                  NULL,
                                  system.file("extdata", "pd_fasta.fasta", 
                                              package="MSstatsPTM"),
                                  which_proteinid = "Master.Protein.Accessions"))

expect_error(PDtoMSstatsPTMFormat(pd_psm_input, 
                                  data.frame(columns=c("Condition", "BioReplicate")),
                                  system.file("extdata", "pd_fasta.fasta", 
                                              package="MSstatsPTM"),
                                  which_proteinid = "Master.Protein.Accessions"))

input = system.file("tinytest/raw_data/PD/pd-ptm-input.csv", 
                    package = "MSstatsPTM")
input = data.table::fread(input)
annot = system.file("tinytest/raw_data/PD/pd-ptm-annot.csv",
                    package = "MSstatsPTM")
annot = data.table::fread(annot)
input_protein = system.file("tinytest/raw_data/PD/protein-input.csv",
                            package = "MSstatsPTM")
input_protein = data.table::fread(input_protein)
annot_protein = system.file("tinytest/raw_data/PD/protein-annot.csv",
                            package = "MSstatsPTM")
annot_protein = data.table::fread(annot_protein)
fasta_path=system.file("extdata", "pd_with_proteome.fasta", 
                       package="MSstatsPTM")
pd_imported = PDtoMSstatsPTMFormat(
    input,
    annotation = annot,
    protein_input = input_protein,
    annotation_protein = annot_protein,
    fasta_path = fasta_path,
    mod_id = "\\(GG\\)",
    labeling_type = "TMT",
    use_localization_cutoff = FALSE,
    which_proteinid = "Master.Protein.Accessions")

.validatePositiveNumberOfRows(pd_imported)
.validateProteinId(pd_imported$PROTEIN, "P52480", 30)
.validateProteinId(pd_imported$PTM, "P52480_K305", 12)
.validateProteinId(pd_imported$PTM, "P52480", 0)
.validatePtmSubstring(
    pd_imported$PTM, "\\*",
    length(pd_imported$PTM$PeptideSequence))
.validatePtmSubstring(
    pd_imported$PROTEIN, "\\*", 0)

## Metamorpheus

#### Setup
input = system.file("tinytest/raw_data/Metamorpheus/AllQuantifiedPeaks.tsv", 
                                package = "MSstatsPTM")
input = data.table::fread(input)
annot = system.file("tinytest/raw_data/Metamorpheus/ExperimentalDesign.tsv",
                                package = "MSstatsPTM")
annot = data.table::fread(annot)
input_protein = system.file("tinytest/raw_data/Metamorpheus/AllQuantifiedPeaksGlobalProteome.tsv",
                                package = "MSstatsPTM")
input_protein = data.table::fread(input_protein)
annot_protein = system.file("tinytest/raw_data/Metamorpheus/ExperimentalDesignGlobalProteome.tsv",
                                package = "MSstatsPTM")
annot_protein = data.table::fread(annot_protein)
fasta_path=system.file("extdata", "metamorpheus_fasta.fasta", 
                       package="MSstatsPTM")

#### Happy Case
expect_silent(MetamorpheusToMSstatsPTMFormat(
    input,
    annot,
    fasta_path=fasta_path,
    input_protein=input_protein,
    annotation_protein=annot_protein,
    mod_ids = c("\\[Common Fixed:Carbamidomethyl on C\\]")
))

#### Global profiling run removed
expect_silent(MetamorpheusToMSstatsPTMFormat(
    input,
    annot,
    fasta_path=fasta_path,
    use_unmod_peptides = FALSE,
    mod_ids = c("\\[Common Fixed:Carbamidomethyl on C\\]")
))

#### Global profiling run removed with use_unmod_peptides = TRUE
expect_silent(MetamorpheusToMSstatsPTMFormat(
    input,
    annot,
    fasta_path=fasta_path,
    use_unmod_peptides = TRUE,
    mod_ids = c("\\[Common Fixed:Carbamidomethyl on C\\]")
))

#### FASTA file missing
expect_error(MetamorpheusToMSstatsPTMFormat(
    input,
    annot,
    input_protein=input_protein,
    annotation_protein=annot_protein,
    mod_ids = c("\\[Common Fixed:Carbamidomethyl on C\\]")
))

#### Annotation missing
expect_error(MetamorpheusToMSstatsPTMFormat(
    input,
    fasta_path=fasta_path,
    input_protein=input_protein,
    annotation_protein=annot_protein,
    mod_ids = c("\\[Common Fixed:Carbamidomethyl on C\\]")
))

#### Output validation tests
metamorpheus_imported = MetamorpheusToMSstatsPTMFormat(
    input,
    annot,
    fasta_path=fasta_path,
    input_protein=input_protein,
    annotation_protein=annot_protein,
    mod_ids = c("\\[Common Fixed:Carbamidomethyl on C\\]")
)
.validatePositiveNumberOfRows(metamorpheus_imported)
.validateProteinId(metamorpheus_imported$PROTEIN, "O95817", 12)
.validateProteinId(metamorpheus_imported$PTM, "O95817_C207", 6)
.validateProteinId(metamorpheus_imported$PTM, "O95817", 0)
.validatePtmSubstring(
    metamorpheus_imported$PTM, "\\[Common Fixed:Carbamidomethyl on C\\]", 
    length(metamorpheus_imported$PTM$PeptideSequence))
.validatePtmSubstring(
    metamorpheus_imported$PROTEIN, "\\[Common Fixed:Carbamidomethyl on C\\]", 0)

metamorpheus_imported = MetamorpheusToMSstatsPTMFormat(
    input,
    annot,
    fasta_path=fasta_path,
    use_unmod_peptides = TRUE,
    mod_ids = c("\\[Common Fixed:Carbamidomethyl on C\\]")
)
.validatePositiveNumberOfRows(metamorpheus_imported)
.validateProteinId(metamorpheus_imported$PROTEIN, "O95817", 6)
.validateProteinId(metamorpheus_imported$PTM, "O95817_C207", 6)
.validateProteinId(metamorpheus_imported$PTM, "O95817", 0)
.validatePtmSubstring(
    metamorpheus_imported$PTM, "\\[Common Fixed:Carbamidomethyl on C\\]", 
    length(metamorpheus_imported$PTM$PeptideSequence))
.validatePtmSubstring(
    metamorpheus_imported$PROTEIN, "\\[Common Fixed:Carbamidomethyl on C\\]", 0)


# Progenesis
input = system.file("tinytest/raw_data/Progenesis/progenesis_peptide.csv", 
                    package = "MSstatsPTM")
input = data.table::fread(input)
colnames(input) = unlist(input[1,])
input = input[-1,]
annot = system.file("tinytest/raw_data/Progenesis/phospho_annotation.csv",
                    package = "MSstatsPTM")
annot = data.table::fread(annot)

expect_silent(ProgenesistoMSstatsPTMFormat(
    input,
    annot,
))

expect_error(ProgenesistoMSstatsPTMFormat(
    input
))

prog_imported = ProgenesistoMSstatsPTMFormat(
    input,
    annot,
)

.validateProteinId(prog_imported$PTM, 
    "sp|A2ASS6|TITIN_MOUSE_AVTSPPRVKSPEPR_[4] Phospho (ST)|[10] Phospho (ST)", 
    18
)


# Fragpipe
#### Happy Case
expect_silent(FragPipetoMSstatsPTMFormat(fragpipe_input,
                                         fragpipe_annotation,
                                         fragpipe_input_protein,
                                         fragpipe_annotation_protein,
                                         label_type="TMT",
                                         mod_id_col = "STY",
                                         localization_cutoff=.75,
                                         remove_unlocalized_peptides=TRUE)
              )

#### Global profiling run removed
expect_silent(FragPipetoMSstatsPTMFormat(fragpipe_input,
                                         fragpipe_annotation,
                                         label_type="TMT",
                                         mod_id_col = "STY",
                                         localization_cutoff=.75,
                                         remove_unlocalized_peptides=TRUE)
              )

#### use_unmod_peptides = TRUE, no unmodified proteins in dataset
expect_error(FragPipetoMSstatsPTMFormat(fragpipe_input,
                                         fragpipe_annotation,
                                         label_type="TMT",
                                         mod_id_col = "STY",
                                         localization_cutoff=.75,
                                         remove_unlocalized_peptides=TRUE,
                                         use_unmod_peptides=TRUE)
              )

#### Annotation missing
expect_error(FragPipetoMSstatsPTMFormat(fragpipe_input,
                                        fragpipe_input_protein,
                                        fragpipe_annotation_protein,
                                        label_type="TMT",
                                        mod_id_col = "STY",
                                        localization_cutoff=.75,
                                        remove_unlocalized_peptides=TRUE)
             )
fragpipe_input = FragPipetoMSstatsPTMFormat(fragpipe_input,
                                          fragpipe_annotation,
                                          fragpipe_input_protein,
                                          fragpipe_annotation_protein,
                                          label_type="TMT",
                                          mod_id_col = "STY",
                                          localization_cutoff=.75,
                                          remove_unlocalized_peptides=TRUE)
.validatePositiveNumberOfRows(fragpipe_input)
.validateProteinId(fragpipe_input$PROTEIN, "sp|Q9Y2W1|TR150_HUMAN", 110)
.validateProteinId(fragpipe_input$PTM, "sp|Q9Y2W1|TR150_HUMAN_S805", 20)
.validateProteinId(fragpipe_input$PTM, "sp|Q9Y2W1|TR150_HUMAN_S248_S253", 20)
.validatePtmSubstring(
    fragpipe_input$PTM, "\\*", 
    length(fragpipe_input$PTM$PeptideSequence))
.validatePtmSubstring(
    fragpipe_input$PROTEIN, "\\*", 0)

# LFQ Example (w/out global profiling run)
input = system.file("tinytest/raw_data/Fragpipe/MSstats.csv",
                                        package = "MSstatsPTM")
input = data.table::fread(input)
annot = system.file("tinytest/raw_data/Fragpipe/experiment_annotation.tsv",
                                        package = "MSstatsPTM")
annot = data.table::fread(annot)

#### Happy Case
expect_silent(FragPipetoMSstatsPTMFormat(input,
                                         annot,
                                         label_type="LF",
                                         mod_id_col = "STY",
                                         localization_cutoff=.75,
                                         protein_id_col = "ProteinName",
                                         peptide_id_col = "PeptideSequence")
)

#### Annotation file not provided but present in input
expect_silent(FragPipetoMSstatsPTMFormat(input,
                                        label_type="LF",
                                        mod_id_col = "STY",
                                        localization_cutoff=.75,
                                        protein_id_col = "ProteinName",
                                        peptide_id_col = "PeptideSequence")
)

fragpipe_input = FragPipetoMSstatsPTMFormat(input,
                                          annot,
                                          label_type="LF",
                                          mod_id_col = "STY",
                                          localization_cutoff=.75,
                                          protein_id_col = "ProteinName",
                                          peptide_id_col = "PeptideSequence")

.validateProteinId(fragpipe_input$PTM, "sp|P02400|RLA4_YEAST_S100", 8)
.validateProteinId(fragpipe_input$PTM, "sp|O13563|RPN13_YEAST_S132", 4)
.validatePtmSubstring(
    fragpipe_input$PTM, "\\*", 
    length(fragpipe_input$PTM$PeptideSequence))


## Protein Prospector

#### Setup
input = system.file("tinytest/raw_data/ProteinProspector/Prospector_PhosphoTMT.txt",
                    package = "MSstatsPTM")
input = data.table::fread(input)
annot = system.file("tinytest/raw_data/ProteinProspector/Annotation.csv",
                    package = "MSstatsPTM")
annot = data.table::fread(annot)
input_protein = system.file("tinytest/raw_data/ProteinProspector/Prospector_TotalTMT.txt",
                            package = "MSstatsConvert")
input_protein = data.table::fread(input_protein)
annot_protein = system.file("tinytest/raw_data/ProteinProspector/Annotation.csv",
                            package = "MSstatsConvert")
annot_protein = data.table::fread(annot_protein)

#### Happy Case
expect_silent(ProteinProspectortoMSstatsPTMFormat(
    input,
    annot,
    input_protein,
    annot_protein
))

#### Global profiling run removed
expect_silent(ProteinProspectortoMSstatsPTMFormat(
    input,
    annot
))

#### Global profiling run removed with use_unmod_peptides = TRUE
expect_silent(ProteinProspectortoMSstatsPTMFormat(
    input,
    annot,
    use_unmod_peptides = TRUE
))

#### Annotation missing
expect_error(ProteinProspectortoMSstatsPTMFormat(
    input,
    input_protein,
    annot_protein
))

#### Annotation global profiling missing
expect_error(ProteinProspectortoMSstatsPTMFormat(
    input,
    annot,
    input_protein
))

#### Output validation tests
protein_prospector_imported = ProteinProspectortoMSstatsPTMFormat(
    input,
    annot,
    input_protein,
    annot_protein
)
.validatePositiveNumberOfRows(protein_prospector_imported)
.validateProteinId(protein_prospector_imported$PROTEIN, "P16546", 234)
.validateProteinId(protein_prospector_imported$PTM, "P16546_S572", 6)
.validateProteinId(protein_prospector_imported$PTM, "Q9QYX7_S617_S625", 6)
.validateProteinId(protein_prospector_imported$PTM, "P16546", 0)
.validatePtmSubstring(
    protein_prospector_imported$PTM, "\\(Phospho\\)", 
    length(protein_prospector_imported$PTM$PeptideSequence))
.validatePtmSubstring(
    protein_prospector_imported$PROTEIN, "\\(Phospho\\)", 0)

#' Read and tidy a FASTA file
#'
#' \code{tidyFasta} reads and tidys FASTA file. Use this function as the first 
#' step in identifying modification sites.
#'
#' @param path A string of path to a FASTA file.
#'
#' @return A data.table with columns named \code{header}, \code{sequence},
#'   \code{uniprot_ac}, \code{uniprot_iso}, \code{entry_name}.
#'
#' @examples
#' tidyFasta(system.file("extdata", "O13297.fasta", package="MSstatsPTM"))
#'
#' @export
#' @importFrom Biostrings readAAStringSet
#' @importFrom data.table data.table is.data.table
#' @importFrom dplyr mutate group_by ungroup bind_rows
tidyFasta = function(path) {
  
  # Check input
  if (missing(path))
    stop(paste0("The input ", sQuote("path"), " is missing"))
  if (!is.character(path))
    stop("Provide the path to the FASTA file as a string")
  if (length(path) != 1)
    stop("Provide only one path to the FASTA file at a time")
  
  aa = readAAStringSet(path)
  aa = as.character(aa)  # named vector
  header = names(aa)
  
  pat_ac = paste0(
    "([OPQ][0-9][A-Z0-9]{3}[0-9]|[A-NR-Z][0-9]",
    "([A-Z][A-Z0-9]{2}[0-9]){1,2})"
  )
  pat_iso = paste0(
    "([OPQ][0-9][A-Z0-9]{3}[0-9]|[A-NR-Z][0-9]",
    "([A-Z][A-Z0-9]{2}[0-9]){1,2})([-]\\d{1,}){0,1}"
  )
  
  mch_head = regexpr(
    pattern = "([^\\s]*)(?=\\s)", text = header, perl = TRUE
  )
  shead = regmatches(header, m = mch_head)
  
  mch_uniprot = regexpr(pattern = pat_ac, text = shead)
  matched = mch_uniprot != (-1)
  ac = regmatches(shead, regexpr(pattern = pat_ac, text = shead))
  iso = regmatches(shead, regexpr(pattern = pat_iso, text = shead))
  
  trimmed = sub("^(sp|tr)(?=\\|)", "", shead, perl = TRUE)
  trimmed = sub(pat_iso, "", trimmed)
  entry = gsub("\\|", "", trimmed)
  
  data.table(
    header = header[matched], sequence = as.vector(aa[matched]),
    uniprot_ac = ac, uniprot_iso = iso, entry_name = entry[matched]
  )
}

#' Annotate modified sites with associated peptides
#'
#' \code{PTMlocate} annotates modified sites with associated peptides.
#'
#' @param peptide A string vector of peptide sequences. The peptide sequence
#'   does not include its preceding and following AAs.
#' @param uniprot A string vector of Uniprot identifiers of the peptides'
#'   originating proteins. UniProtKB entry isoform sequence is used.
#' @param fasta A data.table with FASTA information. Output of \code{tidyFasta}.
#' @param modResidue A string. Modifiable amino acid residues.
#' @param modSymbol A string. Symbol of a modified site.
#' @param rmConfound A logical. \code{TRUE} removes confounded
#'   unmodified sites, \code{FALSE} otherwise. Default is \code{FALSE}.
#'
#' @return A data frame with three columns: \code{uniprot_iso}, \code{peptide},
#'   \code{site}.
#'
#' @examples
#' fasta = tidyFasta(system.file("extdata", "O13297.fasta", package="MSstatsPTM"))
#' locatePTM("DRVSYIHNDSC*TR", "O13297", fasta, "C", "\\*")
#'
#' @export
locatePTM = function(peptide, uniprot, fasta, modResidue, modSymbol,
                      rmConfound=FALSE) {
  
  if (!.locateCheck(peptide, uniprot, fasta, modResidue, modSymbol))
    stop("Input checking fails!")
  
  peptide_seq = data.table(uniprot_iso = uniprot, peptide = peptide)
  peptide_seq$is_mod = grepl(modSymbol, peptide_seq$peptide)
  peptide_seq$peptide_unmod = gsub(modSymbol, "", peptide_seq$peptide)
  
  # Locate modifiable sites
  col_seq = c("uniprot_iso", "sequence")
  sub_fasta = fasta[fasta$uniprot_iso %in% uniprot, ..col_seq]
  loc = gregexpr(modResidue, sub_fasta$sequence, fixed = TRUE)
  sub_fasta$idx_site_full = lapply(loc, .location_start)
  
  # Locate peptides (use extended AAs for specific matching when possible)
  peptide_fasta = merge(peptide_seq, sub_fasta, all.x = TRUE, 
                         by = "uniprot_iso")
  loc = Map(function(p, s) gregexpr(p, s, fixed = TRUE)[[1]],
             as.list(peptide_fasta$peptide_unmod), as.list(peptide_fasta$sequence))
  n = vapply(loc, .num_match, FUN.VALUE = integer(1))
  peptide_fasta = peptide_fasta[n == 1L, ]
  loc = loc[n == 1L]
  peptide_fasta$idx_peptide = lapply(loc, .location)
  peptide_fasta$aa_start = vapply(loc, .location_start, numeric(1))
  
  # Pattern of modified sites
  mod_pattern = paste0(modResidue, modSymbol)
  
  # Locate modifiable, modified sites (AA residues) associated with peptides
  peptide_fasta$idx_site = Map(.covered_set,
                                peptide_fasta$idx_site_full, 
                                peptide_fasta$idx_peptide)
  peptide_fasta$idx_mod = Map(
    function(p, a) locateMod(p, a, residueSymbol = mod_pattern),
    as.list(peptide_fasta$peptide), as.list(peptide_fasta$aa_start)
  )
  peptide_fasta$mod_aa = Map(
    function(s, i) if (length(i) == 0) character() else substring(s, i, i),
    as.list(peptide_fasta$sequence), peptide_fasta$idx_mod
  )
  # Annotate modified sites
  peptide_fasta$len_site = lapply(
    peptide_fasta$idx_site_full, function(x) nchar(x[length(x)])
  )
  peptide_fasta$site = Map(annotSite,
                            peptide_fasta$idx_mod, peptide_fasta$mod_aa, 
                            peptide_fasta$len_site)
  peptide_fasta$site = as.character(peptide_fasta$site)
  col_fasta = c("uniprot_iso", "peptide", "peptide_unmod", "is_mod",
                 "idx_site", "idx_mod", "site")
  peptide_fasta = peptide_fasta[, ..col_fasta]
  
  .rmConfounded(peptide_fasta, rmConfound)
}

#' @noRd
.locateCheck = function(peptide, uniprot, fasta, modResidue, modSymbol) {
  if (missing(peptide))
    stop("Input peptide is missing!")
  if (missing(uniprot))
    stop("Input uniprot is missing!")
  if (missing(fasta))
    stop("Input fasta is missing!")
  if (missing(modResidue))
    stop("Input modResidue is missing!")
  if (missing(modSymbol))
    stop("Input modSymbol is missing!")
  if (!is.character(peptide))
    stop("Please provide peptide sequence as character in peptide!")
  if (!is.character(uniprot))
    stop("Please provide Uniprot protein ID as character in uniprot!")
  if (length(peptide) != length(uniprot))
    stop("peptide and uniprot must be of the same length")
  if (!is.data.table(fasta))
    stop("Please provide the FASTA information in a data table!")
  if (!all(c("uniprot_iso", "sequence") %in% names(fasta)))
    stop("Uniprot_iso or sequence is missing from FASTA data table!")
  TRUE
}

#' @noRd
#' @importFrom dplyr group_by ungroup bind_rows
.rmConfounded = function(peptideFasta, rmConfound) {
  # Handle confounded unmodified sites
  col_res = c("uniprot_iso", "peptide", "site")
  if (rmConfound) {
    by_prot = group_by(peptideFasta, .data$uniprot_iso)
    by_prot = mutate(
      by_prot, idx_mod_full = list(unique(unlist(.data$idx_mod)))
    )
    by_prot = ungroup(by_prot)
    by_prot = by_prot[!by_prot$is_mod, ]
    cnfnd = vector("logical", nrow(by_prot))
    for (i in seq_along(cnfnd)) {
      cnfnds = by_prot$idx_site[[i]] %in% by_prot$idx_mod_full[[i]]
      cnfnd[i] = any(cnfnds)
    }
    s_unmod = by_prot[!cnfnd, col_res]
    
    n1 = vapply(peptideFasta$idx_mod, .length_one, FUN.VALUE = logical(1))
    s_mod = peptideFasta[peptideFasta$is_mod & n1, ..col_res]
    res = bind_rows(s_mod, s_unmod)
  } else {
    n1 = vapply(peptideFasta$idx_mod, .length_one, FUN.VALUE = logical(1))
    res = peptideFasta[!peptideFasta$is_mod | n1, ..col_res]
  }
  res
}

#' @noRd
.num_match = function(x) {
  ifelse(
    is.na(x) || identical(as.vector(x), -1L),
    0L, length(attr(x, "match.length"))
  )
}

#' @noRd
.location_start = function(x) {
  start = as.vector(x)
  if (identical(start, -1L)) {
    return(integer())
  }
  start
}

#' @noRd
.location = function(x) {
  start = as.vector(x)
  if (identical(start, -1L)) {
    return(cbind(start = integer(), end = integer()))
  }
  end = as.vector(x) + attr(x, "match.length") - 1
  
  cbind(start = start, end = end)
}

#' @noRd
.covered_set = function(x, y) {
  x[x >= y[1] & x <= y[2]]
}

#' @noRd
.length_one = function(x) length(x) == 1

#' Locate modified sites with a peptide
#'
#' \code{locateMod} locates modified sites with a peptide.
#'
#' @param peptide A string. Peptide sequence.
#' @param aaStart An integer. Starting index of the peptide.
#' @param residueSymbol A string. Modification residue and denoted symbol.
#'
#' @return A string.
#'
#' @examples
#' locateMod("P*EP*TIDE", 3, "\\*")
#'
#' @export
locateMod = function(peptide, aaStart, residueSymbol) {
  x = gregexpr(residueSymbol, peptide)[[1]]
  start = as.vector(x)
  if (identical(start, -1L)) {
    return(integer())
  }
  start - seq_along(start) + aaStart
}

#' Annotate modification site
#'
#' \code{annotSite} annotates modified sites as their residues and locations.
#'
#' @param aaIndex An integer vector. Location of the sites.
#' @param residue A string vector. Amino acid residue.
#' @param lenIndex An integer. Default is \code{NULL}
#'
#' @return A string.
#'
#' @examples
#' annotSite(10, "K")
#' annotSite(10, "K", 3L)
#'
#' @export
annotSite = function(aaIndex, residue, lenIndex=NULL) {
  if (length(aaIndex) == 0) {
    return("None")
  } else {
    if (length(residue) != 1 && length(aaIndex) != length(residue))
      stop("Lengths of aaIndex and residue don't match!")
    if (!is.null(lenIndex)) {
      if (!is.integer(lenIndex))
        stop("lenIndex should be an integer!")
      if (max(nchar(aaIndex)) > lenIndex)
        stop("lenIndex doesn't reserve enough space for aaIndex!")
      
      aa_idx_pad = formatC(
        aaIndex, width = lenIndex, format = "d", flag = "0"
      )
      site = paste0(residue, aa_idx_pad, collapse = "-")
    } else {
      site = paste0(residue, aaIndex, collapse = "-")
    }
  }
  site
}

#' MSstatsPTM: A package for detecting differentially abundant 
#' post-translational modifications (PTM) in mass spectrometry-based 
#' proteomic experiments.
#'
#' A set of tools for detecting differentially abundant PTMs and proteins in
#' shotgun mass spectrometry-based proteomic experiments. The package can handle
#' a variety of acquisition types, including label free and TMT experiments, 
#' acquired with DDA, DIA, SRM or PRM acquisition methods. The 
#' package includes tools to convert raw data from different spectral processing
#' tools, summarize feature intensities, and fit a linear mixed effects model. A
#' major advantage of the package is to leverage a separate global profiling run
#' and adjust the PTM fold change for changes in the unmodified protein, showing
#' the unconvoluted PTM fold change. Finally, the package includes functionality
#' to plot a variety of data visualizations.
#'
#' @section functions :
#' \itemize{
#'   \item \code{\link{FragPipetoMSstatsPTMFormat}} : Generates MSstatsPTM 
#'   required input format for TMT FragePipe outputs.
#'   \item \code{\link{MaxQtoMSstatsPTMFormat}} : Generates MSstatsPTM required
#'   input format for label-free and TMT MaxQuant outputs.
#'   \item \code{\link{ProgenesistoMSstatsPTMFormat}} : Generates MSstatsPTM 
#'   required input format for label-free Progenesis outputs.
#'   \item \code{\link{SpectronauttoMSstatsPTMFormat}} : Generates MSstatsPTM 
#'   required input format for label-free Spectronaut outputs.
#'   \item \code{\link{SkylinetoMSstatsPTMFormat}} : Generates 
#'   MSstatsPTM required input format for Skyline outputs.
#'   \item \code{\link{PStoMSstatsPTMFormat}} : Generates 
#'   MSstatsPTM required input format for PEAKS outputs.
#'   \item \code{\link{PDtoMSstatsPTMFormat}} : Generates 
#'   MSstatsPTM required input format for Proteome Discoverer outputs.
#'   \item \code{\link{dataSummarizationPTM}} : Summarizes PSM level 
#'   quantification to peptide (modification) and protein level quantification.
#'   For use in label-free analysis
#'   \item \code{\link{dataSummarizationPTM_TMT}} : Summarizes PSM level 
#'   quantification to peptide (modification) and protein level quantification.
#'   For use in TMT analysis.
#'   \item \code{\link{dataProcessPlotsPTM}} : Visualization for explanatory
#'   data analysis. Specifically gives ability to plot Profile and Quality 
#'   Control plots.
#'   \item \code{\link{groupComparisonPTM}} : Tests for significant changes in 
#'   PTM and protein abundance across conditions. Adjusts PTM fold change for 
#'   changes in protein abundance.
#'   \item \code{\link{groupComparisonPlotsPTM}} : Visualization for model-based
#'    analysis and summarization
#' }
#'
#' @name MSstatsPTM
#' @keywords internal
"_PACKAGE"

#' Planning future experimental designs of PTM experiments in sample size calculation
#'
#' @description Calculate sample size for future experiments of a PTM 
#' experiment based on intensity-based linear model. Calculation is only 
#' available for group comparison experimental designs (not including time 
#' series).
#' Two options of the calculation: 
#' (1) number of biological replicates per condition, 
#' (2) power.
#' 
#' @param data output of the groupComparisonPTM function.
#' @param desiredFC the range of a desired fold change which includes the lower 
#' and upper values of the desired fold change.
#' @param FDR a pre-specified false discovery ratio (FDR) to control the overall 
#' false positive rate. Default is 0.05
#' @param numSample minimal number of biological replicates per condition. 
#' TRUE represents you require to calculate the sample size for this category, 
#' else you should input the exact number of biological replicates.
#' @param power a pre-specified statistical power which defined as the probability 
#' of detecting a true fold change. TRUE represent you require to calculate the power 
#' for this category, else you should input the average of power you expect. Default is 0.9
#' @param use_log_file logical. If TRUE, information about data processing
#' will be saved to a file.
#' @param append logical. If TRUE, information about data processing will be 
#' added to an existing log file.
#' @param verbose logical. If TRUE, information about data processing will be 
#' printed to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. 
#' If not provided, such a file will be created automatically.
#' If `append = TRUE`, has to be a valid path to a file.
#' @param base start of the file name.
#' 
#' @details The function fits the model and uses variance components to calculate 
#' sample size. The underlying model fitting with intensity-based linear model with 
#' technical MS run replication. Estimated sample size is rounded to 0 decimal.
#' The function can only obtain either one of the categories of the sample size 
#' calculation (numSample, numPep, numTran, power) at the same time.
#' 
#' @return data.frame - sample size calculation results including varibles:
#' desiredFC, numSample, FDR,  and power.
#' 
#' @importFrom MSstats designSampleSize
#' @importFrom stats median
#' 
#' @export
#' 
#' @examples
#' model.lf.msstatsptm = groupComparisonPTM(summary.data, 
#'                                      ptm_label_type="LF",
#'                                      protein_label_type="LF",
#'                                      verbose = FALSE)
#'                                      
#' #(1) Minimal number of biological replicates per condition
#' designSampleSizePTM(data=model.lf.msstatsptm, numSample=TRUE,
#'                  desiredFC=c(2.0,2.75), FDR=0.05, power=0.8)
#' #(2) Power calculation
#' designSampleSizePTM(data=model.lf.msstatsptm, numSample=5,
#'                  desiredFC=c(2.0,2.75), FDR=0.05, power=TRUE)                                  
#' 
#' 
designSampleSizePTM = function(
  data, desiredFC, FDR = 0.05, numSample = TRUE, power = 0.8, 
  use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL,
  base = "MSstatsPTM_log_"
){
  
  ## Start log  
  if (is.null(log_file_path) & use_log_file == TRUE){
    time_now = Sys.time()
    path = paste0(base, gsub("[ :\\-]", "_", time_now), 
                  ".log")
    file.create(path)
  } else {path = log_file_path}
  
  # if (data.type == 'TMT'){
  #   pkg = "MSstatsTMT"
  #   option_log = "MSstatsTMTLog"
  # } else {
  #   pkg = "MSstats"
  #   option_log = "MSstatsLog"
  # }
  pkg = "MSstats"
  option_log = "MSstatsLog"
  
  MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                      log_file_path, 
                                      "MSstatsPTM_sampleSize_log_")
  getOption(option_log)("INFO", "** MSstatsPTM - designSampleSizePTM function")
  getOption(option_log)("INFO", paste0("Desired fold change = ", 
                                         paste(desiredFC, collapse=" - ")))
  getOption(option_log)("INFO", paste0("FDR = ", FDR))
  getOption(option_log)("INFO", paste0("Power = ", power))
  
  fitted_models = data$Model.Details
  
  if (!is.null(fitted_models$PROTEIN)){
    
    ptm_var_component = MSstats:::.getVarComponent(fitted_models$PTM)
    protein_var_component = MSstats:::.getVarComponent(fitted_models$PROTEIN)
    
    ptm_median_sigma_error = median(ptm_var_component[["Error"]], na.rm = TRUE)
    protein_median_sigma_error = median(protein_var_component[["Error"]], 
                                        na.rm = TRUE)
    
    ptm_median_sigma_subject = MSstats:::.getMedianSigmaSubject(
      ptm_var_component)
    protein_median_sigma_subject = MSstats:::.getMedianSigmaSubject(
      protein_var_component)
    
    ## power calculation
    if (isTRUE(power)) {
      delta = log2(seq(desiredFC[1], desiredFC[2], 0.025))
      desiredFC = 2 ^ delta
      power_output = .calculatePowerPTM(desiredFC, FDR, delta, 
                                        ptm_median_sigma_error, 
                                        protein_median_sigma_error, 
                                        ptm_median_sigma_subject,
                                        protein_median_sigma_subject, numSample)
      getOption("MSstatsLog")("INFO", "Power is calculated. - okay")
      sample_size = data.frame(desiredFC, numSample, FDR, 
                               power = power_output)
    }	
    
    if (is.numeric(power)) {
      delta = log2(seq(desiredFC[1], desiredFC[2], 0.025))
      desiredFC = 2 ^ delta
      ## Large portion of proteins are not changing
      m0_m1 = 99 ## it means m0/m1=99, m0/(m0+m1)=0.99
      alpha = power * FDR / (1 + (1 - FDR) * m0_m1)
      if (isTRUE(numSample)) {
        numSample = .getNumSamplePTM(desiredFC, power, alpha, delta,
                                     ptm_median_sigma_error, 
                                     protein_median_sigma_error, 
                                     ptm_median_sigma_subject,
                                     protein_median_sigma_subject)
        
        getOption(option_log)("INFO", 
                              "The number of sample is calculated. - okay")
        sample_size = data.frame(desiredFC, numSample, FDR, power)
      }
    } 
    
    
  } else {
    sample_size = designSampleSize(fitted_models$PTM, desiredFC, FDR, numSample,
                                   power, use_log_file, append, verbose, 
                                   log_file_path)
  }
  
  return(sample_size)
  
}

#' Power calculation for PTM experiment
#' @inheritParams designSampleSizePTM
#' @importFrom stats qnorm
#' @return `float` of power
#' @keywords internal
.calculatePowerPTM = function(desiredFC, FDR, delta, ptm_median_sigma_error, 
                           protein_median_sigma_error, ptm_median_sigma_subject,
                           protein_median_sigma_subject, numSample) {
  m0_m1 = 99
  t = delta / sqrt(2 * (ptm_median_sigma_error/numSample + 
                          protein_median_sigma_error/numSample + 
                          ptm_median_sigma_subject/numSample + 
                          protein_median_sigma_subject/numSample))
  powerTemp = seq(0, 1, 0.01)
  power = numeric(length(t))
  for (i in seq_along(t)) {
    diff = qnorm(powerTemp) + qnorm(1 - powerTemp * 
                                      FDR / (1 + (1 - FDR) * m0_m1) / 2) - t[i]
    min(abs(diff), na.rm = TRUE)
    power[i] = powerTemp[order(abs(diff))][1]
  }
  return(power)
}

#' Get sample size for PTM experiment
#' @inheritParams designSampleSizePTM
#' @inheritParams .calculatePowerPTM
#' @importFrom stats qnorm
#' @return `int` of samples
#' @keywords internal
.getNumSamplePTM = function(desiredFC, power, alpha, delta, 
                            ptm_median_sigma_error, protein_median_sigma_error, 
                            ptm_median_sigma_subject, 
                            protein_median_sigma_subject){
  z_alpha = qnorm(1 - alpha / 2)
  z_beta = qnorm(power)
  aa = (delta / (z_alpha + z_beta)) ^ 2
  numSample = round(2 * (ptm_median_sigma_error + protein_median_sigma_error + 
                           ptm_median_sigma_subject + 
                           protein_median_sigma_subject) / aa, 0)
  return(numSample)
}

---
title: "MSstatsPTM TMT Workflow"
author: "Devon Kohler (<kohler.d@northeastern.edu>)"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{MSstatsPTM TMT Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width=8, 
  fig.height=8
)
```

```{r, message=FALSE, warning=FALSE}
library(MSstatsPTM)
```

This Vignette provides an example workflow for how to use the package 
MSstatsPTM for a TMT dataset.

## Installation

To install this package, start R (version "4.0") and enter:

``` {r, eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("MSstatsPTM")
```

## 1. Workflow

### 1.1 Raw Data Format

**Note: We are actively developing dedicated converters for MSstatsPTM. If you 
have data from a processing tool that does not have a dedicated converter in 
MSstatsPTM please add a github issue 
`https://github.com/Vitek-Lab/MSstatsPTM/issues` and we will add the 
converter.**

We go in depth on all converters included in this package in the 
`MSstatsPTM_LabelFree_Workflow`. For more information about data conversion 
please review the relevant sections there.

### 1.2 Summarization - dataSummarizationPTM_TMT

After loading in the input data, the next step is to use the 
dataSummarizationPTM_TMT function This provides the summarized dataset needed to 
model the protein/PTM abundance. The function will summarize the 
Protein dataset up to the protein level and will summarize the PTM dataset up to
the peptide level. There are multiple options for normalization and missing 
value imputation. These options should be reviewed in the package documentation.

```{r summarize, echo=FALSE, message=FALSE, warning=FALSE}

MSstatsPTM.summary <- dataSummarizationPTM_TMT(raw.input.tmt, 
                                               use_log_file = FALSE, 
                                               append = FALSE, verbose = FALSE)
```

```{r show_summ}
head(MSstatsPTM.summary$PTM$ProteinLevelData)
head(MSstatsPTM.summary$PROTEIN$ProteinLevelData)
```

The summarize function returns a list with PTM and Protein summarization 
information.

### 1.2.1 QCPlot

Once summarized, MSstatsPTM provides multiple plots to analyze the experiment. 
Here we show the quality control boxplot. The first plot shows the modified data
and the second plot shows the global protein dataset.

```{r qcplot, message=FALSE, warning=FALSE}

dataProcessPlotsPTM(MSstatsPTM.summary,
                    type = 'QCPLOT',
                    which.PTM = "allonly",
                    address = FALSE)
```

### 1.2.2 Profile Plot

Here we show a profile plot. Again the top plot shows the modified peptide, and 
the bottom shows the overall protein.

```{r profileplot, message=FALSE, warning=FALSE}
dataProcessPlotsPTM(MSstatsPTM.summary,
                    type = 'PROFILEPLOT',
                    which.Protein = c("Protein_12"),
                    address = FALSE)
```

### 1.3 Modeling - groupComparisonPTM

After summarization, the summarized datasets can be modeled using the 
groupComparisonPTM function. This function will model the PTM and Protein 
summarized datasets, and then adjust the PTM model for changes in overall 
protein abundance. The output of the function is a list containing these three 
models named: `PTM.Model`, `PROTEIN.Model`, `ADJUSTED.Model`.

```{r model, message=FALSE, warning=FALSE}

# Specify contrast matrix
comparison <- matrix(c(1,0,0,-1,0,0,
                       0,1,0,0,-1,0,
                       0,0,1,0,0,-1,
                       1,0,-1,0,0,0,
                       0,1,-1,0,0,0,
                       0,0,0,1,0,-1,
                       0,0,0,0,1,-1),nrow=7, ncol=6, byrow=TRUE)

# Set the names of each row
row.names(comparison)<-c('1-4', '2-5', '3-6', '1-3', 
                         '2-3', '4-6', '5-6')
colnames(comparison) <- c('Condition_1','Condition_2','Condition_3',
                          'Condition_4','Condition_5','Condition_6')
MSstatsPTM.model <- groupComparisonPTM(MSstatsPTM.summary,
                                       ptm_label_type = "TMT",
                                       protein_label_type = "TMT",
                                       contrast.matrix = comparison,
                                       use_log_file = FALSE, append = FALSE)
head(MSstatsPTM.model$PTM.Model)
head(MSstatsPTM.model$PROTEIN.Model)
head(MSstatsPTM.model$ADJUSTED.Model)
```

### 1.3.1 Volcano Plot

The models from the `groupComparisonPTM` function can be used in the model 
visualization function, `groupComparisonPlotsPTM`. Here we show Volcano Plots 
for the models.

``` {r volcano, message=FALSE, warning=FALSE}
groupComparisonPlotsPTM(data = MSstatsPTM.model,
                        type = "VolcanoPlot",
                        which.Comparison = c('1-4'),
                        which.PTM = 1:50,
                        address=FALSE)
```


### 1.3.2 Heatmap Plot

Here we show a Heatmap for the models.

``` {r meatmap, message=FALSE, warning=FALSE}
groupComparisonPlotsPTM(data = MSstatsPTM.model,
                        type = "Heatmap",
                        which.PTM = 1:49,
                        address=FALSE)
```

---
title: "MSstatsPTM LabelFree Workflow"
author: "Devon Kohler (<kohler.d@northeastern.edu>)"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{MSstatsPTM LabelFree Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width=8, 
  fig.height=8
)
```

This vignette provides an example workflow for how to use the package 
`MSstatsPTM` for a mass spectrometry based proteomics experiment acquired with 
a labelfree acquisition methods, such as DDA/DIA/SRM/PRM. It also provides 
examples and an analysis of how adjusting for global protein levels allows for 
better interpretations of PTM modeling results.

## Installation

To install this package, start R (version "4.0") and enter:

``` {r, eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("MSstatsPTM")
```

```{r, message=FALSE, warning=FALSE}
library(MSstatsPTM)
library(MSstats)
```

## 1. Workflow

### 1.1 Raw Data Format

**Note: We are actively developing dedicated converters for `MSstatsPTM`. If you 
have data from a processing tool that does not have a dedicated converter in 
MSstatsPTM please add a github issue 
`https://github.com/Vitek-Lab/MSstatsPTM/issues` and we will add the 
converter.**

The first step is to load in the raw dataset for both the PTM and Protein 
datasets. Each dataset can formatted using dedicated converters in `MSstatsPTM`,
such as `FragePipetoMSstatsPTMFormat`. If there is not a dedicated converter 
for your tool in `MSstatsPTM`, you can alternatively leverage converters from 
base `MSstats`. If using converters from `MSstats` note they will need to be 
run both on the global protein and PTM datasets.

Please note for the PTM dataset, both the protein and modification site (or 
peptide), must be added into the `ProteinName` column. This allows for the 
package to summarize to the peptide level, and avoid the off chance there are 
matching peptides between proteins. For an example of how this can be done 
please see the code below.

The output of the converter is a list with two formatted data.tables. One each 
for the PTM and Protein datasets. If a global profiling run was not performed, 
the Protein data.table will just be `NULL`

#### 1.1.1 MaxQuant - `MaxQtoMSstatsPTMFormat`

`MSstatsPTM` includes a dedicated converter for MaxQuant output. Experiments can
be acquired with label-free or TMT labeling methods. No matter the experiment, 
we recommend using the `evidence.txt` file, and not a PTM specific file such as 
the `Phospho (STY).txt` file. The `MaxQtoMSstatsPTMFormat` converter includes a
variety of parameters. Examples for processing a TMT and LF dataset can be seen 
below.

``` {r maxq, eval=TRUE}
# TMT experiment
head(maxq_tmt_evidence)
head(maxq_tmt_annotation)
 
msstats_format_tmt = MaxQtoMSstatsPTMFormat(evidence=maxq_tmt_evidence,
                                    annotation=maxq_tmt_annotation,
                                    fasta=system.file("extdata", 
                                                      "maxq_tmt_fasta.fasta", 
                                                      package="MSstatsPTM"),
                                    fasta_protein_name="uniprot_ac",
                                    mod_id="\\(Phospho \\(STY\\)\\)",
                                    use_unmod_peptides=TRUE,
                                    labeling_type = "TMT",
                                    which_proteinid_ptm = "Proteins")

head(msstats_format_tmt$PTM)
head(msstats_format_tmt$PROTEIN)

# LF experiment
head(maxq_lf_evidence)
head(maxq_lf_annotation)

msstats_format_lf = MaxQtoMSstatsPTMFormat(evidence=maxq_lf_evidence,
                                     annotation=maxq_lf_annotation,
                                     fasta=system.file("extdata", 
                                                       "maxq_lf_fasta.fasta", 
                                                       package="MSstatsPTM"),
                                     fasta_protein_name="uniprot_ac",
                                     mod_id="\\(Phospho \\(STY\\)\\)",
                                     use_unmod_peptides=TRUE,
                                     labeling_type = "LF",
                                     which_proteinid_ptm = "Proteins")

head(msstats_format_lf$PTM)
head(msstats_format_lf$PROTEIN)

```

#### 1.1.2 FragPipe - `FragPipetoMSstatsPTMFormat`

`MSstatsPTM` includes a dedicated converter for FragPipe output. The input is the 
`msstats.csv` and annotation files automatically generated by FragPipe. 
FragPipe provides additional info on the  localization of modification sites, 
and the `FragPipetoMSstatsPTMFormat` converter includes localization options 
that are not present in other converters.

``` {r fragpipe, eval=TRUE}
# TMT example
head(fragpipe_input)
head(fragpipe_annotation)
head(fragpipe_input_protein)
head(fragpipe_annotation_protein)

msstats_data = FragPipetoMSstatsPTMFormat(fragpipe_input,
                                          fragpipe_annotation,
                                          fragpipe_input_protein, 
                                          fragpipe_annotation_protein,
                                          mod_id_col = "STY",
                                          localization_cutoff=.75,
                                          remove_unlocalized_peptides=TRUE)
head(msstats_data$PTM)
head(msstats_data$PROTEIN)

# LFQ Example
input = system.file("tinytest/raw_data/Fragpipe/MSstats.csv", 
                                        package = "MSstatsPTM")
input = data.table::fread(input)
annot = system.file("tinytest/raw_data/Fragpipe/experiment_annotation.tsv", 
                                        package = "MSstatsPTM")
annot = data.table::fread(annot)    
input_protein = system.file("tinytest/raw_data/Fragpipe/msstats_proteome_lf.csv",
                                        package = "MSstatsPTM")

input_protein = data.table::fread(input_protein) # Global profiling run

msstats_data = FragPipetoMSstatsPTMFormat(input,
                                          annot,
                                          input_protein = input_protein,
                                          label_type="LF",
                                          mod_id_col = "STY",
                                          localization_cutoff=.75,
                                          protein_id_col = "ProteinName",
                                          peptide_id_col = "PeptideSequence")
head(msstats_data$PTM)
head(msstats_data$PROTEIN)
```





#### 1.1.3 Proteome Discoverer - `PDtoMSstatsPTMFormat`

`MSstatsPTM` includes a dedicated converter for Proteome Discoverer output. 
Experiments can be acquired with label-free or TMT labeling methods. The input 
is the `psm` file and a custom built annotation file. Proteome Discoverer 
provides additional info on the localization of modification sites, and the 
`PDtoMSstatsPTMFormat` converter includes localization options that are not 
present in other converters.

``` {r pd, eval=TRUE}
head(pd_psm_input)
head(pd_annotation)

msstats_format = PDtoMSstatsPTMFormat(pd_psm_input, 
                                 pd_annotation, 
                                 system.file("extdata", "pd_fasta.fasta", 
                                             package="MSstatsPTM"),
                                 use_unmod_peptides=TRUE, 
                                 which_proteinid = "Master.Protein.Accessions")

head(msstats_format$PTM)
head(msstats_format$PROTEIN)
```


#### 1.1.4 Spectronaut - `SpectronauttoMSstatsPTMFormat`

`MSstatsPTM` includes a dedicated converter for Spectronaut output. 
Experiments can be acquired with label-free labeling methods.

``` {r Spectronaut, eval=TRUE}
head(spectronaut_input)
head(spectronaut_annotation)

msstats_input = SpectronauttoMSstatsPTMFormat(spectronaut_input, 
                  annotation=spectronaut_annotation, 
                  fasta_path=system.file("extdata", "spectronaut_fasta.fasta", 
                                         package="MSstatsPTM"),
                  use_unmod_peptides=TRUE,
                  mod_id = "\\[Phospho \\(STY\\)\\]",
                  fasta_protein_name = "uniprot_iso"
                  )
 
head(msstats_input$PTM)
head(msstats_input$PROTEIN)

```


#### 1.1.5 Skyline - `SkylinetoMSstatsPTMFormat`

`MSstatsPTM` includes a dedicated converter for Skyline output. 
Experiments can be acquired with label-free labeling methods.


#### 1.1.6 Peak Studio - `PStoMSstatsPTMFormat`

`MSstatsPTM` includes a dedicated converter for Peak Studio output. 
Experiments can be acquired with label-free labeling methods.

#### 1.1.6 Progenesis - `ProgenesistoMSstatsPTMFormat`

`MSstatsPTM` includes a dedicated converter for Progenesis output. 
Experiments can be acquired with label-free labeling methods.

#### 1.1.7 FASTA File

You might notice a FASTA file is also needed for some converters. This
FASTA file can be obtained by querying
[Uniprot](https://www.uniprot.org/id-mapping) with all of the protein
IDs present in your PTM dataset.  

Follow these steps to download a FASTA file from UniProt:

1. **Prepare your protein IDs** Make a list of your protein IDs (e.g., UniProt accessions like P31749, Q9Y243, etc.). You can copy-paste them into the UniProt tool.
2. **Go to the UniProt ID mapping tool** Open the UniProt ID mapping page: https://www.uniprot.org/id-mapping
3. **Select input database** In the "From" dropdown, choose: UniProtKB AC/ID
4. **Select output database** In the "To" dropdown:
    + If isoform-specific data is needed, select: UniProtKB/Swiss-Prot
    + Otherwise, select: UniProtKB
5. **Submit your IDs** Paste your protein IDs into the text box and click "Submit".
6. **Download the FASTA file** Once the mapping is complete, you’ll see a results page.
    + Click the "Download" button on the top of the table.
    + Select "FASTA (canonical)" and click download

The FASTA file is a necessary input because some tools (e.g. MaxQuant) do not 
report the specific amino acid that is modified relative to the whole *protein* 
sequence. Rather, they report the specific amino acid relative to the reported 
*peptide*. This distinction is important because modifications like phosphorylation,
methylation, or acetylation often have specific roles depending on where
they occur within the full-length protein. With the help of a FASTA
file, MSstatsPTM can determine the specific amino acid that is modified
in the context of the whole protein sequence.

#### 1.1.8 Additional tools

If there is not a dedicated `MSstatsPTM` converter for a processing tool, 
the existing converters in `MSstats` and `MSstatsTMT` converters can be used as 
described below. 

Note, in order to do this, it is critical that the ProteinName column be a 
combination of the Protein Name and modification site.

As an example, if you would like to analyze an experiment processed with DIANN, 
you can leverage the `DIANNtoMSstatsFormat` in `MSstats`. Given two datasets, 
named `raw_ptm_df` and `raw_protein_df`, and an annotation file, we can process 
the data as follows.

```{r raw_data, eval = FALSE}
# Add site into ProteinName column
raw_ptm_df$ProteinName = paste(raw_ptm_df$ProteinName,
                                raw_ptm_df$Site, sep = "_")

# Run MSstats Converters
PTM_data = MSstats::DIANNtoMSstatsFormat(raw_ptm_df, annotation)
PROTEIN_data = MSstats::DIANNtoMSstatsFormat(raw_protein_df, annotation)

# Combine into one list
msstatsptm_input_data = list(PTM = PTM_data, PROTEIN = PROTEIN_data)
```

The variable `msstatsptm_input_data` can now be used as the input to the 
remainder of the `MSstatsPTM` processing pipeline.

### 1.2 Summarization - `dataSummarizationPTM`

After loading in the input data, the next step is to use the 
`dataSummarizationPTM` function. This provides the summarized dataset needed to 
model the protein/PTM abundance. The function will summarize the 
Protein dataset up to the protein level and will summarize the PTM dataset up to
the peptide level. There are multiple options for normalization and missing 
value imputation. These options should be reviewed in the package documentation.

```{r summarize, message=FALSE, warning=FALSE}

MSstatsPTM.summary = dataSummarizationPTM(raw.input, verbose = FALSE, 
                                          use_log_file = FALSE, append = FALSE)

head(MSstatsPTM.summary$PTM$ProteinLevelData)
head(MSstatsPTM.summary$PROTEIN$ProteinLevelData)
```

The summarize function returns a list with PTM and Protein summarization 
information. Each PTM and Protein include a list of data.tables: `FeatureLevelData`
is a data.table of reformatted input of dataSummarizationPTM, `ProteinLevelData` is 
the run level summarization data.

### 1.2.1 QCPlot

Once summarized, MSstatsPTM provides multiple plots to analyze the experiment. 
Here we show the quality control boxplot. The first plot shows the modified data
and the second plot shows the global protein dataset.

```{r qcplot, message=FALSE, warning=FALSE}
dataProcessPlotsPTM(MSstatsPTM.summary,
                    type = 'QCPLOT',
                    which.PTM = "allonly",
                    address = FALSE)
```

### 1.2.2 Profile Plot

Here we show a profile plot. Again the top plot shows the modified peptide, and 
the bottom shows the overall protein.

```{r profileplot, message=FALSE, warning=FALSE}

dataProcessPlotsPTM(MSstatsPTM.summary,
                    type = 'ProfilePlot',
                    which.Protein = "Q9Y6C9",
                    address = FALSE)
```

### 1.3 Modeling - `groupComparisonPTM`

After summarization, the summarized datasets can be modeled using the 
`groupComparisonPTM` function. This function will model the PTM and Protein 
summarized datasets, and then adjust the PTM model for changes in overall 
protein abundance. The output of the function is a list containing these three 
models named: `PTM.Model`, `PROTEIN.Model`, `ADJUSTED.Model`.

```{r model, message=FALSE, warning=FALSE}

# Specify contrast matrix
comparison = matrix(c(-1,0,1,0),nrow=1)
row.names(comparison) = "CCCP-Ctrl"
colnames(comparison) = c("CCCP", "Combo", "Ctrl", "USP30_OE")

MSstatsPTM.model = groupComparisonPTM(MSstatsPTM.summary, 
                                      ptm_label_type = "LF",
                                      protein_label_type = "LF",
                                      contrast.matrix = comparison,
                                      use_log_file = FALSE, append = FALSE,
                                      verbose = FALSE)
head(MSstatsPTM.model$PTM.Model)
head(MSstatsPTM.model$PROTEIN.Model)
head(MSstatsPTM.model$ADJUSTED.Model)
```

### 1.3.1 Volcano Plot

The models from the `groupComparisonPTM` function can be used in the model 
visualization function, `groupComparisonPlotsPTM`. Here we show Volcano Plots 
for the models.

``` {r volcano, message=FALSE, warning=FALSE}
groupComparisonPlotsPTM(data = MSstatsPTM.model,
                        type = "VolcanoPlot",
                        FCcutoff= 2,
                        logBase.pvalue = 2,
                        address=FALSE)
```


### 1.3.2 Heatmap Plot

Here we show a Heatmap for the models.

``` {r heatmap, message=FALSE, warning=FALSE}
groupComparisonPlotsPTM(data = MSstatsPTM.model,
                        type = "Heatmap",
                        which.PTM = 1:30,
                        address=FALSE)
```

### 1.4 Sample Size Calculation - `designSampleSizePTM`

Finally, sample size calculation can be performed using the output of the model 
and the `designSampleSizePTM`

```{r sample_size, message=FALSE, warning=FALSE}

# Specify contrast matrix
sample_size = designSampleSizePTM(MSstatsPTM.model, c(2.0, 2.75), FDR = 0.05, 
                                  numSample = TRUE, power = 0.8)

head(sample_size)
```

### 1.4.1 Sample Size Plot

The output of the sample size function can be plotted using the `MSstats` 
`designSampleSizePlots` function.

```{r sample_size_plot, message=FALSE, warning=FALSE}

MSstats::designSampleSizePlots(sample_size)

```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{pd_testing_output}
\alias{pd_testing_output}
\title{Example output of Proteome Discoverer converter}
\format{
a list with 2 data.frames
}
\usage{
pd_testing_output
}
\description{
output using example data provided in package
}
\details{
The experiment did not contain a global profiling run, but we show an example
of extracting the unmodified peptides and using them in place of the
profiling run.
}
\examples{
head(pd_testing_output)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/designSampleSizePTM.R
\name{.calculatePowerPTM}
\alias{.calculatePowerPTM}
\title{Power calculation for PTM experiment}
\usage{
.calculatePowerPTM(
  desiredFC,
  FDR,
  delta,
  ptm_median_sigma_error,
  protein_median_sigma_error,
  ptm_median_sigma_subject,
  protein_median_sigma_subject,
  numSample
)
}
\arguments{
\item{desiredFC}{the range of a desired fold change which includes the lower
and upper values of the desired fold change.}

\item{FDR}{a pre-specified false discovery ratio (FDR) to control the overall
false positive rate. Default is 0.05}

\item{numSample}{minimal number of biological replicates per condition.
TRUE represents you require to calculate the sample size for this category,
else you should input the exact number of biological replicates.}
}
\value{
\code{float} of power
}
\description{
Power calculation for PTM experiment
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{ProgenesistoMSstatsPTMFormat}
\alias{ProgenesistoMSstatsPTMFormat}
\title{Converts non-TMT Progenesis output into the format needed for MSstatsPTM}
\usage{
ProgenesistoMSstatsPTMFormat(
  ptm_input,
  annotation,
  global_protein_input = FALSE,
  fasta_path = FALSE,
  useUniquePeptide = TRUE,
  summaryforMultipleRows = max,
  removeFewMeasurements = TRUE,
  removeOxidationMpeptides = FALSE,
  removeProtein_with1Peptide = FALSE,
  mod.num = "Single"
)
}
\arguments{
\item{ptm_input}{name of Progenesis output with modified peptides, which is
wide-format. 'Accession', Sequence', 'Modification', 'Charge' and one column
for each run are required}

\item{annotation}{name of 'annotation.txt' or 'annotation.csv' data which
includes Condition, BioReplicate, Run, and Type (PTM or Protein)
information. It will be matched with the column name of input for MS runs.
Please note PTM and global Protein run names are often different, which is
why an additional Type column indicating Protein or PTM is required.}

\item{global_protein_input}{name of Progenesis output with unmodified
peptides, which is wide-format. 'Accession', Sequence', 'Modification',
'Charge' and one column for each run are required}

\item{fasta_path}{string containing path to the corresponding fasta file for
the modified peptide dataset.}

\item{useUniquePeptide}{TRUE(default) removes peptides that are assigned for
more than one proteins. We assume to use unique peptide for each protein.}

\item{summaryforMultipleRows}{max(default) or sum - when there are multiple
measurements for certain feature and certain run, use highest or sum of
multiple intensities.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that
have 1 or 2 measurements across runs.}

\item{removeOxidationMpeptides}{TRUE will remove the modified peptides
including 'Oxidation (M)' sequence. FALSE is default.}

\item{removeProtein_with1Peptide}{TRUE will remove the proteins which have
only 1 peptide and charge. FALSE is default.}

\item{mod.num}{For modified peptide dataset, must be one of \code{Single} or
\code{Total}. The default is \code{Single}. The number modifications per peptide to be
used. If "Single", only peptides with one modification will be
used. Otherwise "Total" includes peptides with more than one modification.
Selecting "Total" may confound the effect of different modifications.}
}
\value{
a list of two data.tables named 'PTM' and 'PROTEIN' in the format
required by MSstatsPTM.
}
\description{
Converts non-TMT Progenesis output into the format needed for MSstatsPTM
}
\examples{

input = system.file("tinytest/raw_data/Progenesis/progenesis_peptide.csv", 
                                package = "MSstatsPTM")
input = data.table::fread(input)
colnames(input) = unlist(input[1,])
input = input[-1,]
annot = system.file("tinytest/raw_data/Progenesis/phospho_annotation.csv", 
                                package = "MSstatsPTM")
annot = data.table::fread(annot)
prog_imported = ProgenesistoMSstatsPTMFormat(
    input, 
    annot
)
head(prog_imported$PTM)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{fragpipe_input_protein}
\alias{fragpipe_input_protein}
\title{Output of FragPipe TMT global profiling experiment}
\format{
A data.table with 27 columns and 47 rows.
}
\usage{
fragpipe_input_protein
}
\description{
This dataset was provided by the FragPipe team at the Nesvilab. It was
processed using Philosopher and targeted Phosphorylation.
}
\examples{
head(fragpipe_input_protein)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/designSampleSizePTM.R
\name{designSampleSizePTM}
\alias{designSampleSizePTM}
\title{Planning future experimental designs of PTM experiments in sample size calculation}
\usage{
designSampleSizePTM(
  data,
  desiredFC,
  FDR = 0.05,
  numSample = TRUE,
  power = 0.8,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  base = "MSstatsPTM_log_"
)
}
\arguments{
\item{data}{output of the groupComparisonPTM function.}

\item{desiredFC}{the range of a desired fold change which includes the lower
and upper values of the desired fold change.}

\item{FDR}{a pre-specified false discovery ratio (FDR) to control the overall
false positive rate. Default is 0.05}

\item{numSample}{minimal number of biological replicates per condition.
TRUE represents you require to calculate the sample size for this category,
else you should input the exact number of biological replicates.}

\item{power}{a pre-specified statistical power which defined as the probability
of detecting a true fold change. TRUE represent you require to calculate the power
for this category, else you should input the average of power you expect. Default is 0.9}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be
printed to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{base}{start of the file name.}
}
\value{
data.frame - sample size calculation results including varibles:
desiredFC, numSample, FDR,  and power.
}
\description{
Calculate sample size for future experiments of a PTM
experiment based on intensity-based linear model. Calculation is only
available for group comparison experimental designs (not including time
series).
Two options of the calculation:
(1) number of biological replicates per condition,
(2) power.
}
\details{
The function fits the model and uses variance components to calculate
sample size. The underlying model fitting with intensity-based linear model with
technical MS run replication. Estimated sample size is rounded to 0 decimal.
The function can only obtain either one of the categories of the sample size
calculation (numSample, numPep, numTran, power) at the same time.
}
\examples{
model.lf.msstatsptm = groupComparisonPTM(summary.data, 
                                     ptm_label_type="LF",
                                     protein_label_type="LF",
                                     verbose = FALSE)
                                     
#(1) Minimal number of biological replicates per condition
designSampleSizePTM(data=model.lf.msstatsptm, numSample=TRUE,
                 desiredFC=c(2.0,2.75), FDR=0.05, power=0.8)
#(2) Power calculation
designSampleSizePTM(data=model.lf.msstatsptm, numSample=5,
                 desiredFC=c(2.0,2.75), FDR=0.05, power=TRUE)                                  


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{FragPipetoMSstatsPTMFormat}
\alias{FragPipetoMSstatsPTMFormat}
\title{Convert output of TMT labeled Fragpipe data into MSstatsPTM format.}
\usage{
FragPipetoMSstatsPTMFormat(
  input,
  annotation = NULL,
  input_protein = NULL,
  annotation_protein = NULL,
  use_unmod_peptides = FALSE,
  label_type = "TMT",
  protein_id_col = "Protein",
  peptide_id_col = "Peptide.Sequence",
  mod_id_col = "STY",
  localization_cutoff = 0.75,
  remove_unlocalized_peptides = TRUE,
  Purity_cutoff = 0.6,
  PeptideProphet_prob_cutoff = 0.7,
  useUniquePeptide = TRUE,
  rmPSM_withfewMea_withinRun = FALSE,
  rmPeptide_OxidationM = TRUE,
  rmProtein_with1Feature = FALSE,
  summaryforMultipleRows = sum,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL
)
}
\arguments{
\item{input}{data.frame of \code{msstats.csv} file produced by Philosopher}

\item{annotation}{annotation with Run, Fraction, TechRepMixture, Mixture, Channel,
BioReplicate, Condition columns or a path to file. Refer to the example 'annotation' for the meaning of each column. Channel column should be
consistent with the channel columns (Ignore the prefix "Channel ") in msstats.csv file. Run column should be consistent with the Spectrum.File columns in msstats.csv file.}

\item{input_protein}{same as \code{input} for global profiling run. Default is NULL.}

\item{annotation_protein}{same as \code{annotation} for global profiling run. Default is NULL.}

\item{use_unmod_peptides}{Boolean if the unmodified peptides in the input
file should be used to construct the unmodified protein output. Only used if
\code{input_protein} is not provided. Default is \code{FALSE}.}

\item{label_type}{Type of labeling used for experiment. Must be one of "LF"
or "TMT". Default is "TMT".}

\item{protein_id_col}{Use 'Protein'(default) column for TMT. This needs to be
changed to "ProteinName" for label free. For TMT, 'Master.Protein.Accessions'
can be used instead to get the protein ID with single protein.}

\item{peptide_id_col}{Use 'Peptide.Sequence'(default) column for TMT. Must be
changed to "PeptideSequence" for label free. "Modified.Peptide.Sequence" can
be used instead to get the modified peptide sequence.}

\item{mod_id_col}{Column containing the modified Amino Acids. For example, a Phosphorylation experiment may pass \code{STY}. The corresponding column with \code{STY} combined with the mass (e.x. \code{STY.79.9663}) will be selected. Default is \code{STY}.}

\item{localization_cutoff}{Minimum localization score required to keep modification. Default is .75.}

\item{remove_unlocalized_peptides}{Boolean indicating if peptides without all sites localized should be kept. Default is TRUE (non-localized sites will be removed).}

\item{Purity_cutoff}{Cutoff for purity. Default is 0.6. Purity refers to how much of the detected ion signal
within a specific inclusion window belongs to the target molecule or its closely related forms,
compared to any other unwanted signals or noise. Higher values indicate greater purity.}

\item{PeptideProphet_prob_cutoff}{Cutoff for the peptide identification probability. Default is 0.7.
The probability is confidence score determined by PeptideProphet and higher values indicate greater confidence.}

\item{useUniquePeptide}{logical, if TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{rmPSM_withfewMea_withinRun}{TRUE will remove the features that have 1 or 2 measurements within each Run. Default is FALSE.}

\item{rmPeptide_OxidationM}{TRUE (default) will remove the peptides including oxidation (M) sequence.}

\item{rmProtein_with1Feature}{TRUE will remove the proteins which have only 1 peptide and charge. Default is FALSE.}

\item{summaryforMultipleRows}{sum (default) or max - when there are multiple measurements for certain feature in certain run,
select the feature with the largest summation or maximal value.}

\item{use_log_file}{logical. If TRUE, information about data processing will
be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing wil be
printed to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved. If not provided, such a file will be created
automatically. If 'append = TRUE', has to be a valid path to a file.}
}
\value{
\code{list} of one or two \code{data.frame} of class \code{MSstatsTMT}, named \code{PTM} and \code{PROTEIN}
}
\description{
Takes as input TMT experiments which are the output of Fragpipe and converts
into MSstatsPTM format. Requires \code{msstats.csv} file and an annotation file.
Optionally an additional \code{msstats.csv} file can be uploaded if a
corresponding global profiling run was performed. Site localization is
performed and only high probability localizations are kept.
}
\examples{
# TMT Example (with global profiling run)
head(fragpipe_input)
head(fragpipe_annotation)
head(fragpipe_input_protein)
head(fragpipe_annotation_protein)

msstats_data = FragPipetoMSstatsPTMFormat(fragpipe_input,
                                          fragpipe_annotation,
                                          fragpipe_input_protein, 
                                          fragpipe_annotation_protein,
                                          label_type="TMT",
                                          mod_id_col = "STY",
                                          localization_cutoff=.75,
                                          remove_unlocalized_peptides=TRUE)
head(msstats_data$PTM)
head(msstats_data$PROTEIN)

# LFQ Example
input = system.file("tinytest/raw_data/Fragpipe/MSstats.csv", 
                                        package = "MSstatsPTM")
input = data.table::fread(input)
annot = system.file("tinytest/raw_data/Fragpipe/experiment_annotation.tsv", 
                                        package = "MSstatsPTM")
annot = data.table::fread(annot)       
input_protein = system.file("tinytest/raw_data/Fragpipe/msstats_proteome_lf.csv",
                                        package = "MSstatsPTM")                                  
input_protein = data.table::fread(input_protein)

msstats_data = FragPipetoMSstatsPTMFormat(input,
                                          annot,
                                          input_protein = input_protein,
                                          label_type="LF",
                                          mod_id_col = "STY",
                                          localization_cutoff=.75,
                                          protein_id_col = "ProteinName",
                                          peptide_id_col = "PeptideSequence")
                                          
# If no global profiling run is available, omit input_protein and set:
# msstats_data = FragPipetoMSstatsPTMFormat(input, annot,
#   label_type = "LF", mod_id_col = "STY",
#   localization_cutoff = .75, protein_id_col = "ProteinName",
#   peptide_id_col = "PeptideSequence", use_unmod_peptides = FALSE)

head(msstats_data$PTM)
head(msstats_data$PROTEIN)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataProcessPlotsPTM.R
\name{dataProcessPlotsPTM}
\alias{dataProcessPlotsPTM}
\title{Visualization for explanatory data analysis}
\usage{
dataProcessPlotsPTM(
  data,
  type = "PROFILEPLOT",
  ylimUp = FALSE,
  ylimDown = FALSE,
  x.axis.size = 10,
  y.axis.size = 10,
  text.size = 4,
  text.angle = 90,
  legend.size = 7,
  dot.size.profile = 2,
  ncol.guide = 5,
  width = 10,
  height = 12,
  ptm.title = "All PTMs",
  protein.title = "All Proteins",
  which.PTM = "all",
  which.Protein = NULL,
  originalPlot = TRUE,
  summaryPlot = TRUE,
  address = "",
  isPlotly = FALSE
)
}
\arguments{
\item{data}{name of the list with PTM and (optionally) Protein data, which
can be the output of the MSstatsPTM
\code{\link[MSstatsPTM]{dataSummarizationPTM}} or
\code{\link[MSstatsPTM]{dataSummarizationPTM_TMT}} functions.}

\item{type}{choice of visualization. "ProfilePlot" represents profile plot of
log intensities across MS runs.
"QCPlot" represents box plots of log intensities across channels and MS runs.}

\item{ylimUp}{upper limit for y-axis in the log scale.
FALSE(Default) for Profile Plot and QC Plot uses the upper limit as rounded
off maximum of log2(intensities) after normalization + 3..}

\item{ylimDown}{lower limit for y-axis in the log scale. FALSE(Default) for
Profile Plot and QC Plot uses 0..}

\item{x.axis.size}{size of x-axis labeling for "Run" and "channel in Profile
Plot and QC Plot.}

\item{y.axis.size}{size of y-axis labels. Default is 10.}

\item{text.size}{size of labels represented each condition at the top of
Profile plot and QC plot. Default is 4.}

\item{text.angle}{angle of labels represented each condition at the top of
Profile plot and QC plot. Default is 0.}

\item{legend.size}{size of legend above Profile plot. Default is 7.}

\item{dot.size.profile}{size of dots in Profile plot. Default is 2.}

\item{ncol.guide}{number of columns for legends at the top of plot. Default
is 5.}

\item{width}{width of the saved pdf file. Default is 10.}

\item{height}{height of the saved pdf file. Default is 10.}

\item{ptm.title}{title of overall PTM QC plot}

\item{protein.title}{title of overall Protein QC plot}

\item{which.PTM}{PTM list to draw plots. List can be names of
PTMs or order numbers of PTMs.
Default is "all", which generates all plots for each protein. For QC plot,
"allonly" will generate one QC plot with all proteins.}

\item{which.Protein}{List of proteins to plot. Will plot all PTMs associated
with listed Proteins. Default is NULL which will default to which.PTM.}

\item{originalPlot}{TRUE(default) draws original profile plots, without
normalization.}

\item{summaryPlot}{TRUE(default) draws profile plots with protein
summarization for each channel and MS run.}

\item{address}{the name of folder that will store the results. Default folder
is the current working directory.}

\item{isPlotly}{Parameter to use Plotly or ggplot2. If set to TRUE, MSstats
will save Plotly plots as HTML files. If set to FALSE MSstats will save ggplot2 plots
as PDF files
The other assigned folder has to be existed under the current working
directory.
An output pdf file is automatically created with the default name of
"ProfilePlot.pdf" or "QCplot.pdf".
The command address can help to specify where to store the file as well as
how to modify the beginning of the file name.
If address=FALSE, plot will be not saved as pdf file but showed in window.}
}
\value{
plot or pdf
}
\description{
To illustrate the quantitative data and quality control of MS runs,
dataProcessPlotsPTM takes the quantitative data from dataSummarizationPTM or
dataSummarizationPTM_TMT to plot the following :
(1) profile plot (specify "ProfilePlot" in option type), to identify the
potential sources of variation for each protein;
(2) quality control plot (specify "QCPlot" in option type), to evaluate the
systematic bias between MS runs.
}
\examples{

# QCPlot
dataProcessPlotsPTM(summary.data,
                    type = 'QCPLOT',
                    which.PTM = "allonly",
                    address = FALSE)
                    
#ProfilePlot
dataProcessPlotsPTM(summary.data,
                    type = 'PROFILEPLOT',
                    which.PTM = "Q9UQ80_K376",
                    address = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_converters.R
\name{.locateSites}
\alias{.locateSites}
\title{Add site location and aa}
\usage{
.locateSites(
  data,
  mod_id,
  protein_name_col,
  unmod_pep_col,
  mod_pep_col,
  mod_id_is_numeric,
  replace_text = FALSE
)
}
\arguments{
\item{data}{data.table}

\item{mod_id}{string}
}
\value{
data.table
}
\description{
Add site location and aa
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_converters.R
\name{MSstatsPTMSiteLocator}
\alias{MSstatsPTMSiteLocator}
\title{Locate modification site number and amino acid}
\usage{
MSstatsPTMSiteLocator(
  data,
  protein_name_col = "ProteinName",
  unmod_pep_col = "PeptideSequence",
  mod_pep_col = "PeptideModifiedSequence",
  clean_mod = FALSE,
  fasta_file = NULL,
  fasta_protein_name = "header",
  mod_id = "\\\\*",
  localization_scores = FALSE,
  localization_cutoff = 0.75,
  remove_unlocalized_peptides = TRUE,
  terminus_included = FALSE,
  terminus_id = "\\\\.",
  mod_id_is_numeric = FALSE,
  remove_underscores = FALSE,
  remove_other_mods = FALSE,
  bracket = FALSE,
  replace_text = FALSE
)
}
\arguments{
\item{data}{\code{data.table} of enriched experimental run. Must include
\code{ProteinName}, \code{PeptideSequence}, \code{PeptideModifiedSequence}, and (optionally)
\code{Start} columns.}

\item{protein_name_col}{Name of column indicating protein. Default is
\code{ProteinName}.}

\item{unmod_pep_col}{Name of column indicating unmodified peptide sequence. Default
is \code{PeptideSequence}.}

\item{mod_pep_col}{Name of column indicating modified peptide sequence. Default
is \code{PeptideModifiedSequence}.}

\item{clean_mod}{Remove special characters and numbers around modification
name. Default is \code{FALSE}}

\item{fasta_file}{File path to FASTA file that matches with proteins in
\code{data}. Can be either string or \code{data.table} processed with \code{tidyFasta()}
function. Default to NULL if peptide number included in \code{data}.}

\item{fasta_protein_name}{Name of fasta file column that matches with
\code{protein_name_col}. Default is \code{header}.}

\item{mod_id}{String that indicates what amino acid was modified in
\code{PeptideSequence}.}

\item{localization_scores}{Boolean indicating if mod id is a localization
score. If TRUE, \code{mod_id} will be ignored and localization cutoff will be
used to determine sites. Default is FALSE.}

\item{localization_cutoff}{Default is .75. Localization probabilities below
cutoffs will be removed. \code{localization_scores} must be TRUE.}

\item{remove_unlocalized_peptides}{Default is TRUE. If \code{localization_scores}
is TRUE and probabilities are below \code{localization_cutoff}, the modification
site will not be able to be determined. These unlocalized peptides can be
kept or removed. If FALSE the unlocalized peptides will still be used in
modeling the sites that could be localized.}

\item{terminus_included}{Boolean indicating if the \code{PeptideSequence} includes
the terminus amino acid.}

\item{terminus_id}{String that indicates what the terminus amino acid is.
Default is '.'.}

\item{mod_id_is_numeric}{Boolean indicating if modification identifier is
a number instead of a character (i.e. +80 vs *).}

\item{remove_underscores}{Boolean indicating if underscores around peptide
exist. These should be removed to properly count where in sequence the
modification occurred.}

\item{remove_other_mods}{keeping mods that are not of interest can mess up
the amino acid count. Remove them if they are causing issues.}

\item{bracket}{bracket type that encompasses PTM (usually \code{[} or \code{(}). Always
pass opening bracket (there is a function to grab the close bracket). Default
is FALSE (i.e. no bracket).}

\item{replace_text}{If PTM is noted by text (i.e. \code{Phospho}) and needs to be
replaced by an indicator (\code{*})}
}
\value{
\code{data.table} with site location added into \code{Protein} column.
}
\description{
Locate modification site number and amino acid
}
\examples{
##TODO

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locatePTM.R
\name{annotSite}
\alias{annotSite}
\title{Annotate modification site}
\usage{
annotSite(aaIndex, residue, lenIndex = NULL)
}
\arguments{
\item{aaIndex}{An integer vector. Location of the sites.}

\item{residue}{A string vector. Amino acid residue.}

\item{lenIndex}{An integer. Default is \code{NULL}}
}
\value{
A string.
}
\description{
\code{annotSite} annotates modified sites as their residues and locations.
}
\examples{
annotSite(10, "K")
annotSite(10, "K", 3L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groupComparisonPTM.R
\name{groupComparisonPTM}
\alias{groupComparisonPTM}
\title{Perform differential analysis on MS-based proteomics experiments targeting PTMs}
\usage{
groupComparisonPTM(
  data,
  data.type = NULL,
  contrast.matrix = "pairwise",
  moderated = FALSE,
  adj.method = "BH",
  log_base = 2,
  save_fitted_models = TRUE,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  base = "MSstatsPTM_log_",
  ptm_label_type = "LF",
  protein_label_type = "LF"
)
}
\arguments{
\item{data}{list of summarized datasets. Output of MSstatsPTM summarization
function \code{\link[MSstatsPTM]{dataSummarizationPTM}}  or
\code{\link[MSstatsPTM]{dataSummarizationPTM_TMT}} depending on acquisition
type.}

\item{data.type}{Type of data. Must be one of \code{LF} or \code{TMT}. Will be deprecated
in favor of ptm_label_type and protein_label_type.}

\item{contrast.matrix}{comparison between conditions of interests. Default
models full pairwise comparison between all conditions}

\item{moderated}{For TMT experiments only. TRUE will moderate t statistic;
FALSE (default) uses ordinary t statistic. Default is FALSE.}

\item{adj.method}{For TMT experiments only. Adjusted method for multiple
comparison. "BH" is default. "BH" is used for all other experiment types}

\item{log_base}{For non-TMT experiments only. The base of the logarithm used
in summarization.}

\item{save_fitted_models}{logical, if TRUE, fitted models will be added to
the output.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be
printed to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{base}{start of the file name.}

\item{ptm_label_type}{Indicator of labeling type for PTM dataset. Must be one
of \code{LF} or \code{TMT}}

\item{protein_label_type}{Indicator of labeling type for PROTEIN dataset.
Must be one of \code{LF} or \code{TMT}}
}
\value{
list of modeling results. Includes PTM, PROTEIN, and ADJUSTED
data.tables with their corresponding model results.
}
\description{
Takes summarized PTM and protein data from \code{dataSummarizationPTM} or
\code{dataSummarizationPTM_TMT} functions and performs differential analysis.
Leverages unmodified protein data to perform adjustment and deconvolute the
effect of the PTM and unmodified protein. If protein data is unavailable,
PTM data can still be passed into the function, however adjustment can not be
performed. All model results are returned for completeness.
}
\examples{

model.lf.msstatsptm = groupComparisonPTM(summary.data, 
                                     ptm_label_type="LF",
                                     protein_label_type="LF",
                                     verbose = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataProcessPTM.R
\name{dataProcessPTM}
\alias{dataProcessPTM}
\title{Data processing and summarization of peptide-level quantification to PTM and protein level quantification}
\usage{
dataProcessPTM(
  data,
  ptm_label_type,
  protein_label_type,
  MBimpute_ptm = FALSE,
  MBimpute_protein = TRUE,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{data}{Name of the output of MSstatsPTM converter function or
peptide-level quantified data from other tools. It should be a list
containing one or two data tables, named PTM and PROTEIN for modified and
unmodified datasets. The list must at least contain the PTM dataset, however
the PROTEIN dataset is optional.}

\item{ptm_label_type}{Indicator of labeling type for PTM dataset. Must be one of
\code{LF} or \code{TMT}}

\item{protein_label_type}{Indicator of labeling type for PROTEIN dataset. Must be
one of \code{LF} or \code{TMT}}

\item{MBimpute_ptm}{Missing value imputation on the feature-level for the PTM
dataset. TRUE (default) imputes missing values by Accelated
failure model. FALSE uses minimum value to impute the missing value for each
peptide precursor ion.}

\item{MBimpute_protein}{Missing value imputation on the feature-level for the
PROTEIN dataset. TRUE (default) imputes missing values by Accelated
failure model. FALSE uses minimum value to impute the missing value for each
peptide precursor ion.}

\item{...}{Additional parameters passed to either the
\link[MSstats]{dataProcess} function from \code{MSstats} or the
\link[MSstatsTMT]{proteinSummarization} function from \code{MSstatsTMT}.}
}
\description{
Function to perform data processing and summarization on an experiment
targeting post-translational modifications. Performs normalization, missing
value imputation, feature selection, and summarization. Can optionally take
an additional global protein quantification experiment for protein-level
correction of PTM changes. Can take either label free or tandem mass tag
(TMT) labeled data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_converters.R
\name{.fixTerminus}
\alias{.fixTerminus}
\title{Fix terminus location adjustments}
\usage{
.fixTerminus(data, terminus_id, unmod_pep_col)
}
\arguments{
\item{data}{data.table containing peptide data}

\item{terminus_id}{character string identifying the terminus
(e.g. N-terminus)}

\item{unmod_pep_col}{character string specifying the column name containing
unmodified peptide sequences}
}
\value{
data.table with corrected Start positions
}
\description{
Fix terminus location adjustments
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{raw.input}
\alias{raw.input}
\title{Example of input PTM dataset for LabelFree/DDA/DIA experiments.}
\format{
A list of two data.tables named PTM and PROTEIN with 1745 and 478
rows respectively.
}
\usage{
raw.input
}
\description{
It can be the output of MSstatsPTM converter ProgenesistoMSstatsPTMFormat or
other MSstats converter functions (Please see MSstatsPTM_LabelFree_Workflow
vignette). The dataset is formatted as a list with two data.tables named PTM
and PROTEIN. In each data.table the variables are as follows:
}
\details{
\itemize{
#'   \item ProteinName : Name of protein with modification site mapped in with
an underscore. ie "Protein_4_Y474"
\item PeptideSequence
\item Condition : Condition (ex. Healthy, Cancer, Time0)
\item BioReplicate : Unique ID for biological subject.
\item Run : MS run ID.
\item Intensity
\item PrecursorCharge
\item FragmentIon
\item ProductCharge
\item IsotopeLabelType
}
}
\examples{
head(raw.input$PTM)
head(raw.input$PROTEIN)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{fragpipe_annotation_protein}
\alias{fragpipe_annotation_protein}
\title{Example annotation file for a global profiling run TMT FragPipe experiment.}
\format{
A data.table with 7 columns.
}
\usage{
fragpipe_annotation_protein
}
\description{
Automatically created by FragPipe, manually checked by the user and input
into the FragPipetoMSstatsPTMFormat converter. Requires the correct columns
and maps the experimental desing into the MSstats format. Specify unique
bioreplicates for group comparison designs, and the same bioreplicate for
repeated measure designs. The columns and descriptions are below.
}
\details{
\itemize{
\item Run : Run name that matches exactly with FragPipe run. Used to join
evidence and metadata in annotation file.
\item Fraction : If multiple fractions were used (i.e. the same mixture
split into multiple fractions) enter that here.
TechRepMixture : Multiple runs using the same bioreplicate
\item Channel : Mixture channel used
\item Condition : Name of condition that was used for each run.
\item Mixture : The unique mixture (plex) name
\item BioReplicate : Name of biological replicate. Repeating the same name
here will tell MSstatsPTM that the experiment is a repeated measure design.
}
}
\examples{
head(fragpipe_annotation_protein)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{PDtoMSstatsPTMFormat}
\alias{PDtoMSstatsPTMFormat}
\title{Convert Proteome Discoverer output into MSstatsPTM format}
\usage{
PDtoMSstatsPTMFormat(
  input,
  annotation,
  fasta_path,
  protein_input = NULL,
  annotation_protein = NULL,
  labeling_type = "LF",
  mod_id = "\\\\(Phospho\\\\)",
  use_localization_cutoff = FALSE,
  keep_all_mods = FALSE,
  use_unmod_peptides = FALSE,
  fasta_protein_name = "uniprot_iso",
  localization_cutoff = 75,
  remove_unlocalized_peptides = TRUE,
  useNumProteinsColumn = FALSE,
  useUniquePeptide = TRUE,
  summaryforMultipleRows = max,
  removeFewMeasurements = TRUE,
  removeOxidationMpeptides = FALSE,
  removeProtein_with1Peptide = FALSE,
  which_quantification = "Precursor.Area",
  which_proteinid = "Protein.Group.Accessions",
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL
)
}
\arguments{
\item{input}{PD report corresponding with enriched experimental data.}

\item{annotation}{name of 'annotation.txt' or 'annotation.csv' data which
includes Condition, BioReplicate, Run information. 'Run' will be matched
with 'Spectrum.File'}

\item{fasta_path}{string containing path to the corresponding fasta file for
the modified peptide dataset.}

\item{protein_input}{PD report corresponding with unmodified experimental
data.}

\item{annotation_protein}{Same format as \code{annotation} corresponding to
unmodified data.}

\item{labeling_type}{type of experimental design, must be one of \code{LF} for
label free or \code{TMT} for tandem mass tag.}

\item{mod_id}{Character that indicates the modification of interest. Default
is \verb{\\\\(Phospho\\\\)}. Note \verb{\\\\} must be included before special characters.}

\item{use_localization_cutoff}{Boolean indicating whether to use a custom
localization cutoff or rely on PD's modifications column. \code{TRUE} is default
and apply custom cutoff \code{localization_cutoff}.}

\item{keep_all_mods}{Boolean indicating whether to keep or remove peptides
not in \code{mod_id}. Default is FALSE.}

\item{use_unmod_peptides}{If \code{protein_input} is not provided,
unmodified peptides can be extracted from \code{input} to be used in place of a
global profiling run. Default is \code{FALSE}.}

\item{fasta_protein_name}{Name of fasta column that matches with protein name
in evidence file. Default is \code{uniprot_iso}.}

\item{localization_cutoff}{Minimum localization score required to keep modification. Default is .75.}

\item{remove_unlocalized_peptides}{Boolean indicating if peptides without all sites localized should be kept. Default is TRUE (non-localized sites will be removed).}

\item{useNumProteinsColumn}{TRUE removes peptides which have more than 1 in
Proteins column of PD output.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned
for more than one proteins. We assume to use unique peptide for each protein.}

\item{summaryforMultipleRows}{max(default) or sum - when there are multiple
measurements for certain feature and certain run, use highest or sum of
multiple intensities.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that
have 1 or 2 measurements across runs.}

\item{removeOxidationMpeptides}{TRUE will remove the peptides including
'oxidation (M)' in modification. FALSE is default.}

\item{removeProtein_with1Peptide}{TRUE will remove the proteins which have
only 1 peptide and charge. FALSE is default.}

\item{which_quantification}{Use 'Precursor.Area'(default) column for
quantified intensities. 'Intensity' or 'Area' can be used instead.}

\item{which_proteinid}{Use 'Protein.Accessions'(default) column for protein
name. 'Master.Protein.Accessions' can be used instead.}

\item{use_log_file}{logical. If TRUE, information about data processing will
be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be
printed to the console}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved. If not provided, such a file will be created
automatically. If 'append = TRUE', has to be a valid path to a file.}
}
\value{
\code{list} of \code{data.table}
}
\description{
Import Proteome Discoverer files, identify modification site location.
}
\examples{
# Global profiling example
input = system.file("tinytest/raw_data/PD/pd-ptm-input.csv", 
package = "MSstatsPTM")
input = data.table::fread(input)
annot = system.file("tinytest/raw_data/PD/pd-ptm-annot.csv",
                    package = "MSstatsPTM")
annot = data.table::fread(annot)
input_protein = system.file("tinytest/raw_data/PD/protein-input.csv",
                            package = "MSstatsPTM")
input_protein = data.table::fread(input_protein)
annot_protein = system.file("tinytest/raw_data/PD/protein-annot.csv",
                            package = "MSstatsPTM")
annot_protein = data.table::fread(annot_protein)
fasta_path=system.file("extdata", "pd_with_proteome.fasta", 
                       package="MSstatsPTM")
pd_imported = PDtoMSstatsPTMFormat(
    input,
    annotation = annot,
    protein_input = input_protein,
    annotation_protein = annot_protein,
    fasta_path = fasta_path,
    mod_id = "\\\\(GG\\\\)",
    labeling_type = "TMT",
    use_localization_cutoff = FALSE,
    which_proteinid = "Master.Protein.Accessions")
   
head(pd_imported$PTM)
head(pd_imported$PROTEIN)

# No global profiling example
head(pd_psm_input)
head(pd_annotation)

msstats_format = PDtoMSstatsPTMFormat(pd_psm_input, 
                                      pd_annotation, 
                                      system.file("extdata", "pd_fasta.fasta", package="MSstatsPTM"),
                                      use_unmod_peptides=TRUE, 
                                      which_proteinid = "Master.Protein.Accessions")

head(msstats_format$PTM)
head(msstats_format$PROTEIN)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{spectronaut_input}
\alias{spectronaut_input}
\title{Example Spectronaut evidence file from the output of a label free experiment}
\format{
a data.table with 23 columns and 2683 rows, the output of Spectronaut
}
\usage{
spectronaut_input
}
\description{
Experiment was performed by the Olsen lab and published on Nat. Commun.
(citation below).
}
\details{
Bekker-Jensen, D.B., Bernhardt, O.M., Hogrebe, A. et al. Rapid and
site-specific deep phosphoproteome profiling by data-independent acquisition
without the need for spectral libraries. Nat Commun 11, 787 (2020).
https://doi.org/10.1038/s41467-020-14609-1

The experiment was processed using Spectronaut by the computational proteomics
team at Pfizer (Liang Xue and Pierre Jean).

The experiment did not contain a global profiling run, but we show an example
of extracting the unmodified peptides and using them in place of the
profiling run.
}
\examples{
head(spectronaut_input)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_converters.R
\name{.joinFasta}
\alias{.joinFasta}
\title{Add FASTA data into dataframe}
\usage{
.joinFasta(
  data,
  fasta_file,
  fasta_protein_name,
  protein_name_col,
  unmod_pep_col,
  mod_pep_col
)
}
\arguments{
\item{data}{data.table}

\item{fasta_file}{string or data.table}
}
\value{
data.table
}
\description{
Add FASTA data into dataframe
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locatePTM.R
\name{tidyFasta}
\alias{tidyFasta}
\title{Read and tidy a FASTA file}
\usage{
tidyFasta(path)
}
\arguments{
\item{path}{A string of path to a FASTA file.}
}
\value{
A data.table with columns named \code{header}, \code{sequence},
\code{uniprot_ac}, \code{uniprot_iso}, \code{entry_name}.
}
\description{
\code{tidyFasta} reads and tidys FASTA file. Use this function as the first
step in identifying modification sites.
}
\examples{
tidyFasta(system.file("extdata", "O13297.fasta", package="MSstatsPTM"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locatePTM.R
\name{locateMod}
\alias{locateMod}
\title{Locate modified sites with a peptide}
\usage{
locateMod(peptide, aaStart, residueSymbol)
}
\arguments{
\item{peptide}{A string. Peptide sequence.}

\item{aaStart}{An integer. Starting index of the peptide.}

\item{residueSymbol}{A string. Modification residue and denoted symbol.}
}
\value{
A string.
}
\description{
\code{locateMod} locates modified sites with a peptide.
}
\examples{
locateMod("P*EP*TIDE", 3, "\\\\*")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{maxq_tmt_evidence}
\alias{maxq_tmt_evidence}
\title{Example MaxQuant evidence file from the output of a TMT experiment}
\format{
a data.table with 96 columns and 199 rows, the output of MaxQuant
}
\usage{
maxq_tmt_evidence
}
\description{
Experiment was performed by the Olsen lab and published on Nat. Commun.
(citation below).
}
\details{
Hogrebe, A., von Stechow, L., Bekker-Jensen, D.B. et al. Benchmarking common
quantification strategies for large-scale phosphoproteomics. Nat Commun 9,
1045 (2018). https://doi.org/10.1038/s41467-018-03309-6

The experiment was processed using MaxQuant by the computational proteomics
team at Pfizer (Liang Xue and Pierre Jean).

The experiment did not contain a global profiling run, but we show an example
of extracting the unmodified peptides and using them in place of the
profiling run.
}
\examples{
head(maxq_tmt_evidence)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{summary.data}
\alias{summary.data}
\title{Example of output from dataSummarizationPTM function for non-TMT data}
\format{
A list of two lists with four data.tables.
}
\usage{
summary.data
}
\description{
It is made from \code{\link{raw.input}}.
It is the output of dataSummarizationPTM function from MSstatsPTM.
It should include a list with two names \code{PTM} and \code{PROTEIN}. Each
of these list values is also a list with two names \code{ProteinLevelData}
and \code{FeatureLevelData}, which correspond to two data.tables.The columns
in these two data.tables are listed below. The variables are as follows:
\itemize{
\item FeatureLevelData : \itemize{
\item PROTEIN : Protein ID with modification site mapped in. Ex.
Protein_1002_S836
\item PEPTIDE : Full peptide with charge
\item TRANSITION: Charge
\item FEATURE : Combination of Protien, Peptide, and Transition Columns
\item LABEL :
\item GROUP : Condition (ex. Healthy, Cancer, Time0)
\item RUN : Unique ID for technical replicate of one TMT
mixture.
\item SUBJECT : Unique ID for biological subject.
\item FRACTION : Unique Fraction ID
\item originalRUN : Run name
\item censored :
\item INTENSITY : Unique ID for TMT mixture.
\item ABUNDANCE : Unique ID for TMT mixture.
\item newABUNDANCE : Unique ID for TMT mixture.
\item predicted : Unique ID for TMT mixture.
}
\item ProteinLevelData : \itemize{
\item RUN : MS run ID
\item Protein : Protein ID with modification site mapped in. Ex.
Protein_1002_S836
\item LogIntensities: Protein-level summarized abundance
\item originalRUN : Labeling information (126, ... 131)
\item GROUP : Condition (ex. Healthy, Cancer, Time0)
\item SUBJECT : Unique ID for biological subject.
\item TotalGroupMeasurements : Unique ID for technical replicate of one TMT
mixture.
\item NumMeasuredFeature : Unique ID for TMT mixture.
\item MissingPercentage : Unique ID for TMT mixture.
\item more50missing : Unique ID for TMT mixture.
\item NumImputedFeature : Unique ID for TMT mixture.
}

}
}
\examples{
head(summary.data)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groupComparisonPlotsPTM.R
\name{groupComparisonPlotsPTM}
\alias{groupComparisonPlotsPTM}
\title{Visualization for model-based analysis and summarization}
\usage{
groupComparisonPlotsPTM(
  data = data,
  type,
  sig = 0.05,
  FCcutoff = FALSE,
  logBase.pvalue = 10,
  ylimUp = FALSE,
  ylimDown = FALSE,
  xlimUp = FALSE,
  x.axis.size = 10,
  y.axis.size = 10,
  dot.size = 3,
  text.size = 4,
  text.angle = 0,
  legend.size = 13,
  ProteinName = TRUE,
  colorkey = TRUE,
  numProtein = NULL,
  width = 10,
  height = 10,
  which.Comparison = "all",
  which.PTM = "all",
  address = "",
  isPlotly = FALSE
)
}
\arguments{
\item{data}{name of the list with models, which can be the output of the
MSstatsPTM \code{\link[MSstatsPTM]{groupComparisonPTM}} function}

\item{type}{choice of visualization, one of VolcanoPlot or Heatmap}

\item{sig}{FDR cutoff for the adjusted p-values in heatmap and volcano plot.
level of significance for comparison plot. 100(1-sig)\% confidence interval
will be drawn. sig=0.05 is default.}

\item{FCcutoff}{For volcano plot or heatmap, whether involve fold change
cutoff or not. FALSE (default) means no fold change cutoff is applied for
significance analysis. FCcutoff = specific value means specific fold change
cutoff is applied.}

\item{logBase.pvalue}{for volcano plot or heatmap, (-) logarithm
transformation of adjusted p-value with base 2 or 10(default).}

\item{ylimUp}{for all three plots, upper limit for y-axis. FALSE (default)
for volcano plot/heatmap use maximum of -log2 (adjusted p-value) or -log10
(adjusted p-value). FALSE (default) for comparison plot uses maximum of
log-fold change + CI.}

\item{ylimDown}{for all three plots, lower limit for y-axis. FALSE (default)
for volcano plot/heatmap use minimum of -log2 (adjusted p-value) or -log10
(adjusted p-value). FALSE (default) for comparison plot uses minimum of
log-fold change - CI.}

\item{xlimUp}{for Volcano plot, the limit for x-axis. FALSE (default) for
use maximum for absolute value of log-fold change or 3 as default if maximum
for absolute value of log-fold change is less than 3.}

\item{x.axis.size}{size of axes labels, e.g. name of the comparisons in
heatmap, and in comparison plot. Default is 10.}

\item{y.axis.size}{size of axes labels, e.g. name of targeted proteins in
heatmap. Default is 10.}

\item{dot.size}{size of dots in volcano plot and comparison plot. Default is
3.}

\item{text.size}{size of ProteinName label in the graph for Volcano Plot.
Default is 4.}

\item{text.angle}{angle of x-axis labels represented each comparison at the
bottom of graph in comparison plot. Default is 0.}

\item{legend.size}{size of legend for color at the bottom of volcano plot.
Default is 7.}

\item{ProteinName}{for volcano plot only, whether display protein names or
not. TRUE (default) means protein names, which are significant, are
displayed next to the points. FALSE means no protein names are displayed.}

\item{colorkey}{TRUE(default) shows colorkey.}

\item{numProtein}{The number of proteins which will be presented in each
heatmap. Default is 50.}

\item{width}{width of the saved file. Default is 10.}

\item{height}{height of the saved file. Default is 10.}

\item{which.Comparison}{list of comparisons to draw plots. List can be
labels of comparisons or order numbers of comparisons from levels(data$Label)
, such as levels(testResultMultiComparisons$ComparisonResult$Label).
Default is "all", which generates all plots for each protein.}

\item{which.PTM}{Protein list to draw comparison plots. List can be
names of Proteins or order numbers of Proteins from
levels(testResultMultiComparisons$ComparisonResult$Protein). Default is
"all", which generates all comparison plots for each protein.}

\item{address}{the name of folder that will store the results. Default
folder is the current working directory. The other assigned folder has to
be existed under the current working directory. An output pdf file is
automatically created with the default name of "VolcanoPlot.pdf" or
"Heatmap.pdf". The command address can help to specify where to store the
file as well as how to modify the beginning of the file name. If
address=FALSE, plot will be not saved as pdf file but showed in window}

\item{isPlotly}{Parameter to use Plotly or ggplot2. If set to TRUE, MSstats
will save Plotly plots as HTML files. If set to FALSE MSstats will save ggplot2 plots
as PDF files}
}
\value{
plot or pdf
}
\description{
To analyze the results of modeling changes in abundance of modified peptides
and overall protein, groupComparisonPlotsPTM takes as input the results of
the groupComparisonPTM function. It asses the results of three models:
unadjusted PTM, adjusted PTM, and overall protein. To asses the results of
the model, the following visualizations can be created:
(1) VolcanoPlot (specify "VolcanoPlot" in option type), to plot peptides or
proteins and their significance for each model.
(2) Heatmap (specify "Heatmap" in option type), to evaluate the fold change
between conditions and peptides/proteins
}
\examples{

model.lf.msstatsptm = groupComparisonPTM(summary.data, 
                                     ptm_label_type="LF",
                                     protein_label_type="LF")
groupComparisonPlotsPTM(data = model.lf.msstatsptm,
                        type = "VolcanoPlot",
                        FCcutoff= 2,
                        logBase.pvalue = 2,
                        address=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{SpectronauttoMSstatsPTMFormat}
\alias{SpectronauttoMSstatsPTMFormat}
\title{Convert Spectronaut output into MSstatsPTM format}
\usage{
SpectronauttoMSstatsPTMFormat(
  input,
  annotation = NULL,
  fasta_path = NULL,
  protein_input = NULL,
  annotation_protein = NULL,
  use_unmod_peptides = FALSE,
  intensity = "PeakArea",
  mod_id = "\\\\[Phospho \\\\(STY\\\\)\\\\]",
  fasta_protein_name = "uniprot_iso",
  remove_other_mods = TRUE,
  filter_with_Qvalue = TRUE,
  qvalue_cutoff = 0.01,
  useUniquePeptide = TRUE,
  removeFewMeasurements = TRUE,
  removeProtein_with1Feature = FALSE,
  summaryforMultipleRows = max,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL
)
}
\arguments{
\item{input}{name of Spectronaut PTM output, which is long-format.
ProteinName, PeptideSequence, PrecursorCharge, FragmentIon, ProductCharge,
IsotopeLabelType, Condition, BioReplicate, Run, Intensity,
F.ExcludedFromQuantification are required. Rows with
F.ExcludedFromQuantification=True will be removed.}

\item{annotation}{name of 'annotation.txt' data which includes Condition,
BioReplicate, Run. If annotation is already complete in Spectronaut,
use annotation=NULL (default). It will use the annotation information from
input.}

\item{fasta_path}{string containing path to the corresponding fasta file for
the modified peptide dataset.}

\item{protein_input}{name of Spectronaut global protein output, which is
as in the same format as \code{input} parameter.}

\item{annotation_protein}{name of annotation file for global protein data, in
the same format as above.}

\item{use_unmod_peptides}{If \code{protein_input} is not provided,
unmodified peptides can be extracted from \code{input} to be used in place of a
global profiling run. Default is \code{FALSE}.}

\item{intensity}{'PeakArea'(default) uses not normalized peak area.
'NormalizedPeakArea' uses peak area normalized by Spectronaut. Default is
NULL}

\item{mod_id}{Character that indicates the modification of interest. Default
is \verb{\\\\(Phospho\\\\)}. Note \verb{\\\\} must be included before special characters.}

\item{fasta_protein_name}{Name of fasta column that matches with protein name
in evidence file. Default is \code{uniprot_iso}.}

\item{remove_other_mods}{Remove peptides which include modfications other
than the one listed in \code{mod_id}. Default is \code{TRUE}. For example, in an
experiment targeting Phosphorylation, setting this parameter to \code{TRUE} would
remove peptides like
(Acetyl (Protein N-term))AAAAPDSRVS(Phospho (STY))EEENLK. Set this parameter
to \code{FALSE} to keep peptides with extraneous modifications.}

\item{filter_with_Qvalue}{TRUE(default) will filter out the intensities that
have greater than qvalue_cutoff in EG.Qvalue column. Those intensities will
be replaced with zero and will be considered as censored missing values for
imputation purpose.}

\item{qvalue_cutoff}{Cutoff for EG.Qvalue. Default is 0.01.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for
more than one proteins. We assume to use unique peptide for each protein.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that
have 1 or 2 measurements across runs.}

\item{removeProtein_with1Feature}{TRUE will remove the proteins which have
only 1 feature, which is the combination of peptide, precursor charge,
fragment and charge. FALSE is default.}

\item{summaryforMultipleRows}{max(default) or sum - when there are multiple
measurements for certain feature and certain run, use highest or sum of
multiple intensities.}

\item{use_log_file}{logical. If TRUE, information about data processing will
be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing wil be
printed to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved. If not provided, such a file will be created
automatically. If 'append = TRUE', has to be a valid path to a file.}
}
\value{
a list of two data.tables named 'PTM' and 'PROTEIN' in the format
required by MSstatsPTM.
}
\description{
Converters label-free Spectronaut data into MSstatsPTM format. Requires PSM
output from Spectronaut and a custom made annotation file, mapping the run
name to the condition and bioreplicate. Can optionally take a seperate PSM
file for a global profiling run. If no global profiling run provided, the
function can extract the unmodified peptides from the PTM PSM file and use
them as a global profiling run (not recommended).
}
\examples{

head(spectronaut_input)
head(spectronaut_annotation)

msstats_input = SpectronauttoMSstatsPTMFormat(spectronaut_input, 
                  annotation=spectronaut_annotation, 
                  fasta_path=system.file("extdata", "spectronaut_fasta.fasta", package="MSstatsPTM"),
                  use_unmod_peptides=TRUE,
                  mod_id = "\\\\[Phospho \\\\(STY\\\\)\\\\]",
                  fasta_protein_name = "uniprot_iso"
                  )

head(msstats_input$PTM)
head(msstats_input$PROTEIN)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{summary.data.tmt}
\alias{summary.data.tmt}
\title{Example of output from dataSummarizationPTM_TMT function for TMT data}
\format{
A list of two lists with four data.tables.
}
\usage{
summary.data.tmt
}
\description{
It is made from \code{\link{raw.input.tmt}}.
It is the output of dataSummarizationPTM_TMT function from MSstatsPTM.
It should include a list with two names \code{PTM} and \code{PROTEIN}. Each
of these list values is also a list with two names \code{ProteinLevelData}
and \code{FeatureLevelData}, which correspond to two data.tables.The columns
in these two data.tables are listed below. The variables are as follows:
\itemize{
\item FeatureLevelData : \itemize{
\item ProteinName : MS run ID
\item PSM : Protein ID with modification site mapped in. Ex.
Protein_1002_S836
\item censored: Protein-level summarized abundance
\item predicted : Labeling information (126, ... 131)
\item log2Intensity : Condition (ex. Healthy, Cancer, Time0)
\item Run : Unique ID for biological subject.
\item Channel : Unique ID for technical replicate of one TMT
mixture.
\item BioReplicate : Unique ID for TMT mixture.
\item Condition : Unique ID for TMT mixture.
\item Mixture : Unique ID for TMT mixture.
\item TechRepMixture : Unique ID for TMT mixture.
\item PeptideSequence : Unique ID for TMT mixture.
\item Charge : Unique ID for TMT mixture.
}
\item ProteinLevelData : \itemize{
\item Mixture : MS run ID
\item TechRepMixture : Protein ID with modification site mapped in. Ex.
Protein_1002_S836
\item Run: Protein-level summarized abundance
\item Channel : Labeling information (126, ... 131)
\item Protein : Condition (ex. Healthy, Cancer, Time0)
\item Abundance : Unique ID for biological subject.
\item BioReplicate : Unique ID for technical replicate of one TMT
mixture.
\item Condition : Unique ID for TMT mixture.
}

}
}
\examples{
head(summary.data.tmt)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{DIANNtoMSstatsPTMFormat}
\alias{DIANNtoMSstatsPTMFormat}
\title{Convert the output of DIA-NN PSM file into MSstatsPTM format}
\usage{
DIANNtoMSstatsPTMFormat(
  input,
  annotation,
  input_protein = NULL,
  annotation_protein = NULL,
  fasta_path = NULL,
  use_unmod_peptides = FALSE,
  protein_id_col = "Protein.Group",
  fasta_protein_name = "uniprot_ac",
  global_qvalue_cutoff = 0.01,
  qvalue_cutoff = 0.01,
  pg_qvalue_cutoff = 0.01,
  useUniquePeptide = TRUE,
  removeFewMeasurements = TRUE,
  removeOxidationMpeptides = TRUE,
  removeProtein_with1Feature = FALSE,
  MBR = TRUE,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL
)
}
\arguments{
\item{input}{data.frame of \code{report.tsv} file produced by Philosopher}

\item{annotation}{annotation with Run, Fraction, TechRepMixture, Mixture, Channel,
BioReplicate, Condition columns or a path to file. Refer to the example 'annotation' for the meaning of each column.}

\item{input_protein}{same as \code{input} for global profiling run. Default is NULL.}

\item{annotation_protein}{same as \code{annotation} for global profiling run. Default is NULL.}

\item{fasta_path}{A string of path to a FASTA file, used to match PTM peptides.}

\item{use_unmod_peptides}{Boolean if the unmodified peptides in the input
file should be used to construct the unmodified protein output. Only used if
\code{input_protein} is not provided. Default is \code{FALSE}.}

\item{protein_id_col}{Use 'Protein.Groups'(default) column for protein name.}

\item{fasta_protein_name}{Name of column that matches with the protein names
in \code{protein_id_col}. The protein names in these two columns must match in
order to join the FASTA file with the DIA-NN output. Default is "uniprot_ac"
for uniprot ID. For uniprot mnemonic ID, use "entry_name"}

\item{global_qvalue_cutoff}{The global qvalue cutoff. Default is 0.01.}

\item{qvalue_cutoff}{local qvalue cutoff for library. Default is 0.01.}

\item{pg_qvalue_cutoff}{local qvalue cutoff for protein groups Run should be
the same as filename. Default is 0.01.}

\item{useUniquePeptide}{logical, if TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that have 1 or 2 measurements within each Run.}

\item{removeOxidationMpeptides}{TRUE (default) will remove the peptides including oxidation (M) sequence.}

\item{removeProtein_with1Feature}{TRUE will remove the proteins which have only 1 peptide and charge. Defaut is FALSE.}

\item{MBR}{If analaysis was done with match between runs or not. Default is TRUE.}

\item{use_log_file}{logical. If TRUE, information about data processing will
be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing wil be
printed to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved. If not provided, such a file will be created
automatically. If 'append = TRUE', has to be a valid path to a file.}
}
\value{
\code{list} of one or two \code{data.frame} of class \code{MSstatsTMT}, named \code{PTM} and \code{PROTEIN}
}
\description{
Takes as input the \code{report.tsv} file from DIA-NN and converts it into
MSstatsPTM format. Requires PSM and an annotation file. Optionally an
additional \code{report.tsv} file for a corresponding global profiling run can
be included.
}
\examples{
# Example from PRIDE ID PXD053502
input = system.file("tinytest/raw_data/DIANN/report.tsv", 
                                        package = "MSstatsPTM")
input = data.table::fread(input)
annot = system.file("tinytest/raw_data/DIANN/annot.csv", 
                                        package = "MSstatsPTM")
annot = data.table::fread(annot)
fasta_path = system.file("extdata", "diann.fasta", 
                       package="MSstatsPTM")

msstatsptm_format = DIANNtoMSstatsPTMFormat(
    input, 
    annot, 
    protein_id_col = "Protein.Names", 
    fasta_path = fasta_path, 
    fasta_protein_name = "entry_name", 
    use_log_file = FALSE
)

head(msstatsptm_format$PTM)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{spectronaut_annotation}
\alias{spectronaut_annotation}
\title{Example annotation file for a label-free Spectronaut experiment.}
\format{
A data.table with 5 columns.
}
\usage{
spectronaut_annotation
}
\description{
Must be manually created by the user and input into the
SpectronauttoMSstatsPTMFormat converter. Requires the correct columns and
maps the experimental desing into the MSstats format. Specify unique
bioreplicates for group comparison designs, and the same bioreplicate for
repeated measure designs. The columns and descriptions are below.
}
\details{
\itemize{
\item Run : Run name that matches exactly with Spectronaut run. Used to join
evidence and metadata in annotation file.
\item Condition : Name of condition that was used for each run.
\item BioReplicate : Name of biological replicate. Repeating the same name
here will tell MSstatsPTM that the experiment is a repeated measure design.
\item Raw.file : Run name that matches exactly with Spectronaut run. Used to join
evidence and metadata in annotation file.
}
}
\examples{
head(spectronaut_annotation)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locatePTM.R
\name{locatePTM}
\alias{locatePTM}
\title{Annotate modified sites with associated peptides}
\usage{
locatePTM(peptide, uniprot, fasta, modResidue, modSymbol, rmConfound = FALSE)
}
\arguments{
\item{peptide}{A string vector of peptide sequences. The peptide sequence
does not include its preceding and following AAs.}

\item{uniprot}{A string vector of Uniprot identifiers of the peptides'
originating proteins. UniProtKB entry isoform sequence is used.}

\item{fasta}{A data.table with FASTA information. Output of \code{tidyFasta}.}

\item{modResidue}{A string. Modifiable amino acid residues.}

\item{modSymbol}{A string. Symbol of a modified site.}

\item{rmConfound}{A logical. \code{TRUE} removes confounded
unmodified sites, \code{FALSE} otherwise. Default is \code{FALSE}.}
}
\value{
A data frame with three columns: \code{uniprot_iso}, \code{peptide},
\code{site}.
}
\description{
\code{PTMlocate} annotates modified sites with associated peptides.
}
\examples{
fasta = tidyFasta(system.file("extdata", "O13297.fasta", package="MSstatsPTM"))
locatePTM("DRVSYIHNDSC*TR", "O13297", fasta, "C", "\\\\*")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_converters.R
\name{.removeCutoffSites}
\alias{.removeCutoffSites}
\title{Remove sites below cutoff probability}
\usage{
.removeCutoffSites(data, mod_pep_col, cutoff, remove_unlocalized_peptides)
}
\arguments{
\item{data}{data.table}

\item{mod_pep_col}{column in data with modified sites}

\item{cutoff}{numeric cutoff. Default is .75.}

\item{remove_unlocalized_peptides}{Boolean if to remove peptides that
could not be fully localized.}
}
\value{
data.table with modifications below cutoff removed
}
\description{
Remove sites below cutoff probability
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstatsPTM.R
\docType{package}
\name{MSstatsPTM}
\alias{MSstatsPTM-package}
\alias{MSstatsPTM}
\title{MSstatsPTM: A package for detecting differentially abundant
post-translational modifications (PTM) in mass spectrometry-based
proteomic experiments.}
\description{
A set of tools for detecting differentially abundant PTMs and proteins in
shotgun mass spectrometry-based proteomic experiments. The package can handle
a variety of acquisition types, including label free and TMT experiments,
acquired with DDA, DIA, SRM or PRM acquisition methods. The
package includes tools to convert raw data from different spectral processing
tools, summarize feature intensities, and fit a linear mixed effects model. A
major advantage of the package is to leverage a separate global profiling run
and adjust the PTM fold change for changes in the unmodified protein, showing
the unconvoluted PTM fold change. Finally, the package includes functionality
to plot a variety of data visualizations.
}
\section{functions }{

\itemize{
\item \code{\link{FragPipetoMSstatsPTMFormat}} : Generates MSstatsPTM
required input format for TMT FragePipe outputs.
\item \code{\link{MaxQtoMSstatsPTMFormat}} : Generates MSstatsPTM required
input format for label-free and TMT MaxQuant outputs.
\item \code{\link{ProgenesistoMSstatsPTMFormat}} : Generates MSstatsPTM
required input format for label-free Progenesis outputs.
\item \code{\link{SpectronauttoMSstatsPTMFormat}} : Generates MSstatsPTM
required input format for label-free Spectronaut outputs.
\item \code{\link{SkylinetoMSstatsPTMFormat}} : Generates
MSstatsPTM required input format for Skyline outputs.
\item \code{\link{PStoMSstatsPTMFormat}} : Generates
MSstatsPTM required input format for PEAKS outputs.
\item \code{\link{PDtoMSstatsPTMFormat}} : Generates
MSstatsPTM required input format for Proteome Discoverer outputs.
\item \code{\link{dataSummarizationPTM}} : Summarizes PSM level
quantification to peptide (modification) and protein level quantification.
For use in label-free analysis
\item \code{\link{dataSummarizationPTM_TMT}} : Summarizes PSM level
quantification to peptide (modification) and protein level quantification.
For use in TMT analysis.
\item \code{\link{dataProcessPlotsPTM}} : Visualization for explanatory
data analysis. Specifically gives ability to plot Profile and Quality
Control plots.
\item \code{\link{groupComparisonPTM}} : Tests for significant changes in
PTM and protein abundance across conditions. Adjusts PTM fold change for
changes in protein abundance.
\item \code{\link{groupComparisonPlotsPTM}} : Visualization for model-based
analysis and summarization
}
}

\seealso{
Useful links:
\itemize{
  \item Report bugs at \url{https://github.com/Vitek-Lab/MSstatsPTM/issues}
}

}
\author{
\strong{Maintainer}: Anthony Wu \email{wu.anthon@northeastern.edu}

Authors:
\itemize{
  \item Devon Kohler \email{kohler.d@northeastern.edu}
  \item Tsung-Heng Tsai \email{tsai.tsungheng@gmail.com}
  \item Deril Raju \email{raju.d@northeastern.edu}
  \item Ting Huang \email{thuang0703@gmail.com}
  \item Mateusz Staniak \email{mtst@mstaniak.pl}
  \item Meena Choi \email{mnchoi67@gmail.com}
  \item Olga Vitek \email{o.vitek@northeastern.edu}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataSummarizationPTM_TMT.R
\name{dataSummarizationPTM_TMT}
\alias{dataSummarizationPTM_TMT}
\title{Data summarization function for TMT labelled MS experiments targeting PTMs.}
\usage{
dataSummarizationPTM_TMT(
  data,
  method = "msstats",
  global_norm = TRUE,
  global_norm.PTM = TRUE,
  reference_norm = TRUE,
  reference_norm.PTM = TRUE,
  remove_norm_channel = TRUE,
  remove_empty_channel = TRUE,
  MBimpute = TRUE,
  MBimpute.PTM = TRUE,
  maxQuantileforCensored = NULL,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL
)
}
\arguments{
\item{data}{Name of the output of MSstatsPTM converter function or
peptide-level quantified data from other tools. It should be a list
containing one or two data tables, named PTM and PROTEIN for modified and
unmodified datasets. The list must at least contain the PTM dataset. The data
should have columns ProteinName, PeptideSequence, Charge, PSM, Mixture,
TechRepMixture, Run, Channel, Condition, BioReplicate, Intensity}

\item{method}{Four different summarization methods to protein-level can be
performed : "msstats"(default), "MedianPolish", "Median", "LogSum".}

\item{global_norm}{Global median normalization on for unmodified peptide
level data (equalizing the medians across all the channels and MS runs).
Default is TRUE. It will be performed before protein-level summarization.}

\item{global_norm.PTM}{Same as above for modified peptide level data. Default
is TRUE.}

\item{reference_norm}{Reference channel based normalization between MS runs
on unmodified protein level data. TRUE(default) needs at least one reference
channel in each MS run, annotated by 'Norm' in Condtion column. It will be
performed after protein-level summarization. FALSE will not perform this
normalization step. If data only has one run, then reference_norm=FALSE.}

\item{reference_norm.PTM}{Same as above for modified peptide level data.
Default is TRUE.}

\item{remove_norm_channel}{TRUE(default) removes 'Norm' channels from
protein level data.}

\item{remove_empty_channel}{TRUE(default) removes 'Empty' channels from
protein level data.}

\item{MBimpute}{only for method="msstats". TRUE (default) imputes missing
values by Accelated failure model. FALSE uses minimum value to impute the
missing value for each peptide precursor ion.}

\item{MBimpute.PTM}{Same as above for modified peptide level data. Default is
TRUE}

\item{maxQuantileforCensored}{We assume missing values are censored.
maxQuantileforCensored is Maximum quantile for deciding censored missing
value, for instance, 0.999. Default is Null.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be
printed to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}
}
\value{
list of two data.tables
}
\description{
Utilizes functionality from MSstatsTMT to clean, summarize, and
normalize PTM and protein level data. Imputes missing values, performs
normalization, and summarizes data. PTM data is summarized up to the
modification and protein data is summarized up to the protein level. Takes
as input the output of the included converters (see included \code{raw.input.tmt}
data object for required input format).
}
\examples{
head(raw.input.tmt$PTM)
head(raw.input.tmt$PROTEIN)

quant.tmt.msstatsptm = dataSummarizationPTM_TMT(raw.input.tmt,
                                                 method = "msstats", 
                                                 verbose = FALSE)
head(quant.tmt.msstatsptm$PTM$ProteinLevelData)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{raw.input.tmt}
\alias{raw.input.tmt}
\title{Example of input PTM dataset for TMT experiments.}
\format{
A list of two data.tables named PTM and PROTEIN with 1716 and 29221
rows respectively.
}
\usage{
raw.input.tmt
}
\description{
It can be the output of MSstatsPTM converter MaxQtoMSstatsPTMFormat or other
MSstatsTMT converter functions (Please see MSstatsPTM_TMT_Workflow vignette).
The dataset is formatted as a list with two data.tables named PTM and
PROTEIN. In each data.table the variables are as follows:
}
\details{
\itemize{
\item ProteinName : Name of protein with modification site mapped in with
an underscore. ie "Protein_4_Y474"
\item PeptideSequence
\item Charge
\item PSM
\item Mixture : Mixture of samples labeled with different TMT reagents,
which can be analyzed in
a single mass spectrometry experiment. If the channal doesn't have sample,
please add \verb{Empty' under Condition. \\item TechRepMixture : Technical replicate of one mixture. One mixture may have multiple technical replicates. For example, if }TechRepMixture' = 1, 2 are the two technical replicates of
one mixture, then they should match
with same \verb{Mixture' value. \\item Run : MS run ID. \\item Channel : Labeling information (126, ... 131). \\item Condition : Condition (ex. Healthy, Cancer, Time0) \\item BioReplicate : Unique ID for biological subject. If the channal doesn't have sample, please add }Empty' under BioReplicate.
\item Intensity
}
}
\examples{
head(raw.input.tmt$PTM)
head(raw.input.tmt$PROTEIN)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataSummarizationPTM.R
\name{dataSummarizationPTM}
\alias{dataSummarizationPTM}
\title{Data summarization function for label-free MS experiments targeting PTMs.}
\usage{
dataSummarizationPTM(
  data,
  logTrans = 2,
  normalization = "equalizeMedians",
  normalization.PTM = "equalizeMedians",
  nameStandards = NULL,
  nameStandards.PTM = NULL,
  featureSubset = "all",
  featureSubset.PTM = "all",
  remove_uninformative_feature_outlier = FALSE,
  remove_uninformative_feature_outlier.PTM = FALSE,
  min_feature_count = 2,
  min_feature_count.PTM = 1,
  n_top_feature = 3,
  n_top_feature.PTM = 3,
  summaryMethod = "TMP",
  equalFeatureVar = TRUE,
  censoredInt = "NA",
  MBimpute = TRUE,
  MBimpute.PTM = TRUE,
  remove50missing = FALSE,
  fix_missing = NULL,
  maxQuantileforCensored = 0.999,
  use_log_file = TRUE,
  append = TRUE,
  verbose = TRUE,
  log_file_path = NULL,
  base = "MSstatsPTM_log_"
)
}
\arguments{
\item{data}{name of the list with PTM and (optionally) unmodified protein
data.tables, which can be the output of the MSstatsPTM converter functions}

\item{logTrans}{logarithm transformation with base 2(default) or 10}

\item{normalization}{normalization for the protein level dataset, to remove
systematic bias between MS runs. There are three different normalizations
supported. 'equalizeMedians'(default) represents constant normalization
(equalizing the medians) based on reference signals is performed. 'quantile'
represents quantile normalization based on reference signals is performed.
'globalStandards' represents normalization with global standards proteins.
FALSE represents no normalization is performed}

\item{normalization.PTM}{normalization for PTM level dataset. Default is
"equalizeMedians" Can be adjusted to any of the options described above.}

\item{nameStandards}{vector of global standard peptide names for protein
dataset. only for normalization with global standard peptides.}

\item{nameStandards.PTM}{Same as above for PTM dataset.}

\item{featureSubset}{"all" (default) uses all features that the data set has.
"top3" uses top 3 features which have highest average of log-intensity across runs.
"topN" uses top N features which has highest average of log-intensity across runs.
It needs the input for n_top_feature option.
"highQuality" flags uninformative feature and outliers.}

\item{featureSubset.PTM}{For PTM dataset only. Options same as above.}

\item{remove_uninformative_feature_outlier}{For protein dataset only. It only
works after users used featureSubset="highQuality" in dataProcess. TRUE
allows to remove 1) the features are flagged in the column,
feature_quality="Uninformative" which are features with bad quality, 2)
outliers that are flagged in the column, is_outlier=TRUE, for run-level
summarization. FALSE (default) uses all features and intensities for
run-level summarization.}

\item{remove_uninformative_feature_outlier.PTM}{For PTM dataset only. Options
same as above.}

\item{min_feature_count}{optional. Only required if featureSubset = "highQuality".
Defines a minimum number of informative features a protein needs to be considered
in the feature selection algorithm.}

\item{min_feature_count.PTM}{For PTM dataset only. Options the same as above.
Default is 1 due to low average feature count for PTMs.}

\item{n_top_feature}{For protein dataset only. The number of top features for
featureSubset='topN'. Default is 3, which means to use top 3 features.}

\item{n_top_feature.PTM}{For PTM dataset only. Options same as above.}

\item{summaryMethod}{"TMP"(default) means Tukey's median polish, which is
robust estimation method. "linear" uses linear mixed model.}

\item{equalFeatureVar}{only for summaryMethod="linear". default is TRUE.
Logical variable for whether the model should account for heterogeneous
variation among intensities from different features. Default is TRUE, which
assume equal variance among intensities from features. FALSE means that we
cannot assume equal variance among intensities from features, then we will
account for heterogeneous variation from different features.}

\item{censoredInt}{Missing values are censored or at random. 'NA' (default)
assumes that all 'NA's in 'Intensity' column are censored. '0' uses zero
intensities as censored intensity. In this case, NA intensities are missing
at random. The output from Skyline should use '0'. Null assumes that all NA
intensites are randomly missing.}

\item{MBimpute}{For protein dataset only. only for summaryMethod="TMP" and
censoredInt='NA' or '0'. TRUE (default) imputes 'NA' or '0' (depending on
censoredInt option) by Accelated failure model. FALSE uses the values
assigned by cutoffCensored.}

\item{MBimpute.PTM}{For PTM dataset only. Options same as above.}

\item{remove50missing}{only for summaryMethod="TMP". TRUE removes the runs
which have more than 50\% missing values. FALSE is default.}

\item{fix_missing}{Default is Null. Optional, same as the 'fix_missing'
parameter in MSstatsConvert::MSstatsBalancedDesign function}

\item{maxQuantileforCensored}{Maximum quantile for deciding censored missing
values. default is 0.999}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be
printed to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{base}{start of the file name.}
}
\value{
list of summarized PTM and Protein results. These results contain
the reformatted input to the summarization function, as well as run-level
summarization results.
}
\description{
Utilizes functionality from MSstats to clean, summarize, and
normalize PTM and protein level data. Imputes missing values, performs
normalization, and summarizes data. PTM data is summarized up to the
modification and protein data is summarized up to the protein level. Takes
as input the output of the included converters (see included \code{raw.input}
data object for required input format).
}
\examples{
head(raw.input$PTM)
head(raw.input$PROTEIN)

quant.lf.msstatsptm = dataSummarizationPTM(raw.input, verbose = FALSE)
head(quant.lf.msstatsptm$PTM$ProteinLevelData)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/designSampleSizePTM.R
\name{.getNumSamplePTM}
\alias{.getNumSamplePTM}
\title{Get sample size for PTM experiment}
\usage{
.getNumSamplePTM(
  desiredFC,
  power,
  alpha,
  delta,
  ptm_median_sigma_error,
  protein_median_sigma_error,
  ptm_median_sigma_subject,
  protein_median_sigma_subject
)
}
\arguments{
\item{desiredFC}{the range of a desired fold change which includes the lower
and upper values of the desired fold change.}

\item{power}{a pre-specified statistical power which defined as the probability
of detecting a true fold change. TRUE represent you require to calculate the power
for this category, else you should input the average of power you expect. Default is 0.9}
}
\value{
\code{int} of samples
}
\description{
Get sample size for PTM experiment
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{MetamorpheusToMSstatsPTMFormat}
\alias{MetamorpheusToMSstatsPTMFormat}
\title{Import Metamorpheus files into PTM format}
\usage{
MetamorpheusToMSstatsPTMFormat(
  input,
  annotation,
  fasta_path,
  input_protein = NULL,
  annotation_protein = NULL,
  use_unmod_peptides = FALSE,
  mod_ids = c("\\\\[Common Biological:Phosphorylation on S\\\\]"),
  useUniquePeptide = TRUE,
  removeFewMeasurements = TRUE,
  removeProtein_with1Feature = FALSE,
  summaryforMultipleRows = max,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL
)
}
\arguments{
\item{input}{name of Metamorpheus output file, which is tabular format. Use the AllQuantifiedPeaks.tsv file from the Metamorpheus output.}

\item{annotation}{name of 'annotation.txt' data which includes Condition, BioReplicate.}

\item{fasta_path}{string containing path to the corresponding fasta file for
the modified peptide dataset.}

\item{input_protein}{same as \code{input} for global profiling run. Default is NULL.}

\item{annotation_protein}{same as \code{annotation} for global profiling run. Default is NULL.}

\item{use_unmod_peptides}{If \code{protein_input} is not provided,
unmodified peptides can be extracted from \code{input} to be used in place of a
global profiling run. Default is \code{FALSE}.}

\item{mod_ids}{List of modifications of interest. Default
is a list with only \verb{Common Biological:Phosphorylation on S}.
Please note that the 'mod_ids' parameter currently supports lists of size 1 only.
Future updates aim to extend its functionality to accommodate lists of greater sizes.
Note \verb{\\\\} must be included before special characters.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for
more than one proteins. We assume to use unique peptide for each protein.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that
have 1 or 2 measurements across runs.}

\item{removeProtein_with1Feature}{TRUE will remove the proteins which have
only 1 feature, which is the combination of peptide, precursor charge,
fragment and charge. FALSE is default.}

\item{summaryforMultipleRows}{max(default) or sum - when there are multiple
measurements for certain feature and certain run, use highest or sum of
multiple intensities.}

\item{use_log_file}{logical. If TRUE, information about data processing will
be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing wil be
printed to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved. If not provided, such a file will be created
automatically. If 'append = TRUE', has to be a valid path to a file.}
}
\value{
a list of two data.tables named 'PTM' and 'PROTEIN' in the format
required by MSstatsPTM.
}
\description{
Import Metamorpheus files into PTM format
}
\examples{
input = system.file("tinytest/raw_data/Metamorpheus/AllQuantifiedPeaks.tsv", 
                                package = "MSstatsPTM")
input = data.table::fread(input)
annot = system.file("tinytest/raw_data/Metamorpheus/ExperimentalDesign.tsv", 
                                package = "MSstatsPTM")
annot = data.table::fread(annot)
input_protein = system.file("tinytest/raw_data/Metamorpheus/AllQuantifiedPeaksGlobalProteome.tsv",
                                package = "MSstatsPTM")
input_protein = data.table::fread(input_protein)
annot_protein = system.file("tinytest/raw_data/Metamorpheus/ExperimentalDesignGlobalProteome.tsv", 
                                package = "MSstatsPTM")
annot_protein = data.table::fread(annot_protein)
fasta_path=system.file("extdata", "metamorpheus_fasta.fasta", 
                                package="MSstatsPTM")
metamorpheus_imported = MetamorpheusToMSstatsPTMFormat(
    input, 
    annot, 
    fasta_path=fasta_path,
    input_protein=input_protein,
    annotation_protein=annot_protein,
    use_unmod_peptides=FALSE,
    mod_ids = c("\\\\[Common Fixed:Carbamidomethyl on C\\\\]")
)
head(metamorpheus_imported$PTM)
head(metamorpheus_imported$PROTEIN)
}
\author{
Anthony Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{SkylinetoMSstatsPTMFormat}
\alias{SkylinetoMSstatsPTMFormat}
\title{Convert Skyline output into MSstatsPTM format}
\usage{
SkylinetoMSstatsPTMFormat(
  input,
  fasta_path,
  fasta_protein_name = "uniprot_iso",
  annotation = NULL,
  input_protein = NULL,
  annotation_protein = NULL,
  use_unmod_peptides = FALSE,
  removeiRT = TRUE,
  filter_with_Qvalue = TRUE,
  qvalue_cutoff = 0.01,
  use_unique_peptide = TRUE,
  remove_few_measurements = FALSE,
  remove_oxidation_peptides = FALSE,
  removeProtein_with1Feature = FALSE,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL
)
}
\arguments{
\item{input}{name of Skyline PTM output}

\item{fasta_path}{A string of path to a FASTA file, used to match PTM peptides.}

\item{fasta_protein_name}{Name of fasta column that matches with protein name
in evidence file. Default is \code{uniprot_iso}.}

\item{annotation}{name of 'annotation.txt' data which includes Condition,
BioReplicate, Run. If annotation is already complete in Skyline, use
annotation=NULL (default). It will use the annotation information from input.}

\item{input_protein}{name of Skyline unmodified protein output (optional)}

\item{annotation_protein}{name of 'annotation.txt' data which includes Condition,
BioReplicate, Run for unmodified protein output. This can be the same as
\code{annotation}.}

\item{use_unmod_peptides}{Boolean if the unmodified peptides in the input
file should be used to construct the unmodified protein output. Only used if
\code{input_protein} is not provided. Default is \code{FALSE}.}

\item{removeiRT}{TRUE (default) will remove the proteins or peptides which
are labeld 'iRT' in 'StandardType' column. FALSE will keep them.}

\item{filter_with_Qvalue}{TRUE(default) will filter out the intensities that
have greater than qvalue_cutoff in DetectionQValue column. Those intensities
will be replaced with zero and will be considered as censored missing values
for imputation purpose.}

\item{qvalue_cutoff}{Cutoff for DetectionQValue. default is 0.01.}

\item{use_unique_peptide}{TRUE (default) removes peptides that are assigned
for more than one proteins. We assume to use unique peptide for each protein.}

\item{remove_few_measurements}{TRUE will remove the features that
have 1 or 2 measurements across runs. FALSE is default.}

\item{remove_oxidation_peptides}{TRUE will remove the peptides including
'oxidation (M)' in modification. FALSE is default.}

\item{removeProtein_with1Feature}{TRUE will remove the proteins which have
only 1 feature, which is the combination of peptide, precursor charge,
fragment and charge. FALSE is default.}

\item{use_log_file}{logical. If TRUE, information about data processing will
be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing wil be
printed to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved. If not provided, such a file will be created
automatically. If 'append = TRUE', has to be a valid path to a file.}
}
\value{
\code{list} of \code{data.table}
}
\description{
Currently only supports label-free quantification.
}
\examples{
# The output should be in the following format.
head(raw.input$PTM)
head(raw.input$PROTEIN)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{PStoMSstatsPTMFormat}
\alias{PStoMSstatsPTMFormat}
\title{Convert Peaks Studio output into MSstatsPTM format}
\usage{
PStoMSstatsPTMFormat(
  input,
  annotation,
  input_protein = NULL,
  annotation_protein = NULL,
  use_unmod_peptides = FALSE,
  target_modification = NULL,
  remove_oxidation_peptides = FALSE,
  remove_multi_mod_types = FALSE,
  summaryforMultipleRows = max,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL
)
}
\arguments{
\item{input}{name of Peaks Studio PTM output}

\item{annotation}{name of annotation file which includes Raw.file, Condition,
BioReplicate, Run. For example annotation see example below.}

\item{input_protein}{name of Peaks Studio unmodified protein output
(optional)}

\item{annotation_protein}{name of annotation file which includes Raw.file,
Condition,
BioReplicate, Run for unmodified protein output.}

\item{use_unmod_peptides}{Boolean if the unmodified peptides in the input
file should be used to construct the unmodified protein output. Only used if
\code{input_protein} is not provided. Default is \code{FALSE}}

\item{target_modification}{Character name of modification of interest. To
use all mod types, leave as \code{NULL}. Default is \code{NULL}. Note that if the name
includes special characters, you must include "\\" before the characters. Ex.
"Phosphorylation \\(STY\\)"}

\item{remove_oxidation_peptides}{Boolean if Oxidation (M) modifications
should be removed. Default is \code{FALSE}}

\item{remove_multi_mod_types}{Used if \code{target_modification} is not \code{NULL}.
\code{TRUE} will remove peptides with multiple types of modifications
(ie acetylation and phosphorylation). \code{FALSE} will keep these peptides and
summarize them seperately.}

\item{summaryforMultipleRows}{max(default) or sum - when there are multiple
measurements for certain feature and certain run, use highest or sum of
multiple intensities.}

\item{use_log_file}{logical. If TRUE, information about data processing will
be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing wil be
printed to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved. If not provided, such a file will be created
automatically. If 'append = TRUE', has to be a valid path to a file.}
}
\value{
\code{list} of \code{data.table}
}
\description{
Currently only supports label-free quantification.
}
\examples{
# The output should be in the following format.
head(raw.input$PTM)
head(raw.input$PROTEIN)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{MaxQtoMSstatsPTMFormat}
\alias{MaxQtoMSstatsPTMFormat}
\title{Convert output of label-free or TMT MaxQuant experiments into MSstatsPTM format}
\usage{
MaxQtoMSstatsPTMFormat(
  evidence = NULL,
  annotation = NULL,
  fasta_path,
  fasta_protein_name = "uniprot_ac",
  mod_id = "\\\\(Phospho \\\\(STY\\\\)\\\\)",
  sites_data = NULL,
  evidence_prot = NULL,
  proteinGroups = NULL,
  annotation_protein = NULL,
  use_unmod_peptides = FALSE,
  labeling_type = "LF",
  mod_num = "Single",
  TMT_keyword = "TMT",
  ptm_keyword = "phos",
  which_proteinid_ptm = "Proteins",
  which_proteinid_protein = "Proteins",
  remove_other_mods = TRUE,
  removeMpeptides = FALSE,
  removeOxidationMpeptides = FALSE,
  removeProtein_with1Peptide = FALSE,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL
)
}
\arguments{
\item{evidence}{name of 'evidence.txt' data, which includes feature-level
data for enriched (PTM) data.}

\item{annotation}{data frame annotation file for the ptm level data.
Contains column Run, Fraction, TechRepMixture, Mixture, Channel,
BioReplicate, Condition.}

\item{fasta_path}{A string of path to a FASTA file, used to match PTM peptides.}

\item{fasta_protein_name}{Name of fasta column that matches with protein name
in evidence file. Default is \code{uniprot_ac}.}

\item{mod_id}{Character that indicates the modification of interest. Default
is \verb{\\\\(Phospho\\\\)}. Note \verb{\\\\} must be included before special characters.}

\item{sites_data}{(Not recommended. Only used if evidence file not provided.
Only works for TMT labeled data) Modified peptide output from MaxQuant. For
example, a phosphorylation experiment would require the Phospho(STY)Sites.txt
file}

\item{evidence_prot}{name of 'evidence.txt' data, which includes
feature-level data for global profiling (unmodified) data.}

\item{proteinGroups}{name of 'proteinGroups.txt' data. It needs to matching
protein group ID in \code{evidence_prot}.}

\item{annotation_protein}{data frame annotation file for the protein level data.
Contains column Run, Fraction, TechRepMixture, Mixture, Channel,
BioReplicate, Condition.}

\item{use_unmod_peptides}{Boolean if the unmodified peptides in the input
file should be used to construct the unmodified protein output. Only used if
\code{input_protein} is not provided. Default is \code{FALSE}.}

\item{labeling_type}{Either \code{TMT} or \code{LF} (Label-Free) depending on
experimental design. Default is \code{LF}.}

\item{mod_num}{(Only if \code{sites.data} is used) For modified peptide dataset.
The number modifications per peptide to be used. If "Single", only peptides
with one modification will be used. Otherwise "Total" can be selected which
does not cap the number of modifications per peptide. "Single" is the
default. Selecting "Total" may confound the effect of different
modifications.}

\item{TMT_keyword}{(Only if \code{sites.data} is used) the sub-name of columns
in sites.data file. Default is \code{TMT}. This corresponds to the columns in the
format \code{Reporter.intensity.corrected.1.TMT1phos___1}. Specifically, this
parameter indicates the first section of the string \code{TMT1phos} (Before the
mixture number). If \code{TMT} is present in the string, set this value to \code{TMT}.
Else if \code{TMT} is not there (ie string is in the format \verb{1phos}) leave this
parameter as an empty string ('').}

\item{ptm_keyword}{(Only if \code{sites.data} is used) the sub-name of columns in
the sites.data file. Default is
\code{phos}. This corresponds to the columns in the format
\code{Reporter.intensity.corrected.1.TMT1phos___1}. Specifically, this parameter
indicates the second section of the string \code{TMT1phos} (After the mixture
number). If the string is present, set this parameter. Else if this part of
the string is empty (ie string is in the format \code{TMT1}) leave this parameter
as an empty string ('').}

\item{which_proteinid_ptm}{For PTM dataset, which column to use for protein
name. Use 'Proteins'(default) column for protein name. 'Leading.proteins' or
'Leading.razor.protein' or 'Gene.names' can be used instead to get the
protein ID with single protein. However, those can potentially have the
shared peptides.}

\item{which_proteinid_protein}{For Protein dataset, which column to use for
protein name. Same options as above.}

\item{remove_other_mods}{Remove peptides which include modfications other
than the one listed in \code{mod_id}. Default is \code{TRUE}. For example, in an
experiment targeting Phosphorylation, setting this parameter to \code{TRUE} would
remove peptides like
(Acetyl (Protein N-term))AAAAPDSRVS(Phospho (STY))EEENLK. Set this parameter
to \code{FALSE} to keep peptides with extraneous modifications.}

\item{removeMpeptides}{If Oxidation (M) modifications should be removed.
Default is TRUE.}

\item{removeOxidationMpeptides}{TRUE will remove the peptides including
'oxidation (M)' in modification. FALSE is default.}

\item{removeProtein_with1Peptide}{TRUE will remove the proteins which have
only 1 peptide and charge. FALSE is default.}

\item{use_log_file}{logical. If TRUE, information about data processing will
be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be
added to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing wil be
printed to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved. If not provided, such a file will be created
automatically. If 'append = TRUE', has to be a valid path to a file.}
}
\value{
a list of two data.tables named 'PTM' and 'PROTEIN' in the format
required by MSstatsPTM.
}
\description{
Takes as input LF/TMT experiments from MaxQ and converts the data into the
format needed for MSstatsPTM. Requires modified evidence.txt file from MaxQ
and an annotation file for PTM data. To adjust modified peptides for changes
in global protein level, unmodified TMT experimental data must also be
returned. Optionally can use \verb{Phospho(STY)Sites.txt} (or other PTM specific
files) from MaxQuant, but this is not recommended. If PTM specific file
provided, the raw intensities must be provided, not a ratio.
}
\examples{
# TMT experiment
head(maxq_tmt_evidence)
head(maxq_tmt_annotation)

msstats_format_tmt = MaxQtoMSstatsPTMFormat(evidence=maxq_tmt_evidence,
                        annotation=maxq_tmt_annotation,
                        fasta=system.file("extdata", "maxq_tmt_fasta.fasta", package="MSstatsPTM"),
                        fasta_protein_name="uniprot_ac",
                        mod_id="\\\\(Phospho \\\\(STY\\\\)\\\\)",
                        use_unmod_peptides=TRUE,
                        labeling_type = "TMT",
                        which_proteinid_ptm = "Proteins")

head(msstats_format_tmt$PTM)
head(msstats_format_tmt$PROTEIN)

# LF experiment
head(maxq_lf_evidence)
head(maxq_lf_annotation)

msstats_format_lf = MaxQtoMSstatsPTMFormat(evidence=maxq_lf_evidence,
                        annotation=maxq_lf_annotation,
                        fasta=system.file("extdata", "maxq_lf_fasta.fasta", package="MSstatsPTM"),
                        fasta_protein_name="uniprot_ac",
                        mod_id="\\\\(Phospho \\\\(STY\\\\)\\\\)",
                        use_unmod_peptides=TRUE,
                        labeling_type = "LF",
                        which_proteinid_ptm = "Proteins")
head(msstats_format_lf$PTM)
head(msstats_format_lf$PROTEIN)
}
